/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.impactanalysis.rmps.host.web                            |
 *|                                                                        |
 *| Copyright IBM Corporation 2012, 2014.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function () {
	//
	//	Implement DM diagram utility methods for Impact Analysis
	//

	dojo.provide("impactanalysis.app.rmps.DMDiagramUtilityMethods");

	dojo.require("dm.analysis.app.ui.DiagramUtilityMethods");

	dojo.require("rmps.core.RootServices");
	dojo.require("rmps.app.core.util");
	dojo.require("rmps.core.util.ContextUtils");
	dojo.require("rmps.core.util.IRmpsConstants");
	
	var REGISTERED_DM_URI = "http://jazz.net/ns/dm/",
		_isDMResource,
		getCompactRenderingUri,
		getResourceEditUri;
	
	var ACTION_PREFIX = "#action=";
	var IA_PREFIX = "dm.app.analysis.";

	var DIAGRAM_RESOURCE = "impactanalysis";
	var FILTER_RESOURCE = "impactconfig";
	var PROFILE_RESOURCE = "impactprofile";
	var MODELS_RESOURCE = "models";

	var EDIT_DIAGRAM_ACTION = "impactanalysis_impactAnalysis";
	var EDIT_FILTER_ACTION = "impactanalysis_iaConfig";
	var EDIT_PROFILE_ACTION = "impactanalysis_iaProfile";
	
	var util = rmps.app.core.util;

	/** Determine if a resource URI corresponds to a DM resource 
	 *  which can be rendered and edited. URIs that start with the
	 *  DM front-side URL or URIs that belong to the registered
	 *  DM name space on Jazz.net can be rendered by DM.
	 */
	_isDMResource = function (resourceUri) {
		return (resourceUri.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL) === 0 ||
				resourceUri.indexOf(REGISTERED_DM_URI) === 0);
	};

	/** Compute the URI to perform the compact rendering
	 *  of the corresponding resource. If the resource is
	 *  on DM, use the root services "rendering" URL.
	 *  Otherwise, use the resource URI itself.
	 */
	getCompactRenderingUri = function (resourceUri) {
		// If the resource is not from our server, use the resource URI as the rendering URI
		var renderingUri = resourceUri;
		if (_isDMResource(resourceUri)) {
			// Compute the URI for rendering the resource information
			var object= {
				resourceUri: resourceUri
			};
			var contextUri = rmps.core.util.ContextUtils.getCurrentContextURI();
			if (contextUri) {
				object[rmps.core.util.IRmpsConstants.BASELINE]= contextUri;
			}
			renderingUri = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "rendering") + "?" + dojo.objectToQuery(object);
		}
		
		return renderingUri;
	};


	/** Compute the URI to open the editor of 
	 *  the corresponding resource. If it's a IA resource
	 *  on DM, use design manager IA editor.
	 *  Otherwise, use the normal DM edit URI itself.
	 */
	getResourceEditUri = function (resourceUri) {
		if (_isDMResource(resourceUri)){
			var resource = resourceUri.replace(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL, "");
			resource = resource.split('/');

			var action = "";
			if (resource.length > 1){
				switch(resource[1]){
					case DIAGRAM_RESOURCE:
						action = EDIT_DIAGRAM_ACTION;
						break;
					case FILTER_RESOURCE:
						action = EDIT_FILTER_ACTION;
						break;
					case PROFILE_RESOURCE:
						action = EDIT_PROFILE_ACTION;
						break;
					case MODELS_RESOURCE:
						return util.getDesignViewResourceUri(resourceUri); 
					default:
						return resourceUri;
				}
			}
			
			action = ACTION_PREFIX + IA_PREFIX + action;
			var resource = '&' + dojo.objectToQuery({"resourceUri": resourceUri});
			return action + resource;
		}
	
		return resourceUri;
	};

	// Register the utility methods with the common IA
	dm.analysis.app.ui.DiagramUtilityMethods.set_getCompactRenderingUri(getCompactRenderingUri);
	dm.analysis.app.ui.DiagramUtilityMethods.set_getResourceEditUri(getResourceEditUri);

}());
