/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.impactanalysis.rmps.host.web                            |
 *|                                                                        |
 *| Copyright IBM Corporation 2012.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function () {
	// Integratate Impact Analysis toolbar icons into Design Manager.
	//
	// This code injects Impact Analysis Diagram actions into DM editors
	// via Dojo's aspect-orient features. This allows Impact Analysis to
	// have dependencies on DM, but not vice versa.
	//
	// NOTE: This code is activated by the following code in this plug-ins
	// plugin.xml:
	//   <extension
	//         point="com.ibm.xtools.rmps.web.app.webUiInitializers">
	//     <initializer
	//           jsModule="impactanalysis.app.rmps.DesignTabbedEditorAdvice">
	//        <classBinding
	//              jsClass="rmps.app.ui.internal.pages.design.DesignTabbedEditor">
	//        </classBinding>
	//     </initializer>
	//   </extension>

	dojo.provide("impactanalysis.app.rmps.DesignTabbedEditorAdvice");

	dojo.require("rmps.app.ui.internal.pages.design.DesignTabbedEditor");
	dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");
	dojo.require("dojox.lang.aspect");

	dojo.require("dojo.i18n");
	dojo.requireLocalization("impactanalysis.app.rmps", "Messages");
	var messages = dojo.i18n.getLocalization("impactanalysis.app.rmps", "Messages");

	// Add an Impact Analysis Diagram button to the DesignTabbedEdtior (the standard editor for editing in DM)
	// NOTE: This code depends on the fact that DesignTabbedEditor has a public addButtons method
	// that adds items to the editor's toolbar.
	dojox.lang.aspect.advise(rmps.app.ui.internal.pages.design.DesignTabbedEditor, "addButtons", {
		before: function () {
			dojo.hitch(dojox.lang.aspect.getContext().instance, function () {
				// dojo.hitch let's code run in the context of the DesignTabbedEditor instance (i.e. sets it as this)
				if (!this._iaMyToolbarButton) {
					this._iaMyToolbarButton = this.toolbar.addButton(this,
						dojo.hitch(this, function () {
							dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil.createAnalysisForResource(this.resourceInfo.uri);
						}),
						{
							label: messages.impactAnalysisButtonToolTip,
							imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/impact_select.gif")
						}
						);
				}
			})(); // run hitched function immediately
		}
	});

})();

