/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function () {
	dojo.provide('dm.analysis.app.internal.dtlfilters');
	
	dojo.require('dojox.dtl._base');
	
	// implement the filter function
	dojo.mixin(dm.analysis.app.internal.dtlfilters, {
		hackDoubleQuotes: function (value) {
			// summary:
			//		Implement a dojox.dtl filter to work around dojo diagrammer issues with double quotes in template bindings
			// description:
			//		Dojo diagrammer templates can contain bindings such as {{name}}. These templates are expanded via dojox.dtl
			//		in ibm_ilog.diagram.data.dtlbinding.updateShapeBoundProperty(). The template resource is a JSON string enclosed by
			//		double quotes. Method updateShapeBoundProperty() calls it's _fromJson(), and ultimately dojo.fromJson() to convert
			//		this into a string (without the quotes).
			//		THE PROBLEM: If {{name}} contains a double quote, the JSON string is invalid.
			//		THE SOLUTION: Replace {{name}} with {{name|hackDoubleQuotes|safe}} in the template.
			//		This method implements and registers the 'hackDoubleQuotes' filter. It replaces each double quote
			//		with: "+'"'+"  (all quotes are string literals)
			//		When the JSON evaluates, the above expression fragment yields a single double quote in the result.
			//		EXAMPLE:
			//			Original value of {{name}}:	---"---
			//			Hacked value:				---"+'"'+"---
			//			JSON string eval'd as:		"---"+'"'+"---"
			//			Result:						---"---
			//		

			value = value.replace(/"/g, '"+\'"\'+"');

			return value;
		}
	});

	// register the filter with DTL
	dojox.dtl.register.filters('dm.analysis.app.internal', { dtlfilters: ['hackDoubleQuotes'] });

}());