/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function () {
	dojo.provide('dm.analysis.app.ui.internal.diagram.DiagramLoader');

	dojo.require('dojo.i18n');
	dojo.require("dojo.data.ItemFileReadStore");

	dojo.require('dojox.lang.functional');
	
	dojo.require('dm.analysis.app.ui.internal.diagram.LongRunningWorkSequencer');
	dojo.require('dm.analysis.app.ui.internal.diagram.TimeSlicedIterator');
	
	dojo.requireLocalization('dm.analysis.app.ui.internal.impactanalysis', 'ImpactAnalysisMessages'); 
	
	var df,
		messages;
	
	df = dojox.lang.functional;
	messages = dojo.i18n.getLocalization('dm.analysis.app.ui.internal.impactanalysis', 'ImpactAnalysisMessages');
	
	dojo.declare('dm.analysis.app.ui.internal.diagram.DiagramLoader', [], {
		// summary:
		//		Perform asynchronous graph loading for impact analysis.
		// description:
		//		Extracts portions of ibm_ilog.diagram.widget.DataConnector (the portion of iLog
		//		that loads a graph from a data store), and makes the loading run
		//		'in the background'.  
		//
		//		Methods copied/hacked/modified from DataConnector all begin with _djd_.
		_diagram: null,

		_nodeStore: null,
		_linkStore: null,

		_fullNodeData: null,
		_fullLinkData: null,

		_nodeDataToAdd: null,
		_linkDataToAdd: null,

		iaCommon: null,

		constructor: function(diagram, iaCommon, nodeData, linkData) {
			this._diagram = diagram;
			this.iaCommon = iaCommon;

			this._fullNodeData = dojo.clone(nodeData);
			this._fullLinkData = dojo.clone(linkData);

			//Must clone the data, since 'fetching' an ItemFileReadStore alters the input data permanently
			this._nodeStore = new dojo.data.ItemFileReadStore({data: dojo.clone(this._fullNodeData)});
			this._linkStore = new dojo.data.ItemFileReadStore({data: dojo.clone(this._fullLinkData)});

			this._diagram.set("linksStore", this._linkStore);
			this._diagram.set("nodesStore", this._nodeStore);
		},
		
		_createLoadSequencer: function () {
			// summary:
			//		Defines the loading sequence for a diagram as a series of steps.
			//	description
			//		Much of this is extracted from DataConnector._load
			var sequencer;
			
			sequencer = new dm.analysis.app.ui.internal.diagram.LongRunningWorkSequencer()
				/*.step(messages.diagramLoadInit, dojo.hitch(this, function() {
					// N.B. At least in FF, hiding the surface slows things down !?!
						this._diagram.getSurface().rawNode.style.display = 'none';
				}))*/
				.step(messages.diagramLoadCreateModel, dojo.hitch(this, function(s, data) {
					// summary:
					//		Hack in our a private version on this._diagram._clearNodes 
					this._diagram._clearNodes = function() {
						// summary:
						//		Clear the current collection of nodes created from the nodes store.
						// tags:
						//		private
						// clear graphical shapes
						this._model.forEachEntry(function(e) {
							// remove all links attached to this node (both linksStore and implicits)
							// START_DM_CHANGE
							var links = e.node ? e.node.getLinks() : [];
							// END DM_CHANGE
							dojo.forEach(links, function(link) {
								this._linkDispose(link);
							}, this);
							// and dispose the node
							this._nodeDispose(e.node);
						}, this);
						// clear data model
						this._model.initNodes();
						// also the links since all the links shapes have been removed
						this._model.initLinks();
					};
				}))
				.step(messages.diagramLoadCreateModel, dojo.hitch(this, function(s, data) {
					// summary:
					//		Loads the 'node model' of the diagram. Isn't slow, but refactored
					//		this into a time sliced looper anyhow.
					var stepIncomplete;
					
					s.iterator = s.iterator 
						|| new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('nodes', data.resources);
						
					s.iterator.forWorkIncrement(this._diagram._addStoreItemToModel, this._diagram);
					
					stepIncomplete = s.iterator.hasMore();

					return stepIncomplete;
				}))
				.step(messages.diagramLoadCreateModel, dojo.hitch(this, function(s, data) {
					// summary:
					//		Loads the 'link model' of the diagram. Isn't slow, but refactored
					//		this into a time sliced looper anyhow.
					var stepIncomplete;
					
					s.iterator = s.iterator 
						|| new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('links', data.links);
						
					s.iterator.forWorkIncrement(this._djd_addLinkStoreItemToModel, this._diagram);
					
					stepIncomplete = s.iterator.hasMore();

					return stepIncomplete;
				}))
				.step(messages.diagramLoadCreateModel, dojo.hitch(this, function () {
					// summary:
					//		Complete model construction. Is not typically slow.
					dojo.hitch(this._diagram, this._djd_resolveConnectionsFromNodesStore)();
					dojo.hitch(this._diagram, this._djd_completeModelConstruction)();
				}))
				.step(messages.diagramLoadCreateShapes, dojo.hitch(this, function (s) {
					// summary:
					//		Incrementally create model shapes. Can be terribly slow, esp. in IE.
					var stepIncomplete;
					
					s.iterator = s.iterator || this._djd_init_shapeCreationIncrementalIterator();

					s.iterator.forWorkIncrement(
						this._djd_get_createShapesFromModel_workIncrement(),
						this._diagram);
					
					stepIncomplete = s.iterator.hasMore();

					return stepIncomplete;
				}))
				.step(messages.diagramLoadCreateShapes, dojo.hitch(this, function (s) {
					// summary:
					//		Incrementally create model links defined directly in nodes. 
					//		Can be terribly slow, esp. in IE, but in IA, is not because links are defined
					//		via a separate links store
					var stepIncomplete;
					
					s.iterator = s.iterator || this._djd_init_modelEntryIterator();

					s.iterator.forWorkIncrement(
						this._djd_createLinksFromModelEntry,
						this._diagram);
					
					stepIncomplete = s.iterator.hasMore();

					return stepIncomplete;
				}))
				.step(messages.diagramLoadCreateLinks, dojo.hitch(this, function (s) {
					// summary:
					//		Incrementally create model links in the links store. 
					//		Can be terribly slow, esp. in IE.
					var stepIncomplete;
					
					s.iterator = s.iterator || this._djd_init_linkEntryIterator();

					s.iterator.forWorkIncrement(
						this._djd_createLinksFromModelLinkEntry,
						this._diagram);
					
					stepIncomplete = s.iterator.hasMore();

					return stepIncomplete;
				}))
				.step(messages.diagramLoadLayout, function () { 
					// do nothing, just give UI time to refresh before beginning layout
					return;
				})
				.step(messages.diagramLoadLayout, dojo.hitch(this, function () {
					// summary:
					//		Signal layout completion. In IA, this triggers graph layout, which can be slow,
					//		but should be handled separately...
					dojo.hitch(this._diagram, 'onLoaded')();
				}))
				/*.step(messages.diagramLoadLayout, dojo.hitch(this, function () {
					//N.B. At least in FF, hiding the surface slows things down !?!
						this._diagram.getSurface().rawNode.style.display = '';
				}))*/
				;
			
			return sequencer;
		},

		createNodesAndLinks: function () {
			// summary:
			//		Asynchronously create diagram nodes and links from the node and link stores.
			// description:
			//		Asynchronously create diagram nodes and links from the node and link stores for the diagram.
			//		Returns a dojo.Deferred with with the progress and result of the process can be monitored:
			//			loader.createNodesAndLinks().then( 
			//				function () { /*handle completion*/ {,
			//				function (error) { /* handle error */ },
			//				function (progress) { /* handle progress */ });
			//		The object passed to the progress function has the following fields:
			//			message: a localized message suitable for user display
			var graphData, sequencer;
			
			// Code stolen from ibm_ilog.diagram.widget.DataConnector._load()
			if (this._diagram._isValidModel()) {
				this._diagram._reset();
				// end stolen code

				graphData = { };
				sequencer = this._createLoadSequencer();

				//Refresh the Node Store, since it may no longer represent the data in the graph
				this._nodeStore.clearOnClose = true;
				this._nodeStore.data = dojo.clone(this._fullNodeData);
				this._nodeStore.close();

				//Refresh the Link Store, since it may no longer represent the data in the graph
				this._linkStore.clearOnClose = true;
				this._linkStore.data = dojo.clone(this._fullLinkData);
				this._linkStore.close();
				
				this._nodeStore.fetch({onComplete: dojo.hitch(this, function (items) {
					graphData.resources = items;
					
					this._linkStore.fetch({onComplete: dojo.hitch(this, function (items) {
						graphData.links = items;
						
						sequencer.schedule(graphData);
					})});
				})});
			}
			return sequencer.getPromise();
		},

		_createExpansionSequencer: function () {
			// summary:
			//		Defines the sequence for expanding a node in a diagram as a series of steps.
			var sequencer;
			
			sequencer = new dm.analysis.app.ui.internal.diagram.LongRunningWorkSequencer()
			.step("Expanding node...", dojo.hitch(this, function(s, data){
				/*this._diagram._clearLinks = function() {
					// summary: 
					//     Clear the current collection of links created from the link store
					// tags:
					//	    private
					this._model.forEachLinkEntry(function(e) {
						var link = e.link;
						// START_DM_CHANGE
						var ld = link ? link.getData() : null;
						// END_DM_CHANGE
						if(ld && ld.store && ld.store === this._linksStore) {
							this._linkDispose(link);
						}
					}, this);
					this._model.initLinks();
				};*/
				return;
			}))
			//Step: Copied from full-graph sequencer
			.step(messages.diagramLoadCreateModel, dojo.hitch(this, function(s, data) {
				var stepIncomplete;
				s.iterator = s.iterator || new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('nodes', data.resources);
				s.iterator.forWorkIncrement(this._diagram._addStoreItemToModel, this._diagram);
				
				stepIncomplete = s.iterator.hasMore();
				return stepIncomplete;
			}))
			//Step: Copied from full-graph sequencer
			.step(messages.diagramLoadCreateModel, dojo.hitch(this, function(s, data) {
				var stepIncomplete;
				s.iterator = s.iterator || new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('links', data.links);
				s.iterator.forWorkIncrement(this._djd_addLinkStoreItemToModel, this._diagram);
				
				stepIncomplete = s.iterator.hasMore();
				return stepIncomplete;
			}))
			//Step: Copied from full-graph sequencer
			.step(messages.diagramLoadCreateModel, dojo.hitch(this, function() {
				dojo.hitch(this._diagram, this._djd_resolveConnectionsFromNodesStore)();
				dojo.hitch(this._diagram, this._djd_completeModelConstruction)();
			}))
			//Step: Instead of drawing all node shapes, only draw the new ones
			.step(messages.diagramLoadCreateShapes, dojo.hitch(this, function (s) {
				var stepIncomplete;
				s.iterator = s.iterator || this._djd_init_shapeCreationIncrementalIterator_forExpansion();
				s.iterator.forWorkIncrement(this._djd_get_createShapesFromModel_workIncrement(), this._diagram);
				
				stepIncomplete = s.iterator.hasMore();
				return stepIncomplete;
			}))
			//Step: Copied from full-graph sequencer
			.step(messages.diagramLoadCreateShapes, dojo.hitch(this, function(s) {
				var stepIncomplete;
				s.iterator = s.iterator || this._djd_init_modelEntryIterator();
				s.iterator.forWorkIncrement(this._djd_createLinksFromModelEntry, this._diagram);
				
				stepIncomplete = s.iterator.hasMore();
				return stepIncomplete;
			}))
			//Step: Instead of drawing all the links, only draw the new ones
			.step(messages.diagramLoadCreateLinks, dojo.hitch(this, function(s) {
				var stepIncomplete;
				s.iterator = s.iterator || this._djd_init_linkEntryIterator_forExpansion();
				s.iterator.forWorkIncrement(this._djd_createLinksFromModelLinkEntry, this._diagram);
				
				stepIncomplete = s.iterator.hasMore();
				return stepIncomplete;
			}))
			//Step: Copied from full-graph sequencer
			.step(messages.diagramLoadLayout, function() {
				return; // do nothing, just give UI time to refresh before beginning layout
			})
			//Step: Copied from full-graph sequencer
			.step(messages.diagramLoadLayout, dojo.hitch(this, function() {
				dojo.hitch(this._diagram, 'onLoaded', this._diagram.lastMenuTarget)();
			}));

			return sequencer;
		},

		_removeEncounteredData: function(nodeData, linkData){
			// summary:
			//		Modifies the input node/link data, removing nodes/links that already exist in the graph
			var self = this;

			var seenNodeItems = [];
			var unseenNodeItems = dojo.filter(nodeData.items, function(itemToAdd){
				return !dojo.some(self._fullNodeData.items, function(item){
					var match = itemToAdd.URI == item.URI;
					if (match) { seenNodeItems.push(itemToAdd); }
					return match;
				});
			});
			delete nodeData.items;
			nodeData.items = unseenNodeItems;
			self._mergeExistingNodesIntoDiagramModel(seenNodeItems);

			var unseenLinkItems = dojo.filter(linkData.items, function(itemToAdd){
				return !dojo.some(self._fullLinkData.items, function(item){
					return itemToAdd.URI == item.URI;
				});
			});
			delete linkData.items;
			linkData.items = unseenLinkItems;
		},

		/* 
		 * Draws the given nodes and links. The given nodes and links are assumed to be from an expansion
		 * of a single node.
		 *
		 * @return Promise for the expansion
		 */
		expandNodesAndLinks: function(newNodeData, newLinkData) {
			var graphData, sequencer;

			// Code stolen from ibm_ilog.diagram.widget.DataConnector._load()
			if (this._diagram._isValidModel()) {
				//this._diagram._reset();
				// end stolen code

				//This should trick the diagrammer into thinking there is more to do once we add the extra nodes and links
				//Normally calling _model.initNodes() would take care of this, but since it also empties the node model, we'll just do this ourselves
				this._diagram._model.allEntriesAdded = false;
				this._diagram._model.initialLoadComplete = false;

				//Remove nodes/links that have already been drawn in the graph
				this._removeEncounteredData(newNodeData, newLinkData);

				//If no new links are going to be added, then there can't be anything to draw
				if (!newLinkData || !newLinkData.items || newLinkData.items.length <= 0){
					return null;
				}

				this._nodeDataToAdd = newNodeData;
				this._linkDataToAdd = newLinkData;

				//Set the Node Store to the data you want to draw in the graph:
				this._nodeStore.clearOnClose = true;
				this._nodeStore.data = dojo.clone(this._nodeDataToAdd);
				this._nodeStore.close();

				//Set the Link Store to the data you want to draw in the graph:
				this._linkStore.clearOnClose = true;
				this._linkStore.data = dojo.clone(this._linkDataToAdd);
				this._linkStore.close();

				//Merge new nodes/links into our collection of the full data
				this._fullNodeData.items.push.apply(this._fullNodeData.items, this._nodeDataToAdd.items);
				this._fullLinkData.items.push.apply(this._fullLinkData.items, this._linkDataToAdd.items);

				graphData = { };
				sequencer = this._createExpansionSequencer();

				this._nodeStore.fetch({onComplete: dojo.hitch(this, function (items) {
					graphData.resources = items;
					
					this._linkStore.fetch({onComplete: dojo.hitch(this, function (items) {
						graphData.links = items;
						
						sequencer.schedule(graphData);
					})});
				})});
			}

			return sequencer.getPromise();
		},
		
		_createRemovalSequencer: function () {
			// summary:
			//		Defines the sequence for removing links and nodes from the diagram as a series of steps.
			var sequencer;
			
			sequencer = new dm.analysis.app.ui.internal.diagram.LongRunningWorkSequencer()
			.step("Collapsing node...", dojo.hitch(this, function(s, data){
				return;
			}))
			.step("Disposing nodes", dojo.hitch(this, function(s, data) {
				var stepIncomplete;
				s.iterator = s.iterator || new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('nodes', data.resources);
				s.iterator.forWorkIncrement(this._removeNode, this._diagram);
				
				stepIncomplete = s.iterator.hasMore();
				return stepIncomplete;
			}))
			.step("Disposing links", dojo.hitch(this, function(s, data) {
				var stepIncomplete;
				s.iterator = s.iterator || new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('links', data.links);
				s.iterator.forWorkIncrement(this._removeLink, this._diagram);
				
				stepIncomplete = s.iterator.hasMore();
				return stepIncomplete;
			}))
			//Step: Copied from full-graph sequencer
			.step("Resolving node/link models...", dojo.hitch(this, function() {
				return;
				//TODO: removing nodes/links from the model requires ~~the opposite of _djd_completeModelConstruction,
				//		where entries from the model are disconnected, i.e. entry.tolist, fromlist, childlist, parentlist.
				//dojo.hitch(this._diagram, this._djd_completeModelConstruction)();
			}))
			//Step: Copied from full-graph sequencer
			.step("Layout", function() {
				return; // do nothing, just give UI time to refresh before beginning layout
			})
			//Step: Do layout, then re-center the focus
			.step("Layout", dojo.hitch(this, function() {
				dojo.hitch(this._diagram, 'onLoaded', this._diagram.lastMenuTarget)();
			}));

			return sequencer;
		},

		_removeLinksFromFullLinkData: function(linksToRemove){
			this._fullLinkData.items = dojo.filter(this._fullLinkData.items, dojo.hitch(this, function(item){
				return (dojo.indexOf(linksToRemove, item.URI) === -1);
			}));
		},

		_removeNodesFromFullNodeData: function(nodesToRemove){
			this._fullNodeData.items = dojo.filter(this._fullNodeData.items, dojo.hitch(this, function(item){
				return (dojo.indexOf(nodesToRemove, item.URI) === -1);
			}));
		},

		_updateExistingNodesIntoFullNodeData: function(nodesToMerge){
			dojo.forEach(this._fullNodeData.items, function(item){
				for(var i = 0 ; i < nodesToMerge.length ; i++)
				{
					if (nodesToMerge[i].URI == item.URI){
						item = dojo.clone(nodesToMerge[i]);
						break;
					}
				}
			});
		},

		/* 
		 * Removes given nodes and links.
		 * Also, if updatedNodes !== null, updates expansion information of diagram model.
		 *
		 * @return Promise for the collapse
		 */
		collapseNodesAndLinks: function(linksToRemove, nodesToRemove, updatedNodes) {
			var graphData, sequencer;

			// Code stolen from ibm_ilog.diagram.widget.DataConnector._load()
			if (this._diagram._isValidModel()) {
				//this._diagram._reset();
				// end stolen code

				//This should trick the diagrammer into thinking there is more to do once we remove the nodes and links
				//Normally calling _model.initNodes() would take care of this, but since it also empties the node model, we'll just do this ourselves
				this._diagram._model.allEntriesAdded = false;
				this._diagram._model.initialLoadComplete = false;

				/* Keep only links that are in the diagram */
				var _linksToRemove = [];
				dojo.forEach(linksToRemove, dojo.hitch(this, function(link){
					if (this._linkUriExistsInModel(link)){
						_linksToRemove.push(link);
					}
				}));
				linksToRemove = _linksToRemove;

				/* Keep only nodes that are in the diagram */
				var _nodesToRemove = [];
				dojo.forEach(nodesToRemove, dojo.hitch(this, function(node){
					if (this._nodeUriExistsInModel(node)){
						_nodesToRemove.push(node);
					}
				}));
				nodesToRemove = _nodesToRemove;

				var _updatedNodes = [];
				dojo.forEach(updatedNodes, dojo.hitch(this, function(node){
					if (this._nodeUriExistsInModel(node.URI)){
						_updatedNodes.push(node);
					}
				}));
				updatedNodes = _updatedNodes;

				//If there are no links to remove, then there can't be anything to undraw
				if (!linksToRemove || linksToRemove.length <= 0){
					return null;
				}

				if (updatedNodes){
					this._updateExistingNodesIntoDiagramModel(updatedNodes);
					this._updateExistingNodesIntoFullNodeData(updatedNodes);
				}

				this._removeLinksFromFullLinkData(linksToRemove);
				this._removeNodesFromFullNodeData(nodesToRemove);

				graphData = { };
				sequencer = this._createRemovalSequencer();

				graphData.resources = nodesToRemove;
				graphData.links = linksToRemove;

				/*dojo.forEach(nodesToRemove, dojo.hitch(this, function(uri){
					dojo.hitch(this._diagram, this._removeNode)(uri);
				}));

				dojo.forEach(linksToRemove, dojo.hitch(this, function(uri){
					dojo.hitch(this._diagram, this._removeLink)(uri);
				}));*/

				sequencer.schedule(graphData);
			}

			return sequencer.getPromise();
		},

		_nodeUriExistsInModel: function(uri){
			var entry = this._diagram._model.getEntry(uri);

			return !!entry;
		},

		_linkUriExistsInModel: function(uri){
			var entry = this._diagram._model.getLinkEntry(uri);

			return !!entry;
		},

		/*
		 * This function allows us to pass in useful information (currently just Expansion Parents/Children) onto nodes
		 * in the diagram model.
		 *
		 * To be used ONLY when nodesToMerge contains information to REPLACE information in the model
		 */
		_updateExistingNodesIntoDiagramModel: function(nodesToMerge){
			dojo.forEach(nodesToMerge, dojo.hitch(this, function(nodeToMerge) {
				var existingNode = this._diagram._model.getEntry(nodeToMerge.URI).item;

				existingNode.expansionParents = nodeToMerge.expansionParents;
				existingNode.expansionChildren_Nodes = nodeToMerge.expansionChildren_Nodes;
				existingNode.expansionChildren_Links = nodeToMerge.expansionChildren_Links;

				/*this._completeModelConstruction();*/
			}));
		},

		/*
		 * This function allows us to pass in useful information (currently just Expansion Parents/Children) onto nodes
		 * in the diagram model.
		 * 
		 * To be used ONLY when nodesToMerge contains information to be concatenated into the model (versus replacement)
		 */
		_mergeExistingNodesIntoDiagramModel: function(nodesToMerge){
			dojo.forEach(nodesToMerge, dojo.hitch(this, function(nodeToMerge) {
				var existingNode = this._diagram._model.getEntry(nodeToMerge.URI).item;

				var parentsWithDuplicates = existingNode.expansionParents.concat(nodeToMerge.expansionParents);
				existingNode.expansionParents = []; //soon to be with no duplicates
				dojo.forEach(parentsWithDuplicates, function(parent){
					if (dojo.indexOf(existingNode.expansionParents, parent) === -1){
						existingNode.expansionParents.push(parent);
					}
				});

				//Don't merge children if it already exists, since there is no possible way to add new children to an already expanded node
				if (existingNode.expansionChildren_Nodes && existingNode.expansionChildren_Nodes.length === 0){
					existingNode.expansionChildren_Nodes = existingNode.expansionChildren_Nodes.concat(nodeToMerge.expansionChildren_Nodes);
				}
				if (existingNode.expansionChildren_Links && existingNode.expansionChildren_Links.length === 0){
					existingNode.expansionChildren_Links = existingNode.expansionChildren_Links.concat(nodeToMerge.expansionChildren_Links);
				}

				/*this._completeModelConstruction();*/
			}));
		},

		_djd_resolveConnectionsFromNodesStore: function () {
			// summary:
			//		modified version of _resolveConnectionsFromNodesStore that just does resolution, and not next step processing
			this._model.forEachUnresolvedEntry(function(entry) {
			
				var currentItem = entry.item;
				var connectingItems;
				
				if (this._parentBindingFunction) {
					connectingItems = this._parentBindingFunction.call(this, this._nodesStore, currentItem);
					dojo.forEach(connectingItems, function(connectingItem) {
						var ident = this._nodesStore.getIdentity(connectingItem);
						
						if (this._model.containsEntry(ident)) {
							entry.parentlist.push(ident);
						}
					}, this);
				}
				
				if (this._childBindingFunction) {
					connectingItems = this._childBindingFunction.call(this, this._nodesStore, currentItem);
					dojo.forEach(connectingItems, function(connectingItem) {
						var ident = this._nodesStore.getIdentity(connectingItem);
						if (this._model.containsEntry(ident)) {
							entry.childlist.push(ident);
							
						}
					}, this);
				}
				
				if (this._successorsBindingFunction) {
					connectingItems = this._successorsBindingFunction.call(this, this._nodesStore, currentItem);
					dojo.forEach(connectingItems, function(connectingItem) {
						var ident = this._nodesStore.getIdentity(connectingItem);
						if (this._model.containsEntry(ident)) {
							entry.tolist.push(ident);
						}
					}, this);
				}
				
				if (this._predecessorsBindingFunction) {
					connectingItems = this._predecessorsBindingFunction.call(this, this._nodesStore, currentItem);
					dojo.forEach(connectingItems, function(connectingItem) {
						var ident = this._nodesStore.getIdentity(connectingItem);
						if (this._model.containsEntry(ident)) {
							entry.fromlist.push(ident);
						}
					}, this);
				}
				
				entry.resolved = true;
			}, this);
			
			this._model.setAllEntriesAdded();
			
		},
		
		_djd_completeModelConstruction: function () {
			// summary:
			//		Work incrementalized version of _completeModelConstruction -- does not create shapes
			
			if(!this._model.isModelComplete()) {
				console.error('_djd_completeModelConstruction: model is not complete');
			}
			else { 
				// When all model entries have been creates and their items loaded.. 
				//                
				// Assumptions made for hierarchies
				// - An entry with an empty to-list is a leaf node 
				// - The from-list length will by not longer than 1
				
				this._model.forEachEntry(function(entry) {
					var childlist = entry.childlist;
					var parentlist = entry.parentlist;
					if (childlist.length > 0) {
						var t;
						for ( t = 0; t < childlist.length; t++) {
							var childEntry = this._model.getEntry(childlist[t]);
							childEntry.parent = entry;
							entry.children.push(childEntry);
						}
					}
					if (parentlist.length === 1) {
						var parentEntry = this._model.getEntry(parentlist[0]);
						entry.parent = parentEntry;
						parentEntry.children.push(entry);
					}
				}, this);
				
				this._onModelCreated(this._model);
			}
			
		},
		
		_djd_addLinkStoreItemToModel: function (item) {
			// code copied from ibm_ilog.diagram.widget.DataConnector._resolveConnectionsFromLinksStore
			var linkEntry = this._addLinkStoreItemToModel(item);
			linkEntry.startId = this._getStartLinkItemId(item);
			linkEntry.endId = this._getEndLinkItemId(item);
			
			// If we have enough information to describe a link then
			// update the model. 
			if (linkEntry.startId !== null) {
				linkEntry.start = this._model.getEntry(linkEntry.startId);
			}
			if (linkEntry.endId !== null) {
				linkEntry.end = this._model.getEntry(linkEntry.endId);
			}
			
			if (!linkEntry.start || !linkEntry.end) {
				linkEntry.pending = true;
			}
		},
		
		_djd_init_shapeCreationIncrementalIterator: function () {
			var iter,
				d = this._diagram;
			if (d._createLinksForHierarchy) {
				iter = this._djd_init_modelEntryIterator();
			} else {
				iter = new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('subgraphs', 
					df.filter(d._model.nmodel.getValueList(), function (v) { return !v.parent; })
				);
			}
			return iter;
		},
		
		_djd_init_shapeCreationIncrementalIterator_forExpansion: function () {
			var iter,
				d = this._diagram,
				self = this;
			if (d._createLinksForHierarchy) {
				iter = this._djd_init_modelEntryIterator();
			} else {
				var valueList = d._model.nmodel.getValueList();
				var onlyNewNodes = [];
				dojo.forEach(valueList, function(value){
					dojo.forEach(self._nodeDataToAdd.items, function(newNode){
						if (newNode.URI == value.id){
							onlyNewNodes.push(value);
						}
					});
				});
				iter = new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('subgraphs', 
					df.filter(onlyNewNodes, function (v) { return !v.parent; })
				);
			}
			return iter;
		},
		
		_djd_get_createShapesFromModel_workIncrement: function() {
			// summary:
			//		return a work increment for creating a shape
			
			var workFunction;
			var self = this;
			
			if (this._diagram._createLinksForHierarchy) {
				workFunction = function (entry) {
					entry.item.name[0] = self.toMultilineLabel(entry.item.name[0]);

					// Create nodes
					entry.node = this.createNode(null, entry.item);
					this._applyNodeStoreItemGraphics(entry.node);
				};
			} else {
				workFunction = function (entry) {
					entry.item.name[0] = self.toMultilineLabel(entry.item.name[0]);

					// Create subgraphs/nodes
					if (entry.isLeaf()) {
						//When we want to add custom templates for specific types of nodes, define them in ImpactAnalysisCommon.js and then you can use them here
						//var template = !self.iaCommon ? null : ibm_ilog.diagram.declareTemplate(self.iaCommon.getTemplate());

						entry.node = this.createNode(null, entry.item);
						this._applyNodeStoreItemGraphics(entry.node);
					} else {
						entry.node = this.createSubgraph(null, entry.item);
						this.createChildGraphics(entry);
						this._applyNodeStoreItemGraphics(entry.node);
					}
				};
			}
			return workFunction;
		},

		/**
		 * Converts a label string with no newline characters into a label with multiple readable lines if needed.
		 * @param  {string} singlelineLabel A string with no newline characters, just a single horizontally challenged string.
		 * @return {string}                 The input string split up into multiple lines, between 20 and 35 characters per line (not counting the last).
		 */
		toMultilineLabel: function(singlelineLabel) {
			var DELIMITERS = [' ','-'];
			var START_CHARACTER_POSITION = 20;
			var MAX_CHARACTER_POSITION = 35;

			var multilineLabel = "";

			var charPosition = 0;
			dojo.forEach(singlelineLabel, function(character){
				multilineLabel += character;
				charPosition += 1;

				if ((charPosition >= START_CHARACTER_POSITION && dojo.indexOf(DELIMITERS, character) != -1) || (charPosition >= MAX_CHARACTER_POSITION)){
					multilineLabel += '\n';
					charPosition = 0;
				}
			});

			return multilineLabel;
		},

		_djd_init_modelEntryIterator: function () {
			return new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('nodes', this._diagram._model.nmodel.getValueList());
		},
		
		_djd_init_linkEntryIterator: function () {
			return new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('links', this._diagram._model.lmodel.getValueList());
		},
		
		_djd_init_linkEntryIterator_forExpansion: function () {
			var self = this;
			var valueList = this._diagram._model.lmodel.getValueList();
			var onlyNewLinks = [];

			dojo.forEach(valueList, function(value){
				dojo.forEach(self._linkDataToAdd.items, function(newLink){
					if (newLink.URI == value.id){
						onlyNewLinks.push(value);
					}
				});
			});

			return new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('links', onlyNewLinks);
		},
		
		_djd_init_modelEntryIterator_forExpansion: function () {
			var self = this;
			var valueList = this._diagram._model.nmodel.getValueList();
			var onlyNewNodes = [];

			dojo.forEach(valueList, function(value){
				dojo.forEach(self._linkDataToAdd.items, function(newLink){
					if (newLink.URI == value.id){
						onlyNewNodes.push(value);
					}
				});
			});

			return new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('nodes', onlyNewNodes);
		},

		_djd_createLinksFromModelEntry: function (entry) {
			// summary:
			//		Functionality extracted from DataConnector._createLinksFromModel
			var currentNode;
			var childlist = entry.childlist;
			var parentlist = entry.parentlist;
			var tolist = entry.tolist;
			var fromlist = entry.fromlist;
			var link, t, f;
			if (this._createLinksForHierarchy) {
				if (childlist.length + parentlist.length > 0) {
					currentNode = entry.node;
					for (t = 0; t < childlist.length; t++) {
						var childNode = this._model.getEntry(childlist[t]).node;
						link = this.connectNodes(currentNode, childNode);
					}
					for (f = 0; f < parentlist.length; f++) {
						var parentNode = this._model.getEntry(parentlist[f]).node;
						link = this.connectNodes(parentNode, currentNode);
					}
				}
			}
			if (tolist.length + fromlist.length > 0) {
				currentNode = entry.node;
				for (t = 0; t < tolist.length; t++) {
					var toNode = this._model.getEntry(tolist[t]).node;
					link = this.connectNodes(currentNode, toNode);
				}
				for (f = 0; f < fromlist.length; f++) {
					var fromNode = this._model.getEntry(fromlist[f]).node;
					link = this.connectNodes(fromNode, currentNode);
				}
			}
		},
		
		_djd_createLinksFromModelLinkEntry: function (entry) {
			// summary:
			//		Functionality extracted from DataConnector._createLinksFromModel
			if (!entry.pending) {
				entry.link = this.connectNodes(entry.start.node, entry.end.node, null, entry.item);
				this._applyLinkStoreItemGraphics(entry.link);
			}
		},

		_removeNodesFromModel: function(uri){
			this._model.removeEntry(uri);
		},

		_removeLinksFromModel: function(uri){
			this._model.removeLinkEntry(uri);
		},

		_removeNode: function(uri){
			var entry = this._model.getEntry(uri);
			this._nodeDispose(entry.node);
			this._model.removeEntry(uri);
		},

		_removeLink: function(uri){
			var entry = this._model.getLinkEntry(uri);
			this._linkDispose(entry.link);
			this._model.removeLinkEntry(uri);
		},

		/*
		 * dataset - contains objects whose URI attribute is to be matched against input uri
		 * 
		 * COPIED from ImpactAnalysisDiagram.js
		 */
		_uriAppearsInDataset: function(uri, dataset){
			for(var i = 0 ; i < dataset.length ; i ++)
			{
				if (dataset[i].URI == uri)
					return true;
			}

			return false;
		}
	});
	
}());