/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function () {
	dojo.provide('dm.analysis.app.ui.internal.diagram.TimeSlicedIterator');
	
	dojo.require('dojo.i18n');
	dojo.require('dojo.string');

	dojo.requireLocalization('dm.analysis.app.ui.internal.impactanalysis', 'ImpactAnalysisMessages'); 

	var INCREMENT_LIMIT_IN_MS = 500,
		messages;

	messages = dojo.i18n.getLocalization('dm.analysis.app.ui.internal.impactanalysis', 'ImpactAnalysisMessages');
	
	dojo.declare('dm.analysis.app.ui.internal.diagram.ArrayIterator', [], {
		// summary:
		//		A dojox.lang.functional style iterator for arrays with hasNext() and next() methods.
		_theArray: null,
		_i: 0,
		_len: null,
		
		constructor: function (theArray) {
			this._theArray = theArray;
			this._len = theArray.length;
		},
		
		hasNext: function () {
			return this._i < this._len;
		},
		
		next: function () {
			var result;
			if(!this.hasNext()) {
				throw new Error("dm.analysis.app.ui.internal.diagram.ArrayIterator: Attempt to call next() past end.");
			}
			
			result = this._theArray[this._i];
			this._i += 1;
			
			return result;
		}
			
	});
	
	dojo.declare('dm.analysis.app.ui.internal.diagram.CollectionIterator', [], {
		// summary:
		//		A dojox.lang.functional style iterator for dojo.collections with hasNext() and next() methods.
	
		_colIterator: null,
	
		constructor: function (theCollection) {
			this._colIterator = theCollection.getIterator();
		},
		
		hasNext: function () {
			return ! this._colIterator.atEnd();
		},
		
		next: function () {
			return this._colIterator.get();
		}
	});

	dojo.declare('dm.analysis.app.ui.internal.diagram.TimeSlicedIterator', [], {
		// summary:
		//		Performs collection/array iteration in time slices so that browser responsiveness can be maintained.
		// description:
		//		The principal method is forWorkIncrement, which is a forEach variant, that processes elements in the collection
		//		until the time slice limit is reached. Subsequent calls to forWorkIncrement will resume the iteration. You
		//		can test whether another call is necessary by using hasNext()
		_name: null,
		_iterator: null,
		_stats : [],
		_total: 0,
		_n: 0,

		constructor: function(name, collection) {
			this.name = name;
			this._iterator = dojo.isArray(collection) 
				? new dm.analysis.app.ui.internal.diagram.ArrayIterator(collection) 
				: new dm.analysis.app.ui.internal.diagram.CollectionIterator(collection);
			if (dojo.isArray(collection)) {
				this._total = collection.length;
			}
		},
		
		forWorkIncrement: function (worker, scope) {
			// Summary:
			//		continue looping over the collection until done or the time increment is exceeded
			var start = new Date().getTime(),
				time, 
				numProcessed = 0;
			
			// CAUTION: Putting a break point on the while statement while cause the loop not to run
			// Break on the first statement of the loop body instead.
			while(this._iterator.hasNext() && (new Date().getTime() - start) < INCREMENT_LIMIT_IN_MS) {
				numProcessed += 1;
				worker.call(scope, this._iterator.next());
			}
			time = new Date().getTime() - start;
			this._stats.push({ numProcessed: numProcessed, time: time});
			this._n += numProcessed;
			if(time > INCREMENT_LIMIT_IN_MS && numProcessed === 1) {
				console.debug('TimeSlicedIterator: Exceeded time increment on a single element');
			}
		},
		
		hasMore: function () {
			// summary:
			//		Test whether there is more content to iterate (via forWorkIncrement)
			return this._iterator.hasNext();
		},
		
		getProgressMessage: function () {
			// summary:
			//		Return a simple progress message (e.g. 4 of 10), indicating progress. The message is localized.
			if(this._total > 0) { // we know the collection size
				return dojo.string.substitute(messages.iteratorProgressWithTotal, [this._n, this._total]);
			} else {
				return dojo.string.substitute(messages.iteratorProgress, [this._n]);
			}
		}
	});
}());

