/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.AllImpactAnalysisesEditor");

dojo.require("dijit._Widget");
dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.ui.editor.EditorUtil");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.BaseTable");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");
var AnalysisUtil = dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil;

dojo.require("jazz.ui.SimpleToolbar");
dojo.require('jazz.util.html');
dojo.require("rmps.ui.util.ToolbarFocusUtil");
var ToolbarFocusUtil = rmps.ui.util.ToolbarFocusUtil;
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil");

var UA_KEY = "com.ibm.xtools.impactanalysis.web.app.page_AllImpactAnalysis001";

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.AllImpactAnalysisesEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	isRefreshable: false,
	isReadOnly: true,	
	_diagramWidget: null,
	_content:null,
	editorInput:null,
	projectId:null,
	uaKey: UA_KEY,
	
	constructor: function (args) {
		this.editorInput= args.editorInput;
	},
	
	postCreate: function() {
		this.projectId = jazz.app.context.get().itemId;
		this.inherited(arguments);
		
	},
	
	createContentWidget: function() {		
		this._content = new dm.analysis.app.ui.internal.impactanalysis.AllImpactAnalysisesEditor._Content({editor: this});
		return this._content;
	},

	//Cloned from BaseEditor; prevents domNode from being adjusted
	resize: function(size){
		if (!size) {
			return;
		}

		this._lastSize = size;

		var cbContainer = dojo.contentBox(this.containerNode);
		var mbContainer = dojo.marginBox(this.containerNode);
		var mbHeader = dojo.marginBox(this.headerContainerNode);
		
		var contentSize = {};
		if (size.h) {
			contentSize.h = size.h - mbContainer.h + cbContainer.h - mbHeader.h;
			this.containerNode.style.minHeight = contentSize.h + "px";
		}
		if (size.w) {
			contentSize.w = size.w - mbContainer.w + cbContainer.w;
			this.containerNode.style.minWidth = contentSize.w + "px";
		}

		if (this.contentWidget && this.contentWidget.resize) {
			this.contentWidget.resize(contentSize);
		}
	},

	addButtons: function(toolbar) {
		toolbar.addButton(this, this.onRefreshClick, {
			label: messages.refresh,
			imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/refresh.gif")
		});
		toolbar.addSeparator(1);
	},
	
	addTextActions: function(){
		var self= this;
			self.addTextAction(messages.allAnalysisNew, 
			dojo.moduleUrl("dm.analysis.app", "graphics/icons/impact.gif"),
					dojo.hitch(self, self.onNewClick));
	},
	
	activate: function() {
		// until editing is supported, mark as dirty
	},
	
	onRefreshClick: function() {
		this._content.getAndPresentAllImpacted();
		
	},
	
	onNewClick: function() {
		dojo.hash(AnalysisUtil.getNewAnalysisUri());
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.AllImpactAnalysisesEditor._Content", [dijit._Widget, dijit._Templated], {
	
	messages: null, 
	
	widgetsInTemplate: true,
	templateString: '<div>' +
				'<div dojoAttachPoint="mainDiv">' +
				'</div>' +					
			'</div>',
	// init properties
	editor: null,
	_headers:null,
	_table:null,
	
	postCreate: function() {
		this._headers = this.createTableHeaders();
		this._table = new dm.analysis.app.ui.internal.impactanalysis.BaseTable();
		this._table.noEntriesLabel = messages.noIALabel;
		this._table.noFilterMatchesLabel = messages.noIAFilterMatchesLabel;
		
		this.getAndPresentAllImpacted();
		this.subscribe("dm.analysis.app.ui.internal.impactanalysis.refresh", dojo.hitch(this, this.refreshTable));
		
	},
	refreshTable: function()
	{
		this.getAndPresentAllImpacted();
	},
	
	
	createTableHeader: function(id, args, label) {
		var header = {
				isOrderable: false
			};
		dojo.mixin(header, args);
		header.id = id;
		header.label = label;
		return header;
	},
	
	createTableHeaders: function() {
		var headers = [
			this.createTableHeader("name", {filterParticipant: true, isOrderable: true} , messages.allAnalysisName),
			this.createTableHeader("element", {filterParticipant: true} , messages.allAnalysisElement),
			this.createTableHeader("actions", {}, messages.allAnalysisActions)
			
		];
		return headers;
	},
	
	getAndPresentAllImpacted: function()
	{
		var deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.getAllImpacted(this.editor.projectId);
		deferred.addCallback(this, function(result) {
			var tableData =[];
			dojo.forEach(result.ImpactList, function(entry) 
			{
				var resourceUri = entry.URI;
				var presentationUri = AnalysisUtil.getAnalysisUri(entry.URI);
				
				var row = {
					link: presentationUri,
					linkTarget: "_self",
					resourceUri: resourceUri,
					analysis: entry,
					columns: []
				};
				
				dojo.forEach(this._headers, function(header) {
					var column = this.createColumnObject(entry, header.id, resourceUri, presentationUri);
					row.columns.push(column);
				}, this);
				tableData.push(row);
			
			}, this);
			this._table.setTableData(this._headers, tableData);
			var seconderyOrder = [];
			seconderyOrder.push({id:"name" , isAscending: true});
			this._table.update(false,null,50,result.ImpactList.length,300,"name" , true , seconderyOrder);
			this.mainDiv.appendChild(this._table.domNode);
		});
		deferred.addErrback(this, function(error) {
			var xhr = deferred.ioArgs.xhr;
			if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
				var lqeInfo = dojo.fromJson(xhr.responseText);
				this.editor.showMessage({
					type: lqeInfo.type,
					summary: lqeInfo.summary,
					details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
					closable: true
				});
			} else {
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.errorGetAllIADiagrams,
					errorObject: error,
					closable: true
				});
			}
		});
	},
	
	createColumnObject: function(rowObj, columnId, resourceUri, presentationUri) {
		var result = {};
		if (columnId === "name") {
			result.data = rowObj.name;
			result.link = true;
		} else if (columnId === "element") {
			result.init = function (rowConfig, tableRow, rowIdx, columnConfig, tableCell, colIdx) {
				dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil.createResourceLink(tableCell, {
					uri: rowObj.selectedElementURI,
					label: jazz.util.html.escape(rowObj.selectedElementName || ' - ')
				});
			};
		} else if (columnId === "actions") {
			result.init = dojo.hitch(this, this.createRowActions);
		}	
		return result;
	},
	
	
	createRowActions: function(rowObj, tableRow, rowIdx, columnConfig, cell, colIdx) {
		// Get the impact analysis resource
		var analysis = rowObj.analysis;
		
		// Create a toolbar to be the content of the table cell
		analysis._actionToolbar = new jazz.ui.SimpleToolbar();  // ToolbarFocusUtil requires a toolbar
		dojo.addClass(cell, "actions");
		cell.appendChild(analysis._actionToolbar.domNode);
		
		// Add the "edit" button
		var button = analysis._actionToolbar.addButton(this, dojo.hitch(this, this._editAnalysis, analysis), { 
			imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/edit.gif"), 
			label: messages.allAnalysisEdit
			});		
		dojo.style(button.domNode, "visibility", "hidden");
		
		// Add the "delete" button
		button = analysis._actionToolbar.addButton(this, dojo.hitch(this, this._deleteAnalysis, analysis), {
			imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/remove.gif"), 
			label: messages.allAnalysisDelete
			});
		dojo.style(button.domNode, "visibility", "hidden");

		// Add listeners to show and hide the toolbar actions if necessary.
		analysis._actionToolbar.startup();
		ToolbarFocusUtil.addToolbarFocusListeners( { 
			actionToolbar: analysis._actionToolbar, 
			containerNode: tableRow, 
			actionClassName: ".jazz-ui-SimpleToolbar-button"
		});
	},
	
	_editAnalysis: function(analysis) {
		var analysisUri = AnalysisUtil.getAnalysisUri(analysis.URI);
		window.location.href = analysisUri + "&editMode=true";
	},
	
	_deleteAnalysis: function(analysis) {
		var substitutions = [analysis.name];
		var delMessage = dojo.string.substitute(messages.confirmDelete, substitutions);
		if (! confirm (delMessage))
		{
			return;
		}
		
		// destroy the toolbar so that the listeners in ToolbarFocusUtil are removed.
		if (analysis._actionToolbar) {
			analysis._actionToolbar.destroy();
			delete analysis._actionToolbar;
		}
		
		// The analysis resource in this browser has a "URI" property but the delete looks for a "uri" property.
		analysis.uri = analysis.URI;
		var deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.deleteImpactAnalysis(analysis);
		deferred.addCallback(this, function(ia) {
			dojo.publish("dm.analysis.app.ui.internal.impactanalysis.refresh");
			dojo.hash(AnalysisUtil.getAllAnalysisUri() + "&ts="+ new Date().getTime(), true); // true -> replace in history
			
			// close the editor, if it's open
			iaInput = new dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisInput({resourceUri: analysis.uri});
			pageHelper = rmps.ui.editor.PageHelper.getInstance().closeEditorInput(iaInput, true, true);
		});
		deferred.addErrback(this, function(error) {
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.cantDelete,
				errorObject: error,
				closable: true
			});
		});
	}
	
});
}());
