/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.DiagramNodeMenu");

dojo.require('jazz.ui.Menu'); //$NON-NLS-1$

dojo.require('dm.analysis.app.ui.internal.impactanalysis.util.DiagramMenuUtil');

var DiagramMenuUtil = dm.analysis.app.ui.internal.impactanalysis.util.DiagramMenuUtil;

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages");
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages");

/**
 * A wrapper for jazz.ui.Menu useful only for the iLOG Dojo Diagrammer nodes.
 * This class keeps enabling/disabling information about each menuItem, and will
 * apply them based on the context (i.e. the type of node that was clicked)
 */
dojo.declare("dm.analysis.app.ui.internal.impactanalysis.DiagramNodeMenu", [jazz.ui.Menu], 
{
	diagram: null,
	editor: null,

	_menuItems: null,
	
	constructor: function(args) {
		this.inherited(arguments);

		this.diagram = args.diagram;
		this.editor = args.editor;

		DiagramMenuUtil.init({ diagram: args.diagram, editor: args.editor });

		this._initMenuItems();
		
		//don't allow a browser right click menu on top of our custom context menu UNLESS we're in debug mode
		var debugMode = !DiagramMenuUtil.disableCallback_debug(null);
		if (!debugMode) this.connect(this.domNode, "oncontextmenu", function(e) { dojo.stopEvent(e); } );
	},

	_initMenuItems: function() {
		this._menuItems = new dm.analysis.app.ui.internal.impactanalysis.DiagramNodeMenu_MenuItems();

		this._menuItems.addNodeMenuItem({
			menuItem: this.menuItem_preview(),
			disableCallback: dojo.hitch(this, DiagramMenuUtil.disableCallback_preview)
		});

		this._menuItems.addNodeMenuItem({
			menuItem: this.menuItem_hide(),
			disableCallback: dojo.hitch(this, DiagramMenuUtil.disableCallback_hide)
		});

		var subMenu = new jazz.ui.Menu();
		var menuItem = this.menuItem_hideType(subMenu);
		this._menuItems.addNodeMenuItem({
			menuItem: menuItem,
			subMenu: subMenu,
			subMenuCallback: dojo.hitch(this, this.populateSubMenu_hideType),
			disableCallback: dojo.hitch(this, DiagramMenuUtil.disableCallback_hideType)
		});

		this._menuItems.addNodeMenuItem({
			menuItem: this.menuItem_expand(),
			disableCallback: dojo.hitch(this, DiagramMenuUtil.disableCallback_expand)
		});

		this._menuItems.addNodeMenuItem({
			menuItem: this.menuItem_collapse(),
			disableCallback: dojo.hitch(this, DiagramMenuUtil.disableCallback_collapse)
		});

		this._menuItems.addNodeMenuItem({
			menuItem: this.menuItem_debug(),
			disableCallback: dojo.hitch(this, DiagramMenuUtil.disableCallback_debug)
		});

		this.startup();
	},

	menuItem_expand: function() {
		var self = this;
		return new dijit.MenuItem({
			label: messages.actionExpand,
			iconClass: 'ia-icon-expand', //$NON-NLS-1$
			disabled: false,
			onClick: function(e) {
				if (!self.diagram)
					return;

				var node = self.diagram.impactDiagram.lastMenuTarget;
				DiagramMenuUtil.expandNode(node);
			}
		});

	},

	menuItem_collapse: function() {
		var self = this;
		return new dijit.MenuItem({
			label: messages.actionCollapse,
			iconClass: 'ia-icon-collapse', //$NON-NLS-1$
			disabled: false,
			onClick: function(e) {
				if (!self.diagram)
					return;

				var node = self.diagram.impactDiagram.lastMenuTarget;
				DiagramMenuUtil.collapseNode(node);
			}
		});

	},

	menuItem_debug: function() {
		var self = this;
		return new dijit.MenuItem({
			label: "Debug", //$NON-NLS-1$
			iconClass: 'ia-icon-debug', //$NON-NLS-1$
			disabled: false,
			onClick: dojo.hitch(this, function(){
				if (!this.diagram) return;
				DiagramMenuUtil.debugNode(this.diagram.impactDiagram.lastMenuTarget);
			})
		});
	},

	menuItem_preview: function() {
		var self = this;

		return new dijit.MenuItem({
			label: messages.actionPreview,
			iconClass: 'ia-icon-preview', //$NON-NLS-1$
			disabled: false,
			onClick: dojo.hitch(this, function(e) {
				if (!this.diagram) return;
				DiagramMenuUtil.previewNode(this.diagram.impactDiagram.lastMenuTarget);
			})
		});
	},

	menuItem_hide: function() {
		var self = this;

		return new dijit.MenuItem({
			label: messages.actionHideArtifact,
			iconClass: 'ia-icon-hide-artifact', //$NON-NLS-1$
			disabled: false,
			onClick: dojo.hitch(this, function() {
				var node = self.diagram.impactDiagram.lastMenuTarget;
				DiagramMenuUtil.hideArtifact(node, false);
			})
		});
	},

	menuItem_hideType: function(subMenu) {
		var self = this;

		return new dijit.PopupMenuItem({
			label: messages.actionHideType,
			iconClass: 'ia-icon-hide-type',
			popup: subMenu
		});
	},

	populateSubMenu_hideType: function(subMenu) {
		if (!subMenu){
			console.warn("Unable to populate sub-menu. It appears to be missing."); //$NON-NLS-1$
			return;
		}

		rdfTypes = DiagramMenuUtil.getNodeTypes(this.diagram.impactDiagram.lastMenuTarget);

		if (subMenu.hasChildren()){
			dojo.forEach(subMenu.getChildren(), function (child) {
				subMenu.removeChild(child);
			});
		}

		var self = this;
		dojo.forEach(rdfTypes, function (rdfType) {
			var typeName = DiagramMenuUtil.determineTypeName(rdfType);

			subMenu.addChild(new dijit.MenuItem({
				label: typeName,
				//iconClass: 'ia-icon-filter',
				disabled: false,
				onClick: function() {
					if(confirm(dojo.string.substitute(messages.confirmHideType, [typeName]))) {
						self.diagram.addArtifactQuickFilter(rdfType, typeName);
					}
				}
			}));
		});
	},

	/** 
	 * Call this to reset the items in the visible menu based on the node that was clicked
	 */
	updateMenuForNode: function(node) {
		//If it hasn't already been done, attempt to create a div for the Preview and place it at the node
		//It won't be shown yet, we just need to get it ready.
		DiagramMenuUtil.positionPreview(node);

		var self = this;

		//Remove everything from the visible menu
		dojo.forEach(this.getChildren(), function(child){
			self.removeChild(child);
		});

		//Add everything from our stored items into the visible menu, check with disableCallback
		//first to make sure you should, then populate the sub-menus for any menu item that has one
		dojo.forEach(this._menuItems.getNodeMenuItems(), function(nodeMenuItem) { 
			if (!nodeMenuItem.disableCallback(node)){
				self.addChild(nodeMenuItem.menuItem);
			}
			if (nodeMenuItem.subMenuCallback){
				nodeMenuItem.subMenuCallback(nodeMenuItem.subMenu);
			}
		});

	},

	/**
	 * Open the Menu at the specified node. 
	 * Useful for anytime you want to open the menu without a click.
	 */
	openAtNode: function(node) {
		this.diagram.impactDiagram.lastMenuTarget = node;
		this.diagram.impactDiagram.nodeMenu.updateMenuForNode(node);

		this._openMyself({
			target: node.rawNode
		});
	},

	/**
	 * Open the Menu in the Nav Tree for this node.
	 * Provide the contextMenu event arg so we know where to open the Menu.
	 */
	openAtNavTreeNode: function(uri, evt) {
		var node = this.diagram._getNode(uri);

		this.diagram.impactDiagram.lastMenuTarget = node;
		this.diagram.impactDiagram.nodeMenu.updateMenuForNode(node);

		this._openMyself(evt);
	},

	/** 
	 * When the diagram nodes and links are destroyed, the floating resource links need to be deleted.
	 */
	cleanup: function() {
		// Destroy all of the resource links
		DiagramMenuUtil.destroyResourceLinks();
	},
	
	/** 
	 * DiagramNodeMenu.destroyRecursive() shouldn't ever be called, since the menuItems aren't always
	 * ALL in the Menu, but they are always all in this._menuItems, so let us handle destroying them.
	 */
	destroy: function() {
		this.cleanup();
		this._menuItems.destroy();
		this.inherited(arguments);
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.DiagramNodeMenu_MenuItems", null, {

	_nodeMenuItems: [],

	constructor: function() {
		this._nodeMenuItems = [];
	},
	
	addNodeMenuItem: function(args) {
		if (args.menuItem && dojo.isFunction(args.disableCallback)) {
			var nodeMenuItem = {
				menuItem: args.menuItem,
				subMenu: args.subMenu,
				subMenuCallback: args.subMenuCallback,
				disableCallback: args.disableCallback
			};

			this._nodeMenuItems.push(nodeMenuItem);
		}
	},

	getNodeMenuItems: function() {
		return this._nodeMenuItems;
	},

	destroy: function() {
		dojo.forEach(this._nodeMenuItems, function(nodeMenuItem) { 
			nodeMenuItem.menuItem.destroyRecursive();
		});
	}
	  
});

})();
