 /*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisCommon");

dojo.require('ibm_ilog.diagram.Node');
dojo.require('ibm_ilog.diagram.Link');
dojo.require('ibm_ilog.graphlayout.tree.TreeLayout');
dojo.require('ibm_ilog.graphlayout.forcedirected.ForceDirectedLayout');
dojo.require('ibm_ilog.graphlayout.circular.CircularLayout');
dojo.require('ibm_ilog.graphlayout.hierarchical.HierarchicalLayout');
dojo.require('ibm_ilog.graphlayout.Direction');
dojo.require('ibm_ilog.diagram.overview.OverviewTopLevelTemplateRenderer');
dojo.require('ibm_ilog.diagram.gfxlayout.GridLayout');

dojo.require('dm.analysis.app.internal.dtlfilters');

ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "borderStyle", "solid");
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "borderWidth", 1);
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "cornerRadius", 5);
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "highlightColor", "white");

ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Link, "strokeColor", "#999999");
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Link, "strokeWidth", 1);
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Link, "selectedStrokeColor", "#008ABF");

ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "loadingImage", dojo.moduleUrl('dm.common.app', 'graphics/icons/loading.gif'));

var configs = {
			'Tree': {
				req: ['ibm_ilog.graphlayout.tree.TreeLayout'],
				f: function() {
					var layout = new ibm_ilog.graphlayout.tree.TreeLayout();
					layout.setGlobalLinkStyle(ibm_ilog.graphlayout.tree.TreeLayout.ORTHOGONAL_STYLE);
					layout.setFlowDirection(ibm_ilog.graphlayout.Direction.RIGHT);
					layout.setParentChildOffset(160);
					return layout;
				}
			},
			"Radial Tree" : {
				req: ['ibm_ilog.graphlayout.tree.TreeLayout'],
				f: function() {
					var layout = new ibm_ilog.graphlayout.tree.TreeLayout();
					layout.setGlobalLinkStyle(ibm_ilog.graphlayout.tree.TreeLayout.ORTHOGONAL_STYLE);
					layout.setFlowDirection(ibm_ilog.graphlayout.Direction.BOTTOM);
					layout.setLayoutMode(ibm_ilog.graphlayout.tree.TreeLayout.RADIAL);
					return layout;
				}
			},
			"Force-Directed" :{
				req:['ibm_ilog.graphlayout.forcedirected.ForceDirectedLayout'],
				f:function(){
					var layout = new ibm_ilog.graphlayout.forcedirected.ForceDirectedLayout();
					layout.setPreferredLinksLength(120);
					return layout;
				}
			},
			"Circular":{
				req:['ibm_ilog.graphlayout.circular.CircularLayout'],
				f:function(){
					return new ibm_ilog.graphlayout.circular.CircularLayout();
				}
			},			
			
			"Hierarchical":{
				req:['ibm_ilog.graphlayout.hierarchical.HierarchicalLayout'],
				f: function() {
					var layout = new ibm_ilog.graphlayout.hierarchical.HierarchicalLayout();
					layout.setGlobalLinkStyle(ibm_ilog.graphlayout.hierarchical.HierarchicalLayout.ORTHOGONAL_STYLE);
					layout.setFlowDirection(ibm_ilog.graphlayout.Direction.BOTTOM);
					layout.setVerticalNodeOffset(100);

					return layout;
				}
			}
		};

dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisCommon = {

		LAYOUT_HIERARCHICAL: 'Hierarchical',
		LAYOUT_TREE: 'Tree',
		LAYOUT_RADIAL_TREE: 'Radial Tree',
		LAYOUT_FORCE_DIRECTED: 'Force-Directed',
		LAYOUT_CIRCULAR: 'Circular',

		selectLayout:  function(layoutType){
			var cfg = configs[layoutType];
			if (!cfg) {
				throw new Error("Layout not supported: " + layoutType);
			}
            dojo.forEach(cfg.req, "dojo.require(item);");
			var layout = cfg.f();
			return layout;
		},

	

	addDecoration : function(link, text, position, type, orthogonal, parallel) {

		var deco = link.createGroup();
		deco.isDecoration = true;
		deco.autoRotate = false;
		var bg = deco.createRect({width: 50, height: 16});
		//bg.setFill('#eeeeee');
		//bg.setStroke('#808080');
		var label = deco.createText({text: text, y:10, align: 'middle'}); //$NON-NLS-1$
		label.setFill('black'); //$NON-NLS-1$
		if (dojo.isIE <= 8) {
			label.setFont({
				// IE 8 doesn't reliablly inherit the font size for SVG text
				// elements from parent element.s Set this explicitly to the
				// known body font size. (This looks a little big to me, but
				// it matches what other browsers are getting via
				// inheritance.)
				size: '12pt' //$NON-NLS-1$
			});
		}

		var textWidth = label.getTextWidth();
		bg.setShape({width: textWidth+6});
		label.setShape({x:textWidth/2+3});

		deco.positionType = type;
		deco.position = position;
		deco.orthogonalAlignment = orthogonal;
		deco.parallelAlignment = parallel;
		deco.distance = 1;
	},
	//moved
	doWordWrapping: function(node, width){
		// wrap text in a Task node (we know that the text is the 2nd child)
		// by adding other text shapes below (and centering the whole vertically).
		var textObj = node.textShape;
		var textShape = textObj.getShape();
		var text = textShape.text;
		var words = text.split(" ");
		if (words.length > 0) {
			var lines = [];
			var line = null;
			dojo.forEach(words, function (word) {
				if (!line) {
					line = word;
				} else {
					var newline = line + " " + word;
					textShape.text = newline;
					textObj.setShape(textShape);
					if (textObj.getTextWidth() > width) {
						lines.push(line);
						line = word;
					}
					else {
						line = newline;
					}
				}
			}, this);
			if (line) {
				lines.push(line);
			}
			var lineHeight = 15;
			if (lines.length > 1) {
				textShape.y -= (lineHeight * (lines.length - 1)) / 2;
				textShape.text = lines[0];
				textObj.setShape(textShape);
				dojo.forEach(lines, function (line) {
					textShape = dojo.clone(textShape);
					textShape.y += lineHeight;
					textShape.text = line;
					var font = textObj.getFont();
					var fill = textObj.getFill();
					textObj = node.createText(textShape);
					textObj.setFont(font);
					textObj.setFill(fill);
				}, this);
			}
		}
	} ,

	getTemplate: function()
	{
		var template =
		{
				layout: {
					type: 'ibm_ilog.diagram.gfxlayout.GridLayout', 
					// ensure the selection rectangle has 3px margins around the shape
					padding: [3,3,3,3]
				},
				children: [
					{
						// The primary shape, a rounded-cornered rectangle
						shape: {
							type: 'rect',
							width: 1,
							height: 1,
							r: '{{cornerRadius}}'
						},
						// minimumSize: {height:26},
						fill: '{{backgroundColor}}',
						stroke: {
							color: '{{borderColor}}',
							width: '{{borderWidth}}',
							style: '{{borderStyle}}'
						}, 
						selectedStyle: {
							stroke: {
								color: '#008ABF',
								width: '2px'
							}
						}
					}, 
					{
						layout: {
							type: 'ibm_ilog.diagram.gfxlayout.GridLayout',
							cols: [{
								width: 24
							}, {
								width: -1
							}, {
								width: 10
							}],
							rows: [{
								height: -1
							}]
						},
						
						children:[
						{
							column:0,row:0,
							valign:'center',
							dojoAttachPoint: 'elementImg',
							margins:[4,4,4,4],
							shape: {
								type: 'image',
								src: '{{loadingImage}}',
								height: 16,
								width: 16
							}
						},
						
						{
							column:1,row:0,
							valign:'center',
							margins:[0,4,0,4],
							shape: {
								type:'mltext', 
								text:'{{label|hackDoubleQuotes|safe}}', 
								align:'left',
								wrap: true
							}, 
							fill:'{{textColor}}',
							font:{type:'font', size:'12pt', family:'sans-serif', weight: 700}
						}
					]
				}]
			};

		return template;
	},

	defaultOverviewNodeTemplate : [{
	  dojoAttachPoint: 'baseShape',
	  shape: {
	      x: 0,
	      y: 0,
	      width: 80,
	      height: 40,
	      r: 5,
	      type: 'rect'
	  },
	  fill: '#A4A4A4',
	  stroke: {
	      'color': '{{borderColor}}',
	      'width': 2
	  }
	}],
	getOverviewRenderer: function ()
	{
		var OverviewNodeTemplate = this.defaultOverviewNodeTemplate;
		return  new ibm_ilog.diagram.overview.OverviewTopLevelTemplateRenderer({node:OverviewNodeTemplate});
	}
};

}());


