/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require('dojox.html.entities');

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.Select");
dojo.require("dijit.form.HorizontalSlider");

dojo.require("jazz.util.html");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.tree.IconItem");
dojo.require("jazz.ui.tree.SimpleItem");
dojo.require("rmps.ui.CollapsibleSection");

dojo.require('dm.analysis.app.ui.internal.diagram.DiagramLoader');

dojo.require("dm.analysis.app.ui.internal.impactanalysis.ILogConfig");
dojo.require('ibm_ilog.diagram.base');
dojo.require('ibm_ilog.diagram.data.dtlbinding');
dojo.require('ibm_ilog.diagram.Node');
dojo.require('ibm_ilog.diagram.Link');
dojo.require('ibm_ilog.diagram.widget.Diagram');
dojo.require('ibm_ilog.diagram.Selection');

dojo.require("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisCommon");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisTree");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.LegendDialog");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.IAColorTable");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.DiagramNodeMenu");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.PreviewLoader");
dojo.require('dm.analysis.app.ui.internal.impactanalysis.util.DiagramMenuUtil');

dojo.require("dm.analysis.app.ui.internal.impactanalysis.IAActionsCache");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages");
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages");
var common =  dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisCommon;
var colorTable = dm.analysis.app.ui.internal.impactanalysis.IAColorTable;
var DiagramMenuUtil = dm.analysis.app.ui.internal.impactanalysis.util.DiagramMenuUtil;

var minZoom = 0.1;
var maxZoom = 2;
var zoomFactor = 1.2;

dojo.require('ibm_ilog.diagram.interactors.DragInteractor'); //$NON-NLS-1$

if (dojo.isIE) {
	// IE with dojo 1.8 sometimes fires and even out of order that causes a
	// second drag interator to be added before the first is removed (if you
	// click twice sometimes, or click and hold a node). The end results seems
	// the same, but its temporarily in an invalid state and shows an error.
	// This just protects against that state. This is more common with IE 8,
	// but I can reproduce in IE 9 too, so always protect.
	var oldDragFirstMove = ibm_ilog.diagram.interactors.DragInteractor.prototype._dragFirstMove;
	if (!oldDragFirstMove) console.warn('Try tring to patch a deprecated method of DragInteractor. May need a new fix.'); //$NON-NLS-1$
	ibm_ilog.diagram.interactors.DragInteractor.prototype._dragFirstMove = function() {
		if (dojo.isIE && this._manager._stack.length) {
			console.debug('IE tried to add two interactors at once. Ignoring the second.'); //$NON-NLS-1$
			return;
		}
		oldDragFirstMove.apply(this, arguments)	;
	};
}
	
dojo.declare("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram", [dijit._Widget, dijit._Templated], {

	messages: messages,
	selectedItem: null,
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl('dm.analysis.app', 'ui/internal/impactanalysis/templates/ImpactAnalysisDiagram.html'),

	toolbarHeight: 35,
	TOOLBAR_HEIGHT: 35,

	// init properties
	//impactDiagram: null, //Instantiated in the html template
	editor: null,
	subgraphs: null,
	markSubgraph: null,
	treeSection: null,
	allElements: null, /* needed by ImpactAnalysisTree */
	allRelations: null, /* needed by DiagramMenuUtil when trying to look at node inlinks and outlinks */
	navTree: null, /* ImpactAnalysisTree, populated by loadTree() */
	diagramLoader: null, /* Creates and loads the Model, draws the nodes and links, and computes the layout of the graph */
	expansionSequence: [],
	combinedPartialResults: null, //A merging of all of the expansion data, to be drawn once all expansions are processed
	remainingExpansionsToProcess: null, //The number of result sets we should receive from the server before drawing combinedPartialResults
	warningMessage: null,
	
	// remember the node that has the focus
	_focusNode: null,

	// remember the graph node corresponding to the primary target
	_targetNode: null,

	// for removed elements
	originalResult: null,
	currentMapRes: null,

	/*
	 * capture for each removed elements what are the associated elements, result
	 * {URI:{elements:[] , relations:[]}}
	 */
	removedElement: null,
	distance: null,
	slider: null,
	toolbarSlider: null,
	listenToSlider: true,
	mustUpdateSlider: true,
	zoomChanged: false,

	toolbar: null,
	filterSelect: null,
	resetzoom: null,
	fitzoom: null,
	
	// the Promised returned by the IA Loader...
	_iaLoadPromise: null,

	_updateOverview: function ()
	{
		if (this._enabled && this._viewport.isHudEnabled()) {
			var sz = this._size;
			var hsz = this._viewport.getHudSize();
			this._overview.setSize(sz);
			this._background.setShape(sz);

			var location = { y: hsz.height - sz.height, x: 0 };
			
			if(!this._viewport.isRtl()){
				location.x = 13;
			}

			this._shape.setTransform(dojox.gfx.matrix.translate(location));
		}
	},
	postCreate: function() {
		DiagramMenuUtil.init({ diagram: this, editor: this.editor });
		this._loadToolButtons();
		
		////////////////////////////////////// create the main diagram diagram ///////////////////////////////////////////////////////

		this.impactDiagram.nodeMenu = new dm.analysis.app.ui.internal.impactanalysis.DiagramNodeMenu({
			editor: this.editor,
			diagram: this
		});

		this.impactDiagram.set('nodesQuery', {
			URI: '*'
		});
		this.impactDiagram.set('nodesQueryOptions', {
			deep: true
		});
		this.impactDiagram.set('childBinding', "nodeof");
		this.impactDiagram.set('createLinksForHierarchy', false);
		this.impactDiagram.set('nodeLayout', this._getCurrentLayout());
		this.impactDiagram.set('viewportResizeMode', 1);
		this.impactDiagram.set('automaticNodeLayout', false);
		this.impactDiagram.set('overview', true);
		this.impactDiagram.set('move', true);

		// Note: For Mobile browsers, marquee and mouse wheels are not implemented.
		// Trying to disable them on mobile browsers results in errors.
		if (!dojo.config.diagramForMobile) {
			this.impactDiagram.set('marquee', false); //disable selection rectangle.
			this.impactDiagram.set('wheelPan', false); //disable the vertical scrollbar.
		}
		dojo.style(this.impactDiagram.scrollbarsNode, 'overflow', 'hidden'); //hide the vertical and horizontal scrollbars
		//Because we are hiding the vertical and horizontal scrollbars we need to adjust the diagrams behavior to never to take the scrollbars into account when resizing
		this.impactDiagram._viewport._scrollbarWidth = 0;
		this.impactDiagram._viewport._computeScrollbarWidth = function() {
			return (this._scrollbarWidth = 0);
		};
		
		//Change the input filter for the Pan Interactor's start event to only require a left click (and not the shift key):
		this.impactDiagram.assignInteractorInputs("pan", { start: { filter: { button:0 } }});

		//Change the input filter for the Zoom Interactor's wheelZoom event to no longer require any keypress before zooming:
		this.impactDiagram.assignInteractorInputs("zoom", { wheelZoom: { filter: {} }});

		this.impactDiagram.set('viewportResizeMode', 1);
		//this.impactDiagram.set ('showZoomFactor',true);
		this.impactDiagram.set('showZoomFeedback', true);
		this.impactDiagram.set('viewportAnimated', true);
		this.impactDiagram.set('nodeTemplate', ibm_ilog.diagram.declareTemplate(common.getTemplate()));
		
		this.impactDiagram._setOverviewRendererAttr(common.getOverviewRenderer());
		
		var hudOverview = this.impactDiagram._hudOverview;
		hudOverview._background.setShape({
				'x': -0.5,
				'y': -0.5,
				'width': 160,
				'height': 120,
				'r': 0,
				'type': 'rect'
			});
		hudOverview._background.setStroke({'color': '2E5589', 'width': '1'});
		hudOverview._background.setFill([247, 247, 247, 0.8]);
		
		hudOverview._overview.setViewRectStroke({'color': '#e9c488', 'width': '0'});
		hudOverview._overview.setViewRectFill([254, 221, 168, 0.6]);
		
		hudOverview._update = this._updateOverview;		
		hudOverview._overview._updateViewRectShape();
		
		if (dojo.config.diagramForMobile) {
			this.connect(this.impactDiagram, "onNodeClick", this.onNodeClick);
		}

		this.connect(this.impactDiagram, "onNodeContextMenu", this.onNodeContextMenu);
		this.connect(this.domNode, "oncontextmenu", this.onContextMenu); //Override browser context menu
		this.connect(this.impactDiagram, "onNodeCreated", this.onNodeCreated);
		this.connect(this.impactDiagram, "onLoaded", this.performLayout);
		this.connect(this.impactDiagram, "onLinkCreated", this.onLinkCreated);
		this.connect(this.impactDiagram.getSelection(), "onSelectionChanged", this.onSelectionChanged);
		this.connect(this.impactDiagram, "onFocusedElementChanged", this.onFocusedElementChanged);
		this.connect(this.impactDiagram, "onKeyPress", this.onKeyPress);
		this.connect(this.impactDiagram, "onNodeMouseEnter", this.onNodeMouseEnter);
		this.connect(this.impactDiagram, "onNodeMouseLeave", this.onNodeMouseLeave);
		this.connect(this.impactDiagram, "onSubgraphCreated", this.setGridLayoutOnSubgraph);
		// zooming options
		var VP =  this.impactDiagram.getViewport();

		VP.setMaxZoom(maxZoom);
		VP.setMinZoom(minZoom);
		//VP.setZoom(0.8);
		//VP.setAirFactor(0.05);
		this.connect(VP, "onViewRectChanged", function(){ this.onViewportChange();});
		this.updateScrollbarsAndZoom();

		VP.setAnimated(true);
		this.impactDiagram.set ('overviewSize', {width: 160,height: 120 });
		VP.setHudEnabled(true);
		this.treeSection = this.editor.getTreeSideNav();
		this.expansionSequence = dojo.clone(this.editor.editorInput.impactAnalysis.expansionSequence);
		this.impactDiagram.domNode.style.position = 'absolute';
		
		// for Chrome: doesn't autosize the 'canvasNode' div to its content (the SVG)
		// Explicitly style it to 100% width.
		this.impactDiagram.canvasNode.style.width = '100%';
		
	},

	_isSelectedLayout: function(layoutToCheck)
	{
		if (this.editor.editorInput.impactAnalysis && this.editor.editorInput.impactAnalysis.layout)
		{
			layout = this.editor.editorInput.impactAnalysis.layout;
		}
		else
			layout = common.LAYOUT_HIERARCHICAL;
		
		return (layout === layoutToCheck);
	}, 
	
	_loadToolButtons: function()
	{
		var self = this;
		this.toolbar = new jazz.ui.SimpleToolbar({});
		dojo.style(this.toolbarNode, 'height', this.TOOLBAR_HEIGHT + 'px');
		
		this.toolbarNode.appendChild(this.toolbar.domNode);
		this.filterSelect = new dijit.form.Select({
			name: 'filterSelect',
			style: 'width:150px;',
			'aria-label':'Diagram View',
			options: [{
				label: messages.layoutHierarchical,
				value: common.LAYOUT_HIERARCHICAL,
				selected: this._isSelectedLayout(common.LAYOUT_HIERARCHICAL),
			},
			{
				label: messages.layoutTree,
				value: common.LAYOUT_TREE,
				selected: this._isSelectedLayout(common.LAYOUT_TREE),
			},
			{
				label: messages.layoutRadialTree,
				value: common.LAYOUT_RADIAL_TREE,
				selected: this._isSelectedLayout(common.LAYOUT_RADIAL_TREE),
			},
			{
				label: messages.layoutForceDirected,
				value: common.LAYOUT_FORCE_DIRECTED,
				selected: this._isSelectedLayout(common.LAYOUT_FORCE_DIRECTED),
			},
			{
				label: messages.layoutCircular,
				value: common.LAYOUT_CIRCULAR,
				selected: this._isSelectedLayout(common.LAYOUT_CIRCULAR),
			}]
		});
		this.connect(this.filterSelect, "onChange", function(){
			this.setLayout();
		});
	
		this.toolbar.addWidget(this.filterSelect);
		this.toolbar.addSeparator();
		
		this.toolbarSlider = new dijit.form.HorizontalSlider({
			name: "slider",
			'aria-label': 'Zoom Slider',
			value: 80,
			minimum: 5,
			maximum: maxZoom * 100,
			intermediateChanges: true,
			style: "width:200px;",
			onChange: function(value) {
				self.scalingEvent(value);
			}
		}, "slider");
		this.toolbar.addWidget(this.toolbarSlider);
		
		this.toolbar.addButton(this, this.resetZoom, {
			imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/resetzoom.png'),
			label:  messages.resetZoom
		});

		this.toolbar.addButton(this, this.fitZoom, {
			imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/zoomfit.png'),
			label: messages.zoomFit
		});
		
		this.toolbar.addSeparator();
		
		this.actionsMenu = this._constructActionsMenu();

		if (this.actionsMenu.getChildren().length > 0) {
			this.actionsButton = new jazz.ui.toolbar.MenuButton({
				label: messages.allAnalysisActions,
				labelPressed: messages.allAnalysisActions, // Work around display bug in MenuButton if there is no labelPressed
				imgClass: 'ia-icon-action', //$NON-NLS-1$
				enabled: true,
				menuProvider: function() {
					return self.actionsMenu;
				}
			});

			this.toolbar.addWidget(this.actionsButton, {});
			this.toolbar.addSeparator();
		}

		this.toolbar.addButton(this, this.openLegendDlg, {
			imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/legend.gif'),
			label: messages.openLegend
		});


		/* NODE TOOLBAR */

		this.nodeToolbar = new jazz.ui.SimpleToolbar({});
		this.nodeToolbarButtons = [];

		this.toolbarNode.appendChild(this.nodeToolbar.domNode);
		dojo.style(this.nodeToolbar.domNode, 'float', 'right');
		dojo.style(this.nodeToolbar.domNode, 'marginRight', '10px');

		this.previewArtifactButton = this.nodeToolbar.addButton(this, this.previewArtifact, {
			imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/preview.gif'),
			label:  messages.previewSelectedArtifact
		});
		this.previewArtifactButton.enableForNodes = DiagramMenuUtil.enableForNodes_preview;
		this.nodeToolbar.disableButton(this.previewArtifactButton);
		this.nodeToolbarButtons.push(this.previewArtifactButton);

		this.nodeToolbar.addSeparator();

		this.hideArtifactButton = this.nodeToolbar.addButton(this, this.hideArtifact, {
			imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/remove_grey.gif'),
			label:  messages.hideSelectedArtifacts
		});
		this.hideArtifactButton.enableForNodes = DiagramMenuUtil.enableForNodes_hide;
		this.nodeToolbar.disableButton(this.hideArtifactButton);
		this.nodeToolbarButtons.push(this.hideArtifactButton);

		this.hideArtifactTypeButton = new jazz.ui.toolbar.MenuButton({
			imgClass: 'ia-icon-hide-type',
			enabled: true,
			menuProvider: function() {
				return self._constructHideArtifactTypeMenu();
			},
			title: messages.hideSelectedTypes
		});
		this.nodeToolbar.addWidget(this.hideArtifactTypeButton);
		this.hideArtifactTypeButton.enableForNodes = DiagramMenuUtil.enableForNodes_hideType;
		this.hideArtifactTypeButton.set("enabled", false);
		this.nodeToolbarButtons.push(this.hideArtifactTypeButton);

		this.nodeToolbar.addSeparator();

		this.expandArtifactButton = this.nodeToolbar.addButton(this, this.expandArtifact, {
			imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/expand.png'),
			label:  messages.expandSelectedArtifact
		});
		this.nodeToolbar.disableButton(this.expandArtifactButton);
		this.expandArtifactButton.enableForNodes = DiagramMenuUtil.enableForNodes_expand;
		this.nodeToolbarButtons.push(this.expandArtifactButton);

		this.collapseArtifactButton = this.nodeToolbar.addButton(this, this.collapseArtifact, {
			imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/collapse.png'),
			label:  messages.collapseSelectedArtifact
		});
		this.nodeToolbar.disableButton(this.collapseArtifactButton);
		this.collapseArtifactButton.enableForNodes = DiagramMenuUtil.enableForNodes_collapse;
		this.nodeToolbarButtons.push(this.collapseArtifactButton);

		if (!DiagramMenuUtil.disableCallback_debug()){
			this.nodeToolbar.addSeparator();
			this.debugArtifactButton = this.nodeToolbar.addButton(this, this.debugArtifact, {
				imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/log-selected-item.gif'),
				label:  messages.debugSelectedArtifact
			});
			this.nodeToolbar.disableButton(this.debugArtifactButton);
			this.debugArtifactButton.enableForNodes = DiagramMenuUtil.enableForNodes_debug;
			this.nodeToolbarButtons.push(this.debugArtifactButton);
		}
    },
	
	_constructActionsMenu: function() {
		var self = this;
		var actionsMenu = new jazz.ui.Menu();

		dojo.forEach(dm.analysis.app.ui.internal.impactanalysis.IAActionsCache, function(action) {
			var menuItem = new dijit.MenuItem({
				label: action.label,
				onClick: function(e) {

					if (((action.uri.indexOf("${title}") > -1) || (action.uri.indexOf("${uri}") > -1)) && (!self.isSaved())) {
						self.editor.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
							summary: messages.mustSaveBeforeExecutingAction,
							closable: true
						});
						return;
					}
					var modifiedUri = action.uri.replace(/\$\{title\}/gi, self.editor.editorInput.resourceInfo.shortName);
					modifiedUri = modifiedUri.replace(/\$\{user\}/gi, self._getUserName());
					modifiedUri = modifiedUri.replace(/\$\{uri\}/gi, encodeURIComponent(window.location.href));

					if (action.openInNewWindow) {
						window.open(modifiedUri);
					} else {
						if ((action.uri.indexOf("mailto") == 0) && navigator.appName == 'Microsoft Internet Explorer') {
							//mailto only seems to work in IE on open(), not assign or location.href
							newWindow = window.open(modifiedUri, 'newWindow');
							if (newWindow && !newWindow.closed) {
								newWindow.close();
							}
						} else
							window.location.href = modifiedUri;
					}
				}
			});
			actionsMenu.addChild(menuItem);
		}, this);

		return actionsMenu;
	},
	
	isSaved: function()
	{
		return this.editor.editorInput.resourceInfo.uri != null;
	},
	_getUserName: function()
	{
		return dojo.getObject("jazz.app.currentApplication.config.user.name");
	},
	startup: function(){
		this.inherited(arguments);
	},
	
	//////// layouting
	setLayout: function(){
		var layoutType = this.filterSelect.value,
			i;
		this.impactDiagram.set("nodeLayout",  common.selectLayout(layoutType));
		this.impactDiagram.performGraphLayout();
		for (i = 0 ; i < this.subgraphs.length ; i++)
		{
			this.subgraphs[i].setCollapsed(true);
		}
		
		var oldValue = this.editor.editorInput.impactAnalysis.layout;
		this.editor._updateDirty(oldValue, layoutType, "layout");
	},

	_getCurrentLayout: function () {
		// summary:
		//		Return the Dojo Diagrammer layout corresponding to the current selected layout

		var layoutType = this.filterSelect.value || common.LAYOUT_HIERARCHICAL;
		return common.selectLayout(layoutType);
	},
	
	
	/////// handle zooming functions
	
	
	scalingEvent: function(value) {
		var V = this.impactDiagram.getViewport();
		//	var value = this.slider.value;
		var zoom = this.sliderToZoom(value);
		this.mustUpdateSlider = false;
		if(this.listenToSlider && zoom != V.getZoom()) {
			this.setZoom(zoom);
		}
		this.mustUpdateSlider = true;
	},
	
	
	updateScrollbarsAndZoom: function() {
		this.updateButtons();
		this.updateSlider();
	},
	

	// Update the UI when the zoom is changed
	onViewportChange : function(){
		this.updateScrollbarsAndZoom();
	},

	setZoom: function(zoom) {
		var V = this.impactDiagram.getViewport();
		if(zoom && zoom >= V.getMinZoom() && zoom <= V.getMaxZoom()) {
			V.setZoom(zoom, {
				instant: true
			});
		}
	},
	
	_focusOn: function(target) {
		setTimeout(dojo.hitch(this, function () { this.impactDiagram.centerOnNode(target); }), 100);
	},

	sliderToZoom : function(value){
		return value / 100;
	},

	zoomToSlider : function(zoom){
		return zoom * 100;
	},

	updateSlider : function(){
		if (this.mustUpdateSlider == true) {
			var zoom = this.impactDiagram.getViewport().getZoom();
			this.listenToSlider = false;
			this.toolbarSlider.set('value', this.zoomToSlider(zoom));
			this.listenToSlider = true;
		}
	},

	updateButtons : function(){
		var currentZoom = this.impactDiagram.getViewport().getZoom();
	},

	resetZoom : function(){
		this.setZoom(1);
		this._focusOn(this._targetNode);
	},

	fitZoom : function(){
		var vp = this.impactDiagram.getViewport();
		savedMaxZoom = vp.getMaxZoom();
	    vp.setMaxZoom((this.toolbarSlider.maximum/100));//Do not exceed the maximum zoom of the slider
		vp.setViewRect(vp.getContentRect(), {instant: true, center: true});
		vp.setMaxZoom(savedMaxZoom);
	},
	
	openLegendDlg : function()
	{
		if (this.originalResult){
			var legendDlg = new dm.analysis.app.ui.internal.impactanalysis.LegendDialog(this,this.originalResult.toolsData);
		}
	},
	
	/// finish zooming functions

	resize: function(size)
	{
		try {
			var diagramPos = dojo.position(this.impactDiagram.domNode);
			var contentPos = dojo.position('content-cell');
			var yOffset = diagramPos.y - contentPos.y;
			if (yOffset <= 0) return; //Diagram hasn't been placed within the content-area yet
			
			//Calculate the true content cell height, because attempting to get the height from content-cell directly is inconsistent depending on the browser.  Some browsers
			//send the resize event after content-cell has been resized and some browsers do it before content-cell has been resized.
			var viewport = dijit.getViewport();
			var footer = dojo.query('.jazz-ui-PageTemplate-footer')[0];
			var footerPos = dojo.position(footer);
			var trueContentCellHeight = viewport.h - contentPos.y - footerPos.h;  

			var newSize = { h: trueContentCellHeight - yOffset,
					w: dojo.contentBox(this.editor.contentWidget.domNode).w };
					
			this.impactDiagram.resize(newSize);
			
			this.editor.contentWidget._sidebars[0].resize();
		}
		catch (e) { }
	},

	reset: function(){
		if (this.navTree) this.navTree.destroyRecursive();
		this.navTree = null;

		this.initDiagramsAndTree();

		if (this.slider) this.slider.destroyRecursive();
		this.slider = null;

		this.editor.hideMessage();
	},

	/**
	 * Given the uri of an artifact that exists in the diagram, look into the model and grab the Node that represents it
	 */
	_getNode: function(artifactUri){
		return this.impactDiagram._model.getEntry(artifactUri).node;
	},
	
	selectAndFocusOn: function (uris) {
		// summary:
		//		Find, select and center on the artifact identified by the passed URI
		// var ns = this.impactDiagram.get('nodesStore');
		// this.impactDiagram.getSelection().clear();
		// dojo.forEach(uris, function (artifactUri) {
		// 	ns.fetchItemByIdentity({identity: artifactUri, onItem: dojo.hitch(this, function (item) {
		// 		var node = this.impactDiagram.getGraphElementFromDataItem(item);
		// 		this.impactDiagram.getSelection().add(item);
		// 		node.setSelected(true);
		// 		if (uris.length === 1) {
		// 			this.impactDiagram.centerOnNode(node, {animate: true});
		// 		}
		// 	})});
		// }, this);

		/* Below is a hack which does what the above commented code used to do. The hack is necessary because the addition of
		   expansion and collapse features to DiagramLoader have left the nodesStore unrepresentative of the diagram model */
		//TODO: Low priority since nothing else should be looking at nodesStore, but make the stores accurate again
		//		in DiagramLoader after an expansion/collapse
		this.impactDiagram.getSelection().clear();
		dojo.forEach(uris, function (artifactUri) {
			var node = this._getNode(artifactUri);
			this.impactDiagram.getSelection().add(node);
			node.setSelected(true);
			if (uris.length === 1) {
				this.impactDiagram.centerOnNode(node, {animate: true});
			}
		}, this);

	},

	//@param impactAnalysis - contains the data to show in the diagram, if null diagram will show local saved data
	loadData: function(impactAnalysis)
	{
		var ia;

		this.initDiagramsAndTree();
		ia = impactAnalysis ? impactAnalysis : this.editor.editorInput.impactAnalysis;

		if(ia.isReadyToRun()) {
			dojo.publish(jazz.ui.PageTemplate.StatusMessage.SHOW, [messages.computeImpactAnalysis]);
			var deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.computeImpactAnalysis(ia, messages.computeImpactAnalysis);
			deferred.addCallback(this, dojo.hitch(this, function(result) {
				if (!result || result.elementsData.items.length  === 0) {
					return; // no execution result
				}

				// Translate html escape characters in the result node names:
				dojo.forEach(result.elementsData.items, function(item){
					item.name = jazz.util.html.unescape(item.name);
				});

				// Initialize each of the original results to prepare for expansion/collapse
				dojo.forEach(result.elementsData.items, function(item){
					item.neverCollapse = true; // special tag so we don't remove this ever when collapsing nodes
					item.expansionParents = [];
					item.expansionChildren_Nodes = [];
					item.expansionChildren_Links = [];
				});

				this.originalResult = dojo.clone(result);
			
				this.initImpactData(result);
				this.loadDiagram(result);
				
				if (result.graphCutOff)
				{
					this.warningMessage = {
						type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
						summary: dojo.string.substitute(messages.impactAnalysisGraphCutOff, [result.totalDistance]),
						details: [{message: dojo.string.substitute(messages.toExpandFurther, [result.totalDistance, (result.analysisTimeLimit/1000)])}],
						closable: true
					};
				}
			}));
			deferred.addErrback(this, function(error){
				dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
				var xhr = deferred.ioArgs.xhr;
				if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
					var lqeInfo = dojo.fromJson(xhr.responseText);
					this.editor.showMessage({
						type: lqeInfo.type,
						summary: lqeInfo.summary,
						details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
						closable: true
					});
				} else if (xhr.getResponseHeader('user-facing-exception')){
					var jsonErrorResponse = dojo.fromJson(xhr.responseText);
					this.editor.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: jsonErrorResponse.summary,
						details: this.editor.getErrorMessageDetails(jsonErrorResponse),
						closable: true
					});
				} else if (deferred.cancelled){
					this.editor.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
						summary: messages.impactAnalysisRequestCancelled,
						errorObject: error,
						closable: true
					});
				} else {
					this.editor.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: messages.computeImpactAnalysisError,
						errorObject: error,
						closable: true
					});
				}
			});
		}
	},

	reloadDiagramAndTree: function (changeDistance)
	{
		this.initDiagramsAndTree();
		var result = dojo.clone(this.originalResult);
		this.initImpactData(result, changeDistance);
		this.loadDiagram(result);
	},
	
	loadTree:function(result)
	{
		if(!this.treeSection) return;

		dojo.style(this.treeSection, 'paddingLeft', '4px');
		dojo.style(this.treeSection, 'paddingTop', '4px');

		this.slider = new dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram._Slider({
			sresult: result,
			distance: this.distance,
			diagram: this,
			disabled: (!result.totalDistance || result.totalDistance < 2) 
		});
		this.treeSection.appendChild(this.slider.domNode);

		var viewTreeCaption =  dojo.doc.createElement("div");
		viewTreeCaption.innerHTML = messages.navigationTreeLabel;
		this.treeSection.appendChild(viewTreeCaption);
		
		if (this.navTree) this.navTree.destroyRecursive();
		this.navTree = new dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisTree({diagram: this});
		this.treeSection.appendChild(this.navTree.domNode);
		
		dojo.forEach(result.elementsData.items, function (artifact) {
			if (artifact.nodeof) {
				dojo.some(artifact.nodeof, function (node) {
					var found = false;
					if(node._reference === this.editor.editorInput.impactAnalysis.targetResourceUri.uri) {
						this.markSubgraph = artifact.URI;
						found = true;
					}
					return found;
				}, this);
			}
		}, this);

		this.navTree.loadTreeData(result.treeData, this.markSubgraph);
	},
	
	_checkForDiagramLimits: function (nArtifacts, nLinks) {
		var message,
			substitutions = {
				nArtifacts: nArtifacts,
				nLinks: nLinks
			};
		if (dojo.isIE === 8 && (nArtifacts > 25 || nLinks > 25)) {
			message = messages.largeDiagramWarningIE8;
		} else if (nArtifacts > 250 || nLinks > 250) {
			message = messages.largeDiagramWarning;
		}
		if (message) {
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
				summary: dojo.string.substitute(message, substitutions),
				closable: true
			});
		}
	},

	loadDiagram: function (result) {
		// pick up the data sources...
		this.allElements = result.elementsData;
		this.allRelations = result.relationsData;

		this._checkForDiagramLimits(result.elementsData.items.length, result.relationsData.items.length);

		// Asynchronous diagram load
		this.diagramLoader = new dm.analysis.app.ui.internal.diagram.DiagramLoader(this.impactDiagram, common, result.elementsData, result.relationsData);
		this.impactDiagram.set("nodeLayout", this._getCurrentLayout());
		
		this.editor.enableIAStopButton(true);

		this._iaLoadPromise = this.diagramLoader.createNodesAndLinks();
		var stopSubscription = dojo.subscribe('impactanalysis.stop', function () {
			this._iaLoadPromise.cancel();
			this._iaLoadPromise = null;
		});
		this._iaLoadPromise.then(dojo.hitch(this, function () {
			this.loadTree(result);
			this.editor.enableIAStopButton(false);
			dojo.unsubscribe(stopSubscription);
			dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
			this._iaLoadPromise = null;
			this.loadExpansions();
		}), dojo.hitch(this, function(errResult) {
			this.editor.enableIAStopButton(false);
			dojo.unsubscribe(stopSubscription);
			dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
			if(errResult.cancelled) {
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: messages.renderingCancelled,
					closable: true
				});
			} else {
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.computeImpactAnalysisError,
					errorObject: errResult,
					closable: true
				});
			}
			this._iaLoadPromise = null;
		}), dojo.hitch(this, function(progressResult) {
			dojo.publish(jazz.ui.PageTemplate.StatusMessage.SHOW, [progressResult.message]);
			console.info('Progress: ' + progressResult.message);
		}));
		
	},

	initDiagramsAndTree: function(){
		this.impactDiagram.reset();
		this.subgraphs = [];
		this.initTree();
		
		// Clean up any data that the popup menu may have created
		// for the nodes that were just deleted.
		this.impactDiagram.nodeMenu.cleanup();
	},

	initTree: function(){
		while(this.treeSection && this.treeSection.hasChildNodes()){
			this.treeSection.removeChild(this.treeSection.lastChild);
		}
	},

	/**
	 * Grab the types of the nodes from the tree and attach them to each of the nodes
	 */
	_determineTypes: function(result) {
		var treeDomains = result.treeData.items;

		dojo.forEach(result.elementsData.items, function(node) {
			//Only determine types if none are already determined
			if (!node.rdfTypes || node.rdfTypes.length > 0){
				var typesOfUri = [];

				//DOMAIN level:
				dojo.forEach(treeDomains, function (domain) {
					if (!domain.children || domain.children.length === 0) return;

					//In DM, we don't know the type of External artifacts, which are categorized under "Links"
					//"Links" actually just contains the link type to the artifact, and not the type of the artifact
					if (domain.name == "Links") return;

					//TYPE level:
					dojo.forEach(domain.children, function (type) {
						if (!type.children || type.children.length === 0) return;

						var typeUri = type.URI;

						//ARTIFACT level:
						dojo.some(type.children, function (artifact) {
							if (node.URI == artifact.URI){
								typesOfUri.push(typeUri);
								return true;
							}
						});
					});
				});

				node.rdfTypes = typesOfUri;
			}
		});
	},

	filterDistance : function (result, distance) {
		var updatedItemList = [],
		    updatedLinkList = [],
			updatedItemMap = {},
			updatedNodeofList;

		// include non-container items (those that are not logical dimensions)
		// that are within the specified distance
		dojo.forEach(result.elementsData.items, function (item) {
			if (!item.isLD && item.distance <= distance) {
				updatedItemList.push(item);
				updatedItemMap[item.URI] = item.URI;
			}
		});

		// include container items (i.e. logical dimensions) that
		// still reference items in the update map.
		dojo.forEach(result.elementsData.items, function (item) {
			updatedNodeofList = [];
			if (item.isLD == true) {
				dojo.forEach(item.nodeof, function (nodeof) {
					if (updatedItemMap[nodeof._reference]) {
						updatedNodeofList.push(nodeof);
					}
				});
				if (updatedNodeofList.length > 0) {
					item.nodeof = updatedNodeofList;
					updatedItemList.push(item);
				}
			}
		});

		// prune links, saving only those that join elements left in the graph.
		dojo.forEach(result.relationsData.items, function (link) {
			if (updatedItemMap[link.start] && updatedItemMap[link.end]) {
				updatedLinkList.push(link);
			}
		});
		result.elementsData.items = updatedItemList;
		result.relationsData.items = updatedLinkList;
	},
	
	_getTargetUris: function (ia) {
		// summary:
		//		Return an array of URI's that are targets of the given impact analysis
		var targetUris = [ia.targetResourceUri];
		dojo.forEach(ia.addedElements, function (uri) { targetUris.push(uri); });
		return targetUris;
	},
	
	_calculateReachables: function (urisToRemove, previousLevelUris, reachableUris, linksField, linkEndField) {
		var nextLevelUris = [];
		dojo.forEach(previousLevelUris, function (knownUri) {
			var item = this.currentMapRes.itemMap[knownUri];
			dojo.forEach(item[linksField], function (link) {
				var uri = link[linkEndField];
				if (!reachableUris[uri] && dojo.indexOf(urisToRemove, uri) === -1) {
					nextLevelUris.push(uri);
					reachableUris[uri] = uri;
				}
			}, this);
		}, this);
		
		if (nextLevelUris.length > 0) {
			this._calculateReachables(urisToRemove, nextLevelUris, reachableUris, linksField, linkEndField);
		}
	},
	
	_removeFromArray: function (arr, keyField, key) {
		dojo.some(arr, function (entry, i) {
			var found = false;
			if (entry[keyField] === key) {
				arr.splice(i, 1);
				found = true;
			}
			return found;
		});
	},
	
	filterRemovedElements2: function (result, urisToRemove) {
		var reachableUris = {},
			targetUris,
			unreachables = [];
		
		// start with the target artifacts...
		targetUris = this._getTargetUris(this.editor.editorInput.impactAnalysis);
		
		// make a 'map' of reachable uris
		dojo.forEach(targetUris, function (uri) {
			reachableUris[uri] = uri;
		});
		
		// don't let target artifacts be removed, not matter what...
		urisToRemove = dojo.filter(urisToRemove, function (uri) {
			return !reachableUris[uri];
		}, this);
		
		// figure out which other uri's we can reach, but up and down stream
		this._calculateReachables(urisToRemove, targetUris, reachableUris, "outLinks", "end");
		this._calculateReachables(urisToRemove, targetUris, reachableUris, "inLinks", "start");
		
		// calculate which ones are unreachable
		dojo.forEach(result.elementsData.items, function (artifact) {
			if(!reachableUris[artifact.URI]) {
				unreachables.push(artifact.URI);
			}
		});
		
		// remove the unreachables...
		dojo.forEach(unreachables, function (uri) {
			var item = this.currentMapRes.itemMap[uri],
				artifacts = result.elementsData.items,
				relations = result.relationsData.items;
			this._removeFromArray(artifacts, "URI", uri);
			dojo.forEach(item.inLinks, function (link) {
				this._removeFromArray(relations, "URI", link.URI);
			}, this);
			dojo.forEach(item.outLinks, function (link) {
				this._removeFromArray(relations, "URI", link.URI);
			}, this);
		}, this);
	},

	/**
	 * next - at first this is the targets + added elements.
	 */
	dfsScan : function (result /*result after remove */ ,current /*[]*/ , connectedMap /* - returned result {}*/)
	{
		var next = {},
			i,
			j;
		for(i = 0 ; i <current.length ; i ++)
		{
			
			var item = this.currentMapRes.itemMap[current[i]];
			connectedMap[current[i]] = current[i];
			var inlinks = item.inLinks;
			var outlinks = item.outLinks;
			for (j= 0 ; j < inlinks.length ; j++ )
			{
				if (connectedMap[inlinks[j].start] == null)//if not reached already
				{
					next[inlinks[j].start] = inlinks[j].start;
				}
			}
			for (j= 0 ; j < outlinks.length ; j++ )
			{
				if (connectedMap[outlinks[j].end] == null) //if not reached already
				{
					next[outlinks[j].end] = outlinks[j].end;
				}
			}
		}
		var nextArr = [];
		var key;
		for (key in  next)
		{
			nextArr.push(key);
		}
		if (nextArr.length > 0)
		{
			this.dfsScan(result ,nextArr, connectedMap);
		}
		
	},
	/*
	 * go over all the result elements and remove the elements that are not connceted and their links (in / out)
	 */
	removeDisconnectedElements: function(result, connectedMap /*{}*/) {
		var notConnected = [];
		result.elementsData.items = dojo.filter(result.elementsData.items, function (artifact) {
			var includeArtifact = true;
			if (!connectedMap[artifact.URI]) {
				notConnected.push(artifact.URI);
				includeArtifact = false;
			}
			return includeArtifact;
		});
	},
	
	calcMaxDistance: function(result)
	{
		var maxDistance = 0;
		dojo.forEach(result.elementsData.items, function (artifact) {
			if (artifact.isLD == false) {
				if (maxDistance < artifact.distance) {
					maxDistance = artifact.distance;
				}
			}
		}, this);
		this.distance = result.totalDistance = maxDistance;
	},
	
	_incrementMapCount: function (map, key) {
		map[key] = map[key] + 1 || 1;
	},
	
	_visitTreeData: function (rootItems, visitorMethod) {
		var inspectChildren;
		dojo.forEach(rootItems, function (treeItem) {
			inspectChildren = visitorMethod(treeItem);
			if(inspectChildren && treeItem.children) {
				this._visitTreeData(treeItem.children, visitorMethod);
			}
		}, this);
	},
	
	_visitTreeDataPostFix: function(rootItems, visitorMethod) {
		// summary:
		//		Traverse tree data in a post-fix fashion, visiting children before their parents.
		dojo.forEach(rootItems, function (treeItem, idx) {
			if(treeItem.children) {
				this._visitTreeDataPostFix(treeItem.children, visitorMethod);
			}
			visitorMethod(treeItem);
		}, this);
	},
	
	rebuildTheToolsLegendAndTree: function (result) {
		var artifactMap = [];
		
		var ldMap = {}; //{key:key val:key}
		var typesLDsMap = {}; //{key:key val:key}
		// update num of elements
		var isLDView = false;
		result.numOfElements = result.elementsData.items.length;
		result.treeData.numOfElements = result.elementsData.items.length	;
		
		// build maps
		dojo.forEach(result.elementsData.items, function (artifact) {
			artifactMap[artifact.URI] = artifact;
		});
		dojo.forEach(result.elementsData.items, function (artifact) {
			if (artifact.isLD == false && artifact.LDURI) {
				isLDView = true;
				this._incrementMapCount(ldMap, artifact.LDURI);
				this._incrementMapCount(typesLDsMap, artifact.LDURI + "_" + artifact.type);
			}
		}, this);

		
		if (isLDView) {
			//update ld tree
			var impacted = result.treeData.items[0];
			impacted.numOfElements = result.elementsData.items.length;
			var nonImpacted =  result.treeData.items[1];
			this.rebuildLDTreeElements(impacted,nonImpacted, typesLDsMap , ldMap);
		} else {
			//update structure tree with display counts.
			this._visitTreeDataPostFix(result.treeData.items, function (treeItem) {
				var numOfElements, artifact;
				if(treeItem.type === 'artifact') {
					artifact = artifactMap[treeItem.URI];
					treeItem.numOfElements =  artifact ? 1 : 0;
				} else {
					numOfElements = 0;
					dojo.forEach(treeItem.children, function (child) {
						numOfElements += child.numOfElements;
					});
					treeItem.numOfElements = numOfElements;
				}
			});
		}
	},

	rebuildLDTreeElements: function(treeElement ,nonImpactedList , typesLDsMap , ldMap )
	{
		var tmpChildren = [],
			i;
		for (i= 0; i < treeElement.children.length ; i ++ ){
			var tmpElement = treeElement.children[i];
			
			if (tmpElement.type == "element_type")
			{
				var counter = typesLDsMap[treeElement.URI + "_" + tmpElement.URI];
				if (counter  != undefined )
				{
					tmpElement.numOfElements = counter;
					tmpChildren.push(tmpElement);
				}
			}
			else // elt / e.g. system
			{
				var counter2 = ldMap[tmpElement.URI];
				if (counter2  != undefined)
				{
					tmpElement.numOfElements = counter2;
					tmpChildren.push(tmpElement);
				}
				else{
					var nonImpacted = {};
					nonImpacted.URI = tmpElement.URI;
					nonImpacted.name = tmpElement.name || '';
					nonImpacted.type = tmpElement.type;
					
					nonImpactedList.children.push(nonImpacted);
				}
				if (tmpElement.children && tmpElement.children.length > 0)
				{
					this.rebuildLDTreeElements(tmpElement ,nonImpactedList , typesLDsMap , ldMap);
				}
			}
		}
		treeElement.children = tmpChildren;
	},

	// Given a uri, look through the items of a dataset and remove the item it matches
	removeItemFromDataset: function(uri, dataset){
		for (var i=0; i<dataset.items.length; i++){
			var item = dataset.items[i];
			if (item.URI == uri){
				dataset.items.splice(i, 1);
				break;
			}
		}
	},

	// Given a uri, look through the items of dataset and return the item it matches
	getItemFromDataset: function(uri, dataset){
		var returnItem = null;
		dojo.forEach(dataset.items, function(item){
			if (item.URI == uri) { returnItem = item; }
		});
		
		return returnItem;
	},

	/*
	 * Recursive collapse starting at the node of the uri specified; traversal done with collapse data
	 * found on the nodes.
	 * 
	 * @param uri - uri of node to expand
	 * @param callback - called only if the graph actually changes as a result of this collapse, and
	 *					 is called with a list of all node uris that were actually collapsed
	 */
	collapseNode: function(uri, callback){
		var nodeToCollapse = this.getItemFromDataset(uri, this.originalResult.elementsData);
		var collapseResults = this._recursiveCollapseNode(nodeToCollapse, this.originalResult); //this.originalResult is modified

		this._determineTypes(this.originalResult); //this.originalResult is modified
		this._filterResultAndBuildTree(this.originalResult); //this.originalResult is modified

		var expansionPromise = this.diagramLoader.collapseNodesAndLinks(collapseResults.removedLinkUris, 
																		collapseResults.removedNodeUris, 
																		collapseResults.modifiedNodes);

		if (expansionPromise){
			expansionPromise.then(function() {
				dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
			}, function(progressResult) {
				dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
				console.error("Failed to collapse.");
			}, function(progressResult) {
				dojo.publish(jazz.ui.PageTemplate.StatusMessage.SHOW, [progressResult.message]);
				console.info('Progress: ' + progressResult.message);
			});

			if (collapseResults.collapsedNodeUris.length > 0 && callback && dojo.isFunction(callback)){
				callback(collapseResults.collapsedNodeUris);
			}
		}
	},

	/* 
	 * @param nodeToCollapse - node with URI and collapse data
	 * @param completeDataset - the complete dataset, in which to remove the resulting links/nodes from
	 *
	 * @returns results.removedLinkUris - list of link uris that we need to remove from the diagram,
	 *			results.removedNodeUris - list of node uris that we need to remove from the diagram,
	 *			results.modifiedNodes - list of NODES that need to be updated in the diagram,
	 *			results.collapsedNodeUris - list of node uris that were collapsed as a result of this recursive call
	 */
	_recursiveCollapseNode: function(nodeToCollapse, completeDataset){
		var returnResults = { removedLinkUris: [],
							  removedNodeUris: [],
							  modifiedNodes: [],
							  collapsedNodeUris: [nodeToCollapse.URI] };

		var childLinks = nodeToCollapse.expansionChildren_Links;
		var childNodes = nodeToCollapse.expansionChildren_Nodes;

		// Remove all child Links from nodeToCollapse as well as the completeDataset
		while (childLinks.length > 0){
			var childLinkUri = childLinks.pop();
			returnResults.removedLinkUris.push(childLinkUri);
			this.removeItemFromDataset(childLinkUri, completeDataset.relationsData);
		}

		var childrenToCollapse = [];

		//If there are children (meaning we can collapse this node), we are going to remove them from nodeToCollapse
		//which means we are modifying the node
		if (childNodes.length > 0){
			returnResults.modifiedNodes.push(nodeToCollapse);
		}

		// Remove all child Nodes from nodeToCollapse and then recursively collapse any that no longer
		// have parents. Children who have no parents and no children will be removed from completeDataset.
		while (childNodes.length > 0){
			var childNode = this.getItemFromDataset(childNodes.pop(), completeDataset.elementsData);
			if (childNode.neverCollapse) 
				continue;
			else {
				var parentsOfChild = childNode.expansionParents;
				parentsOfChild.splice(dojo.indexOf(parentsOfChild, nodeToCollapse.URI), 1);
				//If none of modifiedNodes IS already childNode, then add childNode to it
				if (!this._uriAppearsInDataset(childNode.URI, returnResults.modifiedNodes)){
					returnResults.modifiedNodes.push(childNode);
				}

				//If this child of nodeToCollapse has no more expansion parents left, let's kill it
				if (parentsOfChild.length === 0){
					if (childNode.expansionChildren_Links.length > 0){
						childrenToCollapse.push(childNode); //collapse children who have children
					}
					returnResults.removedNodeUris.push(childNode.URI);
					this.removeItemFromDataset(childNode.URI, completeDataset.elementsData);
				}
			}
		}

		//Recursively collapse the children that need collapsing
		dojo.forEach(childrenToCollapse, dojo.hitch(this, function(child){
			var recursiveResults = this._recursiveCollapseNode(child, completeDataset);

			returnResults.removedLinkUris = returnResults.removedLinkUris.concat(recursiveResults.removedLinkUris);
			returnResults.removedNodeUris = returnResults.removedNodeUris.concat(recursiveResults.removedNodeUris);
			returnResults.modifiedNodes = returnResults.modifiedNodes.concat(recursiveResults.modifiedNodes);
			returnResults.collapsedNodeUris = returnResults.collapsedNodeUris.concat(recursiveResults.collapsedNodeUris);
		}));

		//Remove Uris from modifiedNodes that are also listed in removedNodeUris, since we don't care about them anymore
		var onlySurvivingModifiedNodes = [];
		dojo.forEach(returnResults.modifiedNodes, function(modifiedNode){
			//If modifiedNode matches none of removedNodeUris
			if (!dojo.some(returnResults.removedNodeUris, function(removedNodeUri){
				return modifiedNode.URI == removedNodeUri;
			})){
				onlySurvivingModifiedNodes.push(modifiedNode);
			}
		});
		returnResults.modifiedNodes = onlySurvivingModifiedNodes;

		return returnResults;
	},

	_dataIntersection: function(largerDataset, smallerDataset) {
		var intersectData = dojo.clone(largerDataset);

		intersectData.items = dojo.filter(intersectData.items, dojo.hitch(this, function(item){
			return this._uriAppearsInDataset(item.URI, smallerDataset.items);
		}));

		return intersectData;
	},

	/*
	 * Makes a call to the server to figure out what is attached to this uri/node.
	 * Then figures out and draws the things we SHOULD DRAW in this diagram.
	 * 
	 * @param uri - uri of node to expand
	 * @param callback - called only if the graph actually changes as a result of this expansion
	 * @param autoExpansion - flag to indicate that this expansion is not being done by the user
	 *                        but is instead being done while the diagram is loading
	 */
	expandNode: function(uri, callback, autoExpansion) {
		dojo.publish(jazz.ui.PageTemplate.StatusMessage.SHOW, [messages.retrievingNodeInfo]);

		var deferred = this.editor.computeSingleNodeExpansion(uri);
		deferred.addCallback(this, dojo.hitch(this, function(partialResult) {
			dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
			if (!partialResult) {
				return; // no execution result
			}

			// Translate html escape characters in the result node names:
			dojo.forEach(partialResult.elementsData.items, function(item){
				item.name = jazz.util.html.unescape(item.name);
			});

			//Remove all links from the expansion result which are already in the diagram:
			dojo.forEach(this.originalResult.relationsData.items, dojo.hitch(this, function(item){
				this.removeItemFromDataset(item.URI, partialResult.relationsData);
			}));

			//Remove all nodes from the expansion result which don't have a link to/from it
			var nodesToRemove = [];
			dojo.forEach(partialResult.elementsData.items, dojo.hitch(this, function(nodeItem){
				var hasLink = dojo.some(partialResult.relationsData.items, dojo.hitch(this, function(linkItem){
					return (linkItem.start == nodeItem.URI || linkItem.end == nodeItem.URI);
				}));
				var isExpandingNode = (nodeItem.URI == uri);

				if (!hasLink && !isExpandingNode) {
					nodesToRemove.push(nodeItem.URI);
				}
			}));
			dojo.forEach(nodesToRemove, dojo.hitch(this, function(_uri){
				this.removeItemFromDataset(_uri, partialResult.elementsData);
			}));

			/* Mark nodes with what they are responsible for expanding, or what expanded them */
			var expandingItem = null;
			var expandingUris_Links = [];
			dojo.forEach(partialResult.relationsData.items, dojo.hitch(this, function(item){
				expandingUris_Links.push(item.URI);
			}));
			var expandingUris_Nodes = [];
			dojo.forEach(partialResult.elementsData.items, dojo.hitch(this, function(item){
				if (item.URI == uri){
					expandingItem = item;
					expandingItem.expansionParents = [];
				} else {
					expandingUris_Nodes.push(item.URI);
					item.expansionParents = [uri];
					item.expansionChildren_Nodes = [];
					item.expansionChildren_Links = [];
				}
			}));
			expandingItem.expansionChildren_Nodes = expandingUris_Nodes;
			expandingItem.expansionChildren_Links = expandingUris_Links;

			/* Here we are trying to combine all of the expansions quantified by this.remainingExpansionsToProcess,
			   so that we can draw them all at once. */
			if (this.combinedPartialResults === null){
				this.combinedPartialResults = partialResult;
			} else {
				this.combinedPartialResults = this._mergeResults(this.combinedPartialResults, partialResult);
			}
			this.remainingExpansionsToProcess--;

			if (this.remainingExpansionsToProcess <= 0){
				//combinedResult contains everything we SHOULD show in the diagram
				var combinedResult = this.initPartialData(this.combinedPartialResults);

				/* Remove disconnected components (For example when something was expanded earlier but isn't connected to the graph anymore) */
				var sources = dojo.clone(this.editor.editorInput.impactAnalysis.addedElements);
				sources.push(this.editor.editorInput.impactAnalysis.targetResourceUri);
				var connectedMap = {};
				this.dfsScan(combinedResult, sources, connectedMap);
				this.removeDisconnectedElements(combinedResult, connectedMap);

				//Right now, we only want to DRAW nodes/links from partialResult.
				//So "combinedResult INTERSECTION partialResult" gives us what we SHOULD DRAW from partialResult.
				var partialResult_filtered = {};
				partialResult_filtered.elementsData = this._dataIntersection(combinedResult.elementsData, this.combinedPartialResults.elementsData);
				partialResult_filtered.relationsData = this._dataIntersection(combinedResult.relationsData, this.combinedPartialResults.relationsData);

				dojo.publish(jazz.ui.PageTemplate.StatusMessage.SHOW, [messages.expandingNode]);
				var expansionPromise = this.diagramLoader.expandNodesAndLinks(partialResult_filtered.elementsData, partialResult_filtered.relationsData);
				this.combinedPartialResults = null;

				if (expansionPromise){
					expansionPromise.then(function() {
						dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
					}, function(progressResult) {
						dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
						console.error("Failed to draw expansion.");
					}, function(progressResult) {
						dojo.publish(jazz.ui.PageTemplate.StatusMessage.SHOW, [progressResult.message]);
						console.info('Progress: ' + progressResult.message);
					});

					if (callback && dojo.isFunction(callback)){
						callback();
					}
				} else if (!autoExpansion) { //the user doesn't need to see this while the diagram is loading the expansions
					dojo.publish(jazz.ui.PageTemplate.StatusMessage.SHOW, [messages.nothingExpanded]);
					setTimeout(dojo.hitch(this, function () { dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE); }), 3000);
				} else {
					dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
				}
			}
		}));
		deferred.addErrback(this, dojo.hitch(this, function(error){
			dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.errorCannotExpandArtifact,
				errorObject: error,
				closable: true
			});
		}));
	},

	loadExpansions: function() {
		this.remainingExpansionsToProcess = this.expansionSequence.length;
		dojo.forEach(this.expansionSequence, dojo.hitch(this, function(expansionUri) {
			this.expandNode(expansionUri, null, true);
		}));
	},
	
	/*
	 * Merges partial result sets, including the tree data.
	 * Also merges Expansion information for nodes (except when neverCollapse==true)
	 * 
	 * @return The merged result set, with no duplicates.
	 */
	_mergeResults: function(largerDataset, smallerDataset){
		var mergedResult = dojo.clone(largerDataset);

		/* Elements Data (Nodes/Artifacts): */
		dojo.forEach(smallerDataset.elementsData.items, function(partialItem){
			var match = false;
			dojo.forEach(mergedResult.elementsData.items, function(originalItem){
				//Merge EXPANSION information only
				if (partialItem.URI == originalItem.URI){
					match = true;

					//neverCollapse:
					originalItem.neverCollapse = partialItem.neverCollapse ? partialItem.neverCollapse : originalItem.neverCollapse;

					//expansionParents:
					if (!originalItem.neverCollapse){
						var parentsWithDuplicates = originalItem.expansionParents.concat(partialItem.expansionParents);
						originalItem.expansionParents = []; //soon to be with no duplicates
						dojo.forEach(parentsWithDuplicates, function(parent){
							if (dojo.indexOf(originalItem.expansionParents, parent) === -1){
								originalItem.expansionParents.push(parent);
							}
						});
					}

					//expansionChildren:
					originalItem.expansionChildren_Nodes = originalItem.expansionChildren_Nodes.concat(partialItem.expansionChildren_Nodes);
					originalItem.expansionChildren_Links = originalItem.expansionChildren_Links.concat(partialItem.expansionChildren_Links);
				}
			});

			//If partialItem doesn't exist in largerDataset, add it
			if (!match){
				mergedResult.elementsData.items.push(dojo.clone(partialItem));
			}
		});

		/* Relations Data (Links): */
		dojo.forEach(smallerDataset.relationsData.items, dojo.hitch(this, function(partialItem){
			//If partialItem doesn't exist in largerDataset, add it to the mergedResult
			if (!this._uriAppearsInDataset(partialItem.URI, mergedResult.relationsData.items)){
				mergedResult.relationsData.items.push(dojo.clone(partialItem));
			}
		}));

		/* Tools Data: */
		dojo.forEach(smallerDataset.toolsData.items, dojo.hitch(this, function(partialItem){
			//If partialItem doesn't exist in largerDataset, add it to the mergedResult
			if (!this._uriAppearsInDataset(partialItem.URI, mergedResult.toolsData.items)){
				mergedResult.toolsData.items.push(dojo.clone(partialItem));
			}
		}));

		/* Tree Data: */
		var mergedTree = mergedResult.treeData;
		var partialTree = smallerDataset.treeData;

		//DOMAIN level:
		dojo.forEach(partialTree.items, function(pDomain){
			var matchedDomainItem = null;

			if (!dojo.some(mergedTree.items, function(mDomain){
				var match = pDomain.URI == mDomain.URI;
				if (match){ matchedDomainItem = mDomain; }
				return match;
			})){
				mergedTree.items.push(dojo.clone(pDomain));
			}
			else {
				if (!matchedDomainItem || !matchedDomainItem.children || matchedDomainItem.children.length === 0 || 
					!pDomain.children || pDomain.children.length === 0) {
					return;
				}

				//TYPE level:
				dojo.forEach(pDomain.children, function (pType){
					var matchedTypeItem = null;

					if (!dojo.some(matchedDomainItem.children, function(mType){
						var match = pType.URI == mType.URI;
						if (match){ matchedTypeItem = mType; }
						return match;
					})){
						matchedDomainItem.children.push(dojo.clone(pType));
					}
					else {
						if (!matchedTypeItem || !matchedTypeItem.children || matchedTypeItem.children.length === 0 || 
							!pType.children || pType.children.length === 0) {
							return;
						}

						//ARTIFACT level:
						dojo.forEach(pType.children, function (pArtifact){
							if(!dojo.some(matchedTypeItem.children, function(mArtifact) {
								return pArtifact.URI == mArtifact.URI;
							})) {
								matchedTypeItem.children.push(dojo.clone(pArtifact));
							}
						});
					}
				});
			}
		});

		return mergedResult;
	},

	/*
	 * dataset - contains objects whose URI attribute is to be matched against input uri
	 */
	_uriAppearsInDataset: function(uri, dataset){
		for(var i = 0 ; i < dataset.length ; i ++)
		{
			if (dataset[i].URI == uri)
				return true;
		}

		return false;
	},

	/*
	 * Removes hidden/filtered artifacts from the provided dataset, then rebuilds the navigation tree
	 * and Legend Dialog based on that final dataset.
	 *
	 * Input is modified.
	 */
	_filterResultAndBuildTree: function(result){

		//Remove nodes from explicitly 'Hidden Artifacts'
		if (this.editor.editorInput.impactAnalysis.removedElements.length > 0){
			this.filterRemovedElements2(result, this.editor.editorInput.impactAnalysis.removedElements);

			// remove disconnected elements (from the target + added elements non-directed graph)
			var current = dojo.clone(this.editor.editorInput.impactAnalysis.addedElements);
			current.push(this.editor.editorInput.impactAnalysis.targetResourceUri);
			var connectedMap = {};
			this.dfsScan(result, current, connectedMap);
			this.removeDisconnectedElements(result, connectedMap);
		}

		//Remove nodes that have a type appearing in the list of quick filters
		var artifactQuickFilterTypes = this.editor.editorInput.impactAnalysis.artifactQuickFilters;
		if (artifactQuickFilterTypes.length > 0){
			var quickFilterElementsToRemove = [];
			dojo.forEach(result.elementsData.items, function(element) {
				var typeIsFiltered = dojo.indexOf(artifactQuickFilterTypes, element.type) !== -1 || 
									dojo.some(artifactQuickFilterTypes, function (type) {
										return dojo.indexOf(element.rdfTypes, type) !== -1;
									});
				if (typeIsFiltered){
					quickFilterElementsToRemove.push(element.URI);
				}
			});
			this.filterRemovedElements2(result, quickFilterElementsToRemove);

			//Remove any newly disconnected components:
			var current_quickFilters = dojo.clone(this.editor.editorInput.impactAnalysis.addedElements);
			current_quickFilters.push(this.editor.editorInput.impactAnalysis.targetResourceUri);
			var connectedMap_quickFilters = {};
			this.dfsScan(result, current_quickFilters, connectedMap_quickFilters);
			this.removeDisconnectedElements(result, connectedMap_quickFilters);
		}

		this.rebuildTheToolsLegendAndTree(result);
		this.initTree();
		this.loadTree(result);
	},

	/* 
	 * Takes a partial result to be added to the existing graph, merges into the previous results,
	 * then applies all client-side filtering from initImpactData() to that combined data set.
	 *
	 * @return The combined data set with hidden, filtered, and disconnected pieces removed.
	 */
	initPartialData: function(partialResult){
		if (!partialResult || !this.originalResult || this.originalResult.length <= 0) {
			return;
		}

		//Merge the partial data into the original set
		var result = this._mergeResults(this.originalResult, partialResult);

		//Make a copy of this merged data for later
		this.originalResult = dojo.clone(result);

		this.currentMapRes = {};
		var itemMap = {};
		var linkMap = {};
		var nodesMap = {};
		this.currentMapRes.itemMap = itemMap;
		this.currentMapRes.linkMap = linkMap;
		this.currentMapRes.nodesMap = nodesMap;

		dojo.forEach(result.elementsData.items, function (artifact) {
			var item = dojo.clone(artifact);
			if (item.isLD === false) {
				item.inLinks = [];
				item.outLinks = [];
				itemMap[item.URI] = item;
			} else {
				nodesMap[item.URI] = item;
			}
		}, this);

		dojo.forEach(result.relationsData.items, function (link) {
			var clonedLink = itemMap[link.URI] = dojo.clone(link);
			// add links to items
			itemMap[link.start].outLinks.push(clonedLink);
			itemMap[link.end].inLinks.push(clonedLink);
		}, this);

		this._determineTypes(result);
		this._filterResultAndBuildTree(result);

		return result;
	},

	/*
	 * Item object
	 * item.uri , item.original (uri, name ,tool , node , type )
	 * item.inLink , item.outLink
	 *
	 * item.inlinks = map of Links object a-->thisObj , b-->thisObj
	 * item.outlinks = map of Links object thisObj-->c , thisObj-->d
	 *
	 * link object
	 * link.uri
	 * link.original (uri , source ,target)
	 *
	 */
	initImpactData: function (result, changeDistance)
	{
		// changeDistance argument is option. Default it to true if not specified
		if(changeDistance === undefined) {
			changeDistance = true;
		}
		if (result == null) {
			return;
		}

		this.currentMapRes = {};
		var itemMap = {};
		var linkMap = {};
		var nodesMap = {};
		this.currentMapRes.itemMap = itemMap;
		this.currentMapRes.linkMap = linkMap;
		this.currentMapRes.nodesMap = nodesMap;
		
		dojo.forEach(result.elementsData.items, function (artifact) {
			var item = dojo.clone(artifact);
			if (item.isLD == false) {
				item.inLinks = [];
				item.outLinks = [];
				itemMap[item.URI] = item;
			} else {
				nodesMap[item.URI] = item;
			}
		}, this);
		
		dojo.forEach(result.relationsData.items, function (link) {
			var clonedLink = itemMap[link.URI] = dojo.clone(link);
			// add links to items
			itemMap[link.start].outLinks.push(clonedLink);
			itemMap[link.end].inLinks.push(clonedLink);
		}, this);

		//Quick Filtering, determine the types of each node from the nav tree
		this._determineTypes(result);

		//Remove nodes from explicitly 'Hidden Artifacts'
		if (this.editor.editorInput.impactAnalysis.removedElements.length > 0){
			this.filterRemovedElements2(result, this.editor.editorInput.impactAnalysis.removedElements);

			// remove disconnected elements (from the target + added elements non-directed graph)
			var current = dojo.clone(this.editor.editorInput.impactAnalysis.addedElements);
			current.push(this.editor.editorInput.impactAnalysis.targetResourceUri);
			var connectedMap = {};
			this.dfsScan(result, current, connectedMap);
			this.removeDisconnectedElements(result, connectedMap);
		}

		//Remove nodes that have a type appearing in the list of quick filters
		var artifactQuickFilterTypes = this.editor.editorInput.impactAnalysis.artifactQuickFilters;
		if (artifactQuickFilterTypes.length > 0){
			var quickFilterElementsToRemove = [];
			dojo.forEach(result.elementsData.items, function(element) {
				var typeIsFiltered = dojo.indexOf(artifactQuickFilterTypes, element.type) !== -1 || 
									dojo.some(artifactQuickFilterTypes, function (type) {
										return dojo.indexOf(element.rdfTypes, type) !== -1;
									});
				if (typeIsFiltered){
					quickFilterElementsToRemove.push(element.URI);
				}
			});
			this.filterRemovedElements2(result, quickFilterElementsToRemove);

			//Remove any newly disconnected components:
			var current_quickFilters = dojo.clone(this.editor.editorInput.impactAnalysis.addedElements);
			current_quickFilters.push(this.editor.editorInput.impactAnalysis.targetResourceUri);
			var connectedMap_quickFilters = {};
			this.dfsScan(result, current_quickFilters, connectedMap_quickFilters);
			this.removeDisconnectedElements(result, connectedMap_quickFilters);
		}

		if(changeDistance) {
			this.calcMaxDistance(result);
		}
		this.filterDistance(result, this.distance || result.totalDistance);
		this.rebuildTheToolsLegendAndTree(result);
	},

	removeElementAndReload: function (URI, name, type){
		this.removeElement(URI, name, type);

		//reload
		this.reloadDiagramAndTree(true);
	},

	removeElement: function (URI, name, type)
	{
		var ia = this.editor.editorInput.impactAnalysis;
		ia.removedElements.push(URI); // add the element
		this.editor.addRemovedElement(URI, name, type);
	},

	addArtifactQuickFilter: function(type, typeName)
	{
		var ia = this.editor.editorInput.impactAnalysis;
		if (dojo.indexOf(ia.artifactQuickFilters, type) === -1){
			ia.artifactQuickFilters.push(type);
		}

		this.editor.addArtifactQuickFilter(type, typeName);
		this.reloadDiagramAndTree(true);
	},

	setGridLayoutOnSubgraph: function (subgraph){
		subgraph.textShape.setFont({size: '10pt'});
		if (subgraph._data.item.URI ) {
			if (this.markSubgraph == subgraph._data.item.URI) {
				subgraph.setBorderColor('#FF0000');
				subgraph._collapsedGroup.children[0].setStroke('#FF0000');
			}
		}
		this.subgraphs[this.subgraphs.length] = subgraph;
		subgraph.getGraph().setNodeLayout(common.selectLayout(common.LAYOUT_HIERARCHICAL));
        subgraph.getGraph().setAutomaticNodeLayout(false);
    },

	onNodeCreated: function(node){
		var d = node.getData(),
			isTargetNode,
			isFocalPointNode;

		// back link item to node...
		d.item.node = node;
		
		// look for focal nodes
		isTargetNode = (this.editor.editorInput.impactAnalysis.targetResourceUri === d.item.URI[0]);
		isAddedNode = dojo.some( this.editor.editorInput.impactAnalysis.addedElements, function (added) {
				return added == d.item.URI[0];
			});
		isFocalPointNode =  isTargetNode || isAddedNode;

		isBoundTypeNode = this._isABoundTypeNode(d.item);
		
		if (isFocalPointNode) {
			//The following ignores what the d.item.color number was and just formats focal nodes to grab attention
			node.setBorderWidth(colorTable.getFocalBorderWidth());
			node.setBorderColor(colorTable.getFocalBorderColor());
			node.setBackgroundColor(colorTable.getFocalBackgroundColor());
		}
		else if (isBoundTypeNode) {
			node.setBorderColor(colorTable.getBoundTypeBorderColor());
			node.setBackgroundColor(colorTable.getBoundTypeBackgroundColor());
		}
		else {
			node.setBorderColor(colorTable.getElementBorderColor (d.item.color));
			node.setBackgroundColor(colorTable.getElementColor (d.item.color));
		}
		
		
		if (isTargetNode) {
			this._targetNode = node;
		}
	},
	
	_isABoundTypeNode: function(itemNode)
	{
		console.log('itemNode', itemNode);
		
		var traversalDef = this.editor.inlineEditor.traversalDefinition;
		var traversalMode = traversalDef.getTraversalMode();
		var upstreamBoundType = traversalDef.getUpstreamBoundType();
		var downstreamBoundType = traversalDef.getDownstreamBoundType();
		var fullTraceBoundType = traversalDef.getFullTraceBoundType();
		
		if ( (traversalMode === traversalDef.BOTH || traversalMode === traversalDef.UPSTREAM)
				&& traversalDef.getUpstreamBoundType() !== "")
		{
			if (dojo.indexOf(itemNode.rdfTypes, downstreamBoundType) > -1)
				return true;
		}
		if ( (traversalMode === traversalDef.BOTH || traversalMode === traversalDef.DOWNSTREAM)
				&& traversalDef.getDownstreamBoundType() !== "")
		{
			if (dojo.indexOf(itemNode.rdfTypes, downstreamBoundType) > -1)
				return true;
		}
		if ( (traversalMode === traversalDef.FULL_TRACE)
				&& fullTraceBoundType !== "")
		{
			if (dojo.indexOf(itemNode.rdfTypes, fullTraceBoundType) > -1)
				return true;
		}
		return false;
	},

	/**
	 * When a node is left-clicked on, display the context menu.
	 */
	onNodeClick : function(node) {
		this.impactDiagram.nodeMenu.openAtNode(node);
	},

    /**
     * This happens before the context menu is drawn, so use this time to update the menu contents
     * to only display items relevant to the active node.
     */
	onNodeContextMenu: function(node, event) {
		this.impactDiagram.nodeMenu.updateMenuForNode(node);
	},

	/**
	 * Disable browser right-click in the diagram
	 */
	onContextMenu: function(e) {
		dojo.stopEvent(e);
	},

	onSelectionChanged: function(added, removed){
		var selection = this.impactDiagram.getSelection().get();
		var nodes = [];
		selection.forEach(dojo.hitch(this, function(node){
			node = this.impactDiagram.asGraphElement(node);
			if(!node._isIBMDiagramLink){
				nodes.push(node);
			}
		}));

		dojo.forEach(this.nodeToolbarButtons, dojo.hitch(this, function(button){
			if (button.enableForNodes){
				var enableForNodes = dojo.hitch(DiagramMenuUtil, button.enableForNodes);

				if (button == this.hideArtifactTypeButton){ /* Special case since this isn't a button but a widget */
					if (enableForNodes(nodes)){
						this.hideArtifactTypeButton.set("enabled", true);
					} else {
						this.hideArtifactTypeButton.set("enabled", false);
					}
				} else if (enableForNodes(nodes)){
					this.nodeToolbar.enableButton(button);
				} else {
					this.nodeToolbar.disableButton(button);
				}
			}
		}));
	},

	previewArtifact: function(){
		var selection = this.impactDiagram.getSelection().get();

		if (selection.count === 1){
			var node = this.impactDiagram.asGraphElement(selection.item(0));
			if(!node._isIBMDiagramLink){
				DiagramMenuUtil.previewNode(node);
			}
		}
	},

	hideArtifact: function(){
		var self = this;
		var selection = this.impactDiagram.getSelection().get();

		var nodeSelection = [];
		selection.forEach(function(node){
			node = self.impactDiagram.asGraphElement(node);
			if(!node._isIBMDiagramLink){
				nodeSelection.push(node);
			}
		});

		DiagramMenuUtil.hideArtifacts(nodeSelection);
	},

	_constructHideArtifactTypeMenu: function(){
		var subMenu = new jazz.ui.Menu({
			onClose: function() {
				subMenu.destroyRecursive();
			}
		});

		var self = this;
		var selection = this.impactDiagram.getSelection().get();

		var rdfTypes = [];

		selection.forEach(function(node){
			node = self.impactDiagram.asGraphElement(node);

			var types = DiagramMenuUtil.getNodeTypes(node);

			for (var i = types.length - 1; i >= 0; i--) {
				if (rdfTypes.indexOf(types[i]) == -1) {
					rdfTypes.push(types[i]);
				}
			}
		});

		//add rdfTypes to menu with function that hides the type:
		dojo.forEach(rdfTypes, function (rdfType) {
			var typeName = DiagramMenuUtil.determineTypeName(rdfType);

			subMenu.addChild(new dijit.MenuItem({
				label: typeName,
				iconClass: 'ia-icon-filter',
				disabled: false,
				onClick: function() {
					if(confirm(dojo.string.substitute(messages.confirmHideType, [typeName]))) {
						self.addArtifactQuickFilter(rdfType, typeName);
					}
				}
			}));
		});

		return subMenu;
	},

	expandArtifact: function(){
		var selection = this.impactDiagram.getSelection().get();

		if (selection.count === 1){
			var node = this.impactDiagram.asGraphElement(selection.item(0));
			if(!node._isIBMDiagramLink){
				DiagramMenuUtil.expandNode(node);
			}
		}
	},

	collapseArtifact: function(){
		var selection = this.impactDiagram.getSelection().get();

		if (selection.count === 1){
			var node = this.impactDiagram.asGraphElement(selection.item(0));
			if(!node._isIBMDiagramLink){
				DiagramMenuUtil.collapseNode(node);
			}
		}
	},

	debugArtifact: function(){
		var selection = this.impactDiagram.getSelection().get();

		if (selection.count === 1){
			var node = this.impactDiagram.asGraphElement(selection.item(0));
			if(!node._isIBMDiagramLink){
				DiagramMenuUtil.debugNode(node);
			}
		}
	},

	onFocusedElementChanged: function (oldNode, newNode) {
		if (newNode) {
			this.onNodeMouseEnter(newNode);
		}
	},

	onKeyPress: function (event) {
		// If a node has the focus
		if (this._focusNode) {
			var k = dojo.keys;
			if (event.keyCode === k.ENTER) {
				this.onNodeClick(this._focusNode);
			}
		}
	},

	isNode: function(node){
		//This is the method used by the diagrammer to determine whether something is a node or link
		//I was unable to find a public method for this determination offered by the diagrammer
		return node instanceof ibm_ilog.diagram.Node;
	},
	
	onNodeMouseEnter: function(node){
		// If another node had the focus, it does no more.
		if (this._focusNode && this.isNode(this._focusNode)) {
			this.onNodeMouseLeave(this._focusNode);
		}
		
		// The following block is no longer useful, since we don't care about highlighting nodes that are being hovered on
		/*
		this.selectedItem = node.getData();

		//The following would highlight the node that was being hovered on, 
		//but a bug in the diagrammer kept the node highlighted after deselecting it
		// if (! node.isSelected())
		// {
		// 	node.setBorderColor(colorTable.getOnHoverColor());
		// 	node.setBorderWidth(2);
		// }
		*/

		
		// This node now has the focus
		this._focusNode = node;
	},

	onNodeMouseLeave : function(node){
		// The following block is no longer useful, since we don't care about highlighting nodes that are being hovered on
		/*
		this.selectedItem = null;

		var d = node.getData();

		node.setBorderColor(colorTable.getElementBorderColor(d.item.color));
		node.setBorderWidth(1);

		// look for target nodes
		isTargetNode = (this.editor.editorInput.impactAnalysis.targetResourceUri === d.item.URI[0]);
		isAddedNode = dojo.some( this.editor.editorInput.impactAnalysis.addedElements, function (added) {
				return added == d.item.URI[0];
			});
		isFocalPointNode =  isTargetNode || isAddedNode;
		
		if (isFocalPointNode && !node.isSelected()) {
			//The following ignores what the d.item.color number was and just formats focal nodes to grab attention
			node.setBorderWidth(colorTable.getFocalBorderWidth());
			node.setBorderColor(colorTable.getFocalBorderColor());
		}
		*/

		// Clear the node that had focus
		this._focusNode = null;
	},

	onLinkCreated: function(link,start,end)
	{
		try
		{
			link.setStrokeWidth("1");
			//link.setCurveTension(0.01); // not working OK
			if (link._data ) {
				if (link._data.item.intra == 'true') {
					link.setStrokeColor( colorTable.getLinkColor ("intra"));
				}
				else {
					link.setStrokeColor(colorTable.getLinkColor ("inter"));
				}
				if (link._data.item.crossLD == 'true') {
					link.setStrokeColor(colorTable.getLinkColor ("crossDomain"));
				}
				common.addDecoration(link, link._data.item.type, 0.5, ibm_ilog.diagram.LinkDecorationPositionType.Relative, ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Center, ibm_ilog.diagram.LinkDecorationParallelAlignment.Middle);
			}
			else {
				common.addDecoration(link, "contain", 0.5, ibm_ilog.diagram.LinkDecorationPositionType.Relative, ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Center, ibm_ilog.diagram.LinkDecorationParallelAlignment.Middle);
			}
		}
		catch (err) {
		}
	},

	//moved
	performLayout: function(focusNode) {
		setTimeout(dojo.hitch(this, this._performLayout(focusNode)), 20);
	},
	
	_performLayout: function(focusNode) {
		this.impactDiagram.performGraphLayout();
		dojo.forEach(this.subgraphs, function (subgraph) {
			subgraph.setCollapsed(true);
		});

		if (this.editor.editorInput.impactAnalysis.removedElements.length > 0 && this.slider && this.slider.isFilterApplay() == true)
		{
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.removeElementsDistancefilterApplyed,
				closable: false
		});
		}
		else if (this.editor.editorInput.impactAnalysis.removedElements.length > 0){
			
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.removeElementsfilterApplyed,
				closable: false
			});
		}
		else if (this.slider && this.slider.isFilterApplay() == true){
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.distancefilterApplyed,
				closable: false
			});
		}
		else if (this.warningMessage)
		{
			this.editor.showMessage(this.warningMessage);
		}
		else
		{
			this.editor.hideMessage();
		}
		
		// ensure the primary focal point is in view
		this._focusOn(focusNode || this._targetNode);
		
		// resolve images in nodes...
		// TODO: this can be more efficient if we could figure out a single request the return a sprite
		// and have every image source be a reference into that one sprte
		
		var targetUriSet = {};
		var targetUris = this._getTargetUris(this.editor.editorInput.impactAnalysis);
		for (var i = 0; i < targetUris.length; i++) {
			var uri = targetUris[i];
			targetUriSet[uri] = true;
		}

		var targetNodes = [];
 
		
		dojo.forEach(this.impactDiagram.getGraph().getNodes(), function (node) {
			var d = node.getData();
			if ( d.item.remoteServerUri[0]==null ) {
				
				var resourceUri = d.item.URI;
				if (targetUriSet[resourceUri]) {
					targetNodes.push(node);
				}

				if  (node.elementImg != null) {
					var queryObj = {
						icon: resourceUri,
						serverIcon: 'nil'
					};
					var baseline = jazz.app.context.get().baseline;
					if(baseline) {
						dojo.mixin(queryObj, {"rmps.context": baseline});
					}

					//Instead of making a server call for each node, we're setting the icon to a default,
					//then letting the _previewLoader make batch OSLC calls to replace them when they're found:

					//var imgsrc = net.jazz.ajax._contextRoot + "/analysisinfo?" + dojo.objectToQuery(queryObj);
					//var imgsrc = dojo.moduleUrl('dm.analysis.app', 'graphics/icons/default_node_icon.png');
					var imgsrc = dojo.moduleUrl('dm.analysis.app', 'graphics/icons/unloaded_icon.gif');
					node.elementImg.setShape({src:imgsrc}) ;
				}
				
			} else {
				var remoteSrc = dojo.moduleUrl('dm.analysis.app', 'graphics/icons/remote1.gif');
				node.elementImg.setShape({src:remoteSrc}) ;
			}

		}, this);

		this._previewLoader && this._previewLoader.destroyRecursive();
		this._previewLoader = new dm.analysis.app.ui.internal.impactanalysis.PreviewLoader(targetNodes);
		this._previewLoader.loadPreviews();

		
		dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
	},

	uninitialize: function(){
		this._previewLoader && this._previewLoader.destroyRecursive();
		if (this.impactDiagram.nodeMenu) this.impactDiagram.nodeMenu.destroyRecursive();
		if (this.navTree) this.navTree.destroyRecursive();

		if (this.slider) this.slider.destroyRecursive();
		if (this.toolbar) this.toolbar.destroyRecursive();
		if (this.filterSelect) this.filterSelect.destroyRecursive();
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram._Slider", [dijit._Widget, dijit._Templated], {

	slider: null,

	sresult: null,
	distance: null,
	diagram: null,
	disabled: false,

	sliderInput: null,

	messages: messages,
	widgetsInTemplate: true,
	templateString: '<div>' +
				'<div>${messages.setDistanceText}</div>'+
				'<table>' +
				'<tr>'+
				'<td style="width:70%"><div dojoAttachPoint="sliderDiv" ></Div></td>'+
				'<td style="width:30%"><label dojoAttachPoint="sliderInput" style="width:30px"></label></td>'+
				'</tr>'+
				'</table>' +
			'</div>',

	// init properties
	postCreate: function() {
		this.slider = new dijit.form.HorizontalSlider({
			name: "slider",
			minimum: 1,
			maximum: this.sresult.totalDistance,
			value: this.distance ? this.distance : this.sresult.totalDistance,
			intermediateChanges: false,
			clickSelect: false,
			style: "width:155px",
			discreteValues: this.sresult.totalDistance
		});

		this.slider.set("disabled", this.disabled);

		this.connect(this.slider, "onChange", function(value){
			if(this.diagram._iaLoadPromise) {
				// On firefox, at least, dijit.form.Slider appears to occasionally give multiple onChange
				// events, esp. if you click the button at the end of the slider.
				// This doesn't work well with firing a long-running action like re-building
				// the IA diagram.
				// If the IA diagram load is in progress when we get this event,
				// ignore the event.
				return;
			}
			if (value > 0) {
				this.diagram.distance = value;
				this.diagram.reloadDiagramAndTree(false); //mark that don't
			}
		});

		this.sliderInput.innerHTML = this.distance ? this.distance +"/" + this.sresult.totalDistance:
															this.sresult.totalDistance + "/" + this.sresult.totalDistance;
		this.sliderDiv.appendChild(this.slider.domNode);
	},
	startup: function(){

	},
	isFilterApplay: function() {
		return this.sresult.totalDistance != this.slider.value;
	}
});

}());


