/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisEditor");

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisEditor");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("jazz.ui.Breadcrumbs");

dojo.require("rmps.ui.editor.PageHelper");
dojo.require("rmps.ui.SidebarContainer");

dojo.require("dm.common.app.ui.internal.rhlink.IAInfoClient");

dojo.require("dm.analysis.app.ui.internal.impactanalysis.InitDiagramDialog");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.SaveProfileDialog");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.picker.LocateElementControler");

dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysis");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient");

dojo.require("dm.analysis.app.ui.internal.impactanalysis.ImpactDefinitionsInlineEditor");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisPanelsContainer");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.EditorUtil");

var AnalysisUtil = dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil;

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 

var STATUS_DRAFT = "Draft";
var STATUS_IN_PROGRESS = "InProgress";
var PageHelper = rmps.ui.editor.PageHelper;

var PAGE_ID = dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisEditor.PAGE_ID;
var DIRTY_GROUP_ID = "dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisEditor.dirty";

var UA_KEY = "com.ibm.xtools.impactanalysis.web.app.page_CreateIADiagram001";

// enable iLog layout extensions...
dojo.config.useGfxLayout = true;

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisEditor", rmps.app.ui.internal.editors.BaseEditor, {
	firstTime:true,
	isRefreshable: false,
	isReadOnly: true,
	_panelsContainer: null,
	_diagramWidget: null,
	editorInput:null,
	_breadCrumbs: null,
	content:null,
	uaKey: UA_KEY,
	dirtyTitle:null,
	dirtyLocate:null,
	locateController:null,
	initDiagramDialog:null,
	doRun:false,
	
	postCreate: function() {
		this.dirtyTitle = {};
		this.dirtyLocate = {};
		this.isReadOnly = false;
		this.isRefreshable = true;
	
		// Add the "All Impact Analyses" bread crumb at the top of the editor
		this._breadCrumbs = new jazz.ui.Breadcrumbs({segments: [{label: messages.allAnalysisMainCaption, href: AnalysisUtil.getAllAnalysisUri()}]});
		dojo.place(this._breadCrumbs.domNode, this.headerContainerNode, "first");
		
		this._inlineEditorContainer = dojo.create('div', null, this.headerContainerNode);
		dojo.style(this._inlineEditorContainer, 'overflow', 'hidden');
		dojo.style(this._inlineEditorContainer, 'width', '100%');
		dojo.style(this._inlineEditorContainer, 'height', 0);

		this.inlineEditor = new dm.analysis.app.ui.internal.impactanalysis.ImpactDefinitionsInlineEditor({ editor: this, isReadOnly: this.isReadOnly });
		this._inlineEditorContainer.appendChild(this.inlineEditor.domNode);

		//check first if no impactUri exist if not we need to create a new Impactelement
		this.locateController = new dm.analysis.app.ui.internal.impactanalysis.picker.LocateElementControler();
		this.initDiagramDialog = new dm.analysis.app.ui.internal.impactanalysis.InitDiagramDialog(this);
		
		if (this.editorInput.impactAnalysis) {
			var editorInputTargetUri = this.editorInput.impactAnalysis.targetResourceUri;
			var externalInputTargetUri = this.editorInput.getState('targetResource');

			if (editorInputTargetUri){
				this.setSourceInfo(editorInputTargetUri, this.editorInput.impactAnalysis.contextConfigVersionUri);
			}
			else if(externalInputTargetUri){
				this.initDiagramDialog.setDialogTarget(externalInputTargetUri);
				this._openInitDialog();
			}
			else {
				this._openInitDialog();
			}
		}

		this._resizeHandle = jazz.util.ResizeNotifier.register({
			context: this,
			callback: this.resize,
			force: true,
			priority: 5
		});

		this.inherited(arguments);
	},

	startup: function(){
		this.inherited(arguments);

		/*if (this.isEditMode()) {
			this.showInlineEditor(true);
		}*/
		this._diagramWidget.loadData();
	},

	/** Called by the framework when this editor is the active editor */
	activate: function() {
		this.inherited(arguments);

		// Wait for half a second until the editor opens properly
		// otherwise the in-line editor resize does not work.
		setTimeout(dojo.hitch(this, function() {
			this.showInlineEditor(this.isEditMode());
		}), 500);
		
	},

	/** Determines if we should be in Edit Mode. 
			Based on editMode url param, whether this IA is dirty, or 
			does not have a target element associated with it. */
	isEditMode: function() {
		var params = dojo.queryToObject(dojo.hash());
		var urlParamEditMode = (params.editMode ? params.editMode.toLowerCase() === 'true' : false);

		var isDirty = this.isDirty();
		var hasTargetElement = this.editorInput.impactAnalysis.targetResourceUri !== null;

		return urlParamEditMode || isDirty || !hasTargetElement;
	},
	
	isEditableTitle: function(){
		return true;
	},
	
	onTitleChange: function(e){
		var oldValue = this.editorInput.impactAnalysis.shortName;
		this.editorInput.impactAnalysis.shortName = this.getTitleValue();
		this._updateDirty(oldValue, 
					this.getTitleValue(), 
					this.dirtyTitle);
	},
	
	getEditableTitleNodeDetails: function () {
		// summary:
		//		Callback from BaseEditor providing default title for a new impact analysis
		return messages.newImpactAnalysisTitle;
	},
	
	validate: function (/*Array*/errors) {
		// summary:
		//		Callback from BaseEditor to validate data prior to save

		var detailedError;

		if(!this.editorInput.impactAnalysis.targetResourceUri) {
			detailedError = new Error(messages.errorMissingTargetElement);
			detailedError.explanation = messages['errorMissingTargetElement.explanation'];
			detailedError.userAction = messages['errorMissingTargetElement.useraction'];
			errors.push(detailedError);
			this.saveButton.setEnabled(this.isDirty());
		}

		var profileInfo = this.getProfileInfo();

		if (profileInfo.traversal === profileInfo.BOTH || profileInfo.traversal === profileInfo.UPSTREAM){
			if(profileInfo.upstreamType === '' && !profileInfo.upstreamDepth) {
				detailedError = new Error(messages.errorMissingUpstreamDepth);
				//detailedError.explanation = messages['errorMissingUpstreamDepth.explanation'];
				//detailedError.userAction = messages['errorMissingUpstreamDepth.useraction'];
				errors.push(detailedError);
				this.saveButton.setEnabled(this.isDirty());
			}
		}

		if (profileInfo.traversal === profileInfo.BOTH || profileInfo.traversal === profileInfo.DOWNSTREAM){
			if(profileInfo.downstreamType === '' && !profileInfo.downstreamDepth) {
				detailedError = new Error(messages.errorMissingDownstreamDepth);
				//detailedError.explanation = messages['errorMissingDownstreamDepth.explanation'];
				//detailedError.userAction = messages['errorMissingDownstreamDepth.useraction'];
				errors.push(detailedError);
				this.saveButton.setEnabled(this.isDirty());
			}
		}
		
		if (profileInfo.traversal === profileInfo.FULL_TRACE){
			if(profileInfo.fullTraceType === '' && !profileInfo.fullTraceDepth) {
				detailedError = new Error(messages.errorMissingFullTraceDepth);
				//detailedError.explanation = messages['errorMissingFullTraceDepth.explanation'];
				//detailedError.userAction = messages['errorMissingFullTraceDepth.useraction'];
				errors.push(detailedError);
				this.saveButton.setEnabled(this.isDirty());
			}
		}
		
		if(this.editorInput.impactAnalysis.shortName === '') {
			detailedError = new Error(messages.errorMissingTitle);
			detailedError.explanation = messages['errorMissingTitle.explanation'];
			detailedError.userAction = messages['errorMissingTitle.useraction'];
			errors.push(detailedError);
			this.saveButton.setEnabled(this.isDirty());

			dm.analysis.app.ui.internal.impactanalysis.util.EditorUtil.animateMissingEditorTitle(this._editableTitle.domNode);
			this._editableTitle.edit();
		}
	},
	
	_updateDirty: function(originalValue, newValue, dirtyKey) {
		var isEqual = originalValue === newValue;
		if (!isEqual && dojo.isArray(newValue) && originalValue && newValue && originalValue.length === newValue.length) {
			isEqual = true; // empty arrays
			isEqual = dojo.every(newValue, function(item, idx) {
				return item === originalValue[idx];
			});
		}
		if (isEqual) {
			this.removeDirty(dirtyKey, DIRTY_GROUP_ID);
		} else {
			this.addDirty(dirtyKey, DIRTY_GROUP_ID);
		}
	},
	
	getMainSideNav: function()
	{
		return this.editorInput.mainSideNav;
	},
	
	getTreeSideNav: function()
	{
		return this._panelsContainer.getTreeNav();
	},
	
	getImpactDefinition: function() {
		def = this.inlineEditor.getImpactDefinition();
		def.iAComment = this.getCommentText();
		return def;
	},

	getProfileInfo: function(){
		return this.inlineEditor.getProfileInfo();
	},
	
	//FIXME: the domNodeArea, as of now, is always (0,0) when getSize is called to instantiate the impactDiagram, so this function is ALWAYS wrong
	getSize: function(){
		var contentCell = dojo.byId("content-cell");
		var contentArea = dojo.marginBox(contentCell);
		var domNodeArea = dojo.marginBox(this.domNode);
		var height = (contentArea.h > 0 ? contentArea.h : parseInt(contentCell.style.minHeight, 10)) + contentArea.y - domNodeArea.y;
		return {w: domNodeArea.w - 258 , h: height};
	},
		
	createContentWidget: function() {
		var sidebar = new rmps.ui.SidebarContainer();
		this._panelsContainer = new dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisPanelsContainer({editor: this});
		sidebar.addSidebar(this._panelsContainer, false, true, true);
	
		this._diagramWidget = new dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram({
			editor: this,
			size: this.getSize()
		});
		
		sidebar.setContent(this._diagramWidget, true);
		return sidebar;
	},
	
	/* 
	 * Completely override BaseEditor since we want to add our own buttons 
	 * and we do want to have buttons like 'Save' behave a little differently. 
	 */
	addButtons: function(toolbar) {

		this.editButton = toolbar.addButton(this, this._animateInlineEditor, {
					buttonType: jazz.ui.SimpleToolbar.ButtonTypes.TWO_STATE,
					imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/edit.gif'),
					label: messages.showEditLabel,
					imgSrcPressed: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/edit.gif'), 
					labelPressed: messages.hideEditLabel
			});

		toolbar.addSeparator();
		
		//this.inherited(arguments);
		//Refresh and Save were taken from BaseEditor.js:

		if (this.isRefreshable) {
			this.refreshButton = toolbar.addButton(this, this._confirmRefresh, {
				label: messages.refresh,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/refresh.gif")
			});
		}
// 		Temporarily hide the stop button because it didn't really work properly
//		this.stopIAButton = toolbar.addWidget( new rmps.ui.SimpleButton({
//			label: messages.stop,
//			onClick: dojo.hitch(this, "_stopAnalysis")
//		}));
		this.stopIAButton = new rmps.ui.SimpleButton({
			label: messages.stop,
			onClick: dojo.hitch(this, "_stopAnalysis")
		});
		this.enableIAStopButton(false);
		
		toolbar.addWidget(new rmps.ui.SimpleButton({
			label: messages.cancel,
			onClick: dojo.hitch(this, this.doCancel)
		}));
		

		if (!this.isReadOnly) {
			this.saveButton = toolbar.addWidget(new rmps.ui.SimpleButton({
				label: messages.save,
				onClick: dojo.hitch(this, function(){
					if (this.inlineEditor.isProfileInfoModified()){
						var conf = confirm(messages.unsavedProfileChanges);
						if (!conf){
							this._saveProfile();
							return;
						}
					}
					this.save();
				})
			}));
			this.saveButton.setEnabled(this.isDirty());
		}

		this.runButton = this.inlineEditor.editorToolbar.addWidget(new rmps.ui.SimpleButton({
			label:messages.iaEditorExe,
			onClick: dojo.hitch(this, this._apply)
		}));
		this.runButton.setEnabled(false);

		var inlineCloseButton = this.inlineEditor.editorToolbar.addWidget(new rmps.ui.SimpleButton({
			label:messages.close,
			onClick: dojo.hitch(this, this._inlineToggle)
		}));

		//only adding the buttons managed by ImpactAnalysisEditor:222
		this._buttons = [this.editButton, inlineCloseButton]; //, this.runButton
		
	},
	
	enableIAStopButton: function (enable) {
		if(this.stopIAButton) {
			this.stopIAButton.setEnabled(enable);
		}
	},
	
	_stopAnalysis: function () {
		dojo.publish('impactanalysis.stop');
	},
	
	deactivate: function () {
		this.inherited(arguments);

		this._stopAnalysis();
	},
	
	doCancel: function() {
		// summary:
		//		When Cancel is clicked in the analysis configuration editor, go
		//		back to the page for viewing all impact analyses.
		var pageHelper = rmps.ui.editor.PageHelper.getInstance(PAGE_ID);
		if(pageHelper.closeEditor(this, false, true)) {
			window.location.href = AnalysisUtil.getAllAnalysisUri() + "&ts="+ new Date().getTime();
		}
	},
	
	/* HIDDEN ARTIFACTS */
	addRemovedElement: function(uri, name, type)
	{
		this._panelsContainer.addRemovedElement(uri, name, type);
	},
	restoreRemovedElement: function(uri)
	{
		var ia = this.editorInput.impactAnalysis;
		var uriIndex = dojo.indexOf(ia.removedElements, uri);
		if (uriIndex != -1) ia.removedElements.splice(uriIndex, 1);

		this._diagramWidget.reloadDiagramAndTree();
	},
	getRemovedElementName: function(removedElement)
	{
		return this._panelsContainer.getRemovedElementName(removedElement);
	},

	/* HIDDEN ARTIFACT TYPES */
	addArtifactQuickFilter: function(type, typeName)
	{
		this._panelsContainer.addArtifactQuickFilter(type, typeName);
	},
	removeArtifactQuickFilter: function(type)
	{
		var ia = this.editorInput.impactAnalysis;
		var typeIndex = dojo.indexOf(ia.artifactQuickFilters, type);
		if (typeIndex != -1) ia.artifactQuickFilters.splice(typeIndex, 1);

		this._diagramWidget.reloadDiagramAndTree(true);
	},

	/* COMMENT */
	getCommentText: function(){
		return this._panelsContainer.getCommentText();
	},
	
	enable: function()
	{
		var i,
			b;
		for (i = 0; i < this._buttons.length ;i++)
		{
			b = this._buttons[i];
			if (b['setEnabled']) {
				b.setEnabled(true);
			}
		}
		this.inlineEditor.enableButtons(true);
		this.onSaveComplete();
	},
	disable: function()
	{
		var i,
			b;
		this.showLoading();
		for (i = 0; i < this._buttons.length ;i++)
		{
			b = this._buttons[i];
			if (b['setEnabled']) {
				b.setEnabled(false);
			}
		}
		this.inlineEditor.enableButtons(false);
	},
	
	reloadDiagramAndTree: function()
	{
		this._diagramWidget.reloadDiagramAndTree();
	},

	_animateInlineEditor: function()
	{
		var shownBorderWidth = 1;
		var show = this.toolbar.isPressed(this.editButton);
		
		//TODO: also animate editor resizing
		if (show){
			this._anim && this._anim.stop();

			var inlineContent = dojo.query('.split-editor', this.inlineEditor.domNode)[0];

			var containerHeight = dojo.marginBox(inlineContent).h;
			this._anim = dojo.animateProperty({
				node: this._inlineEditorContainer,
				duration: 200,
				onEnd: dojo.hitch(this, function() {
					this.resize({h: dojo.marginBox(this.domNode).h});
				}),
				properties: {
					height: { end: containerHeight, units: 'px'}
				}
			}).play();
		}
		else {
			this._anim && this._anim.stop();

			this._anim = dojo.animateProperty({
				node: this._inlineEditorContainer,
				duration: 200,
				onEnd: dojo.hitch(this, function() {
					this.resize({h: dojo.marginBox(this.domNode).h});
				}),
				properties: { 
					height: { end: 0 } 
				}
			}).play();
		}
	},

	resize: function(size){
		//BaseEditor's resize() function doesn't work properly because it adds minHeight/minWidth,
		//If you need any of its functionality, copy over only those parts
		//this.inherited(arguments);

		if (this._diagramWidget){
			this._diagramWidget.resize(size);
		}
	},

	_inlineToggle: function(e) {
		this.showInlineEditor(!this.toolbar.isPressed(this.editButton));
	},

	showInlineEditor: function(show) {
		this.toolbar.setPressed(this.editButton, show);
		this._animateInlineEditor();
	},

	getExpansionSequence: function() {
		return dojo.clone(this._diagramWidget.expansionSequence);
	},
	
	getLayout: function() {
		return this._diagramWidget.filterSelect.value;
	},

	//overriding BaseEditor since we have the extra editMode url param
	refreshHref: function(timestamp){
		window.location.href = this.editorInput.getPresentationUri() + (this.isEditMode() ? "&editMode=true" : "");
	},

	_apply: function(){
		this._run(true);
	},

	_run: function(useProfileParams)
	{
		//There is no scenario in which the diagram is about to be drawn but the user can press the Run Button
		this.runButton.setEnabled(false);

		var def = this.getImpactDefinition();
		if (!this.editorInput.impactAnalysis.targetResourceUri){
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.cantRunWithoutTargetElement,
				closable: true
			});
			return;
		}

		//makes a copy of the local impact analysis, so that we don't modify it
		var ia = dojo.clone(this.editorInput.impactAnalysis);
		
		ia.iAType = def.iAType;
		ia.iAViewType = def.iAViewType;
		ia.iAComment = def.iAComment;
		ia.iAReduceResult= def.iAReduceResult;
		ia.impactStatus = STATUS_IN_PROGRESS;
		ia.shortName = this.getTitleValue();
		if (useProfileParams){
			ia.iAIncludedProfile = this.getProfileInfo();
		}

		this._diagramWidget.loadData(ia);
	},

	/* 
	 * Performs the save or returns errors on the taskMonitor. 
	 * (Save cannot be cancelled at this point) 
	 */
	doSave: function(taskMonitor){
		var ia,
			def,
			clientPromise,
			isNew = false;
	
		this.disable(); //Disables toolbar buttons
		ia = this.editorInput.impactAnalysis;

		def = this.getImpactDefinition();
		ia.iAType = def.iAType;
		ia.iAViewType = def.iAViewType;
		ia.iAComment = def.iAComment;
		ia.iAReduceResult= def.iAReduceResult;
		ia.impactStatus = STATUS_IN_PROGRESS;
		ia.shortName = this.getTitleValue();
		ia.expansionSequence = this.getExpansionSequence();
		ia.layout = this.getLayout();

		if (ia.eTag) {
			clientPromise = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.updateImpactAnalysis(ia);
		} else {
			clientPromise = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.createImpactAnalysis(ia);
			isNew = true;
		}
		clientPromise.then(
			dojo.hitch(this, function (ia) {
				if (isNew) {
					// refresh the editor it with the now 'saved' editor input.
					this.editorInput.resourceUri = ia.uri;
					this.editorInput.removeState('id');
				
					// clear dirty flag first before reload
					this._clearDirty();
					this.refresh();
				} else {
					this.enable(); //Re-enables toolbar buttons when save is complete
				}
				dojo.publish("dm.analysis.app.ui.internal.impactanalysis.refresh");
				
			}),
			dojo.hitch(this, function (error, ioArgs) {
				if (clientPromise.ioArgs && clientPromise.ioArgs.xhr && clientPromise.ioArgs.xhr.getResponseHeader('user-facing-exception')){
					var jsonErrorResponse = dojo.fromJson(clientPromise.ioArgs.xhr.responseText);
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: jsonErrorResponse.summary,
						details: this.getErrorMessageDetails(jsonErrorResponse),
						closable: true
					});
				} else {
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: messages.errorUnableToSaveIA,
						errorObject: error,
						closable: true
					});
				}
			})
		);

		var task = taskMonitor.beginTask("saving", 1);
		task.handleDeferredWork(clientPromise, 1);
	},

	computeSingleNodeExpansion: function(targetUri) {
		var ia,
			def,
			clientPromise,
			isNew = false;

		ia = dojo.clone(this.editorInput.impactAnalysis);
		def = this.getImpactDefinition();
		
		//if there is no contextConfigVersionUri, and the target artifact happens to be a configVersion
		//then use that as the contextConfigVersionUri.  
		//This is for the case when expanding a node and the target resource uri itself IS the context configuration.
		if (!ia.contextConfigVersionUri)
		{
			if (AnalysisUtil.isConfigVersion(ia.targetResourceUri))
			{
				ia.contextConfigVersionUri = ia.targetResourceUri;
			}
		}
		
		ia.iAType = def.iAType;
		ia.iAViewType = def.iAViewType;
		ia.iAReduceResult= def.iAReduceResult;
		ia.iAComment = def.iAComment;
		ia.iAIncludedProfile = this.getProfileInfo();
		ia.iAIncludedProfile.upstreamDepth = 1;
		ia.iAIncludedProfile.downstreamDepth = 1;
		ia.iAIncludedProfile.fullTraceDepth = 1;

		//For single node expansion these are useless, AND they may cause problems for the server
		ia.addedElements = [];
		ia.removedElements = [];
		ia.artifactQuickFilters = [];
		ia.expansionSequence = [];

		ia.targetResourceUri = targetUri;

		return dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.computeImpactAnalysis(ia, messages.expandingNode);
	},
	
	_openInitDialog: function(){
		this.initDiagramDialog.createDialog();
	},

	_initDiagramDialogCallback: function(targetArtifactUri, profileUri, contextConfigVersionUri){
		this.inlineEditor.changeProfileSelectionTo(profileUri);
		this.updateAfterLocate(targetArtifactUri, contextConfigVersionUri);
	},

	_locateElement: function(){
		//var dialog = new dm.common.app.ui.internal.rhlink.LocateElementDialog(this);
		this.locateController.locateElement(this);
	},

	_saveProfile: function(){
		var profileInfo = this.getProfileInfo();

		new dm.analysis.app.ui.internal.impactanalysis.SaveProfileDialog(this, this.iaProfile, profileInfo, dojo.hitch(this, this._saveProfileDialogCallback)).createDialog();
	},

	_saveProfileDialogCallback: function(wasNew, newProfileName, newProfileUri){
		if (wasNew)
			this.inlineEditor._addProfileOption(newProfileName, newProfileUri);
		
		this.inlineEditor.changeProfileSelectionTo(newProfileUri);
		
		this.inlineEditor.resetProfileToolbar();
	},
	
	getResourceInfo: function(){
		return this.editorInput.resourceInfo;	
	},
	
	getProjectID: function(){
		return this.editorInput.projectId;	
	},
	
	getImpactAanlysis: function()
	{
		return this.editorInput.impactAnalysis;	
	},

	getErrorMessageDetails: function(jsonErrorResponse){
		var errorMessageDetails = [];

		if (jsonErrorResponse.explanation && jsonErrorResponse.explanation.length > 0){
			errorMessageDetails.push({ message: dojox.html.entities.encode(jsonErrorResponse.explanation).replace(/\n/g,"<br>") });
		}
		if (jsonErrorResponse.userAction && jsonErrorResponse.userAction.length > 0){
			errorMessageDetails.push({ message: dojox.html.entities.encode(jsonErrorResponse.userAction).replace(/\n/g,"<br>") });
		}

		return errorMessageDetails;
	},

	setSourceInfo: function(sourceURI, contextConfigVersionUri) {
		var deferred = dm.common.app.ui.internal.rhlink.IAInfoClient.getInfo(sourceURI);
		deferred.addCallback(this, function(infos) {
			this.inlineEditor.reset(infos[0], contextConfigVersionUri);
		});
		deferred.addErrback(this, function(error) {
			var xhr = deferred.ioArgs.xhr;
			if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
				var lqeInfo = dojo.fromJson(xhr.responseText);
				this.showMessage({
					type: lqeInfo.type,
					summary: lqeInfo.summary,
					details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
					closable: true
				});
			} else if (xhr.getResponseHeader('user-facing-exception')){
				var jsonErrorResponse = dojo.fromJson(xhr.responseText);
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: jsonErrorResponse.summary,
					details: this.getErrorMessageDetails(jsonErrorResponse),
					closable: true
				});
			} else {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: dojo.string.substitute(messages.errorUnavailableResource, [sourceURI]),
					details: [ { message: messages['errorUnavailableResource.explanation'] }],
					closable: true
				});
			}
			dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
		});
	},

	updateAfterLocate: function (sourceURI, contextConfigVersionUri)
	{
		var oldValue = this.editorInput.impactAnalysis.targetResourceUri;
		this.editorInput.impactAnalysis.targetResourceUri = sourceURI;
		this.editorInput.impactAnalysis.contextConfigVersionUri = contextConfigVersionUri;
		this._updateDirty(oldValue, sourceURI, this.dirtyLocate);
		this.setSourceInfo(sourceURI, contextConfigVersionUri);
		this.enableRunButton();
		
		this._diagramWidget.reset();
	},

	uninitialize: function(){
		jazz.util.ResizeNotifier.unregister(this._resizeHandle);
		this.inlineEditor.destroyRecursive();
	},

	enableRunButton: function()
	{
		if (this.editorInput.impactAnalysis.targetResourceUri)
		{
			this.runButton.setEnabled(true);
		}
	}
});

}());
