/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisInput");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysis");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient");
dojo.require("dm.common.app.ui.internal.rhlink.IAInfoClient");
dojo.require("rmps.ui.editor.EditorInput");
dojo.require("dojox.encoding.digests.MD5");
dojo.require("dojo.i18n");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 

var AnalysisUtil = dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil,
	messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"),
	runningCount = 1;

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisInput", rmps.ui.editor.EditorInput, {
	
	projectId: null,
	treeSideNav:null,
	mainSideNav:null,
	resourceUri:null,
	targetUri:null,
	impactAnalysis:null,
	targetElement:null,
	_id: null,
	resourceInfo:null,
	
	
	//resourceInfo
	constructor: function(params) {
		// find any state related pamaters
		if (params.targetResource) {
			this.addState('targetResource', params.targetResource);
			delete params.targetResource;
		}
		
		if (params.id)  {
			this.addState('id', params.id);
			delete params.id;
		}
		
		// integrated the remaining pamaters into the editor input.
		dojo.mixin(this, params);
		
		this.targetElement = {};
		this.resourceInfo = {};
	},
	
	
	isLoaded: function() { // load if both ia element and target resource element is loaded
		if (this.impactAnalysis )
		{
			if (this.impactAnalysis.targetResourceUri)
			{
				if(this.targetElement)
				{
					return true;
				} 
			}
			else
			{
				return true;
			}
		} else if(!this.resourceUri) {
			this._initFromIA( new dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysis() );
			this.addState('id', this.getId());
			return true; // input represents a new, unsaved resource, no need to load it...
		} else {
			return false;
		}
	},
	
	load: function() {
		var loadImpactFunc = dojo.hitch(this, this._initFromIA);
		var deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.getImpactAnalysis(this.resourceUri);
		deferred.addCallback(this, loadImpactFunc);
		//TODO: addErrback
		return deferred;
	},
	
	_initFromIA: function(ia)  {
		this.impactAnalysis = ia;
		//if there is a target than load it as well
		this.resourceInfo.uri = ia.uri;
		this.resourceInfo.shortName = ia.getName();
		if (ia.targetResourceUri) {
			dm.common.app.ui.internal.rhlink.IAInfoClient.getInfo(ia.targetResourceUri)
				.then(dojo.hitch(this, this._loadTargetResource));
		} 
	},
	
	_loadTargetResource: function (infos)
	{
		this.targetElement = infos[0];
	},
	
	
	getId: function() {
		if(this._id === null ) {
			if(dojo.isString(this.resourceUri) && this.resourceUri.length > 0) {
				this._id = this.resourceUri;
			} else if (this.getState('id')) {
				this._id = this.getState('id');
			} else {
				// make a fake, unique ID to distinguish from other new inputs
				this._id = dojox.encoding.digests.MD5(this.declaredClass + ',' + runningCount);
				runningCount += 1;
			}
		}
		return this._id;
	},
	 
	getIconUri: function() {
		return dojo.moduleUrl("dm.analysis.app", "graphics/icons/impact.gif");
	},
	
	getName: function() { 
		return this.impactAnalysis && dojo.trim(this.impactAnalysis.shortName).length > 0
			? this.impactAnalysis.shortName
			: messages.newImpactAnalysisTitle;
	},
	
	getTooltip: function() {
		return this.getName();
	},
	
	getBasePresentationUri: function() {
		return AnalysisUtil.getAnalysisUri(this.resourceUri);
	}
	
	
	
});

}());
