/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisTree");

dojo.require("dijit._Widget");
dojo.require("jazz.ui.Tree");
dojo.require("jazz.ui.tree.SimpleItem");
dojo.require("jazz.ui.tree.IconItem");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages");
var thisT;

var navigationTree = dojo.declare(jazz.ui.Tree, {
			onSelectionChanged: function(selection) {
				var urisToSelect = [];

				if (selection.length === 1 && selection[0].nodeType !== 'artifact'){
					this._collectAllLeafNodeUris(selection[0], urisToSelect);
				} else {
					dojo.forEach(selection, function(selection_element){
						if (selection_element.nodeType === 'artifact'){
							urisToSelect.push(selection_element.uri);
						}
					});
				}

				thisT.diagram.selectAndFocusOn(urisToSelect);
			},
			
			_collectAllLeafNodeUris: function (treeNode, uris) {
				// summary:
				//		recursivly collect all the leaf node node URIs from the specified tree node
				if (treeNode.isLeaf()) {
					uris.push(treeNode.uri);
				} else {
					dojo.forEach(treeNode.getChildren(), function (treeNode) {
						this._collectAllLeafNodeUris(treeNode, uris);
					}, this);
				}
			}

			// currently there can be only two options - 
			/*searchForElementsAndMark: function (tool,system,elementType ,selected)
			{
				var node;
				dojo.forEach(thisT.diagram.allElements.items, function (element) {
					if (tool != null && element.tool != tool ) {
						// continue...
					} else if (system != null && element.LDURI != system ) {
						// continue...
					} else if (elementType != null && element.type != elementType ) {
						// continue...
					} else if (selected) {
						// true - mark select false - mark select 
						element.node.setHighlightColor('#EE7AE9');
					} else  {
						element.node.setHighlightColor('white'); 
					}
				}, this);
				return node;
			}*/
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisTree", [dijit._Widget, dijit._Templated], {
	
	messages: null, 
	widgetsInTemplate: true,
	templateString:
		'<div dojoAttachPoint="treeContainer" class="rh-app-pages-rh-impact-tree" style="overflow: none" >'+
		'</div>',

	tree: null, //type: navigationTree
	treeData: null,
	diagram: null,

	postCreate: function() {
		thisT = this;
		this.tree = new navigationTree({ selection: "multi" });
		
		this.treeContainer.appendChild(this.tree.domNode);
	},

	/* No idea what "markSubgraph" is for */
	loadTreeData: function(treeData, markSubgraph) {
		this.treeData = treeData;
		this.addSubtree(this.tree, treeData.items, treeData.numOfElements);
	},

	openMenu: function(e, item, args){
		dojo.stopEvent(e);
		if (args.nodeType === 'artifact'){
			//var selection = this.tree.getSelection();
			this.tree.setSelection(item);
			this.diagram.impactDiagram.nodeMenu.openAtNavTreeNode(args.uri, e);
		}
	},

	// Recursively inserts nodes into the tree
	addSubtree: function(tree, nodes, numOfElements) {
		if (!nodes || !tree) {
			return;
		}
		dojo.forEach(nodes, function (node) {
			if(node.numOfElements > 0) {
				var name = node.name || "";
				name = name.replace("-spa-" , " ");
				var nodeLabel;
				if (node.numOfElements && node.children ) {
					nodeLabel = name + " (" + node.numOfElements + ")";
				} else {
					nodeLabel = name ;
				}
				nodeLabel = dojox.html.entities.encode(nodeLabel);

				var item, args;
				if (node.type === 'artifact') {
					args = { label: nodeLabel, uri: node.URI, nodeType: node.type, isSelected: false };
					item = new jazz.ui.tree.SimpleItem(args);
					tree.addChild(item);
				} else {
					args = { label: nodeLabel, iconClass: node.type, uri: node.URI, nodeType: node.type, isSelected: false };
					item = new jazz.ui.tree.IconItem(args);
					tree.addChild(item);
					this.addSubtree(item, node.children, numOfElements);
				}

				this.connect(item.domNode, "oncontextmenu", dojo.hitch(this, function(e) { this.openMenu(e, item, args); }));
			}
		}, this);
	},
});
}());