/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| Copyright IBM Corporation 2012, 2013.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
 (function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.ImpactDefinitionsInlineEditor");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.NumberTextBox");
dojo.require("dijit.form.SimpleTextarea");

dojo.require("dm.common.app.ui.internal.TabManager");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient");

dojo.require('jazz.util.html');
dojo.require('jazz.app.ua.TextHoverHelp');

dojo.require("dm.analysis.app.ui.internal.impactanalysis.panels.FilterListPanel");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.panels.ArtifactFilterListPanel");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.profile.TraversalDefinition");
dojo.require("dm.analysis.app.ui.internal.IA_Settings");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.picker.IAConfigurationSearchBox");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 

var AnalysisUtil = dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil;
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages");
var IA_Settings = dm.analysis.app.ui.internal.IA_Settings;
var TabManager = dm.common.app.ui.internal.TabManager;

var UA_KEY_IAPROFILELABEL = "com.ibm.xtools.impactanalysis.web.app.text_IAConfig001";
var UA_KEY_MAXDEPTHLABEL = "com.ibm.xtools.impactanalysis.web.app.text_MaxDepth001";

dojo.declare(
	"dm.analysis.app.ui.internal.impactanalysis.ImpactDefinitionsInlineEditor", 
	[dijit._Widget, dijit._Templated], 
	{
		messages: messages,
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl('dm.analysis.app','ui/internal/impactanalysis/templates/ImpactDefinitionsInlineEditor.html'),

		// init properties
		editor: null,

		constructor: function(args) {
			this.editor = args.editor;
		},
		
		postCreate: function() {
			this.inherited(arguments);

			if (!IA_Settings.configurationsSupported)
			{
				this._configurationRow.style.display = "none";
			}
			else
			{
				this._configurationSearchBox = new dm.analysis.app.ui.internal.impactanalysis.picker.IAConfigurationSearchBox({
					   onSelect: dojo.hitch(this, this.updateConfigurationAfterLocate)
					}, this._configurationSearch);
				this._configurationSearchBoxSpan = this._configurationSearchBox._configurationSpan;
			}
			var ia = this.editor.editorInput.impactAnalysis;

			this.fillProfiles();
			this._initializeTabs();

			this._profileTextHoverHelp = new jazz.app.ua.TextHoverHelp({
				uaKey: UA_KEY_IAPROFILELABEL,
				contentNode: this._profileSpan
			});

			this._depthTextHoverHelp = new jazz.app.ua.TextHoverHelp({
				uaKey: UA_KEY_MAXDEPTHLABEL,
				contentNode: this._depthSpan
			});

			// handle dirty awareness
			this.connect(this._selectedProfile, "onchange", dojo.hitch(this, function() {
				this.populateProfileInfo(this._selectedProfile.value);
				this.editor.enableRunButton();
				this.editor._updateDirty(ia.iAType, this._selectedProfile.value, this._selectedProfile);
			}));

			this.subscribe('dm.analysis.app.ui.internal.impactanalysis.profile.refresh', dojo.hitch(this, 'refreshProfiles'));

			var locateButton = this._elementToolbar.addButton(this, dojo.hitch(this.editor, '_locateElement'), {
				label: messages.iaEditorLocate,
				imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/search-icon.gif')
			});

			this.saveProfileButton = this._profileToolbar.addWidget(new rmps.ui.SimpleButton({
				label: messages.iaEditorSaveProfile,
				onClick: dojo.hitch(this.editor, '_saveProfile')
			}));


			this._buttons = [locateButton, this.saveProfileButton];
		},

		_initializeTabs: function(){
			var tabStyleName= "Selected";

			this._tabManagerLeft = new TabManager();
			
			this._tabManagerLeft.registerTab(this._tabImpactDef, this._impactDefContent, tabStyleName);
			this._tabManagerLeft.registerTab(this._tabTraversal, this._traversalContent, tabStyleName);
			this._tabManagerLeft.registerTab(this._tabFilters, this._filtersContent, tabStyleName);

			this.traversalDefinition = new dm.analysis.app.ui.internal.impactanalysis.profile.TraversalDefinition({
				profileInfoChangeCallback: dojo.hitch(this, this._profileInfoChangeCallback),
				editor: this.editor,
				iaProfile: this.iaProfile,
				isReadOnly: this.isReadOnly
			}, this._traversalContentDiv);
			this.traversalDefinition.domNode.style.marginTop = "6px";

			this.filterList = new dm.analysis.app.ui.internal.impactanalysis.panels.FilterListPanel({
				profileInfoChangeCallback: dojo.hitch(this, this._profileInfoChangeCallback),
				editor: this.editor,
				iaProfile: this.iaProfile,
				isReadOnly: this.isReadOnly
			}, this._filterListDiv);


			dojo.addClass(this.filterList.domNode, "element-list");

			this.filterList.startup();

			this._tabManagerLeft.showTab(this._tabImpactDef);
		},

		//Called anytime the Profile data fields are modified
		_profileInfoChangeCallback: function(){
			this.editor.enableRunButton();

			if (this.isProfileInfoModified())
				dojo.removeClass(this._profileToolbar.domNode, "hidden");
			else
				dojo.addClass(this._profileToolbar.domNode, "hidden");
		},
		
		resetProfileToolbar: function() {
			dojo.addClass(this._profileToolbar.domNode, "hidden");
		},

		isProfileInfoModified: function(){
			var profileInfo = this.getProfileInfo();
			if (!this.iaProfile) return false;

			var changed = (profileInfo.traversal !== this.iaProfile.traversal);

			if (profileInfo.traversal === profileInfo.UPSTREAM || profileInfo.traversal === profileInfo.BOTH)
			{
				if (profileInfo.upstreamType === ""){
					var bothUpsEmpty = (!profileInfo.upstreamDepth && !this.iaProfile.upstreamDepth);
					var upstreamDepthEqual = bothUpsEmpty ? true : (profileInfo.upstreamDepth === this.iaProfile.upstreamDepth);
					changed = changed || !upstreamDepthEqual;
				} else {
					changed = changed || (profileInfo.upstreamType !== this.iaProfile.upstreamType);
				}
			}
			
			if (profileInfo.traversal === profileInfo.DOWNSTREAM ||  profileInfo.traversal === profileInfo.BOTH)
			{
				if (profileInfo.downstreamType === ""){
					var bothDownsEmpty = (!profileInfo.downstreamDepth && !this.iaProfile.downstreamDepth);
					var downstreamDepthEqual = bothDownsEmpty ? true : (profileInfo.downstreamDepth === this.iaProfile.downstreamDepth);
					changed = changed || !downstreamDepthEqual;
				} else {
					changed = changed || (profileInfo.downstreamType !== this.iaProfile.downstreamType);
				}
			}
			
			if (profileInfo.traversal === profileInfo.FULL_TRACE)
			{
				if (profileInfo.fullTraceType === ""){
					var bothFullsEmpty = (!profileInfo.fullTraceDepth && !this.iaProfile.fullTraceDepth);
					var fullTraceDepthEqual = bothFullsEmpty ? true : (profileInfo.fullTraceDepth === this.iaProfile.fullTraceDepth);
					changed = changed || !fullTraceDepthEqual;
				} else {
					changed = changed || (profileInfo.fullTraceType !== this.iaProfile.fullTraceType);
				}
			}
			
			if (!changed) //If the save button is going to be enabled already, there's no point in checking the filters
			{
				//This gets the things in profileInfo that don't appear in iaProfile
				var profileInfo_dojoFiltered = dojo.filter(profileInfo.filters, dojo.hitch(this, function(filter){
					return (!dojo.some(this.iaProfile.filters, function(originalFilter){
						return originalFilter.uri === filter.uri;
					}));
				}));

				//This gets the things in iaProfile that don't appear in profileInfo
				var iaProfile_dojoFiltered = dojo.filter(this.iaProfile.filters, dojo.hitch(this, function(originalFilter){
					return (!dojo.some(profileInfo.filters, function(filter){
						return filter.uri === originalFilter.uri;
					}));
				}));

				changed = changed || 
							(profileInfo_dojoFiltered.length !== 0) || 
							(iaProfile_dojoFiltered.length !== 0);
			}

			return changed;
		},

		enableButtons: function(enable){
			var i,
				b;
			for (i = 0; i < this._buttons.length; i++)
			{
				b = this._buttons[i];
				if (b.setEnabled) {
					b.setEnabled(enable);
				}
			}
		},

		_addProfileOption: function(profileName, profileUri)
		{
			var opt = dojo.doc.createElement("option");
			opt.innerHTML = jazz.util.html.escape(profileName);
			opt.value = profileUri;
			this._selectedProfile.appendChild(opt);
		},

		changeProfileSelectionTo: function(profileUri)
		{
			dojo.forEach(this._selectedProfile.options, dojo.hitch(this, function(option){
				if (option.value === profileUri){
					this._selectedProfile.selectedIndex = option.index;
				}
			}));

			this.populateProfileInfo(profileUri);
			this.editor._updateDirty(this.editor.editorInput.impactAnalysis.iAType, this._selectedProfile.value, this._selectedProfile);
		},

		getProfileInfo: function(){
			var tdef = this.traversalDefinition.getTraversalDefinition();

			var profileInfo = {
				UPSTREAM: tdef.UPSTREAM,
				DOWNSTREAM: tdef.DOWNSTREAM,
				BOTH: tdef.BOTH,
				FULL_TRACE: tdef.FULL_TRACE,
				traversal: tdef.traversal,
				upstreamDepth: tdef.upstreamDepth,
				downstreamDepth: tdef.downstreamDepth,
				fullTraceDepth: tdef.fullTraceDepth,
				upstreamType: tdef.upstreamType,
				downstreamType: tdef.downstreamType,
				fullTraceType: tdef.fullTraceType,
				filters: this.filterList.getFilters()
			};

			return profileInfo;
		},

		populateProfileInfo: function(profileUri){
			this._getProfileInfoDeferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.getIAProfile(profileUri);
			this._getProfileInfoDeferred.addCallback(this, function(iap) {
				this.iaProfile = iap;
				this.editor.iaProfile = iap;
				if (this.traversalDefinition)
					this.traversalDefinition.loadTraversalDefinitionFromProfile(this.iaProfile);
				if (this.filterList)
					this.filterList.loadFiltersFromProfile(this.iaProfile);
			});
			this._getProfileInfoDeferred.addErrback(this, function(error) {
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: dojo.string.substitute(messages.errorUnavailableResource, [profileUri]),
					details: [ { message: messages['errorUnavailableResource.explanation'] }],
					closable: true
				});
			});
		},

		refreshProfiles: function(originatingDocument)
		{
			if (originatingDocument === this.ownerDocument.URL)
			{  // If this was the editor that initiated refresh, then don't refresh the profiles
				return;
			}
			this.fillProfiles();
		},
		
		fillProfiles: function()
		{
			while(this._selectedProfile.hasChildNodes())
			{
				this._selectedProfile.removeChild(this._selectedProfile.lastChild);
			}
			
			var deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.getAllIAProfiles(this.editor.editorInput.projectId);
			deferred.addCallback(this, function(result) {
				var i,
					opt,
					profile;
				for(i = 0 ; i < result.ImpactProfilesList.length ; i++) {
					profile = result.ImpactProfilesList[i];
					opt = dojo.doc.createElement("option");
					opt.innerHTML = jazz.util.html.escape(profile.name);
					opt.value = profile.URI;
					if (this.editor.editorInput.impactAnalysis.iAType === profile.URI)
					{
						opt.selected = true;
					} else if(this.editor.editorInput.impactAnalysis.iAType === null && profile.systemDefaultProfile ) {
						opt.selected = true;
					}
					if (opt.selected){
						this.populateProfileInfo(opt.value);
					}

					this._selectedProfile.appendChild(opt);
				}
			});
			deferred.addErrback(this, function(error) {
				var xhr = deferred.ioArgs.xhr;
				if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
					var lqeInfo = dojo.fromJson(xhr.responseText);
					this.editor.showMessage({
						type: lqeInfo.type,
						summary: lqeInfo.summary,
						details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
						closable: true
					});
				} else {
					this.editor.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: messages.errorGetAllIAProfiles,
						errorObject: error,
						closable: true
					});
				}
			});
		},
		
		getImpactDefinition: function() {
			var params = {
				iAType: this._selectedProfile.value,
				iAViewType: 'no_ld'
			};

			return params;
		},

		reset: function(targetElementInfo, contextConfigVersionUri){
			//set image and 
			while(this._locatedElement.hasChildNodes()){
				this._locatedElement.removeChild(this._locatedElement.lastChild);
			}
			if (targetElementInfo)
			{
				var uri = targetElementInfo.uri || targetElementInfo;
				
				var label = targetElementInfo.shortName || " - ";
				if(uri) {
					dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil.createResourceLink(this._locatedElement, {
						uri: uri,
						label: dojox.html.entities.encode(label)
					});
				}
			}
			
			
			if (IA_Settings.configurationsSupported)
			{
				this._configurationSearchBox.setConfigurationUri(contextConfigVersionUri);
				if (AnalysisUtil.isConfigVersion(uri))
				{
					this._configurationSearchBox.disable();
				}
				else
				{
					this._configurationSearchBox.enable();
				}
			}
		},
		
		updateConfigurationAfterLocate: function(configurationUri) {
			var oldContextConfigVersionUri = this.editor.editorInput.impactAnalysis.contextConfigVersionUri;
			this.editor.editorInput.impactAnalysis.contextConfigVersionUri = configurationUri;
			
			this.editor.enableRunButton();
			this.editor._updateDirty(oldContextConfigVersionUri, configurationUri, 'contextConfigVersionUri');
			
			if (configurationUri != null)
			{
				var deferred = dm.common.app.ui.internal.rhlink.IAInfoClient.getInfo(configurationUri);
				
				deferred.addCallback(this, function(infos) {
					this.setConfigurationArtifactSpan(infos[0]);
					this._startAnalysisButton.disabled = false;
				});
				deferred.addErrback(this, function(error) {
					this._messageBox.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: dojo.string.substitute(messages.errorUnavailableResource, [resourceUri]),
						details: [ { message: messages['errorUnavailableResource.explanation'] }],
						closable: true
					});
				});
			}
		},
		
		setConfigurationArtifactSpan: function(configurationElementInfo){
			var uri = configurationElementInfo.uri || configurationElementInfo;
			this._configurationSearchBox.setConfigurationUri(uri);
		},
		
		uninitialize: function(){
			this._profileTextHoverHelp && this._profileTextHoverHelp.destroyRecursive();
			this._depthTextHoverHelp && this._depthTextHoverHelp.destroyRecursive();
			this.inherited(arguments);
		}
});

}());
