/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function () {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.InitDiagramDialog");


dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.app.context");

dojo.require('dojo.data.ItemFileReadStore');
dojo.require('dijit.form.FilteringSelect');

dojo.require('dm.common.app.ui.internal.rhlink.IAInfoClient');
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.picker.IASearchBox");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.picker.IAConfigurationSearchBox");
dojo.require("dm.analysis.app.ui.internal.IA_Settings");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");


dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 

var AnalysisUtil = dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil;
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 
var IA_Settings = dm.analysis.app.ui.internal.IA_Settings;
dojo.declare("dm.analysis.app.ui.internal.impactanalysis.InitDiagramDialog", null,  {

	message: messages,
	parent:null,

	targetUri:null,
	dialogContent:null,
	
	constructor: function(parent) {
		this.parent = parent;
	},

	closeDialog: function(){
		if (this.createDialog){
			this.createDialog.close();
		}
	},

	setDialogTarget: function(targetUri) {
		this.targetUri = targetUri;
	},

	createDialog: function(){
		this.dialogContent = new dm.analysis.app.ui.internal.impactanalysis.InitDiagramDialog_content({ parent: this.parent, dialog: this });

		if (this.targetUri) 
			this.dialogContent.updateAfterLocate(this.targetUri, false);
		
		this.createDialog = new jazz.ui.Dialog({
			primaryTitle: messages.initDialogTitle,
			modal: true,
			center: true,
			closeOnEsc: true,
			headerless: false,
			closable: true,
			width:  "500px",
			moveable: true,
			collapsible: false,
			showBackground: false,
			contentNode: this.dialogContent.domNode,
			onClose: dojo.hitch(this, function() {
				this.dialogContent.destroyRecursive();
			})
		});
	}
	
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.InitDiagramDialog_content", [dijit._Widget, dijit._Templated], {
	
	messages: messages,
	widgetsInTemplate: true,

	parent: null,
	dialog: null,
	newTargetUri: null,
	newProfileUri: null,
	newContextConfigVersionUri: null,

	profileStore: null,
	_profileSelectionChangeHandle: null,
	_startAnalysisButtonClickHandle: null,
		
	templatePath: dojo.moduleUrl('dm.analysis.app', 'ui/internal/impactanalysis/templates/InitDiagramDialog.html'),
		
	postCreate: function()
	{
		if (!IA_Settings.configurationsSupported)
		{
			this._targetArtifactSection.style.height = "55px";
			this._configurationRow.style.display = "none";
		}
		else
		{
			//Configuration Search Box:
			this._configurationSearchBox = new dm.analysis.app.ui.internal.impactanalysis.picker.IAConfigurationSearchBox({
			   onSelect: dojo.hitch(this, this.updateConfigurationAfterLocate)
			}, this._configurationSearch);
			this._configurationSearchBoxSpan = this._configurationSearchBox._configurationSpan;
			
			this.newContextConfigVersionUri = this.parent.editorInput._params.contextConfigVersionUri;
			
			if(this.newContextConfigVersionUri){
				this.retrieveAndSetConfigurationInfo(this.newContextConfigVersionUri);
			}
		}
		//Target Artifact Search Box:
		this._targetArtifactSearchBox = new dm.analysis.app.ui.internal.impactanalysis.picker.IASearchBox({
		   onSelect: dojo.hitch(this, this.updateAfterLocate)
		}, this._targetArtifactSearch);
		this._targetArtifact = this._targetArtifactSearchBox._targetArtifact;

		var targetUri = this.parent.editorInput.impactAnalysis.targetResourceUri;
		if(!!targetUri){
			this.retrieveAndSetTargetInfo(targetUri);
		}


		//Filtering Select data store for Profiles:
		this.profileStore = new dojo.data.ItemFileReadStore({clearOnClose: true});
		// provide some dummy data until we are ready to retrieve
		this.profileStore.data = {
			identifier: 'key',
			label: 'value',
			items: []
		};
			
		//Filtering Select:
		this._profileSelect = new dijit.form.FilteringSelect({
			id: '_profileSelect',
			store: this.profileStore,
			'aria-describedby':'previewContentId',
			//name: 'key',
			searchAttr: 'value',
			style: 'width: 400px'
		});
		this._profileSelectDiv.appendChild(this._profileSelect.domNode);
		this._profileSelectionChangeHandle = dojo.connect(this._profileSelect, "onChange", dojo.hitch(this, this._profileSelectionChanged));

		this.populateProfiles();

		//Start Analysis Button:
		this._startAnalysisButton = dojo.doc.createElement("button");
		this._startAnalysisButton.innerHTML = messages.dialogButtonStartAnalysis;
		this._startAnalysisButton.disabled = true;
		this._startAnalysisButtonClickHandle = dojo.connect(this._startAnalysisButton, "onclick", this, this.startAnalysis);
		this._buttonSectionDiv.appendChild(this._startAnalysisButton);
	},

	_targetArtifactPickerCallback: function(e) {
		e && dojo.stopEvent(e);
		this._targetArtifactSearchBox.cancelSearchResults();
		this.parent.locateController.locateElement(this); //calls this.updateAfterLocate after picker returns
	},

	populateProfiles: function(){
		var projectId = jazz.app.context.get().itemId;

		this._getProfilesDeferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.getAllIAProfiles(projectId);
		this._getProfilesDeferred.addCallback(this, function(_profiles) {
			var profiles = _profiles.ImpactProfilesList;
			var noData = profiles.length === 0;

			// Reload the profileStore data store
			this.profileStore.data = {
				identifier: 'key',
				label: 'value',
				items: dojo.map(profiles, function (profile) {
					return {
						key: profile.URI,
						value: profile.name
					};
				}, this)
			};
			this.profileStore.close();

			dojo.forEach(profiles, dojo.hitch(this, function(profile){
				if (profile.systemDefaultProfile){
					this._profileSelect.set('value', profile.URI);
				}
			}));

			if (noData) { //disable other option
				this._profileSelect.disabled = true;
				//TODO: display noProfiles message
				//this.message.innerHTML = messages.iaEditorSelectionElmDialogNoFilters;
			}
		});
		this._getProfilesDeferred.addErrback(this, function(error) {
			var xhr = this._getProfilesDeferred.ioArgs.xhr;
			this.parent.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.errorGetDomains,
				errorObject: error,
				closable: true
			});
			this.dialog.closeDialog();
		});


	},

	updateAfterLocate: function(resourceUri, updateParent){
		updateParent = typeof updateParent !== 'undefined' ? updateParent : true;  //default value of updateParent is true
		
		this.newTargetUri = resourceUri;
		this.retrieveAndSetTargetInfo(resourceUri);
		
		if (IA_Settings.configurationsSupported)
		{
			if (AnalysisUtil.isConfigVersion(resourceUri))
			{
				this.newContextConfigVersionUri = null;
				this._configurationSearchBox.disable();
			}
			else
			{
				this._configurationSearchBox.enable();
			}
		}
		if (updateParent)
		{
			this.parent.editorInput.impactAnalysis.targetResourceUri = resourceUri;
			this.parent.inlineEditor.reset(resourceUri, this.newContextConfigVersionUri);
		}
		
	},
	
	updateConfigurationAfterLocate: function(configurationUri, updateParent){
		updateParent = typeof updateParent !== 'undefined' ? updateParent : true;  //default value of updateParent is true
		
		this.newContextConfigVersionUri = configurationUri;
		this.retrieveAndSetConfigurationInfo(configurationUri);
		if (updateParent)
		{
			this.parent.editorInput.impactAnalysis.contextConfigVersionUri = configurationUri;
			this.parent.inlineEditor.reset(this.newTargetUri, configurationUri);
		}
		
	},

	retrieveAndSetTargetInfo: function(resourceUri){
		var deferred = dm.common.app.ui.internal.rhlink.IAInfoClient.getInfo(resourceUri);
		this._targetArtifactSearchBox.showSpinner(messages.loading);
		
		deferred.addCallback(this, function(infos) {
			this._targetArtifactSearchBox.hideSpinner();
			this.setTargetArtifactSpan(infos[0]);
			this._startAnalysisButton.disabled = false;
		});
		deferred.addErrback(this, function(error) {
			this._targetArtifactSearchBox.hideSpinner();
			this._messageBox.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: dojo.string.substitute(messages.errorUnavailableResource, [resourceUri]),
				details: [ { message: messages['errorUnavailableResource.explanation'] }],
				closable: true
			});
		});
	},
	
	retrieveAndSetConfigurationInfo: function(resourceUri){
		if (resourceUri != null)
		{
			var deferred = dm.common.app.ui.internal.rhlink.IAInfoClient.getInfo(resourceUri);
			
			deferred.addCallback(this, function(infos) {
				var configurationElementInfo = infos[0];
				var uri = configurationElementInfo.uri || configurationElementInfo;
				this._configurationSearchBox.setConfigurationUri(uri);
			});
			deferred.addErrback(this, function(error) {
				this._messageBox.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: dojo.string.substitute(messages.errorUnavailableResource, [resourceUri]),
					details: [ { message: messages['errorUnavailableResource.explanation'] }],
					closable: true
				});
			});
		}
	},
	
	setTargetArtifactSpan: function(targetElementInfo){
		while(this._targetArtifact.hasChildNodes()){
			this._targetArtifact.removeChild(this._targetArtifact.lastChild);
		}

		var uri = targetElementInfo.uri || targetElementInfo;
		var label = targetElementInfo.shortName || " - ";
		if(uri) {
			dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil.createResourceLink(this._targetArtifact, {
				uri: uri,
				label: dojox.html.entities.encode(label)
			});
		}
	},

	_profileSelectionChanged: function(){
		var profileName = this._profileSelect.item.value[0];
		var profileUri = this._profileSelect.item.key[0];

		if (!profileName) return;

		this._getProfileInfoDeferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.getIAProfile(profileUri);
		this._getProfileInfoDeferred.addCallback(this, function(iap) {
			var previewContent = new dm.analysis.app.ui.internal.impactanalysis.InitDiagramDialog_previewContent({ iaProfile: dojo.clone(iap) });

			while(this._profilePreviewDiv.hasChildNodes()){
				this._profilePreviewDiv.removeChild(this._profilePreviewDiv.lastChild);
			}

			this._profilePreviewDiv.appendChild(previewContent.domNode);
		});
		this._getProfileInfoDeferred.addErrback(this, function(error) {
			this.parent.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: dojo.string.substitute(messages.errorUnavailableResource, [profileUri]),
				details: [ { message: messages['errorUnavailableResource.explanation'] }],
				closable: true
			});
		});

		this.newProfileUri = profileUri;
	},

	startAnalysis: function(){
		//TODO: form field checking, display messages
		if (!this.newTargetUri || !this.newProfileUri)
			return;

		this.parent._initDiagramDialogCallback(this.newTargetUri, this.newProfileUri, this.newContextConfigVersionUri);

		this.parent._run();
		this.dialog.closeDialog();
	},

	destroy: function(){
		if (this._profileSelectionChangeHandle) 
			dojo.disconnect(this._profileSelectionChangeHandle);

		if (this._startAnalysisButtonClickHandle)
			dojo.disconnect(this._startAnalysisButtonClickHandle);

		if (this._profileSelect)
			this._profileSelect.destroy();
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.InitDiagramDialog_previewContent", [dijit._Widget, dijit._Templated], {
	
	messages: messages,
	widgetsInTemplate: true,

	iaProfile: null,
		
	templateString: '<div class="preview-content">'+
						'<table role="presentation" id="previewContentId">'+
							'<tr>'+
								'<td>'+
									'<span class="preview-description-label">${messages.initDialogDescriptionLabel}</span>'+
								'</td>' +
							'</tr>'+
							'<tr>'+
								'<td class="preview-description-column">'+
									'<span class="preview-description" dojoAttachPoint="_descriptionSpan"></span>'+
								'</td>' +
							'</tr>'+
							'<tr>'+
								'<td>'+
									'<span class="preview-filters-label">${messages.initDialogFiltersLabel}</span>'+
								'</td>' +
							'</tr>'+
							'<tr>'+
								'<td class="preview-filters-column">'+
									'<div dojoAttachPoint="_filtersDiv"></div>'+
								'</td>' + 
							'</tr>'+
						'<table>'+
					'</div>',

	postCreate: function()
	{
		this._descriptionSpan.innerHTML =  jazz.util.html.escape(this.iaProfile.description);
		
		if (this.iaProfile.filters && this.iaProfile.filters.length) {
			dojo.forEach(this.iaProfile.filters, dojo.hitch(this, function(filter){
				var filterSpan = dojo.doc.createElement("span");

				filterSpan.innerHTML = filter.name;
				filterSpan.innerHTML += "<br>";

				this._filtersDiv.appendChild(filterSpan);
			}));
		} else {
			this._filtersDiv.innerHTML = messages.noFilters;
		}


	}
});

}());
