
/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.LegendDialog");
dojo.require("jazz.ui.Dialog");

dojo.require("jazz.app.context");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.IAColorTable");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 

var colorTable = dm.analysis.app.ui.internal.impactanalysis.IAColorTable;

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.LegendDialog", null,  {
	
	message: messages,
	parent:null,
	tools:null,
	
	constructor: function(parent , tools) {
		this.parent = parent;
		this.tools = tools;
		this.createDialog(this.text,this.width);
	},
		
	createDialog: function(text, width){
		
		this.dialogContent = new dm.analysis.app.ui.internal.impactanalysis.LegendDialog_content({parent:this});
		
		this.createDialog = new jazz.ui.Dialog({
			primaryTitle: messages.legendDiagramTitle,
			modal: true,
			//trim: jazz.ui.StyledBox.TrimStyles.BLUE_ALPHA_SHADOW,
			center: true,
			closeOnEsc: true,
			headerless: false,
			closable: true,
			width:  "280px",
			moveable: true,
			collapsible: false,
			showBackground: false,
			contentNode: this.dialogContent.domNode,
			onClose: dojo.hitch(this, function() {
				this.dialogContent.destroyRecursive();
			})
		});
	}
	
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.LegendDialog_content", [dijit._Widget, dijit._Templated], {
	
	messages: messages,
	widgetsInTemplate: true,
	parent:null,
		
	templateString:
		'<div class="dm-analysis-LegendDialog" dojoAttachPoint="containerNode" style="overflow: auto; padding: 0px; height: 100%; width: 100%">'+			
			'<table style="overflow: hidden; width:100%">'+
				'<tr>'+
					'<td >'+
						'<span class="dirlabel">${messages.elementsLegend}</span>'+
					'</td>'+
				'</tr>'+
				 '<tr>'+
					'<td dojoAttachPoint="toolsNode">'+
					'</td>'+
				'</tr>'+
				 '<tr>'+
					'<td >'+
						'<span class="dirlabel">${messages.linksLegend}</span>'+
					'</td>'+
				'</tr>'+
				 '<tr>'+
					'<td dojoAttachPoint="linksNode">'+
					'</td>'+
				'</tr>'+
			'</table>'+
		'</div>',
		
	postCreate: function()
	{
		this.toolsLegend = new dm.analysis.app.ui.internal.impactanalysis.LegendDialog_legend();
		
		this.toolsLegend.setElements( this.transfromToolStractToLegendStract(this.parent.tools.items) );
		this.toolsNode.appendChild (this.toolsLegend.domNode);
		
		var linksLegendData = [ { color: colorTable.getLinkColor("intra"),  
							 label: messages.intraLink, 
							 description: messages.intraLink , 
							 line: {width: 1 ,color :colorTable.getLinkBorderColor("intra")}}, 
						   { color: colorTable.getLinkColor("inter"),  
							 label: messages.interLink, 
							 description: messages.intraLink , 
							 line: {width: 1 ,color :colorTable.getLinkBorderColor("inter")}}
//						   { color: colorTable.getLinkColor("crossDomain"),  
//							 label: messages.crossDomainLink, 
//							 description: messages.crossDomainLink , 
//							 line: {width: 1 ,color :colorTable.getLinkBorderColor("crossDomain")}},
							 ];
		
		var linksLegend = new dm.analysis.app.ui.internal.impactanalysis.LegendDialog_legend();
		linksLegend.setElements(linksLegendData );
		this.linksNode.appendChild (linksLegend.domNode);
		
	},
	
	transfromToolStractToLegendStract: function (tools)
	{
		var legendStract = [] ;
		
		var targetElement = {"color" : colorTable.getElementColor(0), "label": messages.focusArtifact, "description": messages.focusArtifact, "line":  {style: "ShortDash", width: 1 , color :colorTable.getElementBorderColor(0)}};
		legendStract[0] = targetElement;
		
		for (var i = 0 ; i < tools.length ; i++)
		{
			var legendElement = {};
			legendElement.color = colorTable.getElementColor(tools[i].color);
			legendElement.label = tools[i].name;
			legendElement.description = tools[i].name;
			legendElement.line = {width: 1 , color :colorTable.getElementBorderColor(tools[i].color)};
			legendStract[(i+1)] = legendElement;
		}
		return legendStract;
	},

	destroy: function(){
		this.toolsLegend.destroyRecursive();
	}
	
	
});




dojo.declare("dm.analysis.app.ui.internal.impactanalysis.LegendDialog_legend", [dijit._Widget, dijit._Templated], {
	
	templateString: '<div>' + '<div dojoAttachPoint="containerNode"></div>' + '</div>',

	keys: null,
	
	postCreate: function(){
		this.inherited( arguments );
		
		this.keys = [];
		
		
		
		var test = [ { color:'blue',  description: 'John' }, 
					{ color:'red', description: 'Paul' },
					{ colors: [{offset: 0, color: [251,238,238, 1]}, {offset: 1, color: [229,127,128, 1]}], description: 'George' },
					{ color:'yellow', description: 'Ringo' }];
	},
	
	fill: function( item, data ){
		
		if( data.color ){
			item.setFill( data.color );
		}else{
			if( !dojo.isIE ){
				item.setFill( {
					type: "linear", x1: 0, y1: 0, x2: 0, y2: 25 ,
					colors: data.colors
				});
			}
		}
		
		if( data.line ){
			item.setStroke( data.line );
		}
	},
	
	
	/* setElements takes and array of entries:
	 * 
	 * [ { color:'blue', description: 'John' }, 
	 *   { color:'red', description: 'Paul' },
	 *   { colors: [{offset: 0, color: [251,238,238, 1]}, {offset: 1, color: [229,127,128, 1]}], description: 'George' },
	 *   { color:'yellow', description: 'Ringo' } ]
	 * 
	 */
	
	setElements: function( elements ){
		
		var gap = 15;
		
		var rows = elements.length ;
		
		var totalHeight = 10 + rows * gap;
		
		this.domNode.style.height = totalHeight + 'px';
		this.surface = dojox.gfx.createSurface(this.containerNode, 255, totalHeight);
		
		var font = {family: "sans-serif", size: "12px"};

	    var measurementHeightError = 4;		
		
		var x = 5;
		var dx = 165;
		
		for( var e=0; e<elements.length;e++ ){
			var row = Math.floor(e);	
			var y = 5 + ( row * gap );
			this.keys[e] = this.surface.createRect( {x: x, y: y, height:10, width:15 } );
			this.fill( this.keys[e], elements[e] );		
			
			/* Floating some clear divs about the vector squares to 
			 * latch some tooltips onto. Might be able to simplify this 
			 * all with some css3.
			 */
			
			var t = document.createElement("div");
			
			var dy = totalHeight - ( gap + ( row * gap ) );
			
			t.style.width = '90px';
			t.style.height = '10px';
			t.style.position = 'absolute';		
			t.style.bottom = dy + 'px';
			t.style.right =  dx + 'px';
			
			t.title = elements[e].description;
			
			/* For testing position of divs 
			
			t.style.background = 'silver';
			t.innerHTML = e; */
									
			this.domNode.appendChild(t);
			
			this.surface.createText( {text: elements[e].label, x: x + 18,
				y: y + 9 }).setFill("black").setFont(font);	
			
			//if( x == 5 ){ x = 100; dx = 10; }else{ x = 5; dx = 165; };
		}
	}
});




})();