/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('dm.analysis.app.ui.internal.impactanalysis.PreviewLoader'); //$NON-NLS-1$

dojo.require('jazz.ui.internal.ResourceLinkAggregator'); //$NON-NLS-1$
dojo.require('jazz.ui.HoverPopup'); //$NON-NLS-1$
dojo.require('jazz.util.internal.login'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$

// TODO: use better string for Log In message. But since it's so late in 4.0.3, we'll reuse an existing string.
// dojo.requireLocalization('dm.analysis.app.ui.internal.impactanalysis', 'PreviewLoaderMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("jazz.app.internal", "LoginLogoutMessages");

(function(){
	var linkAggregator_ = jazz.ui.internal.resourceLinkAggreagtor();
	/**
	 * Returns all incoming and outgoing nodes from a given node.
	 * @param  {object} rootNode dojo diagrammer node.
	 * @return {array}          all incoming and outgoing nodes from the given
	 *     node. Empty if there are none.
	 */
	function getIncomingOutgoingNodes(rootNode) {
		var nextNodes = dojo.map(rootNode.getLinks(true, false), function(link) {
			return link.getEndNode();
		});

		dojo.forEach(rootNode.getLinks(false, true), function(link) {
			nextNodes.push(link.getStartNode());
		});

		return nextNodes;
	}

	/**
	 * Overrides the .postition method of HoverPopup to work with SVG elememnts,
	 * specifically, it calls dojo.position for get the abs position instead of dojo.coords.
	 */
	dojo.declare(
		'dm.analysis.app.ui.internal.impactanalysis.DiagramHoverPopup', //$NON-NLS-1$
		[jazz.ui.HoverPopup],
		{
			position:  function(node) {
				var master = jazz.ui._getMasterPopup();
				if (!node) {
					if (!this._isOpen || !this._target)
						return;
					node = this._target;
				}

				// <CHANGED CODE>
				var posAbs = dojo.position(node, true);
				var posRel = dojo.position(node, false);
				// </CHANGED CODE>

				var v = dijit.getViewport();

				this.updateDimensions(v,posAbs,posRel);

				this.effectiveMaxWidth = Math.min(this.maxWidth, v.w - 60);
				this.effectiveMaxHeight = Math.min(this.maxHeight, v.h - 30);

				master.contentWrapper.style.maxWidth = this.effectiveMaxWidth + "px"; //$NON-NLS-1$
				master.contentWrapper.style.maxHeight = this.effectiveMaxHeight + "px"; //$NON-NLS-1$

				if (this.orientation === "V" ||this.orientation === "v" || this.orientation === "T" || this.orientation === "t" || this.orientation === "B" || this.orientation ===  "b") //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$
					this.positionVer(master, posAbs, posRel, v);
				else
					this.positionHor(master, posAbs, posRel, v);

				master.contentWrapper.scrollTop = 1; // workaround a scrollbar display bug in FF
				master.contentWrapper.scrollTop = 0;
				if (dojo.isIE < 8) {
					// force a reflow of the content in IE7
					master.content.className = master.content.className;
				}
			}
		});

	/**
	 * Recursively loads OSLC previews (icons and titles) for nodes in an IA
	 * graph. Starts with the same target and added artifacts as IA and gets
	 * their previews first. Once done, branches to the incoming and outgoing
	 * links for those nodes. This requires several calls to load the whole
	 * graph, but ensures that the target nodes and "close" nodes will load
	 * quickly. Otherwise, large graphs can take a very long time to load all
	 * details.
	 */
	dojo.declare(
		'dm.analysis.app.ui.internal.impactanalysis.PreviewLoader', //$NON-NLS-1$
		[],
		{
			// messages: dojo.i18n.getLocalization('dm.analysis.app.ui.internal.impactanalysis', 'PreviewLoaderMessages'), //$NON-NLS-1$ //$NON-NLS-2$
			loginMessages: dojo.i18n.getLocalization("jazz.app.internal", "LoginLogoutMessages"), //$NON-NLS-1$ //$NON-NLS-2$

			/**
			 * Constructs a loader that will start from the given node. Does
			 * not start the load until .loadPreviews is called.
			 * @param  {array} startNodes the target and "added" nodes from an IA diagram.
			 * These are dojo diagrammer node instances.
			 */
			constructor: function(startNodes)
			{
				this._startNodes = startNodes;

				// tracks all seen URIs for nodes to treat seen URIs as base
				// cases and stop examining them.
				this._seenUris = {};

				// When a HoverPopup is created to show a login link (needed to get live icons)
				// we track it here so it can be cleaned up once the preview for that URI
				// is loaded.
				this._hoverPopupsByUri = {};
			},

			/**
			 * Destroys all known hover popups for logging in. Done during destroy.
			 */
			_destroyAllHoverPopups: function() {
				for (var uri in this._hoverPopupsByUri) {
					var list = this._hoverPopupsByUri[uri];
					for (var i = 0; i < list.length; i++) {
						var hover = list[i];
						hover.destroyRecursive();
					}
				}

				this._hoverPopupsByUri = {};
			},

			/**
			 * Starts loading previews from the initial nodes, working out
			 * from their to related nodes. Automatically replaces node icons
			 * in the graph as it goes.
			 *
			 * Currently does NOT replace titles, since they cause the graph
			 * to resize and need an expensive relayout.
			 */
			loadPreviews: function() {
				this._seenUris = [];
				this._loadPreviewsForNodes(this._startNodes);
			},

			/**
			 * Called recursively. Requests the previews and skips seen
			 * node URIs.
			 * @param  {array} nodes the nodes the get previews for. Seen
			 *     nodes are skipped.
			 */
			_loadPreviewsForNodes: function(nodes) {
				for (var i = 0; i < nodes.length; i++) {
					var node = nodes[i];
					var artifactUri = node.getData().item.URI;

					if (this._seenUris[artifactUri]) {
						continue;
					}
					this._seenUris[artifactUri] = true;

					linkAggregator_.register(
						artifactUri,
						dojo.hitch(this, this._richPreviewDidLoad, node),
						dojo.hitch(this, this._richPreviewFailed, node));
				}
			},

			_doLogin: function(host, node, e) {
				dojo.stopEvent(e);

				// jazz.client is smart enough to inercept this request and do the login for us.
				// Just need to refresh previews once we're done.
				var self = this;
				jazz.client.xhrGet({
						url: node.getData().item.URI,
						headers: {
							accept: "application/x-oslc-compact+xml" //$NON-NLS-1$
						},
					
						load: dojo.hitch(this, function (result) {
							linkAggregator_.resendOauth(host);
						}),
					
						error: dojo.hitch(this, function(error, ioArgs) {
							// TODO: SHOW ERROR MESSAGE
							
							// this._messageBox.showErrorObject({
							// 	type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
							// 	summary: this.messages.artifactLoginFailed,
							// 	explanation: this.messages.artifactLoginFailed$explanation,
							// 	userAction: dojo.replace(this.messages.artifactLoginFailed$useraction, [this.artifactUri])
							// });
						})
					});
			},

			/**
			 * Creates a hover popup around the node's icon to show a log in link.
			 * @param  {string} host host to log in to, the result.token from the callback.
			 * @param  {object} node node to decorate.
			 */
			_createLoginHoverPopup: function(host, node) {
				var artifactUri = node.getData().item.URI;

				var hover = dm.analysis.app.ui.internal.impactanalysis.DiagramHoverPopup({
					around: node.elementImg.rawNode,
					openDelay: 400,
					orientation: 'V' //$NON-NLS-1$
				});


				this._hoverPopupsByUri[artifactUri] = hover;

				var self = this;
				hover.createContents = function() {
					var contents = dojo.create('span'); //$NON-NLS-1$
					var loginLink = dojo.create('a', {
						innerHTML: self.loginMessages.login,
						href: '#'
					}, contents);

					hover.connect(loginLink, 'onclick', dojo.hitch(self, self._doLogin, host, node)); //$NON-NLS-1$
					return contents;
				};

				this._loadPreviewsForNodes(getIncomingOutgoingNodes(node));	
			},

			/**
			 * Callback after a preview is loaded for a node.
			 * @param  {object} node   the node the preview is form.
			 * @param  {object} result compact preview results. OLSC or legacy.
			 */
			_richPreviewDidLoad: function(node, result) {
				var artifactUri = node.getData().item.URI;
				
				var oldHover = this._hoverPopupsByUri[artifactUri];
				if (oldHover) {
					// Now that we retried and got the icon this time, we don't need to hover any more.
					// Do this no matter how this method was called. We either have the data, or will
					// create a new popup: no need for the old one.
					oldHover.destroyRecursive();
					delete this._hoverPopupsByUri[artifactUri];
				}				 

				if (result.oauthRequired) {

					// Shows an "unknown" icon to indicate that the icon is missing and requires attention.
					node.elementImg.setShape({
						src: dojo.moduleUrl('dm.analysis.app','graphics/icons/unloaded_warn_icon.gif') //$NON-NLS-1$ //$NON-NLS-2$
					});

					this._createLoginHoverPopup(result.token, node);
					return;
				}

				if (result.oslcCompact) {
					// var title = result.oslcCompact.title;
					var icon = result.oslcCompact.icon;
				}
				else if (result.compactRendering) {
					// title = result.compactRendering.title;
					icon = result.compactRendering.icon;
				}
				else {
					console.debug('Unknown preview format:', result); //$NON-NLS-1$
				}			

				if (icon) {
					node.elementImg.setShape({
						src: icon
					});
				}

				this._loadPreviewsForNodes(getIncomingOutgoingNodes(node));
			},

			_richPreviewFailed: function(node) {
				// Ignore. Probably a node that doesn't support OSLC previews.
				var artifactUri = node.getData().item.URI;

				//Don't show any icon when there isn't one for this resource, or there isn't access to this resource
				node.elementImg.setShape({ src: "" });

				var oldHover = this._hoverPopupsByUri[artifactUri];
				if (oldHover) {
					// Now that we retried and got the icon this time, we don't need to hover any more.
					oldHover.destroyRecursive();
					delete this._hoverPopupsByUri[artifactUri];
				}
			},

			/**
			 * Clean up code like a widget (though this isn't).
			 * @return {[type]} [description]
			 */
			destroyRecursive: function() {
				this._destroyAllHoverPopups();
			},

			destroy: function() {
				this.destroyRecursive();
			}
		}
	);

})();
