/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.ProgressDialog");

dojo.require("dijit._Widget");
dojo.require('dijit._Templated');
dojo.require('dijit.ProgressBar');
dojo.require('jazz.ui.Dialog');

dojo.require("dojo.i18n");
dojo.requireLocalization('dm.analysis.app.ui.internal.impactanalysis', 'ImpactAnalysisMessages');

/* Borrowed from RELM: 'com.ibm.relm.ple.web.ui.ProgressDialog' (3-13-2013) */

(function(){
	var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages");

	/**
	 * A simple status to show in dialog with a spinner image and a text message.
	 * By default, this will mirror any message sent to StatusMessage.show.
	 * This is an easy way to add feedback to a dialog without needing
	 * to publish multiple messages. 
	 *
	 * It will show itself only when a messages is being displayed and will
	 * be hidden along with the StatusMessage messsage.
	 */
	dojo.declare(
	'dm.analysis.app.ui.internal.impactanalysis.DialogStatusMessage', //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated ],
	{
		templateString: '<div class="ia-DialogStatusMessage" style="display: none;"> <img src="${spinnerSrc}"> <span dojoAttachPoint="_messageNode" class="j-hint"></span> </div>', //$NON-NLS-1$
		spinnerSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/spinner.gif'),

		/**
		 * If true, will automatically hide and show to match the messages of StatusMessage.
		 * This also implicitly hides the page level (green bar) status message. So that messages
		 * aren't being shown in two places.
		 * @type {Boolean} true to mirror status messages. False to control yourself.
		 */
		mirrorStatusMessage: true,

		postCreate: function() {
			this.inherited(arguments);

			if (this.mirrorStatusMessage) {
				this.subscribe('ia.status_message.show', this.show); //$NON-NLS-1$
				this.subscribe('ia.status_message.hide', this.hide); //$NON-NLS-1$

				this._pageStatus = dojo.query('#content-cell .status-message'); //$NON-NLS-1$
				this._pageStatus.addClass('ia-DialogStatusMessage-hiddenPageStatus'); //$NON-NLS-1$
			}
		},

		/**
		 * Show this widget (with spinner graphic) and show the given message.
		 * The last message, is any, is lost.
		 * @param  {string} message the message to show.
		 */
		show: function(message) {
			dojo.empty(this._messageNode);
			// Adding ... is not right, but it's what PageTemplate.js already does
			this._messageNode.appendChild(document.createTextNode(message + '...')); //$NON-NLS-1$
			
			this.domNode.style.display = ''; //$NON-NLS-1$
		},

		/**
		 * Hide this entire widget.
		 */
		hide: function() {
			this.domNode.style.display = 'none'; //$NON-NLS-1$
		},

		uninitialize: function() {
			if (this._pageStatus) {
				this._pageStatus.removeClass('ia-DialogStatusMessage-hiddenPageStatus'); //$NON-NLS-1$
			}
		}
	});

	dojo.declare(
	'dm.analysis.app.ui.internal.impactanalysis.ProgressDialog',
	[ dijit._Widget, dijit._Templated ],
	{
		templatePath: dojo.moduleUrl('dm.analysis.app', 'ui/internal/impactanalysis/templates/ProgressDialog.html'),
		widgetsInTemplate: true,
		messages: messages,

		primaryTitle: '',
		explanation: '',
		width: '350px',
		canCancel: true,
		mirrorStatusMessage: false,
		iconClass: null,
		
		postCreate: function() {
			this.inherited(arguments);

			if (!this.canCancel) {
				this._cancelButton.style.visibility = 'hidden';
			}

			if (this.explanation) {
				this.dialogStatusMessage.show(this.explanation);
			}

			this._dialog = new jazz.ui.Dialog({
				primaryTitle: this.primaryTitle || messages.progressDialog_defaultTitle,
				width: this.width,
				closable: false,
				destroyContents: true,
				contentNode: this.domNode,
				iconClass: this.iconClass
			});
		},

		onCancel: function() {
		},

		cancel: function() {
			this.onCancel();
			this._dialog.close();
		},

		close: function() {
			this._dialog.close();
		},

		uninitialize: function() {
			this.inherited(arguments);
		}
	});

	var ProgressDialog = dm.analysis.app.ui.internal.impactanalysis.ProgressDialog;

	/**
	 * Runs a normal xhrPost method with standard callbacks, but if the wait time is too long
	 * for the callback, puts up a indeterminate progress dialog with the option to cancel the operation.
	 * After a cancel, normal callbacks will not be called, but an optional onCancel will (with no args).
	 * 
	 * @param  {object} args Normal dojo xhrArgs + args.primaryTitle for title of the progress
	 * dialog (defaults to 'Progress'), and args.explanation for the line describing what's happening.
	 * @return {Deferred}      from the XHR request.
	 */
	ProgressDialog.slowXhrPost = function(args) {

		var original = {
			handle: args.handle,
			load: args.load,
			error: args.error
		};

		delete args.handle;
		delete args.load;
		delete args.error;

		var dialog = null;
		var isCancelled = null;
		var onCancelHandle = null;
		var xhrDeferred = null;

		var showTimeout = setTimeout(function() {
			dialog = new dm.analysis.app.ui.internal.impactanalysis.ProgressDialog({
				primaryTitle: args.primaryTitle || messages.progressDialog_defaultTitle,
				explanation: args.explanation || '' //$NON-NLS-1$
			});

			onCancelHandle = dojo.connect(dialog, 'onCancel', function() { //$NON-NLS-1$
				isCancelled = true;
				xhrDeferred.cancelled = true;
				xhrDeferred.cancel();
				args.onCancel && args.onCancel();
			});

			dialog.startup();

		}, args.showDialogInMillis || 400);

		args.load = function() {
			if (isCancelled) return;
			if (original.load) original.load.apply(null, arguments);
		};

		args.error = function() {
			if (isCancelled) return;
			if (original.error) original.error.apply(null, arguments);
		};

		args.handle = function() {
			clearTimeout(showTimeout);
			if (onCancelHandle) dojo.disconnect(onCancelHandle);
			dialog && dialog.close();
			if (original.handle) original.handle.apply(null, arguments);
		};

		xhrDeferred = jazz.client.xhrPost(args);

		// XXX: not intercepting the callbacks from this if the caller uses it.
		return xhrDeferred;
	};

})();