/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.SaveProfileDialog");

dojo.require("rmps.ui.SimpleButton");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.StyledBox");

dojo.require("jazz.app.context");

dojo.require("dojox.widget.Standby");

dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysisProfile");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.SaveProfileDialog", null, {
	parent: null,
	iaProfile: null,
	newProfileInfo: null,
	_callback: null,
	projectId: null,

	standby: null,
	dialog: null,
	dialogContent: null,
		
	constructor: function(parent, iaProfile, newProfileInfo, _callback) {
		this.parent = parent;
		this.iaProfile = iaProfile;
		this.newProfileInfo = newProfileInfo;
		this._callback = _callback;

		this.projectId = jazz.app.context.get().itemId;
	},

	closeDialog: function(){
		this.dialog && this.dialog.close();
	},

	createDialog: function(){
		this.dialogContent = new dm.analysis.app.ui.internal.impactanalysis.SaveProfileDialog_content({ parent: this.parent, dialog: this });

		this.dialog = new jazz.ui.Dialog({
			primaryTitle: messages.saveProfileDialogTitle,
			modal: true,
			trim: jazz.ui.StyledBox.TrimStyles.BLUE_ALPHA_SHADOW,
			center: true,
			closeOnEsc: true,
			headerless: false,
			closable: true,
			width: 500 + "px",
			moveable: true,
			collapsible: false,
			showBackground: false,
			contentNode: this.dialogContent.domNode,
			destroyContents: true,
			onClose: dojo.hitch(this, function()
			{
				this.dialogContent.destroyRecursive();
			})
		});
	}
});


dojo.declare("dm.analysis.app.ui.internal.impactanalysis.SaveProfileDialog_content", 
	[dijit._Widget, dijit._Templated], {

	parent: null,
	dialog: null,

	saveButton: null,
	cancelButton: null,

	_saveButtonClickHandle: null,

	messages: messages,
	widgetsInTemplate: true,

	templateString: '<div class="save-profile-dialog">'+
		'<div dojoAttachPoint="_messageBoxContainer" style="display:block"></div>'+
		'<span>${messages.nameLabel}</span>'+
		'<input dojoType="dijit.form.TextBox" dojoAttachPoint="nameTextBox"'+
			'dojoAttachEvent="onChange: _onNameChange, onKeyPress: _onNameChange, onPaste: _onNameChange, onInput: _onNameChange"></input>'+
		'<span style="display:block">${messages.descriptionLabel}</span>'+
		'<textarea class="description"'+
			'style="width:95%"'+
			'dojoAttachPoint="descriptionTextArea"'+
			'dojoType="dijit.form.SimpleTextarea"'+
			'intermediateChanges="true" cols="63"'+ 
			'title="${messages.descriptionHover}">'+
		'</textarea>'+
		'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="toolbar" style="display:block; float:right"></div>'+
	'</div>',

	postCreate: function()
	{
		this.standby = new dojox.widget.Standby({target: this.domNode});
		this.domNode.appendChild(this.standby.domNode);
		
		this.saveButton = this.toolbar.addWidget(new rmps.ui.SimpleButton({
			label: messages.dialogButtonSaveProfile,
			onClick: dojo.hitch(this, this._saveProfile)
		}));

		this.cancelButton = this.toolbar.addWidget(new rmps.ui.SimpleButton({
			label: messages.dialogButtonCancel,
			onClick: dojo.hitch(this, this._cancel)
		}));

		this.nameTextBox.set('value', this.dialog.iaProfile.shortName);
		this.descriptionTextArea.set('value', this.dialog.iaProfile.description);

		this.standby.startup();
	},

	_cancel: function(){
		this.dialog.closeDialog();
	},

	_onNameChange: function(){
		this.saveButton.setEnabled(true);
	},

	showMessage: function(args){
		if (args && this._messageBoxContainer) {
			if (!this.messageBox) {
				this.messageBox = new jazz.ui.MessageBox();
				this._messageBoxContainer.appendChild(this.messageBox.domNode);
			}
			this.messageBox.showMessage(args);
			this._messageBoxContainer.style.display = "";
		}
	},

	_saveProfile: function(){
		this.inprogress(true);

		var existingProfile = this.dialog.iaProfile;
		var newName = this.nameTextBox.get('value');
		var newDesc = this.descriptionTextArea.get('value');
		var newProfileInfo = this.dialog.newProfileInfo;

		//when the entered name is different than the existing name, we should create a new profile instead
		//of updating the existing one
		var createNew = newName !== existingProfile.shortName;

		var deferred;

		if (createNew || !this.dialog.iaProfile.eTag){
			var newProfile = new dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysisProfile();

			newProfile.shortName  = newName;
			newProfile.description  = newDesc;
			newProfile.traversal = newProfileInfo.traversal;
			newProfile.upstreamDepth = newProfileInfo.upstreamDepth;
			newProfile.downstreamDepth = newProfileInfo.downstreamDepth;
			newProfile.fullTraceDepth = newProfileInfo.fullTraceDepth;
			newProfile.upstreamType = newProfileInfo.upstreamType;
			newProfile.downstreamType = newProfileInfo.downstreamType;
			newProfile.fullTraceType = newProfileInfo.fullTraceType;
			newProfile.filters = dojo.clone(newProfileInfo.filters);

			deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.createIAProfile(newProfile);
		}
		else {
			existingProfile.description  = newDesc;
			existingProfile.traversal = newProfileInfo.traversal;
			existingProfile.upstreamDepth = newProfileInfo.upstreamDepth;
			existingProfile.downstreamDepth = newProfileInfo.downstreamDepth;
			existingProfile.fullTraceDepth = newProfileInfo.fullTraceDepth;
			existingProfile.upstreamType = newProfileInfo.upstreamType;
			existingProfile.downstreamType = newProfileInfo.downstreamType;
			existingProfile.fullTraceType = newProfileInfo.fullTraceType;
			existingProfile.filters = dojo.clone(newProfileInfo.filters);

			deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.updateIAProfile(existingProfile);
		}

		deferred.addCallback(this, dojo.hitch(this, function() {
			if (createNew)
				this.dialog._callback(createNew, newProfile.shortName, newProfile.uri);
			else
				this.dialog._callback(createNew, existingProfile.shortName, existingProfile.uri);

			this.inprogress(false);
			
			dojo.publish("dm.analysis.app.ui.internal.impactanalysis.profile.refresh", [this.ownerDocument.URL]);
			
			this.dialog.closeDialog();
		}));

		deferred.addErrback(this, dojo.hitch(this, function(error) {
			this.inprogress(false);
			
			if (deferred.ioArgs && deferred.ioArgs.xhr && !dojo._isDocumentOk(deferred.ioArgs.xhr)) {
				error = rmps.core.error.createXhrError(deferred.ioArgs);
			}

			dojo.hitch(this, this.parent._handleSaveErrors)([error]);
			this.saveButton.setEnabled(false);

		}));

	},

	inprogress: function(inProgress){
		if (inProgress === true) {
			this.saveButton.setEnabled(false);
			this.standby.show();
		} else {
			this.saveButton.setEnabled(true);
			this.standby.hide();
		}
	},

	destroy: function(){
		this._saveButtonClickHandle && dojo.disconnect(this._saveButtonClickHandle);

		this.standby.destroyRecursive();
	}





});

}());