/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.application.MenuInitializer"); //$NON-NLS-1$

dojo.require("dm.app.analysis.ui.internal.application.AnalysisActions"); //$NON-NLS-1$
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil"); //$NON-NLS-1$
dojo.require("dm.app.analysis.ui.internal.application.AnalysisMenu"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.pages", "DMAnalysisMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis.pages", "DMAnalysisMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var AnalysisActions = dm.app.analysis.ui.internal.application.AnalysisActions;
var AnalysisUtil = dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil;

/** 
 *  Initializes the Impact Analysis component menu by adding the actions for Browsing and Creating:
 *  Diagrams, Filters (aka Configs), and Profiles
 */

/** Adding the 'Browse' menu items */
dojox.lang.aspect.advise(dm.app.analysis.ui.internal.application.AnalysisMenu, "addAnalysisBrowseMenuItems", { //$NON-NLS-1$
	after: function() {
		var menu = dojox.lang.aspect.getContext().instance;

		// If this call is for IA menu items to be added
		if (menu.getCurrentSection() === AnalysisActions.SECTION_IMPACTANALYSIS) {
			
			menu.addItemByArgs({
				label: messages.allAnalysis,
				href: AnalysisUtil.getAllAnalysisUri(),
				icon: dojo.moduleUrl("dm.analysis.app", "graphics/icons/impact.gif") //$NON-NLS-1$ //$NON-NLS-2$
			});
			menu.addItemByArgs({
				label: messages.allAnalysisConfig,
				href: AnalysisUtil.getAllConfigAnalysisUri(),
				icon: dojo.moduleUrl("dm.analysis.app", "graphics/icons/ia_filter.gif") //$NON-NLS-1$ //$NON-NLS-2$
			});
			menu.addItemByArgs({
				label: messages.allAnalysisProfile,
				href: AnalysisUtil.getAllProfileAnalysisUri(),
				icon: dojo.moduleUrl("dm.analysis.app", "graphics/icons/ia_profile.gif") //$NON-NLS-1$ //$NON-NLS-2$
			});
		}
	}
});

/** Adding the 'Create' menu items */
dojox.lang.aspect.advise(dm.app.analysis.ui.internal.application.AnalysisMenu, "addAnalysisCreateMenuItems", { //$NON-NLS-1$
	after: function() {
		var menu = dojox.lang.aspect.getContext().instance;
		
		// If this call is for IA menu items to be added
		if (menu.getCurrentSection() === AnalysisActions.SECTION_IMPACTANALYSIS) {

			var createDiagramArgs = {
				label: messages.createAnalysis,
				href: AnalysisUtil.getNewAnalysisUri(),
				icon: dojo.moduleUrl("dm.analysis.app", "graphics/icons/impact.gif") //$NON-NLS-1$ //$NON-NLS-2$
			};
			var createFilterArgs = {
				label: messages.createAnalysisConfig,
				href: AnalysisUtil.getNewAnalysisConfigUri(),
				icon: dojo.moduleUrl("dm.analysis.app", "graphics/icons/ia_filter.gif") //$NON-NLS-1$ //$NON-NLS-2$
			};
			var createProfileArgs = {
				label: messages.createAnalysisProfile,
				href: AnalysisUtil.getNewAnalysisProfileUri(),
				icon: dojo.moduleUrl("dm.analysis.app", "graphics/icons/ia_profile.gif") //$NON-NLS-1$ //$NON-NLS-2$
			};

			var menuItems_create = [];
			
			menuItems_create.push({
				args: createDiagramArgs,
				updatedHref: dojo.hitch(AnalysisUtil, AnalysisUtil.getNewAnalysisUri),
				menuItem: menu.addItemByArgs(createDiagramArgs)
			});
			menuItems_create.push({
				args: createFilterArgs,
				updatedHref: dojo.hitch(AnalysisUtil, AnalysisUtil.getNewAnalysisConfigUri),
				menuItem: menu.addItemByArgs(createFilterArgs)
			});
			menuItems_create.push({
				args: createProfileArgs,
				updatedHref: dojo.hitch(AnalysisUtil, AnalysisUtil.getNewAnalysisProfileUri),
				menuItem: menu.addItemByArgs(createProfileArgs)
			});

			dojo.connect(menu, "onOpen", dojo.hitch(menu, this.updateMenuItems, menuItems_create));
		}
	},

	/**
	 * Each time the menu is opened, we must refresh the links in the menu to update their timestamp,
	 * this is a slightly hacky way of dealing with the fact that when you're on a new "Create" page, the
	 * url has no unique ID but just a timestamp, so clicking on the "Create" link in the menu (which has
	 * the same timestamp as the page you are on) would not navigate to a new Create page. Therefore we have to
	 * update the timestamps on the links in the menu.
	 *
	 * NOTE: This is the way RTC and other Jazz applications handle it, albeit they don't update upon every onOpen,
	 * but upon a loosely defined "amount of user activity", which is just page navigation and certain clicks.
	 * 
	 * menuItems: a list of objects each containing the 'menuItem' widget that was added to the menu, the 'args' used
	 *			in adding it to the menu, and the 'updatedHref' function which can generate a new href for the update
	 */
	updateMenuItems: function(menuItems) {
		for (var i = 0; i < menuItems.length; i++){
			var menuItemIndex = this._menu.getIndexOfChild(menuItems[i].menuItem);
			var menuItemArgs = menuItems[i].args;
			menuItemArgs.href = menuItems[i].updatedHref();

			this._menu.removeChild(menuItemIndex);
			menuItems[i].menuItem && menuItems[i].menuItem.destroyRecursive();

			//Create and add the replacement menu item, and make sure to modify the argument to our own function so that it's correct for the next update
			menuItems[i].menuItem = this.addItemByArgs(menuItemArgs, menuItemIndex);
		}
	}
});

})();