/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.config.AddTypeDialog");
dojo.require("jazz.ui.Dialog");

dojo.require("dm.analysis.app.ui.internal.impactanalysis.config.FilterCacheManager");
dojo.require("dm.common.app.ui.internal.rhlink.AnalysisConfigClient");
dojo.require("jazz.app.context");
dojo.require("jazz.ui.StyledBox");
dojo.require("dojox.widget.Standby");

dojo.require('dijit.layout.ContentPane');
dojo.require('dijit.form.FilteringSelect');
dojo.require('dojo.data.ItemFileReadStore');

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.config", "ImpactAnalysisConfigMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis.config", "ImpactAnalysisConfigMessages");

var TYPE_ELEMENT = "ELEMENT";

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.AddTypeDialog", null, {
	parent: null,
	alreadySelectedElements: [],
	alreadySelectedLinks: [],
	filterType: "",

	projectId: null,
	typeStore: null,

	domainCombo: null,
	standby: null,
	addSelectionButton: null,
	typeSelected: null,

	_emptyListData: {
		identifier: 'key',
		label: 'value',
		items: []
	},

	updateAfterAddCallback: null,
	dialog: null, //handle to the dialog
	
	constructor: function(parent, filterType, updateAfterAddCallback, alreadySelectedElements, alreadySelectedLinks, title) {
		this.parent = parent;
		this.filterType = dojo.isString(filterType) ? filterType : "";
		this.updateAfterAddCallback = updateAfterAddCallback;
		this.alreadySelectedElements = alreadySelectedElements || [];
		this.alreadySelectedLinks = alreadySelectedLinks || [];

		this.dialogTitle = messages.iaEditorSelectionElmDialogTitle + " (" + (this.filterType === TYPE_ELEMENT ? 
																				messages.addTypeDialogResourceLabel : 
																				messages.addTypeDialogPredicateLabel) + ")";
		if (dojo.isString(title)) this.dialogTitle = title;

		this.projectId = jazz.app.context.get().itemId;
		
		this.refreshHandle = dojo.subscribe("com.ibm.xtools.impactanalysis.FilterCacheManager.typesModified", dojo.hitch(this, function(projectID, toolType, filterType, results) 
		{
			if (this.typeStore && this.domainCombo &&
				this.projectId == projectID &&
				this.domainCombo.value == toolType &&
				this.filterType == filterType)
			{
				this.typeSelected.closeDropDown();
				if (!this.messageBox)
					this.messageBox = new jazz.ui.MessageBox();
				this.messageBox.placeAt(this.message);
				this.messageBox.showMessage({ 
				   type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
				   summary: messages.valueListUpdated
				});
				this._processTypeData(results);
			}
		}));
	},
		
	createDialog: function(){
		var createTable = dojo.doc.createElement("table");
		
		//TODO: put up a radio button if the user wants to change between Link and Element

		//Domain:
		var domainSelectId = dijit.getUniqueId('domainSelectId');
		this.domainCombo = dojo.doc.createElement("select");
		dojo.attr(this.domainCombo, 'id', domainSelectId);
		this.createRelationRow(createTable, messages.iaEditorSelectionElmDialogTool, this.domainCombo,null,domainSelectId);
		
		//A dojo data store for retrieved types
		this.typeStore = new dojo.data.ItemFileReadStore({clearOnClose: true});
		// provide some dummy data until we are ready to retrieve
		this.typeStore.data = {
			identifier: 'key',
			label: 'value',
			items: []
		};

		//Filtering Select:
		var messageId = dijit.getUniqueId('messageId'); //description id
		var typeSelectedId = dijit.getUniqueId("typeSelected");
		this.typeSelected = new dijit.form.FilteringSelect({
			id: typeSelectedId,
			'aria-describedby': messageId,
			store: this.typeStore,
			//name: 'key',
			searchAttr: 'value',
			style: 'width: 400px'
		});
		this.typeSelectedConnectHandle = dojo.connect(this.typeSelected,"onChange", dojo.hitch(this, this._monitorTypeSelected));
		this.createRelationRow(createTable, messages.iaEditorSelectionElmDialogType, this.typeSelected.domNode,null,typeSelectedId);

		this.standby = new dojox.widget.Standby({target: this.typeSelected.domNode});
		
		//Add button
		this.addSelectionButton = dojo.doc.createElement("button");
		this.addSelectionButton.innerHTML = messages.iaEditorSelectionElmDialogAdd;
		this.addConnectHandle = dojo.connect(this.addSelectionButton,"onclick",this, function() { this.add(); });
		this.message =  dojo.doc.createElement("span");
		dojo.attr(this.message,'id',messageId);
		this.createRelationRow(createTable, null, this.addSelectionButton, this.message);
		
		//Fill Domain
		this._getToolsTypesDeferred = dm.common.app.ui.internal.rhlink.AnalysisConfigClient.getToolsTypes(this.projectId);
		this._getToolsTypesDeferred.addCallback(this, function(tools) {
			var noTools = true;
			dojo.forEach(tools.tools, dojo.hitch(this, function (item) {
				var optionChild = dojo.doc.createElement("option");
				optionChild.innerHTML = item.ToolType;
				optionChild.value = item.ToolURI;
				this.domainCombo.appendChild(optionChild);
				noTools = false;
			}));

			//Fill Filter Value
			this.loadTypeSelected();
		});
		this._getToolsTypesDeferred.addErrback(this, function(error) {
			var xhr = this._getToolsTypesDeferred.ioArgs.xhr;
			if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
				var lqeInfo = dojo.fromJson(xhr.responseText);
				this.parent.editor.showMessage({
					type: lqeInfo.type,
					summary: lqeInfo.summary,
					details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
					closable: true
				});
			} else {
				this.parent.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.errorGetDomains,
					errorObject: error,
					closable: true
				});
			}
			this.dialog && this.dialog.close();
		});
		
		this.domainConnectHandle = dojo.connect(this.domainCombo,"onchange",this, function() { 
			this.loadTypeSelected(); 
		});
		
		//Create Dialog:

		var dialogContent = new dijit.layout.ContentPane({
			id: 'addTypeDialogContents'
		});
		
		dialogContent.domNode.appendChild(createTable);
		
		dialogContent.domNode.appendChild(this.standby.domNode);
		
		this.dialog = new jazz.ui.Dialog({
			primaryTitle: this.dialogTitle,
			modal: true,
			trim: jazz.ui.StyledBox.TrimStyles.BLUE_ALPHA_SHADOW,
			center: true,
			closeOnEsc: true,
			headerless: false,
			closable: true,
			width: 500 + "px",
			moveable: true,
			collapsible: false,
			showBackground: false,
			contentNode: dialogContent.domNode,
			destroyContents: true,
			onClose: dojo.hitch(this, function()
			{
				//must do clean up so this class can be garbage collected after the dialog is closed
				dojo.disconnect(this.typeSelectedConnectHandle);
				dojo.disconnect(this.addConnectHandle);
				dojo.disconnect(this.domainConnectHandle);
				dojo.unsubscribe(this.refreshHandle);
				
				this.standby.destroyRecursive();
			})
		});
		
		this.inprogress(true);
		this.standby.startup();
	},
	
	loadTypeSelected : function()
	{
		this.inprogress(true);
		this.message.innerHTML = "";
		this.typeSelected.disabled = false;
		
		this.typeSelected.reset();

		if (this.filterType === TYPE_ELEMENT) {
			this.fillSelectionWithElementTypes();
		} else {
			this.fillSelectionWithLinkTypes();
		}
	},
	
	fillSelectionWithLinkTypes: function()
	{
		this._getSelectionssDeferred = dm.analysis.app.ui.internal.impactanalysis.config.FilterCacheManager.getLinkTypesForTool(this.projectId , this.domainCombo.value);
		
		this._getSelectionssDeferred.addCallback(this, this._processTypeData);
		this._getSelectionssDeferred.addErrback(this, this.typeErrback);
	},
	
	fillSelectionWithElementTypes: function()
	{
		this._getSelectionssDeferred = dm.analysis.app.ui.internal.impactanalysis.config.FilterCacheManager.getElementTypesForTool(this.projectId , this.domainCombo.value);
		
		this._getSelectionssDeferred.addCallback(this, this._processTypeData);
		this._getSelectionssDeferred.addErrback(this, this.typeErrback);
	},
	
	typeErrback: function(error)
	{
		var xhr = this._getSelectionssDeferred.ioArgs.xhr;
		if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
			var lqeInfo = dojo.fromJson(xhr.responseText);
			this.parent.editor.showMessage({
				type: lqeInfo.type,
				summary: lqeInfo.summary,
				details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
				closable: true
			});
		} else {
			this.parent.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: (this.filterType === TYPE_ELEMENT) ? messages.errorGetElementTypes : messages.errorGetLinkTypes,
				errorObject: error,
				closable: true
			});
		}

		this.dialog.close();
	},
	
	_processTypeData: function (data) {
		var noData = data.types.length === 0, 
			alreadySelected = this.filterType === TYPE_ELEMENT ? this.alreadySelectedElements : this.alreadySelectedLinks;

		// filter for already selected...
		data.types = dojo.filter(data.types, function (type) {
			return dojo.every(alreadySelected, function (s) {
				return type.key !== s.uri;
			});
		});
		
		// reload the typeStore data store
		this.typeStore.data = {
			identifier: 'key',
			label: 'value',
			items: dojo.map(data.types, function (type) {
				return { 
					key: type.key,
					value: type.value + ' (' + type.key + ')'
				};
			}, this)
		};
		this.typeStore.close();

		if (noData) { //disable other option
			this.typeSelected.disabled = true;
			this.message.innerHTML = messages.iaEditorSelectionElmDialogNoElmForToolMsg;
		}
		
		this.inprogress(false);
	},
	
	_monitorTypeSelected: function() {
		if (this.typeSelected.getValue())
			this.addSelectionButton.disabled = false;
		else
			this.addSelectionButton.disabled = true;
	},
		
	inprogress: function(inProgress){
		if (inProgress === true) {
			this.addSelectionButton.disabled = true;
			this.standby.show();
		} else {
			this.standby.hide();
		}
	},
	
	createRelationRow: function(table, name, value, message, labelFor)
	{
		var tr = dojo.doc.createElement("tr");
		if (name != null)
		{
			var td1 = dojo.doc.createElement("td");
			tr.appendChild(td1);
			if(labelFor){
				dojo.create('label',{innerHTML: name, 'for':labelFor},td1);
			}
			else{
				td1.innerHTML = name;
			}
		}

		var td2 = dojo.doc.createElement("td");
		td2.appendChild(value);
		tr.appendChild(td2);
		
		if (message != null)
		{
			var td3 = dojo.doc.createElement("td");
			td3.appendChild(message);
			tr.appendChild(td3);
		}
		
		
		table.appendChild(tr);
	},
	
	
	add: function() {
		if (!this.updateAfterAddCallback){
			console.debug("The Dialog doesn't know what to do with the selected filter, no callback specified.");
			return;
		}
		
		dojo.hitch(this.parent, this.updateAfterAddCallback)(
			this.domainCombo.options[this.domainCombo.selectedIndex].text, 
			this.filterType,
			{ 
				domain: this.domainCombo.value,
				uri: this.typeSelected.item.key[0]
			}
		);

		this.dialog.close();
    }
	
});

}());