/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.config.FilterCacheManager");

dojo.require("dm.common.app.ui.internal.rhlink.AnalysisConfigClient");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.config", "ImpactAnalysisConfigMessages"); 

/**
 * This javascript class is used for caching the filter types, and does a publish to all listeners when the data in the cache is updated. 
 * The cache asynchronously get's refreshed in the background everytime the cache is accessed.
 */
dm.analysis.app.ui.internal.impactanalysis.config.FilterCacheManager = { 
	
	cachedData: {},
	
	/**
	 * Returns the link types for the given project id and tool type.
	 */
	getLinkTypesForTool:function(projectID, tooltype)
	{
		return this._getTypesForTool(projectID, tooltype, true);
	},
	
	/**
	 * Returns the element types for the given project id and tool type.
	 */
	getElementTypesForTool:function(projectID, tooltype)
	{
		return this._getTypesForTool(projectID, tooltype, false);
	},
	
	_getFilterTypesKey: function(forLinks)
	{
		if (forLinks)
		{
			return "LINK";
		}
		else
		{
			return "ELEMENT";
		}
	},
	
	_getCachedData:function(projectID, toolType, forLinks)
	{
		var filterTypes = this._getFilterTypesKey(forLinks);
		 
		if (!this.cachedData[projectID] || !this.cachedData[projectID][toolType] || !this.cachedData[projectID][toolType][filterTypes])
			return null;
		
		return this.cachedData[projectID][toolType][filterTypes];	
	},
	
	_setCachedData:function(projectID, toolType, forLinks, filterTypesToSet)
	{
		var filterTypes = this._getFilterTypesKey(forLinks);
		
		if (!this.cachedData[projectID])
			this.cachedData[projectID] = {};
		if (!this.cachedData[projectID][toolType])
			this.cachedData[projectID][toolType] = {};
		this.cachedData[projectID][toolType][filterTypes] = filterTypesToSet;	
	},
	
	_hasCachedData:function(projectID, toolType, forLinks)
	{
		return this._getCachedData(projectID, toolType, forLinks) != null;
	},
	
	_getTypesForTool: function(projectID, toolType , forLinks) {
		var d = new dojo.Deferred();
		var self = this;
		if (this._hasCachedData(projectID, toolType , forLinks)) 
		{
			this._refresh(projectID, toolType , forLinks);
			d.callback(this._getCachedData(projectID, toolType , forLinks));
		}
		else {
			var getFilterTypesDeferred = dm.common.app.ui.internal.rhlink.AnalysisConfigClient.getTypesForTool(projectID, toolType, forLinks);
			d.ioArgs = getFilterTypesDeferred.ioArgs;
			getFilterTypesDeferred.then(function(results){
				self._setCachedData(projectID, toolType, forLinks, results);
    			d.callback(results);
  			}, 
  			d.errback);
		}

		return d;
	},
	
	_refresh: function(projectID, toolType , forLinks) 
	{
		var self = this;
		//refresh the data and publish event if the data was changed, log and publish event if refresh failed
		var getFilterTypesDeferred = dm.common.app.ui.internal.rhlink.AnalysisConfigClient.getTypesForTool(projectID, toolType, forLinks);
		getFilterTypesDeferred.then(function(results){
			self._setCachedData(projectID, toolType, forLinks, results);
			if (self._resultsModified(projectID, toolType, forLinks, results))
			{
				var filterType = self._getFilterTypesKey(forLinks);
				dojo.publish("com.ibm.xtools.impactanalysis.FilterCacheManager.typesModified", [projectID, toolType, filterType, results]);
			}
		}, 
		function(error){
			console.log("Failed to refresh the cache:"+error);
			dojo.publish("com.ibm.xtools.impactanalysis.FilterCacheManager.refreshFailed", [error]);
		});
	},
	
	_resultsModified:function(projectID, toolType, forLinks, results)
	{
		var cachedTypes = this._getCachedData(projectID, toolType, forLinks);
		if ((!cachedTypes) || 
			(cachedTypes.types.length != results.types.length))
			return true;
			
		for (var i=0; i < results.types.length; i++) {
			var type = results.types[i];
			var matchFound = false;
			for (var j=0; j < cachedTypes.types.length; j++) {
				var cachedType = cachedTypes.types[j];
				if (type == cachedType)
				{
					matchFound = true;
					break;
				}
			}
			if (!matchFound)
				return true;
		}
		return false;
	}
};

})();