/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor");

dojo.require("dojo.string");
dojo.require("dojo.i18n");

dojo.require("dijit._Widget");
dojo.require("dijit.form.SimpleTextarea");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.MultiSelect");

dojo.require("jazz.ui.Breadcrumbs");
dojo.require('jazz.app.ua.TextHoverHelp');

dojo.require("rmps.core.error");
dojo.require("rmps.ui.CollapsibleSection");
dojo.require("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.UnlimitedNumberSpinner");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.config.CreateSelectionToolDialog");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.config.AddTypeDialog");

dojo.require("dm.common.app.ui.internal.rhlink.AnalysisConfigClient");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.EditorUtil");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.config", "ImpactAnalysisConfigMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis.config", "ImpactAnalysisConfigMessages"); 

var AnalysisUtil = dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil;

var PAGE_ID = dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor.PAGE_ID;
var DIRTY_GROUP_ID = "dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor.dirty";

var UA_KEY = "com.ibm.xtools.impactanalysis.web.app.page_IACreateFilters001";
var UA_KEY_DOWNSTREAMLABEL = "com.ibm.xtools.impactanalysis.web.app.text_IAUpstream001";
var UA_KEY_UPSTREAMLABEL =  "com.ibm.xtools.impactanalysis.web.app.text_IADownstream001";

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	isRefreshable: false,
	isReadOnly: false,	

	editorInput:null,
	content:null,
	//data
	_general:null,
	_tools:null,
	_selections:null,
	_advance:null,
	_breadCrumbs:null,
	uaKey: UA_KEY,
	dirtyTitle:null,
	_allTools:null,
	
	constructor: function (args) {
		this.editorInput= args.editorInput;
		
		if (!this.editorInput.getName())
			this.editorInput.setName(messages.newImpactAnalysisConfigTitle);
	},

	postCreate: function() {
		this._allTools = {};
		this.dirtyTitle = {};
		
		this._breadCrumbs= new jazz.ui.Breadcrumbs({segments: [{label: messages.iaConfigTitle, href: AnalysisUtil.getAllConfigAnalysisUri()}]});
		dojo.place(this._breadCrumbs.domNode, this.headerContainerNode, "first");
		
		this.inherited(arguments);
		

	},

	activate: function() {
		// until editing is supported, mark as dirty
		//this.addDirty(this);
	},
	
	deactivate: function()
	{
	},
	
	startup: function() {
		this.content.startup();
	},
	
	isEditableTitle: function(){
		return !this.isReadOnly;
	},
	
	onTitleChange: function () {
		var oldValue = this.editorInput.iaConfig.shortName;
		this.editorInput.iaConfig.shortName = this.getTitleValue();
		this._updateDirty(oldValue, 
					this.getTitleValue(), 
					this.dirtyTitle);
	},
	
	createContentWidget: function() {		
		this.content = new dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor._Content({editor: this, isReadOnly: this.isReadOnly});
		return this.content;
	},
	
	addButtons: function(toolbar) {	
		toolbar.addWidget(new rmps.ui.SimpleButton({
			label: messages.cancel,
			onClick: dojo.hitch(this, this.doCancel)
		}));
		
		this.inherited(arguments);
	},
	
	enable: function()
	{
		//enable the save button if its supposed to be
		var enableSaveButton = false;
		if (!this.isReadOnly && this.isDirty())
			enableSaveButton = true;
		this.saveButton.setEnabled(enableSaveButton);
	},
	disable: function()
	{
		//this.saveButton.setEnabled(false); save button already disabled by default in BaseEditor.save()
	},
	
	getEditableTitleNodeDetails: function () {
		// summary:
		//		Callback from BaseEditor providing default title for a new impact analysis
		return messages.newImpactAnalysisConfigTitle;
	},
	
	validate: function (/*Array*/errors) {
		// summary:
		//		Callback from BaseEditor to validate data prior to save

		var detailedError;

		if(this.editorInput.iaConfig.shortName === '') {
			detailedError = new Error(messages.errorMissingTitle);
			detailedError.explanation = messages['errorMissingTitle.explanation'];
			detailedError.userAction = messages['errorMissingTitle.useraction'];
			errors.push(detailedError);
			this.saveButton.setEnabled(this.isDirty());
			
			dm.analysis.app.ui.internal.impactanalysis.util.EditorUtil.animateMissingEditorTitle(this._editableTitle.domNode);
			this._editableTitle.edit();
		}
	},

	/** When Cancel is clicked in the analysis configuration editor, go
	*  back to the page for viewing all configurations.
	*/
	doCancel: function() {
		var pageHelper = rmps.ui.editor.PageHelper.getInstance(PAGE_ID);
		pageHelper.closeEditor(this, true, true);
		window.location.href = AnalysisUtil.getAllConfigAnalysisUri() + "&ts="+ new Date().getTime();
	},
	
	doSave: function(taskMonitor) {
		this.disable();
		var iac = this.editorInput.iaConfig;
		//general
		iac.shortName  = this.getTitleValue();
		iac.description  = this._general.descriptionTextArea.get('value');
		iac.upStreamLevel  = this._general.upStreamLevelInput.getValue();
		iac.downStreamLevel  = this._general.downStreamLevelInput.getValue();
		//filters
		iac.enableDeselecting  = this._filters.enableDeselectingCheck.get('value') === "on";
		//advance
		iac.selectionInterLinks = [];
		dojo.forEach(this._advance.interLinks.options, function (option) {
			if (option.selected) {
				iac.selectionInterLinks.push(option.value);
			}
		}, this);
		
		var deferred, isNew = false;
		if (iac.eTag) {
			deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.updateIAConfig(iac);
		} else {
			deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.createIAConfig(iac);
			isNew = true;
		}
		
		deferred.addCallback(this, function() {
			//when done , enable all button
			if (isNew) {
				// refresh the editor it with the now 'saved' editor input.
				this.editorInput.resourceUri = iac.uri;
				this.editorInput.removeState('id');
				
				// clear dirty flag first before reload
				this._clearDirty();
				this.refresh();
			} else {
				this.enable();
			}
			dojo.publish("dm.analysis.app.ui.internal.impactanalysis.config.refresh");
			
		});	
		
		deferred.addErrback(this, function(error) {
			this.enable();
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.errorSaving,
				errorObject: error,
				closable: true
			});
		});
		
		var task = taskMonitor.beginTask("saving", 1);
		task.handleDeferredWork(deferred, 1);
	},
	
	getResourceInfo: function(){
		return this.editorInput.resourceInfo;	
	},
	
	getProjectID: function(){
		return this.editorInput.projectId;	
	},
	
	_updateDirty: function(originalValue, newValue, dirtyKey) {
		var isEqual = originalValue === newValue;
		if (!isEqual && dojo.isArray(newValue) && originalValue && newValue && originalValue.length === newValue.length) {
			isEqual = true; // empty arrays
			isEqual = dojo.every(newValue, function(item, idx) {
				return item === originalValue[idx];
			});
		}
		if (isEqual) {
			this.removeDirty(dirtyKey, DIRTY_GROUP_ID);
		} else {
			this.addDirty(dirtyKey, DIRTY_GROUP_ID);
		}
	},

	uninitialize: function(){
		this.inherited(arguments);
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor._Content", [dijit._Widget, dijit._Templated], {
	messages: messages, 
	widgetsInTemplate: true,
	templateString: '<div class="config-editor">'+
		'<div dojoAttachPoint="_generalSection"></div>'+
		'<div dojoAttachPoint="_rulesSection"></div>'+
		'<div dojoAttachPoint="_advancedSection"></div>'+
	'</div>',
	
	// init properties
	editor: null,
	isReadOnly: false,
	
	postCreate: function() {
		//get all tools map to be uses to present the tools name:
		this._getToolsTypesDeferred = dm.common.app.ui.internal.rhlink.AnalysisConfigClient.getToolsTypes(this.editor.getProjectID());
		this._getToolsTypesDeferred.addCallback(this, function(result) {
			dojo.forEach(result.tools, function (tool) {
				this.editor._allTools[tool.ToolURI] = tool.ToolType;
			}, this);

			var sectionArgs = { editor: this.editor, isReadOnly: this.isReadOnly };

			this.editor._general = new dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_General(sectionArgs, this._generalSection);
			this.editor._filters = new dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_Filters(sectionArgs, this._rulesSection);
			this.editor._advance = new dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_Advance(sectionArgs, this._advancedSection);
		});
		this._getToolsTypesDeferred.addErrback(this, function(error) {
			var xhr = this._getToolsTypesDeferred.ioArgs.xhr;
			if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
				var lqeInfo = dojo.fromJson(xhr.responseText);
				this.editor.showMessage({
					type: lqeInfo.type,
					summary: lqeInfo.summary,
					details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
					closable: true
				});
			} else {
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.errorGetDomains,
					errorObject: error,
					closable: true
				});
			}

		});
		
	},

	destroy: function(){
		this.editor._general.destroyRecursive();
		this.editor._filters.destroyRecursive();
		this.editor._advance.destroyRecursive();

		this.inherited(arguments);
	},

	startup: function(){
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_General", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	messages: messages, 
	templateString: '<div>'+
		'<div dojoType="rmps.ui.CollapsibleSection" primaryTitle="${messages.sectionTitle_general}" readOnly="false" collapsible="true" isCollapsed="false">'+
			'<div class="IAConfigEditor">' +
				'<table style="overflow: scroll; width:100%; margin: 5px 0;" class="impact-def-table general" >' +
					'<tr> ' +
						'<td style="padding-right:10px; width:15%" dojoAttachPoint="_generalTitleCell"></td>' +
					'</tr>' +
					'<tr> ' +
						'<td style="padding-right:10px; width:15%" >${messages.iaEditorDesc}</td>' +
						'<td><textarea  class="instructions" dojoAttachPoint="descriptionTextArea"'+
								'dojoType="dijit.form.SimpleTextarea"'+
								'intermediateChanges="true"'+
								'cols="63"' + 
								'title="${messages.iaEditorDescHover}" >' +
							'</textarea></td>'+
					'</tr>' +
					'<tr style="display:none">' +
						'<td style="padding-right:10px; width:15%" ><span dojoAttachPoint="_upstreamSpan">${messages.iaEditorUpStream}</span></td>' +
						'<td style="padding-right:10px; padding-left: 2px" ><input '+
							'dojoType="dm.analysis.app.ui.internal.impactanalysis.UnlimitedNumberSpinner"'+
							'dojoAttachPoint="upStreamLevelInput"' + 
							'title="${messages.iaEditorUpStreamHover}"/>' +
							'</td>' +
					'</tr>' +
					'<tr style="display:none">' +
						'<td style="padding-right:10px; width:15%" ><span dojoAttachPoint="_downstreamSpan">${messages.iaEditorDownStream}</span></td>' +
						'<td style="padding-right:10px; padding-left: 2px" ><input '+
							'dojoType="dm.analysis.app.ui.internal.impactanalysis.UnlimitedNumberSpinner"'+
							'dojoAttachPoint="downStreamLevelInput"' + 
							'title="${messages.iaEditorDownStreamHover}"/>' +
							'</td>' +
					'</tr>' +
				'</table>' +
			'</div>' +
		'</div>' +
	'</div>',
	
	// init properties
	editor: null,
	isReadOnly: false,
	iaConfig:null,

	//connect handles
	_descChangeHandle: null,
	_upStreamChangeHandle: null,
	_downStreamChangeHandle: null,
	
	postCreate: function() {
		this.iaConfig = this.editor.editorInput.iaConfig;
		
		this.inherited(arguments);
		
		if (this.iaConfig.systemDefined) {
			this._generalTitleCell.appendChild(dojo.doc.createTextNode(messages.iaEditorGeneralTitleSystemDefined));
		}
		
		this.descriptionTextArea.set('value',this.iaConfig.description);
		this.descriptionTextArea.set('disabled', this.isReadOnly);
		
		this.upStreamLevelInput.setValue(this.iaConfig.upStreamLevel);
		this.upStreamLevelInput.setDisabled(this.isReadOnly);
		
		this.downStreamLevelInput.setValue(this.iaConfig.downStreamLevel);
		this.downStreamLevelInput.setDisabled(this.isReadOnly);
		
		this._upstreamTextHoverHelp = new jazz.app.ua.TextHoverHelp({
			uaKey: UA_KEY_UPSTREAMLABEL,
			contentNode: this._upstreamSpan
		});

		this._downstreamTextHoverHelp = new jazz.app.ua.TextHoverHelp({
			uaKey: UA_KEY_DOWNSTREAMLABEL,
			contentNode: this._downstreamSpan
		});

		if (!this.isReadOnly) {
			this._descChangeHandle = dojo.connect(this.descriptionTextArea,"onChange", dojo.hitch(this, function() {
				this.editor._updateDirty(this.iaConfig.description, 
								this.descriptionTextArea.get('value'), 
								this.descriptionTextArea);
				}));
				
			this._upStreamChangeHandle = dojo.connect(this.upStreamLevelInput,"onchange", dojo.hitch(this, function() {
				this.editor._updateDirty(this.iaConfig.upStreamLevel, 
								this.upStreamLevelInput.getValue(), 
								this.upStreamLevelInput);
				}));
				
			this._downStreamChangeHandle = dojo.connect(this.downStreamLevelInput,"onchange", dojo.hitch(this, function() {
				this.editor._updateDirty(this.iaConfig.downStreamLevel, 
								this.downStreamLevelInput.getValue(), 
								this.downStreamLevelInput);
				}));
		}
		
	},
	startup: function() {
		
	},

	destroy: function() {
		this._upstreamTextHoverHelp.destroyRecursive();
		this._downstreamTextHoverHelp.destroyRecursive();

		dojo.disconnect(this._descChangeHandle);
		dojo.disconnect(this._upStreamChangeHandle);
		dojo.disconnect(this._downStreamChangeHandle);

		this.inherited(arguments);
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_Filters", [dijit._Widget, dijit._Templated], {

	FILTER_TYPES: {
		ELEMENT: messages.iaEditorSelectionElement,
		LINK: messages.iaEditorSelectionLink
	},

	TYPE_LINK: "LINK",
	TYPE_ELEMENT: "ELEMENT",

	widgetsInTemplate: true,
	messages: messages,
	templateString: '<div>'+
		'<div dojoType="rmps.ui.CollapsibleSection" primaryTitle="${messages.sectionTitle_filters}" readOnly="false" collapsible="false" isCollapsed="false">'+
			'<table style="overflow: scroll; width:100%" class="general" >' +
				'<tr>' +
					'<td style="width:1.5em" ><input type="checkbox" dojoAttachPoint="enableDeselectingCheck" dojoType="dijit.form.CheckBox"></td>' +
					'<td>${messages.iaEditorIgnoreSelected}</td>' +
				'</tr>' +
			'</table>' +
			'<div class="sub-section" dojoType="rmps.ui.CollapsibleSection" primaryTitle="${messages.iaEditorFilteredElements}" readOnly="false" collapsible="false" dojoAttachPoint="resourceTypeSection">' +
				'<div dojoAttachPoint="elementsTableDiv"></div>' +
			'</div>' +
			'<div class="sub-section" dojoType="rmps.ui.CollapsibleSection" primaryTitle="${messages.iaEditorFilteredLinks}" readOnly="false" collapsible="false" dojoAttachPoint="linkTypeSection">' +
				'<div dojoAttachPoint="linksTableDiv"></div>' +
			'</div>' +
		'</div>' +
	'</div>',
	
	// init properties
	editor: null,
	isReadOnly: false,
	iaConfig:null,
	
	postCreate: function() {
		this.iaConfig = this.editor.editorInput.iaConfig;	
		
		this.inherited(arguments);

		this.enableDeselectingCheck.set('checked' ,this.iaConfig.enableDeselecting );
		this.enableDeselectingCheck.set('disabled', this.isReadOnly);
		
		if (!this.isReadOnly) {
			dojo.connect(this.enableDeselectingCheck,"onChange", dojo.hitch(this, function() {
				this.editor._updateDirty(this.iaConfig.enableDeselecting, 
								this.enableDeselectingCheck.get('checked'), 
								this.enableDeselectingCheck);
				}));
		}

		//ELEMENTS:

		if (!this.isReadOnly) {
			// put the button in the section...
			this.resourceTypeSection.addImageAction(
				dojo.moduleUrl("dm.common.app", "graphics/icons/add.gif"),
				messages.iaEditorAddArtifactTypeToolTip,
				dojo.hitch(this, this._addFilter, this.TYPE_ELEMENT)
			);
		}
		
		this._elementsTable = new jazz.ui.StyledTable({
			columns: [
				{ label: messages.iaEditorElementType },
				{ label: messages.actions, className: "actions", width: "25%" }
			]
		});
		this._elementsTable.containerNode = this._elementsTable.tbody;
		dojo.style(this._elementsTable.domNode, 'width', '100%');
		this.elementsTableDiv.appendChild(this._elementsTable.domNode);
		this._elementsTable.startup();

		dojo.forEach(this.iaConfig.selectionElements, function(element) {
			this._addFilterRow(this.editor._allTools[element.domain], this.TYPE_ELEMENT,  element );
		}, this);


		//LINKS:

		if (!this.isReadOnly) {
			// put the button in the section...
			this.linkTypeSection.addImageAction(
				dojo.moduleUrl("dm.common.app", "graphics/icons/add.gif"),
				messages.iaEditorAddLinkTypeToolTip,
				dojo.hitch(this, this._addFilter, this.TYPE_LINK)
			);
		}
		
		this._linksTable = new jazz.ui.StyledTable({
			columns: [
				{ label: messages.iaEditorLinkType },
				{ label: messages.actions, className: "actions", width: "25%"}
			]
		});
		this._linksTable.containerNode = this._linksTable.tbody;
		dojo.style(this._linksTable.domNode, 'width', '100%');
		this.linksTableDiv.appendChild(this._linksTable.domNode);
		this._linksTable.startup();	

		dojo.forEach(this.iaConfig.selectionLinks, function(link) {
			this._addFilterRow(this.editor._allTools[link.domain], this.TYPE_LINK,  link );
		}, this);
	},

	startup: function() {
		
	},
	
	/** Filter type can be LINK or ELEMENT */
	_addFilter: function(filterType) {
		new dm.analysis.app.ui.internal.impactanalysis.config.AddTypeDialog(this, filterType,
																			  this.updateAfterAddFilter,
																			  this.iaConfig.selectionElements,
																			  this.iaConfig.selectionLinks).createDialog();
	},

	updateAfterAddFilter: function(domainText, filterType, filterValue) {
		if (filterType == this.TYPE_ELEMENT) {
			this.iaConfig.selectionElements.push(filterValue);
		} else {
			this.iaConfig.selectionLinks.push(filterValue);
		}
		
		this._addFilterRow(domainText, filterType, filterValue);
		this.editor.addDirty(this.selectionsSection, DIRTY_GROUP_ID);
	},
	
	_addFilterRow: function(domainText, filterType, filterValue) {
		var row = new dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_FilterRow({
			selectionsSection: this,
			domainText: domainText,
			filterType: filterType,
			filterValue: filterValue,
			isReadOnly: this.isReadOnly
		});

		if (filterType == this.TYPE_ELEMENT) {
			this._elementsTable.tbody.insertBefore(row.domNode, this._elementsTable.tbody.firstChild);
		} else {
			this._linksTable.tbody.insertBefore(row.domNode, this._linksTable.tbody.firstChild);
		}

		row.startup();
	},

	destroy: function() {
		this.inherited(arguments);
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_FilterRow", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	messages: messages, 
	templateString: '<tr>' +
			'<td>' +
				'<div dojoAttachPoint="filterTypeUriDiv">' +
				'</div>' +
			'</td>' +
			'<td class="actions" style="width: 25%;">' +
				'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="toolbar"></div>' +
			'</td>' +
		'</tr>',
	
	selectionsSection: null,
	domainText: null,
	filterType: null,
	filterValue: null,
	isReadOnly: false,
	
	postCreate: function() {

		// Create the resource link
		dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil.createResourceLink(this.filterTypeUriDiv, {
			uri: this.filterValue.uri,
			label: this.filterValue.uri
		});
		
		// add remove button
		if (!this.isReadOnly) {
			this.toolbar.addButton(this, function(e) {
				dojo.stopEvent(e);
				this.remove();
				this.destroyRecursive();
				}, 
				{
				label: "Remove Selection",
				imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/remove_grey.gif")
			});
		}			
		
	},
	
	remove: function() {
		var table, collection;

		if (this.filterType === this.selectionsSection.TYPE_ELEMENT){
			table = this.selectionsSection._elementsTable;
			collection = this.selectionsSection.iaConfig.selectionElements;
		} else {
			table = this.selectionsSection._linksTable;
			collection = this.selectionsSection.iaConfig.selectionLinks;
		}

		// remove entry from table
		dojo.some(table.tbody.childNodes, function (node) {
			var found = false;
			if (node.id == this.domNode.id) {
				table.tbody.removeChild(node);
				found = true;
			}
			return found;
		}, this);
		
		// remove entry from IA config collection
		dojo.some(collection, function (item, i) {
			var found = false;
			if(item.uri == this.filterValue.uri) {
				collection.splice(i, 1);
				found = true;
			}
			return found;
		}, this);

		this.selectionsSection.editor.addDirty(this.selectionsSection, DIRTY_GROUP_ID);
		
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_Advance", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	messages: messages, 
	templateString: '<div dojoAttachPoint="_advancedContent" class="hidden">'+
			'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" readOnly="false" collapsible="true">' +
					'<table style="overflow: scroll; width:100%" class="general" >' +
						'<tr> ' +
							'<td style="padding-right:10px" >${messages.iaEditorAdvancedInter}</td>' +
							'<td><select multiple="multiple" dojoAttachPoint="interLinks" size="8" title="${messages.iaEditorAdvancedInterHover}"></td>'+
						'</tr>' +
				'</table>' +
			'</div>'+
		'</div>',
	
	// init properties
	editor: null,
	isReadOnly: false,
	iaConfig:null,
	
	postCreate: function() {
		this.iaConfig = this.editor.editorInput.iaConfig;
		
		var titleNode = dojo.create("span");
		titleNode.appendChild(dojo.doc.createTextNode(messages.iaEditorAdvancedTitle));
		
		this._section.setPrimaryTitle(titleNode);
		
		this.interLinks.disabled = this.isReadOnly;
		
		//handle dirty if editor is not read only
		if (!this.isReadOnly) {
			dojo.connect(this.interLinks,"onchange", dojo.hitch(this, function() {
				var tmpList = [];
				dojo.forEach(this.interLinks.options, function (option) {
					if (option.selected) {
						tmpList.push(option.value);
					}
				}, this);
				this._updateInterLinksCount(tmpList.length);
				this.editor._updateDirty(this.iaConfig.selectionInterLinks, 
									tmpList, 
									this.interLinks);
			}));	
		}
		
		var getLinkTypesDeferred = dm.common.app.ui.internal.rhlink.AnalysisConfigClient.getTypesForTool(this.editor.editorInput.projectId , null , true );
		getLinkTypesDeferred.addCallback(this, function(linkTypes) {
			var counter = 0;
			if (linkTypes.types.length > 0) {
				dojo.removeClass(this._advancedContent, "hidden");
			}
			dojo.forEach(linkTypes.types, function(item) {
				var optionChild = dojo.doc.createElement("option");
				optionChild.innerHTML = item.value;
				optionChild.value = item.key;
				if (this.checkIfInterLinkIsSelected(item.key))
				{
					optionChild.selected = true;
					counter++;
				}
				this.interLinks.appendChild(optionChild);
			}, this);
			this._updateInterLinksCount(counter);
		});
		getLinkTypesDeferred.addErrback(this, function(error) {
			var xhr = this._getSelectionssDeferred.ioArgs.xhr;
			if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
				var lqeInfo = dojo.fromJson(xhr.responseText);
				this.editor.showMessage({
					type: lqeInfo.type,
					summary: lqeInfo.summary,
					details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
					closable: true
				});
			} else {
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.errorGetDomains,
					errorObject: error,
					closable: true
				});
			}
		});
		
	},
	
	_updateInterLinksCount: function(linkCount) {
		this._section.setSecondaryTitle('(' + linkCount + ')');
		if (this.linkCount === 0) {
			this._section.collapse();
		} else
		{
			this._section.expand();
		}
	},
	
	startup: function() {
		this.inherited(arguments);
		
	},
	
	checkIfInterLinkIsSelected: function(interLink)
	{
		return dojo.some(
			this.iaConfig.selectionInterLinks, 
			function (item) { return interLink === item; }, 
			this);
	},
	
	destroy: function() {
		this.inherited(arguments);
	}
});

}());
