/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysisConfig");
	
	dojo.require("dojox.data.dom");

	dojo.require("rmps.core.resource.Resource");
	dojo.require("rmps.core.user.UserClient");
	dojo.require("jazz.util.html");
	
	
	///////////////////////////////////////////
	/* full XML
	<?xml version="1.0" encoding="UTF-8"?>
	<rdf:RDF xmlns:iaConfig="http://jazz.net/xmlns/rh/impacAnalysis/1.0/" xmlns:dcterms="http://purl.org/dc/terms/" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
		<iaConfig:impactConfig rdf:about="https://tp-nirm.haifa.ibm.com:9444/dm/impactconfig/1">
			<iaConfig:shortName>My Analysis Config-Apr 28, 2011 12:20:17 AM</iaConfig:shortName>
			<dcterms:description></dcterms:description>
			<iaConfig:upStreamLevel rdf:datatype="http://www.w3.org/2001/XMLSchema#int">-1</iaConfig:upStreamLevel>
			<iaConfig:downStreamLevel rdf:datatype="http://www.w3.org/2001/XMLSchema#int">-1</iaConfig:downStreamLevel>
			<!--if true then you start with all elements / links and deselect them else you start with nothing and select (only if you select something)-->		
			<iaConfig:enableDeselecting rdf:datatype="http://www.w3.org/2001/XMLSchema#boolean">true</iaConfig:enableDeselecting>
			<!--0 :n selected tool -->
			<iaConfig:selectedTool>
			<rdf:Description>
					<!--true = same flase = opposite  -->
					<iaConfig:linksDirection rdf:datatype="http://www.w3.org/2001/XMLSchema#boolean">true</iaConfig:linksDirection>
					<iaConfig:tool rdf:resource="the tool1"/>
				</rdf:Description>
			</iaConfig:selectedTool>
			<!--0 :n selection elements base on the selected tool above -->
			<iaConfig:selectionElement>
				<rdf:Description>
					<iaConfig:type rdf:resource="the type1 "/>
					<iaConfig:tool rdf:resource="the tool1 "/>		
				</rdf:Description>
			</iaConfig:selectionElement>
			
			<iaConfig:selectionElement>
				<rdf:Description>
					<iaConfig:type rdf:resource="the type2 "/>		
					<iaConfig:tool rdf:resource="the tool2  "/>		
				</rdf:Description>
			</iaConfig:selectionElement>
			<!--0 :n selection elements base on the selected tool above -->
			<iaConfig:selectionLink>
				<rdf:Description>
					<iaConfig:type rdf:resource="the type2 "/>		
					<iaConfig:tool rdf:resource="the tool2  "/>		
				</rdf:Description>
			</iaConfig:selectionLink>
			<!--0 :n selection inter links -->			
			<iaConfig:selectionInterLink>
				<rdf:Description>
					<iaConfig:type rdf:resource="the type2 "/>	
				</rdf:Description>
			</iaConfig:selectionInterLink>		
		</iaConfig:impactConfig>
	</rdf:RDF>

*/
	
	//////////////////////////////////////////
		

	var UserClient = rmps.core.user.UserClient;

	var XMLNS = "http://open-services.net/ns/am#";
	var RH_IA  = "http://jazz.net/xmlns/rh/impacAnalysis/1.0/";
	var	DC    = "http://purl.org/dc/terms/";
	var	RDF   = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	
	var IAC_QUALIFIER ="iaConfig";
	var CONTENT_TYPE_IA = "application/rdf+xml";
	
	var BOOLEAN = "http://www.w3.org/2001/XMLSchema#boolean";
	
	var INT  = "http://www.w3.org/2001/XMLSchema#int";

	dojo.declare("dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysisConfig", rmps.core.resource.Resource, {

	// private properties
	description: '',
	shortName: '',
	eTag: null,

	//Property to help identify which Configs have not been upgraded to Filters yet
	isFilter: true,
	
	upStreamLevel: null,
	downStreamLevel: null,
	enableDeselecting: null,
	/*{tool:"" ,linksDirection:true }*/
	//TODO: once a decision has been made about whether to keep selectedTools
	//		this param should be removed from here AND server-side
	selectedTools: null,
	/*{tool:"" ,type:"" }*/
	selectionElements: null,
	/*{tool:"" ,type:"" }*/
	selectionLinks: null,
	//list of link types
	selectionInterLinks: null,
	//TBD add/Remove list / comments
	
	constructor: function() {
		this.setDefaultValues();
	},
	
	setDefaultValues: function() {
		this.upStreamLevel = -1;
		this.downStreamLevel = -1;
		this.enableDeselecting =  false;
		this.selectedTools = [];
		this.selectionElements =  [];
		this.selectionLinks = [];
		this.selectionInterLinks = [];
	},
	
	getName: function()
	{
		return this.shortName;
	},
	
	getContributor: function() {
		var deferred;
		if (this._contributor && this._contributor.uri === this.contributor) {
			deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._contributor);
			}), 0);
			return deferred;
		}
		deferred = UserClient.getUser(this.contributor);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._contributor = user;
		}));
		return deferred;
	},

	getCreator: function() {
		var deferred;
		if (this._creator && this._creator.uri === this.creator) {
			deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._creator);
			}), 0);
			return deferred;
		}
		deferred = UserClient.getUser(this.creator);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._creator = user;
		}));
		return deferred;
	},
	
	toXMLString: function() {
		var iaXML = '<?xml version="1.0" encoding="UTF-8"?> \n' ;
		iaXML+= '<rdf:RDF xmlns="' + XMLNS + 
		'"\n xmlns:dcterms="' + DC + 
		'"\n xmlns:rdf="'+ RDF +
		'"\n xmlns:' + IAC_QUALIFIER + '="' + RH_IA + '"' +
		'>\n';

		var resourceUri = !this.uri ? '' : (' rdf:about="' + this.uri + '"');

		iaXML += '<iaConfig:impactConfig' + resourceUri + '>';
		iaXML += this.fragmentNode(IAC_QUALIFIER, "shortName", jazz.util.html.escape(this.shortName), true, null);
		iaXML += '<dcterms:description>' + jazz.util.html.escape(this.description) + '</dcterms:description>\n';
		iaXML += this.fragmentNode(IAC_QUALIFIER, "upStreamLevel", this.upStreamLevel, true, INT);
		iaXML += this.fragmentNode(IAC_QUALIFIER, "downStreamLevel", this.downStreamLevel, true, INT);
		iaXML += this.fragmentNode(IAC_QUALIFIER, "enableDeselecting", this.enableDeselecting, true, BOOLEAN);
		if (this.systemDefined){
			iaXML += this.fragmentNode(IAC_QUALIFIER, "systemDefined", this.systemDefined, true, BOOLEAN);
		}
		if (this.systemDefaultConfig){
			iaXML += this.fragmentNode(IAC_QUALIFIER, "systemDefaultConfig", this.systemDefaultConfig, true, BOOLEAN);
		}
		if (this.isFilter){
			iaXML += this.fragmentNode(IAC_QUALIFIER, "isFilter", this.isFilter, true, BOOLEAN);
		}
	
		dojo.forEach(this.selectedTools, function (selectedTool) {
			iaXML += this.fragmentTool(IAC_QUALIFIER, selectedTool.tool, selectedTool.linksDirection);
		}, this);
		dojo.forEach(this.selectionElements, function (selectedElement) {
			iaXML += this.fragmentSelection(IAC_QUALIFIER, "selectionElement", selectedElement.uri, selectedElement.domain);
		}, this);
		dojo.forEach(this.selectionLinks, function (selectedLink) {
			iaXML += this.fragmentSelection(IAC_QUALIFIER, "selectionLink", selectedLink.uri, selectedLink.domain);
		}, this);
		dojo.forEach(this.selectionInterLinks, function (selectedInterLink) {
			iaXML += this.fragmentSelection(IAC_QUALIFIER, "selectionInterLink", selectedInterLink, null);
		}, this);
		//END
		iaXML += '</iaConfig:impactConfig>';
		iaXML += '</rdf:RDF>\n';
		return iaXML;
	},
	
	/*<iaConfig:selectionElement>
				<rdf:Description>
					<iaConfig:type rdf:resource="the type1 "/>
					<iaConfig:tool rdf:resource="the tool1 "/>		
				</rdf:Description>
			</iaConfig:selectionElement>
	*/
	fragmentSelection: function(qn, selectiontype,type , tool ) {
		var fragment = "<" + qn + ":"+ selectiontype +">";
		fragment += "<rdf:Description>";
		fragment += this.fragmentResourceNode(qn,"type", type,false);
		if (tool != null)
		{
			fragment += this.fragmentResourceNode(qn,"tool", tool,false);	
		}
		fragment += "</rdf:Description>";
		fragment += "</" + qn + ":"+ selectiontype +">";
		return fragment;
	},
	
	fragmentTool: function(qn,tool,linksDirection) {
		var fragment = "<" + qn + ":selectedTool>";
		fragment += "<rdf:Description>";
		fragment += this.fragmentResourceNode(qn,"tool", tool,false);
		fragment += this.fragmentNode(IAC_QUALIFIER ,"linksDirection" ,linksDirection , true , null ) ;
		fragment += "</rdf:Description>";
		fragment += "</" + qn + ":selectedTool>";
		return fragment;
	},
	
	
	//rdf:datatype="http://www.w3.org/2001/XMLSchema#boolean"
	fragmentNode: function(qn, property, value, not_encoded , type) {
		var adoptedValue= value? value.toString(): value;
		var xmlEncoded = not_encoded ?  value : dojox.html.entities.encode(adoptedValue);
		var datatype = type === null ? "" : "rdf:datatype=\"" + type + "\"";
		if (xmlEncoded === null)
		{
			return "<" + qn + ":"+ property + "></" + qn + ":"+ property +">\n";
		}
		else{
			return "<" + qn + ":"+ property +" " + datatype + ">" + xmlEncoded + "</" + qn + ":"+ property +">\n";
		}
		
	},
	
	fragmentResourceNode: function(qn, property, value, not_encoded) {
		var adoptedValue= value? value.toString(): value;
		var xmlEncoded = not_encoded ?  value : dojox.html.entities.encode(adoptedValue);
		if (xmlEncoded === null)
		{
			return '<' + qn + ':'+ property +' rdf:resource=""/>\n';
		}
		else{
			return '<' + qn + ':'+ property +' rdf:resource="' + xmlEncoded + '"/>\n';
		}
	},
	/*
	
	*/
	parse: function(iaNode, ioArgs) {
		if (ioArgs) {
			var xhr = ioArgs.xhr;
			if (xhr.getResponseHeader("ETag")) {
				this.eTag = xhr.getResponseHeader("ETag");
			}
			if (xhr.getResponseHeader("Location")) {
				this.uri = xhr.getResponseHeader("Location");
			}
		}
		this.selectedTools = [];
		this.selectionElements= [];
		this.selectionLinks= [];
		this.selectionInterLinks= [];
		
		this.parseNodes(iaNode);
	},
	
	parseNodes: function(iaNode)
	{
		dojo.forEach(iaNode.childNodes, function (node) {
			if (node.nodeName === "iaConfig:shortName") {
				if (node.childNodes.length > 0) 
				{
					this.shortName = node.childNodes[0].nodeValue;
				}
			}
			else if (node.nodeName === "dcterms:description") {
				if (node.childNodes.length > 0) 
				{
					this.description = node.childNodes[0].nodeValue;
				}
			}
			else if (node.nodeName === "iaConfig:systemDefined") {
				if (node.childNodes.length > 0) 
				{
					this.systemDefined = (node.childNodes[0].nodeValue === "true");
				}
			}
			else if (node.nodeName === "iaConfig:systemDefaultConfig") {
				if (node.childNodes.length > 0) 
				{
					this.systemDefaultConfig = (node.childNodes[0].nodeValue === "true");
				}
			}
			else if (node.nodeName === "iaConfig:upStreamLevel") {
				if (node.childNodes.length > 0 && node.childNodes[0].nodeValue) 
				{
					this.upStreamLevel = parseInt(node.childNodes[0].nodeValue, 10);
				}
			}
			else if (node.nodeName === "iaConfig:downStreamLevel") {
				if (node.childNodes.length > 0 && node.childNodes[0].nodeValue) 
				{
					this.downStreamLevel = parseInt(node.childNodes[0].nodeValue, 10);
				}
			}
			else if (node.nodeName === "iaConfig:enableDeselecting") {
				if (node.childNodes.length > 0) 
				{
					this.enableDeselecting = (node.childNodes[0].nodeValue === "true");
				}
			}
			else if (node.nodeName === "iaConfig:selectedTool") {
				this.getSelectedTools(node);
			}
			else if (node.nodeName === "iaConfig:selectionElement") {
				this.getSelectionElements(node);
			}	
			else if (node.nodeName === "iaConfig:selectionLink") {
				this.getSelectionLinks(node);
			}
			else if (node.nodeName === "iaConfig:selectionInterLink") {
				this.getSelectionInterLinks(node);
			}
			else{
				if (node.childNodes) {
					this.parseNodes(node);
				}
			}		
		}, this);
	},
	
	getSelectedTools: function(iaNode)
	{
		var selectedTool = {};
		dojo.forEach(iaNode.childNodes, function(node) {
			if (node.nodeName === "iaConfig:tool") {
				if (node.attributes.length > 0) {
					selectedTool.tool = node.attributes[0].nodeValue;
				}
			} else if (node.nodeName === "iaConfig:linksDirection") {
				if (node.childNodes.length > 0) {
					selectedTool.linksDirection = node.childNodes[0].nodeValue;
				}
			} else {
				if (node.childNodes) {
					this.getSelectedTool(node);
				}
			}
		}, this);
		this.selectedTools[this.selectedTools.length] = selectedTool;
	},
	
	getSelectionElements: function(iaNode)
	{
		var selectionElement = {};
		dojo.forEach(iaNode.childNodes, function (node) {
			if (node.nodeName === "iaConfig:tool") {
				if (node.attributes.length > 0) {
					selectionElement.domain = node.attributes[0].nodeValue;
				}
			} else if (node.nodeName === "iaConfig:type") {
				if (node.attributes.length > 0) {
					selectionElement.uri = node.attributes[0].nodeValue;
				}
			} else {
				console.debug("Unable to identify this node in the model: " + node.nodeName);
			}
		}, this);
		this.selectionElements[this.selectionElements.length] = selectionElement;
	},
	
	getSelectionLinks: function(iaNode)
	{
		var selectionLink = {},
			value = null;
		dojo.forEach(iaNode.childNodes, function (node) {
			if (node.nodeName === "iaConfig:tool") {
				if (node.attributes.length > 0) {
					value = node.attributes[0].nodeValue;
					selectionLink.domain = value;
				}
			} else if (node.nodeName === "iaConfig:type") {
				if (node.attributes.length > 0) {
					value = node.attributes[0].nodeValue;
					selectionLink.uri = value;
				}
			} else {
				console.debug("Unable to identify this node in the model: " + node.nodeName);
			}
		}, this);
		this.selectionLinks[this.selectionLinks.length] = selectionLink;
	},
	
	getSelectionInterLinks: function(iaNode)
	{
		var selectionInterLinks = {};
		dojo.forEach(iaNode.childNodes, function (node) {
			if (node.nodeName === "iaConfig:type") {
				if (node.attributes.length > 0) {
					this.selectionInterLinks[this.selectionInterLinks.length] = node.attributes[0].nodeValue;
				}
			} else {
				console.debug("Unable to identify this node in the model: " + node.nodeName);
			}
		}, this);
	}
	
	});

}());