/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysisProfile");

	dojo.require("dojox.data.dom");

	dojo.require("rmps.core.resource.Resource");
	dojo.require("rmps.core.user.UserClient");
	dojo.require("jazz.util.html");

	///////////////////////////////////////////
	/*
	<rdf:RDF xmlns:iaProfile="http://jazz.net/xmlns/rh/impacAnalysis/1.0/" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:dcterms="http://purl.org/dc/terms/">
		
		<iaProfile:impactProfile rdf:about="https://harbinger:9444/dm/impactprofile/738d38cb-7a34-4e33-aa8f-9d838d549e24">
			<iaProfile:shortName>My Profile</iaProfile:shortName>
			<dcterms:description>this is the description of this profile</dcterms:description>
			<iaProfile:impactTimestamp>2013-05-08T17:31:50-07:00</iaProfile:impactTimestamp>

			<iaProfile:traversal>Downstream</iaProfile:traversal>
			<iaProfile:downstreamDepth rdf:datatype="http://www.w3.org/2001/XMLSchema#int">1</iaProfile:downstreamDepth>

			<iaProfile:impactConfig rdf:parseType="Resource">
				<iaProfile:configName rdf:datatype="http://www.w3.org/2001/XMLSchema#string">Default</iaProfile:configName>
				<iaProfile:profile rdf:resource="https://harbinger:9444/dm/impactconfig/defaultConfig-_Psxqk60nEeKTdpb27Gwr4w"></iaProfile:profile>
			</iaProfile:impactConfig>

			<iaProfile:impactConfig rdf:parseType="Resource">
				<iaProfile:configName rdf:datatype="http://www.w3.org/2001/XMLSchema#string">All Cross Domain Links</iaProfile:configName>
				<iaProfile:profile rdf:resource="https://harbinger:9444/dm/impactconfig/ba6b92b4-fa39-4556-bc4b-17921137f999"></iaProfile:profile>
			</iaProfile:impactConfig>

			...
			...

		</iaProfile:impactProfile>

	</rdf:RDF>

	*/
	//////////////////////////////////////////

	var UserClient = rmps.core.user.UserClient;

	var XMLNS = "http://open-services.net/ns/am#";
	var RH_IA  = "http://jazz.net/xmlns/rh/impacAnalysis/1.0/";
	var	DC    = "http://purl.org/dc/terms/";
	var	RDF   = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	var CONTENT_TYPE_IA = "application/rdf+xml";

	var IAP_QUALIFIER ="iaProfile";

	var BOOLEAN = "http://www.w3.org/2001/XMLSchema#boolean";
	var INT  = "http://www.w3.org/2001/XMLSchema#int";

	var UPSTREAM = "Upstream";
	var DOWNSTREAM = "Downstream";
	var BOTH = "Both";
	var FULL_TRACE = "Full_Trace";

	dojo.declare("dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysisProfile", rmps.core.resource.Resource, {

	// private properties
	shortName: '',
	description: '',
	eTag: null,

	//list of filters
	filters: null,

	traversal: '',
	upstreamDepth: null,
	downstreamDepth: null,
	fullTraceDepth: null,

	upstreamType: '',
	downstreamType: '',
	fullTraceType: '',

	constructor: function() {
		this.filters = [];
	},

	getName: function()
	{
		return this.shortName;
	},

	getContributor: function() {
		var deferred;
		if (this._contributor && this._contributor.uri === this.contributor) {
			deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._contributor);
			}), 0);
			return deferred;
		}
		deferred = UserClient.getUser(this.contributor);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._contributor = user;
		}));
		return deferred;
	},

	getCreator: function() {
		var deferred;
		if (this._creator && this._creator.uri === this.creator) {
			deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._creator);
			}), 0);
			return deferred;
		}
		deferred = UserClient.getUser(this.creator);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._creator = user;
		}));
		return deferred;
	},
 
	toXMLString: function() {
		var iaXML = '<?xml version="1.0" encoding="UTF-8"?> \n';
		iaXML += '<rdf:RDF xmlns="' + XMLNS +
			'"\n xmlns:dcterms="' + DC +
			'"\n xmlns:rdf="' + RDF +
			'"\n xmlns:' + IAP_QUALIFIER + '="' + RH_IA + '"' +
			'>\n';

		var resourceUri = !this.uri ? '' : (' rdf:about="' + this.uri + '"');

		iaXML += '<iaProfile:impactProfile'+ resourceUri +'>';
		iaXML += this.fragmentNode(IAP_QUALIFIER, "shortName", jazz.util.html.escape(this.shortName), true, null);
		iaXML += '<dcterms:description>' + jazz.util.html.escape(this.description) + '</dcterms:description>\n';
		if (this.systemDefined) {
			iaXML += this.fragmentNode(IAP_QUALIFIER, "systemDefined", this.systemDefined, true, BOOLEAN);
		}
		if (this.systemDefaultProfile) {
			iaXML += this.fragmentNode(IAP_QUALIFIER, "systemDefaultProfile", this.systemDefaultProfile, true, BOOLEAN);
		}

		iaXML += this.fragmentNode(IAP_QUALIFIER, "traversal", jazz.util.html.escape(this.traversal), true, null);

		if (this.traversal === BOTH || this.traversal === UPSTREAM){
			if (this.upstreamType === "" && this.upstreamDepth !== null)
				iaXML += this.fragmentNode(IAP_QUALIFIER, "upstreamDepth", this.upstreamDepth, false, INT);
			else
				iaXML += this.fragmentResourceNode(IAP_QUALIFIER, "upstreamType", jazz.util.html.escape(this.upstreamType), true, null);
		}

		if (this.traversal === BOTH || this.traversal === DOWNSTREAM){
			if (this.downstreamType === "" && this.downstreamDepth !== null)
				iaXML += this.fragmentNode(IAP_QUALIFIER, "downstreamDepth", this.downstreamDepth, false, INT);
			else
				iaXML += this.fragmentResourceNode(IAP_QUALIFIER, "downstreamType", jazz.util.html.escape(this.downstreamType), true, null);
		}
		
		if (this.traversal === FULL_TRACE){
			if (this.fullTraceType === "" && this.fullTraceDepth !== null)
				iaXML += this.fragmentNode(IAP_QUALIFIER, "fullTraceDepth", this.fullTraceDepth, false, INT);
			else
				iaXML += this.fragmentResourceNode(IAP_QUALIFIER, "fullTraceType", jazz.util.html.escape(this.fullTraceType), true, null);
		}

		dojo.forEach(this.filters, function(filter) {
			iaXML += this.fragmentFilter(filter);
		}, this);

		iaXML += '</iaProfile:impactProfile>';
		iaXML += '</rdf:RDF>\n';
		return iaXML;
	},

	//rdf:datatype="http://www.w3.org/2001/XMLSchema#boolean"
	fragmentNode: function(qn, property, value, not_encoded, type) {
		var adoptedValue = value ? value.toString() : value;
		var xmlEncoded = not_encoded ? value : dojox.html.entities.encode(adoptedValue);
		var datatype = type === null ? "" : "rdf:datatype=\"" + type + "\"";
		if (xmlEncoded === null) {
			return "<" + qn + ":" + property + "></" + qn + ":" + property + ">\n";
		} else {
			return "<" + qn + ":" + property + " " + datatype + ">" + xmlEncoded + "</" + qn + ":" + property + ">\n";
		}
	},
	//rdf:resource="..."
	fragmentResourceNode: function(qn, property, value, not_encoded) {
		var adoptedValue= value? value.toString(): value;
		var xmlEncoded = not_encoded ?  value : dojox.html.entities.encode(adoptedValue);
		if (xmlEncoded === null)
		{
			return '<' + qn + ':'+ property +' rdf:resource=""/>\n';
		}
		else{
			return '<' + qn + ':'+ property +' rdf:resource="' + xmlEncoded + '"/>\n';
		}
	},

	fragmentFilter: function(filter){
		 /*
		<iaProfile:impactConfig>
			<rdf:Description>
				<iaProfile:profile rdf:resource="https://harbinger:9444/dm/impactconfig/ba6b92b4-fa39-4556-bc4b-17921137f999"></iaProfile:profile>
			</rdf:Description>
		</iaProfile:impactConfig>
		*/

		var xml = "";
		var configUri = filter.uri ? filter.uri.toString() : filter.uri;
		xml += '<' + IAP_QUALIFIER + ':impactConfig>\n';
		xml += '<rdf:Description>\n';
			xml += this.fragmentResourceNode(IAP_QUALIFIER, "profile", configUri, false);
		xml += '</rdf:Description>\n';
		xml += '</' + IAP_QUALIFIER + ':impactConfig>\n';

		//xml += '<' + IAP_QUALIFIER + ':impactConfig rdf:resource=\"' + dojox.html.entities.encode(configUri) + '\">\n';

		return xml;
	},

	parse: function(iaNode, ioArgs) {
		if (ioArgs) {
			var xhr = ioArgs.xhr;
			if (xhr.getResponseHeader("ETag")) {
				this.eTag = xhr.getResponseHeader("ETag");
			}
			if (xhr.getResponseHeader("Location")) {
				this.uri = xhr.getResponseHeader("Location");
			}
		}
		this.filters = [];
		
		this.parseNodes(iaNode);
	},
	
	parseNodes: function(iaNode)
	{
		dojo.forEach(iaNode.childNodes, function (node){
			if (node.nodeName === "iaProfile:shortName"){
				if (node.childNodes.length > 0)
				{
					this.shortName = node.childNodes[0].nodeValue;
				}
			}
			else if (node.nodeName === "dcterms:description"){
				if (node.childNodes.length > 0)
				{
					this.description = node.childNodes[0].nodeValue;
				}
			}
			else if (node.nodeName === "iaProfile:traversal"){
				if (node.childNodes.length > 0)
				{
					this.traversal = node.childNodes[0].nodeValue;
				}
			}
			else if (node.nodeName === "iaProfile:upstreamDepth"){
				if (node.childNodes.length > 0)
				{
					var upVal = parseInt(node.childNodes[0].nodeValue, 10);
					if (upVal > 0)
						this.upstreamDepth = upVal;
				}
			}
			else if (node.nodeName === "iaProfile:downstreamDepth"){
				if (node.childNodes.length > 0)
				{
					var downVal = parseInt(node.childNodes[0].nodeValue, 10);
					if (downVal > 0)
						this.downstreamDepth = downVal;
				}
			}
			else if (node.nodeName === "iaProfile:fullTraceDepth"){
				if (node.childNodes.length > 0)
				{
					var fullTraceVal = parseInt(node.childNodes[0].nodeValue, 10);
					if (fullTraceVal > 0)
						this.fullTraceDepth = fullTraceVal;
				}
			}
			else if (node.nodeName === "iaProfile:upstreamType"){
				if (node.attributes.length > 0)
				{
					this.upstreamType = node.attributes[0].nodeValue;
				}
			}
			else if (node.nodeName === "iaProfile:downstreamType"){
				if (node.attributes.length > 0)
				{
					this.downstreamType = node.attributes[0].nodeValue;
				}
			}
			else if (node.nodeName === "iaProfile:fullTraceType"){
				if (node.attributes.length > 0)
				{
					this.fullTraceType = node.attributes[0].nodeValue;
				}
			}
			else if (node.nodeName === "iaProfile:systemDefined"){
				if (node.childNodes.length > 0)
				{
					this.systemDefined = (node.childNodes[0].nodeValue === "true");
				}
			}
			else if (node.nodeName === "iaProfile:systemDefaultProfile"){
				if (node.childNodes.length > 0)
				{
					this.systemDefaultProfile = (node.childNodes[0].nodeValue === "true");
				}
			}
			else if (node.nodeName === "iaProfile:impactConfig"){
				this.parseFilter(node);
			}
			else{
				if (node.childNodes){
					this.parseNodes(node);
				}
			}
		}, this);
	},


	/*
	<iaProfile:impactConfig rdf:parseType="Resource">
		<iaProfile:configName rdf:datatype="http://www.w3.org/2001/XMLSchema#string">All Cross Domain Links</iaProfile:configName>
		<iaProfile:profile rdf:resource="https://harbinger:9444/dm/impactconfig/ba6b92b4-fa39-4556-bc4b-17921137f999"></iaProfile:profile>
	</iaProfile:impactConfig>
	*/
	parseFilter: function(node)
	{
		var filter = {};
		dojo.forEach(node.childNodes, function (childNode) {
			if (childNode.nodeName === "iaProfile:profile") {
				if (childNode.attributes.length > 0) {
					filter.uri = childNode.attributes[0].nodeValue;
				}
			} else if (childNode.nodeName === "iaProfile:configName") {
				if (childNode.childNodes.length > 0) {
					filter.name = childNode.childNodes[0].nodeValue;
				}
			} else {
				console.debug("Unable to identify this node in the model: " + childNode.nodeName);
			}
		}, this);
		this.filters.push(filter);
	}
	
	});

}());