/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.panels.ElementListPanel");

dojo.require("dijit._Widget");
dojo.require("dojo.i18n");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.StyledTable");
dojo.require("dm.common.app.ui.internal.rhlink.IAInfoClient");
dojo.require('dojox.html.entities');
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.panels", "ImpactAnalysisPanelsMessages");
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis.panels", "ImpactAnalysisPanelsMessages");
var DIRTY_GROUP_ID = "dm.analysis.app.ui.internal.impactanalysis.panels.ElementListPanel.dirty";

dojo.declare(
	"dm.analysis.app.ui.internal.impactanalysis.panels.ElementListPanel", 
	[dijit._Widget, dijit._Templated], 
	{
		messages: messages,
		widgetsInTemplate: true,	
		templateString:
			'<div dojoAttachPoint="containerNode" class="rh-app-pages-rh-impact" style="overflow: auto; padding: 0px; height: 100%;">'+
				'<table style="width: 100%"">'+
				'<tr><td>'+
					'<div dojoAttachPoint="elementToolbar" style="float:right"/>'+
				'</td></tr>'+
				'<tr><td>'+
					'<div dojoAttachPoint="elementTableDiv"/>'+
				'</td></tr>'+
			'</div>',
		
		// init properties
		editor: null,
		aContainer: null,
		_elementList: null,
		_toolbar: null,
		_table: null,
		counterText: null,
		
		nameColumnLabel: messages.elementName,
		
		postCreate: function() {
			// init the tool bar
			this._toolbar = new jazz.ui.SimpleToolbar();
			this._toolbar.startup();
			this.elementToolbar.appendChild(this._toolbar.domNode);

			var table = new jazz.ui.StyledTable({
				topBorder: true,
				columns: [
					{
						label: this.nameColumnLabel,
						style: {width: "85%"}
					},
					{
						label: messages.actions,
						style: {width: "15%"}
					}
				]
			});
			this.elementTableDiv.appendChild(table.domNode);
			table.containerNode = table.tbody;
			this._table = table.tbody;
			
			table.domNode.style.width = "100%";
			this.updateToolbar(this._toolbar);
			this.counterText = dojo.doc.createElement("span");
		},

		//abstract
		updateToolbar: function(toolbar){
		},
		
		getElementName: function(elementURI){
			var name;
			dojo.some(this._table.childNodes, function(node) {
				var found = false;
				if (node.id === elementURI) {
					found = true;
					name = node.shortName;
				}
				return found;
			});
			return name;
		},

		resize: function(size){
			if (size && size.h) {
				dojo.marginBox(this.domNode, {h: size.h});
			}
		},

		reset: function(resourceInfo){
			while(this._table.hasChildNodes()){
				this._table.removeChild(this._table.lastChild);
			}
		},
		
		load: function(elementsArr)
		{
			this.reset();
			if(elementsArr.length > 0)
			{
				var deferred = dm.common.app.ui.internal.rhlink.IAInfoClient.getInfo(elementsArr);
				deferred.addCallback(this, function(infos) {
					dojo.forEach(infos, function (info) {
						this.addElement(info.uri, info.shortName, info.typeName, true);
					}, this);
					this.counterText.innerHTML = "&nbsp;(" + this._table.childNodes.length + ")";
				});	
			}
		},
		
		addElement: function(uri, shortName, type, clean)
		{
			this.addRow({"uri": uri, "shortName" : shortName, "parent":this});	
			this.counterText.innerHTML = "&nbsp;(" + this._table.childNodes.length + ")";
			if (!clean) this.editor.addDirty(this, DIRTY_GROUP_ID);
		},

		addElementNoResourceLink: function(uri, shortName, type, clean)
		{
			this.addRowNoResourceLink({"uri": uri, "shortName" : shortName, "parent":this});	
			this.counterText.innerHTML = "&nbsp;(" + this._table.childNodes.length + ")";
			if (!clean) this.editor.addDirty(this, DIRTY_GROUP_ID);
		},

		addRow: function(resource) {
			var row = dojo.doc.createElement("tr");
			row.id = resource.uri;
			row.shortName = resource.shortName;
			row.parent = this;
			var elementName = dojo.create("td" , {style: {width: "85%"}});
			var action = dojo.create("td" , {style: {width: "15%"}});

			// Create the resource link
			dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil.createResourceLink(elementName, {
				uri: resource.uri,
				label: dojox.html.entities.encode(resource.shortName || ' - ')
			});

			var actionBar = new jazz.ui.SimpleToolbar();
			actionBar.startup();
			if (this.iconType && this.iconType == "remove") {
				actionBar.addButton(resource, this._onRestoreRow, {
						label: messages.remove,
						imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/remove.gif")
				});
			} else {			
				actionBar.addButton(resource, this._onRestoreRow, {
						label: messages.restore,
						imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/restore.gif")
				});
			}
			action.appendChild(actionBar.domNode);
			row.appendChild(elementName);
			row.appendChild(action);
			this._table.appendChild(row);
		},

		addRowNoResourceLink: function(resource) {
			var row = dojo.doc.createElement("tr");
			row.id = resource.uri;
			row.shortName = resource.shortName;
			row.parent = this;
			var elementName = dojo.create("td" , {style: {width: "85%"}});
			var action = dojo.create("td" , {style: {width: "15%"}});

			var elementLink = dojo.create("a", {
				href: dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil.getAnalysisConfigUri(resource.uri),
				innerHTML: dojox.html.entities.encode(resource.shortName || ' - ')
			}, elementName);
			
			var actionBar = new jazz.ui.SimpleToolbar();
			actionBar.startup();
			if (this.iconType && this.iconType == "remove") {
				actionBar.addButton(resource, this._onRestoreRow, {
						label: messages.remove,
						imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/remove.gif")
				});
			} else {			
				actionBar.addButton(resource, this._onRestoreRow, {
						label: messages.restore,
						imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/restore.gif")
				});
			}
			action.appendChild(actionBar.domNode);
			row.appendChild(elementName);
			row.appendChild(action);
			this._table.appendChild(row);
		},
		
		_onRestoreRow:function(){
			if (!confirm (messages.confirmRestore))
				return;
			
			dojo.some(this.parent._table.childNodes, function (node) {
				var found = false;
				if (node.id === this.uri) {
					this.parent._table.removeChild(node);
					found = true;
				}
				return found;
			}, this);
			this.parent.editor.addDirty(this, DIRTY_GROUP_ID);
			this.parent.restoreElement(this, this.parent);
		},

		removeDirty: function(){
			this.editor.removeDirty(this, DIRTY_GROUP_ID);
		},
		
		//abstract
		restoreElement: function(row, parent){
		}
	
	});

}());
