/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.panels.FilterListPanel");

dojo.require("dijit._Widget");

dojo.require("dm.analysis.app.ui.internal.impactanalysis.panels.ElementListPanel");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.picker.LocateElementControler");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.profile.AddFilterDialog");
dojo.require("dm.common.app.ui.internal.rhlink.IAInfoClient");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.panels", "ImpactAnalysisPanelsMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis.panels", "ImpactAnalysisPanelsMessages");

dojo.declare(
	"dm.analysis.app.ui.internal.impactanalysis.panels.FilterListPanel",
	dm.analysis.app.ui.internal.impactanalysis.panels.ElementListPanel, 
	{
		messages: messages,
		iconType: "remove",
		
		// init properties
		profileInfoChangeCallback: null,
		editor: null,
		iaProfile: null,			//contains the original iaProfile.filters
		iaProfileModified: null,	//contains the modified iaProfile.filters
		isReadOnly: false,
		
		postCreate: function() {
			this.nameColumnLabel = messages.filterName;
			this.inherited(arguments);
		},

		startup: function() {
			this.loadFiltersFromProfile(this.iaProfile);
		},

		updateToolbar: function(toolbar)
		{
			if (!this.isReadOnly) {
				toolbar.addButton(this, dojo.hitch(this, this._addFilter), {
					label: messages.addFilter,
					imgSrc: dojo.moduleUrl("dm.common.app", "graphics/icons/add.gif")
				});
			}
		},

		_addFilter: function(){
			new dm.analysis.app.ui.internal.impactanalysis.profile.AddFilterDialog(this, this.iaProfile).createDialog();
		},

		//AddFilterDialog callback:
		updateAfterAddFilter: function(filter) {
			this.addElement(filter.uri, filter.name, null, false);
			this.iaProfileModified.filters.push(filter);
			this.profileInfoChangeCallback && this.profileInfoChangeCallback();
		},

		getFilters: function() {
			if (!this.iaProfileModified)
				return [];
			else
				return this.iaProfileModified.filters;
		},

		loadFiltersFromProfile: function(iaProfile){
			if (!iaProfile)
				return;

			this.reset();
			this.removeDirty(); //this really should be done by reset(), but I'm trying to prevent regressions at the moment

			this.iaProfile = iaProfile;
			this.iaProfileModified = dojo.clone(iaProfile);
			dojo.forEach(iaProfile.filters, function(filter) {
				this.addElementNoResourceLink(filter.uri, filter.name, null, true);
			}, this);
		},

		//override	
		restoreElement: function(row, listPanel){
			var iap = this.iaProfileModified;
			for(var i = 0 ; i < iap.filters.length; i++)
			{
				if (iap.filters[i].uri === row.uri)
				{
					iap.filters.splice(i,1);
					this.profileInfoChangeCallback && this.profileInfoChangeCallback();
					break;
				}
			}
		},

		destroy: function() {
			this.inherited(arguments);
		}
	});

})();
