/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("dm.analysis.app.ui.internal.impactanalysis.picker.IASearchBox");

dojo.require("jazz.ui.SearchBox");

dojo.require("dojo.i18n"); 
dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 


(function(){

	dojo.declare(
		"dm.analysis.app.ui.internal.impactanalysis.picker.IASearchBox",
		[ jazz.ui.SearchBox ],
		{
			tooltip: messages.search,
			_cancelSearchResults: false,
			_spinnerSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/spinner.gif'),

			constructor: function(args, node) {
				this.scopes = [{	
		    		id: "scope", 
		    		label: this.tooltip, 
					icon: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/all-media_16.gif') //$NON-NLS-1$ //$NON-NLS-2$
		      	}];
			},

			buildRendering: function(){
				this.inherited(arguments);
				this.domNode.className += ' iaSearchBox blurred';
				this.spinner = dojo.create('img', {
					alt: messages.loading,
					src: this._spinnerSrc,
					"class": "spinner j-hide"
				}, this.domNode, 'first');
				this._targetArtifact = dojo.toDom('<span class="artifactDisplay showing">\
						<span class="j-hint">' + messages.initDialogNoArtifactSelected + '</span>\
					</span>');
				this.domNode.appendChild(this._targetArtifact);
			},

			postCreate: function() {
				this.inherited(arguments);
			},

			_getDefaultText: function() {
				return "";
			},

			onFocus: function(e) {
				dojo.removeClass(this.domNode, "blurred");
				var self = this;
				setTimeout(function() {
					self._searchText.focus();
				}, 0);
			},


			onBlur: function() {
				dojo.addClass(this.domNode, "blurred");
			}, 

			set_performUpdate: function(performUpdate) {
				this.performUpdate = performUpdate;
			},

			_previousQuery: null,

			/**
			 * The function that peforms the actual search. This must be supplied by the provider and must return a deffered
			 * 
			 * @param  {String}   query         the search string
			 * @param  {Object}   scope         a search scope. This should be ignored
			 * @param  {Function} callback      to be called when the search has completed. It should be supplied with formatted
			 *                                  results. See documentation for jazz.ui.SearchBox for details. 
			 * @param  {Function} errorCallback standard hook provided by jazz.ui.iaSearchBox
			 */
			performUpdate: function(query, scope, callback, errorCallback) {
				throw new Error("Provider must provide performUpdate");
			},

			hideSpinner: function()
			{
				dojo.addClass(this.spinner, "j-hide");
			},
			
			showSpinner: function(hintText)
			{
				dojo.removeClass(this.spinner, "j-hide");
				if (hintText)
				{
					while(this._targetArtifact.hasChildNodes()){
						this._targetArtifact.removeChild(this._targetArtifact.lastChild);
					}
					this._targetArtifact.appendChild(dojo.toDom('<span class="j-hint">' + hintText + '</span>'));
				}
					
			},
			
			onUpdate: function(query, scope, callback, errorCallback) {
				this.showSpinner();

				var self = this;
				var newCallback = function(results) {
					self.hideSpinner();
					if (self._cancelSearchResults == true)
						return;
					callback(results);
				};
				this.performUpdate(query, scope, newCallback, errorCallback);
			},

			getHandler: function(resourceUri, label) {
				var self = this;
				var labelDom = dojo.create('span', {
					innerHTML: label
				});
				var text = labelDom.textContent || labelDom.innerText;
				return function() {
					self.onSelect(resourceUri);
					self._searchText.value = text;
					dijit.focus(self.domNode.parentNode); //manually blur 'self' by focusing on something else
				};	
			},
			
			_closeLegacyResults: function() {
				this.inherited(arguments);
				this._lastQuery = null;
			},
			
			cancelSearchResults: function() {
				this._cancelSearchResults = true;
			},
			
			_onkeypress: function(e) {
				this._cancelSearchResults = false;
				return jazz.ui.SearchBox.prototype._onkeypress.apply(this, arguments);
			}
		});

})();
