/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.profile.AddFilterDialog");

dojo.require("jazz.ui.Dialog");
dojo.require("jazz.app.context");
dojo.require("jazz.ui.StyledBox");

dojo.require('dijit.layout.ContentPane');
dojo.require('dijit.form.FilteringSelect');
dojo.require('dojo.data.ItemFileReadStore');
dojo.require("dojox.widget.Standby");

dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.profile", "ImpactAnalysisProfileMessages");
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis.profile", "ImpactAnalysisProfileMessages");

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.profile.AddFilterDialog", null, {
	parent: null,
	iaProfile: null,
	projectId: null,

	filterSelected: null,
	filterStore: null,
	addSelectionButton: null,
	standby: null,
    
	dialog: null, //handle to the dialog
		
	constructor: function(parent, iaProfile) {
		this.parent = parent;
		this.iaProfile = iaProfile;
		this.projectId = jazz.app.context.get().itemId;
	},
		
	createDialog: function(){
		var createTable = dojo.doc.createElement("table");
		
		//A dojo data store for retrieved types
		this.filterStore = new dojo.data.ItemFileReadStore({clearOnClose: true});
		// provide some dummy data until we are ready to retrieve
		this.filterStore.data = {
			identifier: 'key',
			label: 'value',
			items: []
		};
			
		//Filtering Select:
		var filterSelectedId = dijit.getUniqueId("filterSelected");
		this.filterSelected = new dijit.form.FilteringSelect({
			id: filterSelectedId,
			store: this.filterStore,
			//name: 'key',
			searchAttr: 'value',
			style: 'width: 400px'
		});
		this.filterSelectedConnectHandle = dojo.connect(this.filterSelected, "onChange", dojo.hitch(this, this._monitorFilterSelected));
		this.createRelationRow(createTable, messages.iaEditorSelectionElmDialogFilter, this.filterSelected.domNode,null, filterSelectedId);

		this.standby = new dojox.widget.Standby({target: this.filterSelected.domNode});
		
		//Add button
		this.addSelectionButton = dojo.doc.createElement("button");
		this.addSelectionButton.innerHTML = messages.iaEditorSelectionElmDialogAdd;
		this.addConnectHandle = dojo.connect(this.addSelectionButton, "onclick", this, function() { this.add(); });
		this.message =  dojo.doc.createElement("span");
		this.createRelationRow(createTable, null, this.addSelectionButton, this.message);
		
		//Retrieve Filters
		this._getFiltersDeferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.getAllIAConfigs(this.projectId);
		this._getFiltersDeferred.addCallback(this, function(filters) {
			var noFilters = true;
			var configs = filters.ImpactConfigsList;
			var noData = configs.length === 0;
			var alreadySelected = this.iaProfile.filters;

			// Remove configs that are already in the Profile
			configs = dojo.filter(configs, dojo.hitch(this, function(config){
				return dojo.every(this.iaProfile.filters, function(existingFilter){
					return config.URI !== existingFilter.uri;
				});
			}));

			// Reload the filterStore data store
			this.filterStore.data = {
				identifier: 'key',
				label: 'value',
				items: dojo.map(configs, function (config) {
					return {
						key: config.URI,
						value: config.name
					};
				}, this)
			};
			this.filterStore.close();

			if (noData) { //disable other option
				this.filterSelected.disabled = true;
				this.message.innerHTML = messages.iaEditorSelectionElmDialogNoFilters;
			}
		
			this.inprogress(false);
		});
		this._getFiltersDeferred.addErrback(this, function(error) {
			var xhr = this._getFiltersDeferred.ioArgs.xhr;
			this.parent.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.errorGetDomains,
				errorObject: error,
				closable: true
			});

			this.dialog && this.dialog.close();
		});
		
		//Create Dialog:
		var dialogContent = new dijit.layout.ContentPane({
			id: 'addFilterDialogContents'
		});
		
		dialogContent.domNode.appendChild(createTable);
		dialogContent.domNode.appendChild(this.standby.domNode);
		
		this.dialog = new jazz.ui.Dialog({
			primaryTitle: messages.iaEditorSelectionElmDialogTitle,
			modal: true,
			trim: jazz.ui.StyledBox.TrimStyles.BLUE_ALPHA_SHADOW,
			center: true,
			closeOnEsc: true,
			headerless: false,
			closable: true,
			width: 500 + "px",
			moveable: true,
			collapsible: false,
			showBackground: false,
			contentNode: dialogContent.domNode,
			destroyContents: true,
			onClose: dojo.hitch(this, function()
			{
				dojo.disconnect(this.filterSelectedConnectHandle);
				dojo.disconnect(this.addConnectHandle);
				dojo.unsubscribe(this.refreshHandle);
				
				this.standby.destroyRecursive();
			})
		});
		
		// Only show standby progress after dialog is visible. Otherwise fails
		// in IE 8.
		this.inprogress(true);
		this.standby.startup();
	},
	
	_monitorFilterSelected: function() {
		if (this.filterSelected.getValue())
			this.addSelectionButton.disabled = false;
		else
			this.addSelectionButton.disabled = true;
	},
		
	inprogress: function(inProgress){
		if (inProgress === true) {
			this.addSelectionButton.disabled = true;
			this.standby.show();
		} else {
			this.standby.hide();
		}
	},
	
	createRelationRow: function(table, name, value, message, labelFor)
	{
		var tr = dojo.doc.createElement("tr");
		if (name != null)
		{
			var td1 = dojo.doc.createElement("td");
			if(labelFor){
				dojo.create('label',{innerHTML: name, 'for':labelFor},td1);
			}
			else{
				td1.innerHTML = name;
			}
			tr.appendChild(td1);
		}
		var td2 = dojo.doc.createElement("td");
		td2.appendChild(value);
		tr.appendChild(td2);
		
		if (message != null)
		{
			var td3 = dojo.doc.createElement("td");
			td3.appendChild(message);
			tr.appendChild(td3);
		}
		
		table.appendChild(tr);
	},

	add: function() {
		dojo.hitch(this.parent, this.parent.updateAfterAddFilter)({
			uri: this.filterSelected.item.key[0],
			name: this.filterSelected.item.value[0]
		});
		this.dialog.close();
    }
	
});

}());