/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.profile.AllIAProfilesEditor");

dojo.require("dijit._Widget");
dojo.require("dojo.string");
dojo.require("dojo.i18n");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.ui.editor.EditorUtil");
dojo.require("rmps.ui.editor.PageHelper");
dojo.require("rmps.ui.TextFilterInput");

dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysisProfile");
dojo.require('dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileInput');

dojo.require("jazz.util.html");
dojo.require("jazz.app.context");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.StyledTable");
dojo.require("rmps.ui.util.ToolbarFocusUtil");
var ToolbarFocusUtil = rmps.ui.util.ToolbarFocusUtil;

var UA_KEY = "com.ibm.xtools.impactanalysis.web.app.page_ImpactAnalysisConfigurations001";

var AnalysisUtil = dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil;

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.profile", "ImpactAnalysisProfileMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis.profile", "ImpactAnalysisProfileMessages"); 

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.profile.AllIAProfilesEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	isRefreshable: false,
	isReadOnly: true,	
	_diagramWidget: null,
	_content:null,
	editorInput:null,
	projectId:null,
	uaKey: UA_KEY,
	
	constructor: function (args) {
		this.editorInput= args.editorInput;
	},
	
	postCreate: function() {
		this.projectId = jazz.app.context.get().itemId;
		this.inherited(arguments);
		
	},
	
	createContentWidget: function() {		
		this._content = new dm.analysis.app.ui.internal.impactanalysis.profile.AllIAProfilesEditor._Content({editor: this});
		return this._content;
	},

	//Cloned from BaseEditor; prevents domNode from being adjusted
	resize: function(size){
		if (!size) {
			return;
		}

		this._lastSize = size;

		var cbContainer = dojo.contentBox(this.containerNode);
		var mbContainer = dojo.marginBox(this.containerNode);
		var mbHeader = dojo.marginBox(this.headerContainerNode);
		
		var contentSize = {};
		if (size.h) {
			contentSize.h = size.h - mbContainer.h + cbContainer.h - mbHeader.h;
			this.containerNode.style.minHeight = contentSize.h + "px";
		}
		if (size.w) {
			contentSize.w = size.w - mbContainer.w + cbContainer.w;
			this.containerNode.style.minWidth = contentSize.w + "px";
		}

		if (this.contentWidget && this.contentWidget.resize) {
			this.contentWidget.resize(contentSize);
		}
	},
	
	addTextActions: function(){
		var self= this;
			self.addTextAction(messages.newProfile, 
			dojo.moduleUrl("dm.analysis.app", "graphics/icons/ia_profile.gif"),
					dojo.hitch(self, self.onNewClick));
	},

	addButtons: function(toolbar) {
		toolbar.addButton(this, this.onRefreshClick, {
			label: messages.refresh,
			imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/refresh.gif")
		});

		this._deleteButton = toolbar.addButton(this, this.onMultiDeleteClick, {
			label: messages.deleteSelected,
			imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/remove.gif")
		});

		this._exportButton = toolbar.addButton(this, this.onExportClick, {
			label: messages.exportProfile,
			imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/download.gif")
		});

		toolbar.addSeparator(1); //adds separator in position 1
	},
	
	activate: function() {
		// until editing is supported, mark as dirty
	},
	
	onRefreshClick: function() {
		this._content.getAndPresentAllIAProfiles();
	},

	onMultiDeleteClick: function() {
		if (!window.confirm(messages.confirmDeleteSelected)) { return; }

		if (!this._content || !this._content.getCheckedItems) return;

		var selectedItems = this._content.getCheckedItems();
		if (!selectedItems.length || selectedItems.length === 0) return;

		var deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.deleteIAProfiles(selectedItems);
		deferred.addCallback(this, function(results) {
			results = dojo.fromJson(results);
			dojo.forEach(results.deleted, function(deletedProfile){
				dojo.forEach(selectedItems, function(iaProfile){
					if (deletedProfile === iaProfile.URI){
						var iapInput, pageHelper;
						// close the editor, if it's open
						iapInput = new dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileInput({resourceUri: deletedProfile});
						pageHelper = rmps.ui.editor.PageHelper.getInstance().closeEditorInput(iapInput, true, true);
						rmps.ui.editor.PageHelper.updateDirtyStatus(pageHelper.pageId);
						
						// destroy the toolbar so that the listeners in ToolbarFocusUtil are removed.
						if (iaProfile._actionToolbar) {
							iaProfile._actionToolbar.destroy();
							delete iaProfile._actionToolbar;
						}
					}
				});
			});

			var messageType = "";
			var messageSummary = "";

			if (results.connected && results.connected.length && results.connected.length > 0){
				messageType = jazz.ui.MessageBox.STATUS_TYPES.ERROR;
				messageSummary = messages.multiDelConnected;
			} else if (results.failed && results.failed.length && results.failed.length > 0){
				messageType = jazz.ui.MessageBox.STATUS_TYPES.ERROR;
				messageSummary = messages.multiDelFailed;
			} else {
				messageType = jazz.ui.MessageBox.STATUS_TYPES.OK;
				messageSummary = messages.multiDelSuccess;
			}

			this.showMessage({
				type: messageType,
				summary: messageSummary,
				closable: true
			});
			
			dojo.publish("dm.analysis.app.ui.internal.impactanalysis.profile.refresh");
			window.location.href = AnalysisUtil.getAllProfileAnalysisUri() + "&ts="+ new Date().getTime();
		});
		deferred.addErrback(this, function(error) {
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.multiDelFailed,
				errorObject: error,
				closable: true
			});
		});
	},

	onExportClick: function() {
		if (!window.confirm(messages.confirmExport)) { return; }

		if (!this._content || !this._content.getCheckedItems) return;

		var selectedItems = this._content.getCheckedItems();
		if (!selectedItems.length || selectedItems.length === 0) return;

		var downloadForm = dojo.create('form', { //$NON-NLS-1$
			action: net.jazz.ajax._contextRoot + '/impactprofile/content_package', //$NON-NLS-1$
			target: '_blank', //$NON-NLS-1$
			method: 'POST' //$NON-NLS-1$
		}, this.domNode);

		dojo.forEach(selectedItems, function(iaProfile){
			dojo.create('input', { //$NON-NLS-1$
				name: 'profileUri', //$NON-NLS-1$
				type: 'hidden', //$NON-NLS-1$
				value: iaProfile.URI,
			}, downloadForm);
		});

		downloadForm.submit();
		downloadForm.parentNode.removeChild(downloadForm);
	},
	
	onNewClick: function() {
		dojo.hash(AnalysisUtil.getNewAnalysisProfileUri());
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.profile.AllIAProfilesEditor._Content", [dijit._Widget, dijit._Templated], {

	messages: null,

	widgetsInTemplate: true,
	templateString: '<div>' +
			'<div dojoAttachPoint="mainDiv">' +
				'<table width= "100%" nowrap="nowrap">' +
					'<tr>' +
						'<td width= "33%"></td>' +
						'<td width= "33%"></td>' +
						'<td align= "right" width= "33%"><div dojoAttachPoint="_filterSearch"></div></td>' +
					'</tr>' +
				'</table>'+
			'</div>' +
		'</div>',

	editor: null,
	_table: null,
	
	postCreate: function() {
		this._table = new jazz.ui.StyledTable({ columns: [{label: messages.name, width: "20%", sortable: true}, //$NON-NLS-1$
														  {label: messages.description, width: "50%", sortable: false}, //$NON-NLS-1$
														  {label: messages.lastModified, width: "10%", sortable: true}, //$NON-NLS-1$
														  {label: messages.modifier, width: "10%", sortable: false}, //$NON-NLS-1$
														  {label: messages.actions, width: "10%", sortable: false}], //$NON-NLS-1$
												trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
												width: "100%",
												handleSort: dojo.hitch(this, this._handleSort),
												sortBy: 0,
												sortByAscending: true,
												showHover: true });

		this._textFilterInput = new rmps.ui.TextFilterInput({
			filterChanged: dojo.hitch(this, this._onFilterChanged),
		});
		dojo.style(this._textFilterInput.domNode, 'width', '150px'); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.style(this._textFilterInput.domNode, 'verticalAlign', 'bottom'); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.style(this._textFilterInput.domNode, 'paddingTop', '5px'); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.style(this._textFilterInput.domNode, 'paddingBottom', '5px'); //$NON-NLS-1$ //$NON-NLS-2$
		this._filterSearch.appendChild(this._textFilterInput.domNode);

		this.getAndPresentAllIAProfiles();
		this.subscribe("dm.analysis.app.ui.internal.impactanalysis.profile.refresh", dojo.hitch(this, this.refreshTable));
	},

	_onFilterChanged: function(filterText) {
		var updateRowDisplay = dojo.trim(filterText) !== "" ? function(row, fieldValue) {
				if (fieldValue && fieldValue.toLowerCase().indexOf(filterText.toLowerCase()) < 0) {
					row.style.display = "none";
				} else {
					row.style.display = "";
				}
			} : function(row, fieldValue) {
				row.style.display = "";
			};

		var tableRows = this._table.tbody.children;
		for (var index = 0; index < tableRows.length; index++) {
			var currentRow = tableRows[index];
			updateRowDisplay(currentRow, currentRow._name);
		}
	},

	_handleSort: function() {
		var sortBy = this._table.get('sortBy'); //$NON-NLS-1$
		var ascending = this._table.get('sortByAscending'); //$NON-NLS-1$

		this._emptyTable();

		//TODO: see QueryList for how to set up this sorting function
	},
	
	_emptyTable: function() {
		var widgets = dijit.findWidgets(this._table.domNode);
		dojo.forEach(widgets, function(widget) {
			widget.destroyRecursive();
		});
		this._table.clearBody();
	},
	
	refreshTable: function()
	{
		this.getAndPresentAllIAProfiles();
	},
	
	createTableHeader: function(id, args, label) {
		var header = {
				isOrderable: false
			};
		dojo.mixin(header, args);
		header.id = id;
		header.label = label;
		return header;
	},

	getCheckedItems: function(){
		var checkedItems = [];

		var tableRows = this._table.tbody.children;
		for (var i = 0; i < tableRows.length; i++)
		{
			var row = tableRows[i];
			if (this.isRowChecked(row)){
				checkedItems.push(row.iaProfile);
			}
		}

		return checkedItems;
	},

	numberOfCheckedItems: function(){
		var num = 0;

		var tableRows = this._table.tbody.children;
		for (var i = 0; i < tableRows.length; i++)
		{
			var row = tableRows[i];
			if (this.isRowChecked(row))
				num++;
		}
		return num;
	},

	isRowChecked: function(row){
		return row.children[0].children[0].checked; //crappy method
	},
	
	getAndPresentAllIAProfiles: function(){
		var deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.getAllIAProfiles(this.editor.projectId);
		deferred.addCallback(this, function(result) {
			this._emptyTable();
			dojo.forEach(result.ImpactProfilesList, function(profile){
				var resourceUri = profile.URI;
				var presentationUri = AnalysisUtil.getAnalysisProfileUri(profile.URI);
				
				this._table.tbody.appendChild(this._createRow(profile));


				/*var row = {
					checked: false,
					link: presentationUri,
					linkTarget: "_self",
					resourceUri: resourceUri,
					iaProfile: profile,
					columns: []
				};*/
			
			}, this);
			//var seconderyOrder = [];
			//seconderyOrder.push({id:"lastModified" , isAscending: false});
			//this._table.update(false, null, 50, result.ImpactProfilesList.length, 300, "name", true, seconderyOrder);
			this.mainDiv.appendChild(this._table.domNode);

			this.updateToolbarOnSelection();
		});
		deferred.addErrback(this, function(error) {
			var xhr = deferred.ioArgs.xhr;
			if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
				var lqeInfo = dojo.fromJson(xhr.responseText);
				this.editor.showMessage({
					type: lqeInfo.type,
					summary: lqeInfo.summary,
					details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
					closable: true
				});
			} else {
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.errorGetAllIAProfiles,
					errorObject: error,
					closable: true
				});
			}
		});
	},

	_createRow: function(profile){
		var row = document.createElement("tr"); //$NON-NLS-1$
		row._indent = 29;
		row._name = jazz.util.html.escape(profile.name);
		row._description = jazz.util.html.escape(profile.description);
		row._lastModified = dojo.date.locale.format(dojo.date.stamp.fromISOString(profile.lastModified), { formatLength: 'medium' }); //$NON-NLS-1$
		row._modifier = profile.modifier;
		row.id = profile.URI;
		row.iaProfile = profile;

		//NAME COLUMN

		var td = document.createElement("td"); //$NON-NLS-1$

		// Create the checkbox
		var cb = document.createElement("input"); //$NON-NLS-1$
		cb.style.marginRight = "10px"; //$NON-NLS-1$
		cb.style.marginTop = "4px"; //$NON-NLS-1$
		cb.type = "checkbox"; //$NON-NLS-1$
		cb.checked = false;
		cb.title = messages.toggleRowSelection;
		this.connect(cb, "ondijitclick", this._checkBoxOnClick); //$NON-NLS-1$

		td.appendChild(cb);
		this._formatAndInsertProfileName(td, profile);
		row.appendChild(td);

		//DESCRIPTION COLUMN

		td = document.createElement("td"); //$NON-NLS-1$
		td.innerHTML = row._description;
		row.appendChild(td);

		//LAST MODIFIED COLUMN

		td = document.createElement("td"); //$NON-NLS-1$
		td.innerHTML = row._lastModified;
		dojo.style(td, 'color', '#808080'); //$NON-NLS-1$ //$NON-NLS-2$
		row.appendChild(td);

		//MODIFIER COLUMN

		td = document.createElement("td"); //$NON-NLS-1$
		td.innerHTML = row._modifier;
		row.appendChild(td);

		//ACTIONS COLUMN

		td = document.createElement("td"); //$NON-NLS-1$
		this.createRowActions(profile, row, td); //inserts actions available to this profile into the cell
		row.appendChild(td);


		return row;
	},

	_checkBoxOnClick: function(ev){
		this.updateToolbarOnSelection();
	},

	updateToolbarOnSelection: function(){
		if (this.numberOfCheckedItems() > 0){
			this.editor.toolbar.enableButton(this.editor._exportButton);
			this.editor.toolbar.enableButton(this.editor._deleteButton);
		} else {
			this.editor.toolbar.disableButton(this.editor._exportButton);
			this.editor.toolbar.disableButton(this.editor._deleteButton);
		}
	},

	_formatAndInsertProfileName: function(td, profile){
		var link = dojo.create("a", {href: "#", target: "", innerHTML: profile.name, title: profile.name}, td); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

		link.profile = profile;
		link.uri = profile.URI;
		link.name = profile.name;

		var resourceLink = new jazz.ui.ResourceLink({ 
			uri: profile.URI, 
			lazyFetch: false, 
			retainLinkText: false,
			target: "", //$NON-NLS-1$
			useAbbreviation: false }, link);

		var self = this;
		this.connect(link, "onclick", function(evt){ //$NON-NLS-1$
			self._editIAProfile(evt.target.profile); //evt.target is 'link'
			dojo.stopEvent(evt);
		});

	},
	
	createRowActions: function(iaProfile, row, td) {
	    // Create a toolbar to be the content of the table cell
		iaProfile._actionToolbar = new jazz.ui.SimpleToolbar();  // ToolbarFocusUtil requires a toolbar
		dojo.style(iaProfile._actionToolbar.domNode, 'minWidth','8em');
		dojo.addClass(td, "actions");
		td.appendChild(iaProfile._actionToolbar.domNode);
		
		// Add the "edit" action
		var button;
		button = iaProfile._actionToolbar.addButton(this, dojo.hitch(this, this._editIAProfile, iaProfile), 
					{imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/edit.gif"), label: messages.edit});
		dojo.style(button.domNode, "visibility", "hidden");
		
		// For system profiles, add the "reset" action and 
		// for user profiles, add a "delete" action.
		// systemDefined sometimes returns as "false" which is a truthy value
		var buttonLabel = (iaProfile.systemDefined ? messages.reset : messages.deleteProfile);
		var buttonImage = (iaProfile.systemDefined ? "graphics/icons/restore.gif" : "graphics/icons/remove.gif");
		button = iaProfile._actionToolbar.addButton(this, dojo.hitch(this, this._deleteIAProfile, iaProfile), 
					{imgSrc: dojo.moduleUrl("dm.analysis.app", buttonImage), label: buttonLabel});
		dojo.style(button.domNode, "visibility", "hidden");
		
		// Add listeners to show and hide the toolbar actions if necessary.
		iaProfile._actionToolbar.startup();
		ToolbarFocusUtil.addToolbarFocusListeners( { 
			actionToolbar: iaProfile._actionToolbar, 
			containerNode: row, 
			actionClassName: ".jazz-ui-SimpleToolbar-button"
		});
	},
	
	_editIAProfile: function(iaProfile) {		
		var editActionUri = AnalysisUtil.getAnalysisProfileUri(iaProfile.URI);
		window.location.href = editActionUri;
	},
	
	_deleteIAProfile: function(iaProfile) {
		var substitutions = [jazz.util.html.escape(iaProfile.name)];
		var delConfirm = dojo.string.substitute((iaProfile.systemDefined ? messages.confirmReset : messages.confirmDelete), 
			substitutions);
		var delError = dojo.string.substitute((iaProfile.systemDefined ? messages.cantReset : messages.cantDelete), 
			substitutions);
		var delSuccess = dojo.string.substitute((iaProfile.systemDefined ? messages.successReset : messages.successDelete), 
			substitutions);
		if (! window.confirm (delConfirm))
		{
			return;
		}
		
		
		// The IA Profile summary has a "URI" property but the delete looks for a "uri" property.
		iaProfile.uri = iaProfile.URI;
		var deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.deleteIAProfile(iaProfile, false);
		deferred.addCallback(this, function(ia) {
			var iapInput,
				pageHelper;
			// close the editor, if it's open
			iapInput = new dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileInput({resourceUri: iaProfile.uri});
			pageHelper = rmps.ui.editor.PageHelper.getInstance().closeEditorInput(iapInput, true, true);
			rmps.ui.editor.PageHelper.updateDirtyStatus(pageHelper.pageId);
			
			// destroy the toolbar so that the listeners in ToolbarFocusUtil are removed.
			if (iaProfile._actionToolbar) {
				iaProfile._actionToolbar.destroy();
				delete iaProfile._actionToolbar;
			}
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.OK,
				summary: delSuccess,
				closable: true
			});
			
			dojo.publish("dm.analysis.app.ui.internal.impactanalysis.profile.refresh");
			window.location.href = AnalysisUtil.getAllProfileAnalysisUri() + "&ts="+ new Date().getTime();
			
			
		});
		deferred.addErrback(this, function(error) {
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: delError,
				errorObject: error,
				closable: true
			});
		});
	}
	
	
});
}());
