/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor");

dojo.require("dojo.string");
dojo.require("dojo.i18n");

dojo.require("dijit._Widget");
dojo.require("dijit.form.SimpleTextarea");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.MultiSelect");

dojo.require("jazz.ui.Breadcrumbs");
dojo.require('jazz.app.ua.TextHoverHelp');

dojo.require("rmps.core.error");
dojo.require("rmps.ui.CollapsibleSection");
dojo.require("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.UnlimitedNumberSpinner");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.profile.AddFilterDialog");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.profile.TraversalDefinition");

dojo.require("dm.common.app.ui.internal.TabManager");
dojo.require("dm.common.app.ui.internal.rhlink.AnalysisConfigClient");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.EditorUtil");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.profile", "ImpactAnalysisProfileMessages");
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis.profile", "ImpactAnalysisProfileMessages");

var TabManager = dm.common.app.ui.internal.TabManager;
var AnalysisUtil = dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil;

var PAGE_ID = dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor.PAGE_ID;
var DIRTY_GROUP_ID = "dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor.dirty";

var UA_KEY = "com.ibm.xtools.impactanalysis.web.app.page_CreateIAConfig001";

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	isRefreshable: false,
	isReadOnly: false,	

	editorInput:null,
	content:null,
	//data
	_general:null,
	_tools:null,
	_selections:null,
	_advance:null,
	_breadCrumbs:null,
	uaKey: UA_KEY,
	dirtyTitle:null,
	_allTools:null,
	
	constructor: function (args) {
		this.editorInput= args.editorInput;
		
		if (!this.editorInput.getName())
			this.editorInput.setName(messages.newProfileTitle);
	},

	postCreate: function() {
		this._allTools = {};
		this.dirtyTitle = {};
		
		this._breadCrumbs= new jazz.ui.Breadcrumbs({segments: [{label: messages.iaProfileTitle, href: AnalysisUtil.getAllProfileAnalysisUri()}]});
		dojo.place(this._breadCrumbs.domNode, this.headerContainerNode, "first");
		
		this.inherited(arguments);
		

	},

	activate: function() {
		// until editing is supported, mark as dirty
		//this.addDirty(this);
	},
	
	deactivate: function()
	{
	},
	
	startup: function() {
		this.content.startup();
	},
	
	isEditableTitle: function(){
		return !this.isReadOnly;
	},
	
	onTitleChange: function () {
		var oldValue = this.editorInput.iaProfile.shortName;
		this.editorInput.iaProfile.shortName = this.getTitleValue();
		this._updateDirty(oldValue, 
					this.getTitleValue(), 
					this.dirtyTitle);
	},
	
	createContentWidget: function() {		
		this.content = new dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor._Content({editor: this, isReadOnly: this.isReadOnly});
		return this.content;
	},
	
	addButtons: function(toolbar) {	
		toolbar.addWidget(new rmps.ui.SimpleButton({
			label: messages.cancel,
			onClick: dojo.hitch(this, this.doCancel)
		}));
		
		this.inherited(arguments);
	},
	
	enable: function()
	{
		//enable the save button if its supposed to be
		var enableSaveButton = false;
		if (!this.isReadOnly && this.isDirty())
			enableSaveButton = true;
		this.saveButton.setEnabled(enableSaveButton);
	},
	disable: function()
	{
		//this.saveButton.setEnabled(false); save button already disabled by default in BaseEditor.save()
	},
	
	getEditableTitleNodeDetails: function () {
		// summary:
		//		Callback from BaseEditor providing default title for a new impact analysis
		return messages.newProfileTitle;
	},
	
	validate: function (/*Array*/errors) {
		// summary:
		//		Callback from BaseEditor to validate data prior to save

		var detailedError;

		var traversalDef = this.content.getTraversalDef();

		if (traversalDef.traversal === traversalDef.BOTH || traversalDef.traversal === traversalDef.UPSTREAM){
			if(traversalDef.upstreamType === '' && !traversalDef.upstreamDepth) {
				detailedError = new Error(messages.errorMissingUpstreamDepth);
				//detailedError.explanation = messages['errorMissingUpstreamDepth.explanation'];
				//detailedError.userAction = messages['errorMissingUpstreamDepth.useraction'];
				errors.push(detailedError);
				this.saveButton.setEnabled(this.isDirty());
			}
		}

		if (traversalDef.traversal === traversalDef.BOTH || traversalDef.traversal === traversalDef.DOWNSTREAM){
			if(traversalDef.downstreamType === '' && !traversalDef.downstreamDepth) {
				detailedError = new Error(messages.errorMissingDownstreamDepth);
				//detailedError.explanation = messages['errorMissingDownstreamDepth.explanation'];
				//detailedError.userAction = messages['errorMissingDownstreamDepth.useraction'];
				errors.push(detailedError);
				this.saveButton.setEnabled(this.isDirty());
			}
		}
		
		if (traversalDef.traversal === traversalDef.FULL_TRACE){
			if(traversalDef.fullTraceType === '' && !traversalDef.fullTraceDepth) {
				detailedError = new Error(messages.errorMissingFullTraceDepth);
				//detailedError.explanation = messages['errorMissingFullTraceDepth.explanation'];
				//detailedError.userAction = messages['errorMissingFullTraceDepth.useraction'];
				errors.push(detailedError);
				this.saveButton.setEnabled(this.isDirty());
			}
		}

		if(this.editorInput.iaProfile.shortName === '') {
			detailedError = new Error(messages.errorMissingTitle);
			detailedError.explanation = messages['errorMissingTitle.explanation'];
			detailedError.userAction = messages['errorMissingTitle.useraction'];
			errors.push(detailedError);
			this.saveButton.setEnabled(this.isDirty());
			
			dm.analysis.app.ui.internal.impactanalysis.util.EditorUtil.animateMissingEditorTitle(this._editableTitle.domNode);
			this._editableTitle.edit();
		}
	},

	/** When Cancel is clicked in the analysis profile editor, go
	*  back to the page for viewing all profiles.
	*/
	doCancel: function() {
		var pageHelper = rmps.ui.editor.PageHelper.getInstance(PAGE_ID);
		pageHelper.closeEditor(this, true, true);
		window.location.href = AnalysisUtil.getAllProfileAnalysisUri() + "&ts="+ new Date().getTime();
	},
	
	doSave: function(taskMonitor) {
		this.disable();
		var iap = this.editorInput.iaProfile;
		//general
		iap.shortName  = this.getTitleValue();
		iap.description  = this._general.descriptionTextArea.get('value');

		var traversalDef = this.content.getTraversalDef();
		iap.traversal = traversalDef.traversal;
		iap.upstreamDepth = traversalDef.upstreamDepth;
		iap.downstreamDepth = traversalDef.downstreamDepth;
		iap.fullTraceDepth = traversalDef.fullTraceDepth;
		iap.upstreamType = traversalDef.upstreamType;
		iap.downstreamType = traversalDef.downstreamType;
		iap.fullTraceType = traversalDef.fullTraceType;
		
		//iap.filters - filters were already put directly into iaProfile when they were added

		var deferred, isNew = false;
		if (iap.eTag) {
			deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.updateIAProfile(iap);
		} else {
			deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.createIAProfile(iap);
			isNew = true;
		}
		
		deferred.addCallback(this, function() {
			//when done , enable all button
			if (isNew) {
				// refresh the editor it with the now 'saved' editor input.
				this.editorInput.resourceUri = iap.uri;
				this.editorInput.removeState('id');
				
				// clear dirty flag first before reload
				this._clearDirty();
				this.refresh();
			} else {
				this.enable();
			}
			dojo.publish("dm.analysis.app.ui.internal.impactanalysis.profile.refresh");
		});	
		
		deferred.addErrback(this, function(error) {
			this.enable();
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.errorSaving,
				errorObject: error,
				closable: true
			});
		});
		
		var task = taskMonitor.beginTask("saving", 1);
		task.handleDeferredWork(deferred, 1);
	},
	
	getResourceInfo: function(){
		return this.editorInput.resourceInfo;	
	},
	
	getProjectID: function(){
		return this.editorInput.projectId;	
	},
	
	_updateDirty: function(originalValue, newValue, dirtyKey) {
		var isEqual = originalValue === newValue;
		if (!isEqual && dojo.isArray(newValue) && originalValue && newValue && originalValue.length === newValue.length) {
			isEqual = true; // empty arrays
			isEqual = dojo.every(newValue, function(item, idx) {
				return item === originalValue[idx];
			});
		}
		if (isEqual) {
			this.removeDirty(dirtyKey, DIRTY_GROUP_ID);
		} else {
			this.addDirty(dirtyKey, DIRTY_GROUP_ID);
		}
	},

	uninitialize: function(){
		this.inherited(arguments);
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor._Content", [dijit._Widget, dijit._Templated], {
	messages: messages,
	widgetsInTemplate: true,
	templateString: '<div class="profile-editor">' +
		'<div dojoAttachPoint="_generalSection"></div>' +
		'<div dojoAttachPoint="_traversalSection"></div>' +
		'<div dojoAttachPoint="_filterSection"></div>' +
	'</div>',
	
	// init properties
	editor: null,
	isReadOnly: false,
	
	postCreate: function() {
		//get all tools map to be uses to present the tools name:
		this._getDomainsDeferred = dm.common.app.ui.internal.rhlink.AnalysisConfigClient.getToolsTypes(this.editor.getProjectID());
		this._getDomainsDeferred.addCallback(this, function(result) {
			dojo.forEach(result.tools, function (tool) {
				this.editor._allTools[tool.ToolURI] = tool.ToolType;
			}, this);

			var sectionArgs = { editor: this.editor, isReadOnly: this.isReadOnly };

			this.editor._general = new dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor_General(sectionArgs, this._generalSection);
			this.editor._traversal = new dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor_Traversal(sectionArgs, this._traversalSection);
			this.editor._filters = new dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor_Filters(sectionArgs, this._filterSection);
		});
		this._getDomainsDeferred.addErrback(this, function(error) {
			var xhr = this._getDomainsDeferred.ioArgs.xhr;
			if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
				var lqeInfo = dojo.fromJson(xhr.responseText);
				this.editor.showMessage({
					type: lqeInfo.type,
					summary: lqeInfo.summary,
					details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
					closable: true
				});
			} else {
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.errorGetDomains,
					errorObject: error,
					closable: true
				});
			}

		});
		
	},

	destroy: function(){
		this.editor._general.destroyRecursive();
		this.editor._traversal.destroyRecursive();
		this.editor._filters.destroyRecursive();

		this.inherited(arguments);
	},

	getTraversalDef: function(){
		return this.editor._traversal.getTraversalDef();
	},

	startup: function(){
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor_General", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	messages: messages, 
	templateString: '<div>' +
		'<div dojoType="rmps.ui.CollapsibleSection" primaryTitle="${messages.sectionTitle_general}" readOnly="false" collapsible="true" isCollapsed="false">'+
			'<div class="IAProfileEditor">' +
				'<table style="overflow: scroll; width:100%; margin: 5px 0;" class="impact-def-table general" >' +
					'<tr> ' +
						'<td style="padding-right:10px; width:15%" dojoAttachPoint="_generalTitleCell"></td>' +
					'</tr>' +
					'<tr> ' +
						'<td style="padding-right:10px; width:15%" >${messages.profileDescription}</td>' +
						'<td><textarea  class="instructions" dojoAttachPoint="descriptionTextArea"'+
								'dojoType="dijit.form.SimpleTextarea"'+
								'intermediateChanges="true"'+
								'cols="63"' + 
								'title="${messages.profileDescriptionHover}" >' +
							'</textarea></td>'+
					'</tr>' +
				'</table>' +
			'</div>'+
		'</div>'+
	'</div>',

	// init properties
	editor: null,
	isReadOnly: false,
	iaProfile: null,

	//connect handles
	_descChangeHandle: null,

	postCreate: function() {
		this.iaProfile = this.editor.editorInput.iaProfile;
		
		this.inherited(arguments);
		
		if (this.iaProfile.systemDefined) {
			this._generalTitleCell.appendChild(dojo.doc.createTextNode(messages.profileGeneralTitleSystemDefined));
		}
		
		this.descriptionTextArea.set('value',this.iaProfile.description);
		this.descriptionTextArea.set('disabled', this.isReadOnly);
		
		if (!this.isReadOnly) {
			this._descChangeHandle = dojo.connect(this.descriptionTextArea,"onChange", dojo.hitch(this, function() {
				this.editor._updateDirty(this.iaProfile.description, 
								this.descriptionTextArea.get('value'), 
								this.descriptionTextArea);
				}));
		}
		
	},

	startup: function() {
		
	},

	destroy: function() {
		dojo.disconnect(this._descChangeHandle);

		this.inherited(arguments);
	}
});


dojo.declare("dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor_Traversal", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	messages: messages, 
	templateString: '<div>' +
		'<div dojoType="rmps.ui.CollapsibleSection" primaryTitle="${messages.sectionTitle_traversal}" readOnly="false" collapsible="true" isCollapsed="false">'+
			'<div dojoAttachPoint="_traversalDef"/>'+
		'</div>'+
	'</div>',

	// init properties
	editor: null,
	isReadOnly: false,
	iaProfile: null,

	traversalDefinition: null,

	postCreate: function() {
		this.iaProfile = this.editor.editorInput.iaProfile;
		
		this.inherited(arguments);
		
		this.traversalDefinition = new dm.analysis.app.ui.internal.impactanalysis.profile.TraversalDefinition({
			editor: this.editor,
			iaProfile: this.iaProfile,
			isReadOnly: this.isReadOnly
		});

		this._traversalDef.appendChild(this.traversalDefinition.domNode);
	},

	getTraversalDef: function(){
		return this.traversalDefinition.getTraversalDefinition();
	},

	destroy: function() {
		if (this.traversalDefinition)
			this.traversalDefinition.destroyRecursive();

		this.inherited(arguments);
	}
});


dojo.declare("dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor_Filters", [dijit._Widget, dijit._Templated], {

	widgetsInTemplate: true,
	messages: messages,
	templateString: '<div>' +
		'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_filterCollapseSection" primaryTitle="${messages.sectionTitle_filters}" readOnly="false" collapsible="false">'+
			'<div dojoAttachPoint="filtersTableDiv"/>' +
		'</div>'+
	'</div>',

	
	// init properties
	editor: null,
	isReadOnly: false,
	iaProfile: null,
	
	postCreate: function() {
		this.iaProfile = this.editor.editorInput.iaProfile;
		this.inherited(arguments);

		if (!this.isReadOnly) {
			// put the button in the collapsible section...
			this._filterCollapseSection.addImageAction(
				dojo.moduleUrl("dm.common.app", "graphics/icons/add.gif"),
				messages.iaEditorFilterAddToolTip,
				dojo.hitch(this, this._addFilter)
			);
		}
		
		this.filtersTable = new jazz.ui.StyledTable({
			columns: [
				{ label: messages.iaEditorFilterNames },
				{ label: messages.actions, className: "actions", width: "25%" }
			]
		});
		this.filtersTable.containerNode = this.filtersTable.tbody;
		dojo.style(this.filtersTable.domNode, 'width', '100%');
		this.filtersTableDiv.appendChild(this.filtersTable.domNode);
		this.filtersTable.startup();

		this.loadFiltersFromProfile(this.iaProfile);
	},

	startup: function() {
		
	},

	_addFilter: function(){
		new dm.analysis.app.ui.internal.impactanalysis.profile.AddFilterDialog(this, this.iaProfile).createDialog();
	},

	loadFiltersFromProfile: function(iaProfile){
		if (iaProfile){
			this.iaProfile = iaProfile;
			dojo.forEach(iaProfile.filters, function(filter) {
				this._addFilterRow({
					name: filter.name,
					uri: filter.uri
				});
			}, this);
		}
	},

	updateAfterAddFilter: function(filter) {
		this.iaProfile.filters.push(filter);
		
		this._addFilterRow(filter);
		this.editor.addDirty(this.editor._filters, DIRTY_GROUP_ID);
	},
	
	_addFilterRow: function(filterValue) {
		var row = new dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor_FilterRow({
			selectionsSection: this,
			filterValue: filterValue,
			isReadOnly: this.isReadOnly
		});

		this.filtersTable.tbody.insertBefore(row.domNode, this.filtersTable.tbody.firstChild);

		row.startup();
	},

	destroy: function() {
		this.inherited(arguments);
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileEditor_FilterRow", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	messages: messages, 
	templateString: '<tr>'+
			'<td>' +
				'<div dojoAttachPoint="filterTypeUriDiv">' +
				'</div>' +
			'</td>' +
			'<td class="actions" style="width: 25%;">' +
				'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="toolbar"></div>' +
			'</td>' +
		'</tr>',
	
	selectionsSection: null,
	filterValue: null,
	isReadOnly: false,

	_checkboxHandle: null,
	
	postCreate: function() {
		this.filterTypeUriDiv.appendChild(dojo.doc.createTextNode(this.filterValue.name));

		if (!this.isReadOnly) {

			/* TODO: make this.modify() actually do something useful before putting the Edit button back in
			this.toolbar.addButton(this, function(e) {
				dojo.stopEvent(e);
				this.modify();
			},{
				label: "Modify Filter",
				imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/edit.gif")
			});
			*/
			
			this.toolbar.addButton(this, function(e) {
				dojo.stopEvent(e);
				this.remove();
				this.destroyRecursive();
			},{
				label: "Remove Filter",
				imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/remove_grey.gif")
			});
		}
		
	},

	modify: function() {
		var uri = filterValue.uri;

		//TODO: open new dialog with uri OR redirect to Filter editor page for this uri
	},
	
	remove: function() {

		var table = this.selectionsSection.filtersTable;
		var collection = this.selectionsSection.iaProfile.filters;

		// remove entry from table
		dojo.some(table.tbody.childNodes, function (node) {
			var found = false;
			if (node.id == this.domNode.id) {
				table.tbody.removeChild(node);
				found = true;
			}
			return found;
		}, this);
		
		// remove entry from IA profile collection
		dojo.some(collection, function (item, i) {
			var found = false;
			if(item.uri == this.filterValue.uri) {
				collection.splice(i, 1);
				found = true;
			}
			return found;
		}, this);

		this.selectionsSection.editor.addDirty(this.selectionsSection, DIRTY_GROUP_ID);

		//destroy:
		if (this._checkboxHandle) dojo.disconnect(this._checkboxHandle);
	}
});

}());
