/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileInput");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");
var AnalysisUtil = dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil;

dojo.require("rmps.ui.editor.EditorInput");
dojo.require("dojox.encoding.digests.MD5");
dojo.require("dojo.i18n");

var runningCount = 1;

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.profile.IAProfileInput", rmps.ui.editor.EditorInput, {
	
	projectId: null,
	resourceUri: null,
	iaProfile: null,
	_name: null,
	_id: null,
	resourceInfo: null,
	
	constructor: function(params) {
		if (params.id)  {
			this.addState('id', params.id);
			delete params.id;
		}
		
		// integrated the remaining parameters into the EditorInput.
		dojo.mixin(this, params);
		
		this.resourceInfo = {};
	},
	
	isLoaded: function() {
		var loaded;
		if (this.iaProfile) {
			loaded = true;
		} else if (!this.resourceUri) {
			this._loadImpactProfile(new dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysisProfile());
			this.addState('id', this.getId());
			loaded = true;
		} else {
			loaded = false;
		}
		return loaded;
	},
	
	load: function() {
		var deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.getIAProfile(this.resourceUri);
		deferred.addCallback(this, dojo.hitch(this, this._loadImpactProfile));
		return deferred;
	},
	
	_loadImpactProfile: function(iap)  {
		this.iaProfile = iap;
		if (iap.getName() != ""){
			this._name = iap.getName();
			//if there is a target than load it as well
			this.resourceInfo.uri = iap.uri;
			this.resourceInfo.shortName = this._name;
		}
	},
	
	_loadTargetResource: function (infos)
	{
		this.targetElement = infos[0];
	},
	
	
	getId: function() {
		if (this._id === null ) {
			if(dojo.isString(this.resourceUri) && this.resourceUri.length > 0) {
				this._id = this.resourceUri;
			} else if (this.getState('id')) {
				this._id = this.getState('id');
			} else {
				// make a fake, unique ID to distinguish from other new inputs
				this._id = dojox.encoding.digests.MD5(this.declaredClass + ',' + runningCount);
				runningCount += 1;
			}
		}
		return this._id;
	},
	
	getIconUri: function() {
		return dojo.moduleUrl("dm.analysis.app", "graphics/icons/ia_profile.gif");
	},
	
	getName: function() { 
		
		return this._name;
	},
	
	setName: function (name) {
		this._name = name;
	},
	
	getTooltip: function() {
		return this.getName() || '';
	},
	
	getBasePresentationUri: function() {
		return AnalysisUtil.getAnalysisProfileUri(this.resourceUri);
	}
	
});

}());
