/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.profile.TraversalDefinition");

dojo.require("dijit.form.NumberTextBox");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.config.AddTypeDialog");
dojo.require('dm.analysis.app.ui.DiagramUtilityMethods'); //$NON-NLS-1$

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.profile", "ImpactAnalysisProfileMessages");
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis.profile", "ImpactAnalysisProfileMessages");

var DiagramUtilityMethods = dm.analysis.app.ui.DiagramUtilityMethods;

var DIRTY_GROUP_ID = "dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisEditor.dirty";
var TYPE_ELEMENT = "ELEMENT";

var UA_KEY_IATRAVERALLABEL = "com.ibm.xtools.impactanalysis.web.app.text_Traversal001";

var DEFAULT_DEPTH = 2;

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.profile.TraversalDefinition", [dijit._Widget, dijit._Templated], {
	messages: messages,
	widgetsInTemplate: true,
	templateString: '<div class="traversal-def">'+
						'<table>'+
							'<tr>'+
								'<td>'+
									'<span class="traversal-label" dojoAttachPoint="_traversalLabel">${messages.traversalLabel}</span>'+
								'</td>' +
								'<td>'+
									'<select dojoAttachPoint="_selectedTraversal" style="width:100%" title="${messages.traversalSelectionHover}"/>'+
								'</td>' +
							'</tr>'+
							'<tr class="upstream-row">'+
								'<td>'+
									'<span class="upstream-label">${messages.traversalUpstreamLabel}</span>'+
								'</td>' +
								'<td>'+
									'<div dojoAttachPoint="_upstreamTypeDiv"></div>'+
									'<div style="padding-right:2px">'+
										'<input dojoAttachPoint="_upstreamDepth" dojoType="dijit.form.NumberTextBox" intermediateChanges="true" maxlength="2"' +
											'class="number-text-box" constraints="{pattern: \'#\', min: 1, max:99}" title="${messages.traversalUpstreamHover}" value="2"/>'+
									'</div>'+
								'</td>' +
								'<td style="width:40px">'+
									'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="_upstreamActionToolbar"></div>'+
								'</td>' +
							'</tr>'+
							'<tr class="downstream-row">'+
								'<td>'+
									'<span class="downstream-label">${messages.traversalDownstreamLabel}</span>'+
								'</td>' +
								'<td>'+
									'<div dojoAttachPoint="_downstreamTypeDiv"></div>'+
									'<div style="padding-right:2px">'+
										'<input dojoAttachPoint="_downstreamDepth" dojoType="dijit.form.NumberTextBox" intermediateChanges="true" maxlength="2"' +
											'class="number-text-box" constraints="{pattern: \'#\', min: 1, max:99}" title="${messages.traversalDownstreamHover}" value="2"/>'+
									'</div>'+
								'</td>' +
								'<td style="width:40px">'+
									'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="_downstreamActionToolbar"></div>'+
								'</td>' +
							'</tr>'+
							'<tr class="fullTrace-row hidden">'+
							'<td>'+
								'<span class="fullTrace-label">${messages.traversalFullTraceLabel}</span>'+
							'</td>' +
							'<td>'+
								'<div dojoAttachPoint="_fullTraceTypeDiv"></div>'+
								'<div style="padding-right:2px">'+
									'<input dojoAttachPoint="_fullTraceDepth" dojoType="dijit.form.NumberTextBox" intermediateChanges="true" maxlength="2"' +
										'class="number-text-box" constraints="{pattern: \'#\', min: 1, max:99}" title="${messages.traversalFullTraceHover}" value="2"/>'+
								'</div>'+
							'</td>' +
							'<td style="width:40px">'+
								'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="_fullTraceActionToolbar"></div>'+
							'</td>' +
						'</tr>'+
						'</table>'+
					'</div>',
	
	// init properties
	profileInfoChangeCallback: null,
	editor: null,
	iaProfile: null,
	isReadOnly: false,

	// Traversal Definition Properties:
	traversal: "",

	upstreamLink: null,
	downstreamLink: null,
	fullTraceLink: null,
	
	//Event handles:
	_traversalChangeHandle: null,
	_upstreamChangeHandle: null,
	_downstreamChangeHandle: null,
	_fullTraceChangeHandle: null,

	_upstreamLinkRemoveHandler: null,
	_downstreamLinkRemoveHandler: null,
	_fullTraceLinkRemoveHandler: null,

	UPSTREAM: "Upstream",
	DOWNSTREAM: "Downstream",
	BOTH: "Both",
	FULL_TRACE: "Full_Trace",
	TRAVERSAL_OPTIONS: [],

	constructor: function(){
		this.inherited(arguments);

		this.TRAVERSAL_OPTIONS = [ 
		                          { label: this.UPSTREAM, value: this.UPSTREAM}, 
		                          { label: this.DOWNSTREAM, value: this.DOWNSTREAM},
		                          { label: this.BOTH, value: this.BOTH}];
		if (DiagramUtilityMethods.isFullTraceEnabled())
		{
			this.TRAVERSAL_OPTIONS.push({ label: "Full Traceability", value: this.FULL_TRACE});
		}
	},

	postCreate: function(){
		this.inherited(arguments);

		var traversalToChangeTo = this.BOTH;
		var profileAndTraversalLoaded = this.iaProfile && this.iaProfile.traversal && dojo.isString(this.iaProfile.traversal) && this.iaProfile.traversal !== '';
		if (profileAndTraversalLoaded){
			this.loadTraversalDefinitionFromProfile(this.iaProfile);
			traversalToChangeTo = this.iaProfile.traversal;
			this.traversalChange(traversalToChangeTo);
		}

		dojo.forEach(this.TRAVERSAL_OPTIONS, dojo.hitch(this, function(traversalOption){
			option = dojo.doc.createElement("option");
			option.innerHTML = jazz.util.html.escape(traversalOption.label);
			option.value = traversalOption.value;

			if (traversalOption.value === traversalToChangeTo){
				option.selected = true;
			}

			this._selectedTraversal.appendChild(option);
		}));

		// handle dirty awareness
		this._traversalChangeHandle = dojo.connect(this._selectedTraversal, "onchange", dojo.hitch(this, this._traversalChange));

		this._upstreamChangeHandle = dojo.connect(this._upstreamDepth,"onChange", dojo.hitch(this, this._upstreamDepthChanged));
		this._downstreamChangeHandle = dojo.connect(this._downstreamDepth,"onChange", dojo.hitch(this, this._downstreamDepthChanged));
		this._fullTraceChangeHandle = dojo.connect(this._fullTraceDepth,"onChange", dojo.hitch(this, this._fullTraceDepthChanged));

		// Add upstream/downstream type buttons
		this._addBoundaryTypeButtons();

		//Add CSH to Traversal
		this._traversalHoverHelp = new jazz.app.ua.TextHoverHelp({
			uaKey: UA_KEY_IATRAVERALLABEL,
			contentNode: this._traversalLabel
		});
	},

	startup: function() {
		
	},

	_upstreamDepthChanged: function(newValue){
		if (this.iaProfile) this.editor._updateDirty(parseInt(this.iaProfile.upstreamDepth, 10), newValue, this._upstreamDepth);
		if (this.profileInfoChangeCallback) this.profileInfoChangeCallback();
	},

	_downstreamDepthChanged: function(newValue){
		if (this.iaProfile) this.editor._updateDirty(parseInt(this.iaProfile.downstreamDepth, 10), newValue, this._downstreamDepth);
		if (this.profileInfoChangeCallback) this.profileInfoChangeCallback();
	},
	
	_fullTraceDepthChanged: function(newValue){
		if (this.iaProfile) this.editor._updateDirty(parseInt(this.iaProfile.fullTraceDepth, 10), newValue, this._fullTraceDepth);
		if (this.profileInfoChangeCallback) this.profileInfoChangeCallback();
	},

	_addBoundaryTypeButtons: function()
	{
		this._upstreamActionToolbar.addButton(this, dojo.hitch(this, function(){
			new dm.analysis.app.ui.internal.impactanalysis.config.AddTypeDialog(this, TYPE_ELEMENT, 
																				dojo.hitch(this, this.addUpstreamTypeCallback), 
																				[], [], 
																				messages.traversalSetTypeBoundaryDialogTitle).createDialog();
		}), {
			label: messages.traversalSetTypeBoundary,
			imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/search-icon.gif')
		});

		this._downstreamActionToolbar.addButton(this, dojo.hitch(this, function(){
			new dm.analysis.app.ui.internal.impactanalysis.config.AddTypeDialog(this, TYPE_ELEMENT, 
																				dojo.hitch(this, this.addDownstreamTypeCallback), 
																				[], [], 
																				messages.traversalSetTypeBoundaryDialogTitle).createDialog();
		}), {
			label: messages.traversalSetTypeBoundary,
			imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/search-icon.gif')
		});
		
		this._fullTraceActionToolbar.addButton(this, dojo.hitch(this, function(){
			new dm.analysis.app.ui.internal.impactanalysis.config.AddTypeDialog(this, TYPE_ELEMENT, 
																				dojo.hitch(this, this.addFullTraceTypeCallback), 
																				[], [], 
																				messages.traversalSetTypeBoundaryDialogTitle).createDialog();
		}), {
			label: messages.traversalSetTypeBoundary,
			imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/search-icon.gif')
		});

		//Hide these two since they don't work in DM yet.
		//Any dependent application that wants to, can override this function and show these buttons again
		dojo.addClass(this._upstreamActionToolbar.domNode, 'hidden');
		dojo.addClass(this._downstreamActionToolbar.domNode, 'hidden');
		dojo.addClass(this._fullTraceActionToolbar.domNode, 'hidden');
	},

	loadTraversalDefinitionFromProfile: function(iaProfile)
	{
		if (!iaProfile) {
			console.warning("No profile specified for loading");
			return;
		}
		this.iaProfile = iaProfile;

		// Remove anything that may have been around from a previous Profile:
		this.removeUpstreamType();
		this.removeDownstreamType();

		/////////  Start setting values from the profile: /////////

		if (iaProfile.traversal && dojo.isString(iaProfile.traversal) && iaProfile.traversal !== ''){
			dojo.forEach(this._selectedTraversal.options, dojo.hitch(this, function(option){
				if (option.value === iaProfile.traversal){
					this._selectedTraversal.selectedIndex = option.index;
				}
			}));
			this.traversalChange(iaProfile.traversal);
		}

		if (!iaProfile || iaProfile.downstreamDepth === null || iaProfile.downstreamDepth === ''){
			if (iaProfile.downstreamType && iaProfile.downstreamType !== ''){
				this.addDownstreamTypeCallback('', '', { uri: iaProfile.downstreamType });
			} else {
				this._downstreamDepth.set('value', DEFAULT_DEPTH);
			}
		}
		else {
			var downVal = (dojo.isString(iaProfile.downstreamDepth) ? parseInt(iaProfile.downstreamDepth, 10) : iaProfile.downstreamDepth);
			this._downstreamDepth.set('value', (downVal > 0)?(downVal):(DEFAULT_DEPTH));
		}

		if (!iaProfile || iaProfile.upstreamDepth === null|| iaProfile.upstreamDepth === ''){
			if (iaProfile.upstreamType && iaProfile.upstreamType !== ''){
				this.addUpstreamTypeCallback('', '', { uri: iaProfile.upstreamType });
			} else {
				this._upstreamDepth.set('value', DEFAULT_DEPTH);
			}
		}
		else {
			var upVal = (dojo.isString(iaProfile.upstreamDepth) ? parseInt(iaProfile.upstreamDepth, 10) : iaProfile.upstreamDepth);
			this._upstreamDepth.set('value', (upVal > 0)?(upVal):(DEFAULT_DEPTH));
		}
		
		if (!iaProfile || iaProfile.fullTraceDepth === null|| iaProfile.fullTraceDepth === ''){
			if (iaProfile.fullTraceType && iaProfile.fullTraceType !== ''){
				this.addFullTraceTypeCallback('', '', { uri: iaProfile.fullTraceType });
			} else {
				this._fullTraceDepth.set('value', DEFAULT_DEPTH);
			}
		}
		else {
			var fullTraceVal = (dojo.isString(iaProfile.fullTraceDepth) ? parseInt(iaProfile.fullTraceDepth, 10) : iaProfile.fullTraceDepth);
			this._fullTraceDepth.set('value', (fullTraceVal > 0)?(fullTraceVal):(DEFAULT_DEPTH));
		}

		///////// Everything we just set above cannot be dirty yet, so don't let the editor think that we are: /////////

		this.editor.removeDirty(this._upstreamDepth, DIRTY_GROUP_ID);
		this.editor.removeDirty(this._downstreamDepth, DIRTY_GROUP_ID);
		this.editor.removeDirty(this._fullTraceDepth, DIRTY_GROUP_ID);
		this.editor.removeDirty(this._selectedTraversal, DIRTY_GROUP_ID);
	},

	addUpstreamTypeCallback: function(typeText, filterType, filterValue){
		if (this.upstreamLink) this.upstreamLink.destroyRecursive();
		if (this._upstreamLinkRemoveHandler) dojo.disconnect(this._upstreamLinkRemoveHandler);

		while(this._upstreamTypeDiv.hasChildNodes()){
			this._upstreamTypeDiv.removeChild(this._upstreamTypeDiv.lastChild);
		}

		dojo.addClass(this._upstreamDepth.domNode, 'hidden');
		dojo.removeClass(this._upstreamTypeDiv, 'hidden');

		this.upstreamLink = dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil.createResourceLink(this._upstreamTypeDiv, {
			uri: filterValue.uri,
			label: filterValue.uri
		});

		var link = dojo.create('a', {
			"class": "removeLink",
			title: "Remove"
		});
		link.appendChild(dojo.create('span'));
		this._upstreamTypeDiv.appendChild(link);
		this._upstreamLinkRemoveHandler = dojo.connect(link, "onclick", dojo.hitch(this, this._removeUpstreamTypeCallback));

		if (this.iaProfile) this.editor._updateDirty(this.iaProfile.upstreamType, filterValue.uri, this._upstreamDepth);
		if (this.profileInfoChangeCallback) this.profileInfoChangeCallback();
	},

	addDownstreamTypeCallback: function(typeText, filterType, filterValue){
		if (this.downstreamLink) this.downstreamLink.destroyRecursive();
		if (this._downstreamLinkRemoveHandler) dojo.disconnect(this._downstreamLinkRemoveHandler.destroyRecursive);

		while(this._downstreamTypeDiv.hasChildNodes()){
			this._downstreamTypeDiv.removeChild(this._downstreamTypeDiv.lastChild);
		}

		dojo.addClass(this._downstreamDepth.domNode, 'hidden');
		dojo.removeClass(this._downstreamTypeDiv, 'hidden');

		this.downstreamLink = dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil.createResourceLink(this._downstreamTypeDiv, {
			uri: filterValue.uri,
			label: filterValue.uri
		});

		var link = dojo.create('a', {
			"class": "removeLink",
			title: "Remove"
		});
		link.appendChild(dojo.create('span'));
		this._downstreamTypeDiv.appendChild(link);
		this._downstreamLinkRemoveHandler = dojo.connect(link, "onclick", dojo.hitch(this, this._removeDownstreamTypeCallback));

		if (this.iaProfile) this.editor._updateDirty(this.iaProfile.downstreamType, filterValue.uri, this._downstreamDepth);
		if (this.profileInfoChangeCallback) this.profileInfoChangeCallback();
	},

	addFullTraceTypeCallback: function(typeText, filterType, filterValue){
		if (this.fullTraceLink) this.fullTraceLink.destroyRecursive();
		if (this._fullTraceLinkRemoveHandler) dojo.disconnect(this._fullTraceLinkRemoveHandler.destroyRecursive);

		while(this._fullTraceTypeDiv.hasChildNodes()){
			this._fullTraceTypeDiv.removeChild(this._fullTraceTypeDiv.lastChild);
		}

		dojo.addClass(this._fullTraceDepth.domNode, 'hidden');
		dojo.removeClass(this._fullTraceTypeDiv, 'hidden');

		this.fullTraceLink = dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil.createResourceLink(this._fullTraceTypeDiv, {
			uri: filterValue.uri,
			label: filterValue.uri
		});

		var link = dojo.create('a', {
			"class": "removeLink",
			title: "Remove"
		});
		link.appendChild(dojo.create('span'));
		this._fullTraceTypeDiv.appendChild(link);
		this._fullTraceLinkRemoveHandler = dojo.connect(link, "onclick", dojo.hitch(this, this.removeFullTraceTypeCallback));

		if (this.iaProfile) this.editor._updateDirty(this.iaProfile.fullTraceType, filterValue.uri, this._fullTraceDepth);
		if (this.profileInfoChangeCallback) this.profileInfoChangeCallback();
	},
	
	/**
	 * Clear/destroy/disconnect/hide the Upstream Type widgets so that they are no longer visible.
	 * (Also, unhides Upstream Depth)
	 */
	removeUpstreamType: function(){
		if (this.upstreamLink){
			this.upstreamLink.destroyRecursive();
			this.upstreamLink = null;
		}
		if (this._upstreamLinkRemoveHandler) 
			dojo.disconnect(this._upstreamLinkRemoveHandler);

		while(this._upstreamTypeDiv.hasChildNodes()){
			this._upstreamTypeDiv.removeChild(this._upstreamTypeDiv.lastChild);
		}
		
		dojo.removeClass(this._upstreamDepth.domNode, 'hidden');
		dojo.addClass(this._upstreamTypeDiv, 'hidden');
	},

	_removeUpstreamTypeCallback: function(){
		var originalValue = "";
		var iaProfile = this.iaProfile;
		if (!iaProfile || iaProfile.upstreamDepth === null || iaProfile.upstreamDepth === ''){
			if (iaProfile.upstreamType && iaProfile.upstreamType !== ''){
				originalValue = iaProfile.upstreamType;
			}
		}
		else {
			originalValue = (dojo.isString(iaProfile.upstreamDepth) ? parseInt(iaProfile.upstreamDepth, 10) : iaProfile.upstreamDepth);
		}

		this.removeUpstreamType();
		
		if (this.iaProfile) this.editor._updateDirty(originalValue, this._upstreamDepth.get('value'), this._upstreamDepth);
		if (this.profileInfoChangeCallback) this.profileInfoChangeCallback();
	},

	/**
	 * Clear/destroy/disconnect/hide the Downstream Type widgets so that they are no longer visible.
	 * (Also, unhides Downstream Depth)
	 */
	removeDownstreamType: function(){
		if (this.downstreamLink){
			this.downstreamLink.destroyRecursive();
			this.downstreamLink = null;
		}
		if (this._downstreamLinkRemoveHandler)
			dojo.disconnect(this._downstreamLinkRemoveHandler.destroyRecursive);

		while(this._downstreamTypeDiv.hasChildNodes()){
			this._downstreamTypeDiv.removeChild(this._downstreamTypeDiv.lastChild);
		}

		dojo.removeClass(this._downstreamDepth.domNode, 'hidden');
		dojo.addClass(this._downstreamTypeDiv, 'hidden');
	},

	_removeDownstreamTypeCallback: function(){
		var originalValue = "";
		var iaProfile = this.iaProfile;
		if (!iaProfile || iaProfile.downstreamDepth === null || iaProfile.downstreamDepth === ''){
			if (iaProfile.downstreamType && iaProfile.downstreamType !== ''){
				originalValue = iaProfile.downstreamType;
			}
		}
		else {
			originalValue = (dojo.isString(iaProfile.downstreamDepth) ? parseInt(iaProfile.downstreamDepth, 10) : iaProfile.downstreamDepth);
		}

		this.removeDownstreamType();
		
		if (this.iaProfile) this.editor._updateDirty(originalValue, this._downstreamDepth.get('value'), this._downstreamDepth);
		if (this.profileInfoChangeCallback) this.profileInfoChangeCallback();
	},
	
	removeFullTraceTypeCallback: function(){
		var originalValue = this.fullTraceLink ? this.fullTraceLink.get('label') : "";
		
		if (this.fullTraceLink){
			this.fullTraceLink.destroyRecursive();
			this.fullTraceLink = null;
		}
		if (this._fullTraceLinkRemoveHandler)
			dojo.disconnect(this._fullTraceLinkRemoveHandler.destroyRecursive);

		while(this._fullTraceTypeDiv.hasChildNodes()){
			this._fullTraceTypeDiv.removeChild(this._fullTraceTypeDiv.lastChild);
		}

		dojo.removeClass(this._fullTraceDepth.domNode, 'hidden');
		dojo.addClass(this._fullTraceTypeDiv, 'hidden');
		
		if (this.iaProfile) this.editor._updateDirty(originalValue, this._fullTraceDepth.get('value'), this._fullTraceDepth);
		if (this.profileInfoChangeCallback) this.profileInfoChangeCallback();
	},

	getTraversalDefinition: function() {
		var upVal = this._upstreamDepth.get('value');
		var downVal = this._downstreamDepth.get('value');
		var fullTraceVal = this._fullTraceDepth.get('value');
		var upType = this.upstreamLink ? this.upstreamLink.get('label') : "";
		var downType = this.downstreamLink ? this.downstreamLink.get('label') : "";
		var fullType = this.fullTraceLink ? this.fullTraceLink.get('label') : "";
		
		var upEmpty = !upVal; //upVal is null or zero
		var downEmpty = !downVal; //downVal is null or zero
		var fullEmpty = !fullTraceVal;
		var upTypeSpecified = upType !== "";
		var downTypeSpecified = downType !== "";
		var fullSpecified = fullType !== "";
		
		var params = {
			UPSTREAM: this.UPSTREAM,
			DOWNSTREAM: this.DOWNSTREAM,
			BOTH: this.BOTH,
			FULL_TRACE: this.FULL_TRACE,
			traversal: this._selectedTraversal.value,
			upstreamDepth: (upTypeSpecified || upEmpty) ? null : upVal,
			downstreamDepth: (downTypeSpecified || downEmpty) ? null : downVal,
			fullTraceDepth: (fullSpecified || fullEmpty) ? null : fullTraceVal,		
			upstreamType: upType,
			downstreamType: downType,
			fullTraceType: fullType
		};

		return params;
	},
	
	getTraversalMode: function() {
		return this._selectedTraversal.value;
	},
	
	getUpstreamBoundType: function() {
		return this.upstreamLink ? this.upstreamLink.get('label') : "";
	},
	
	getDownstreamBoundType: function() {
		return this.downstreamLink ? this.downstreamLink.get('label') : "";
	},
	
	getFullTraceBoundType: function() {
		return this.fullTraceLink ? this.fullTraceLink.get('label') : "";
	},

	_traversalChange: function() {
		if (this.iaProfile)
			this.editor._updateDirty(this.iaProfile.traversal, this._selectedTraversal.value, this._selectedTraversal);
	
		this.traversalChange(this._selectedTraversal.value);
		
		if (this.profileInfoChangeCallback) this.profileInfoChangeCallback();
	},

	// Shows or hides the up downstream rows based on which traversal is selected
	traversalChange: function(traversal){
		switch(traversal){
			case this.UPSTREAM:
				this._toggleRowVisibility(true, false, false);
				break;
			case this.DOWNSTREAM:
				this._toggleRowVisibility(false, true, false);
				break;
			case this.BOTH:
				this._toggleRowVisibility(true, true, false);
				break;
			case this.FULL_TRACE:
				this._toggleRowVisibility(false, false, true);
				break;
			default:
				this._toggleRowVisibility(false, false, false);
		}
	},

	// For up and down stream rows
	_toggleRowVisibility: function(showUpstream, showDownstream, showFullTrace) {
		var upstreamRow = dojo.query('.upstream-row', this.domNode)[0];
		var downstreamRow = dojo.query('.downstream-row', this.domNode)[0];
		var fullTraceRow = dojo.query('.fullTrace-row', this.domNode)[0];

		if (showUpstream)
			dojo.removeClass(upstreamRow, "hidden");
		else
			dojo.addClass(upstreamRow, "hidden");

		if (showDownstream)
			dojo.removeClass(downstreamRow, "hidden");
		else
			dojo.addClass(downstreamRow, "hidden");
		
		if (showFullTrace)
			dojo.removeClass(fullTraceRow, "hidden");
		else
			dojo.addClass(fullTraceRow, "hidden");
	},


	destroy: function() {
		dojo.disconnect(this._traversalChangeHandle);
		dojo.disconnect(this._upstreamChangeHandle);
		dojo.disconnect(this._downstreamChangeHandle);
		dojo.destroy(this._traversalHoverHelp);

		if (this.upstreamLink) this.upstreamLink.destroyRecursive();
		if (this.downstreamLink) this.downstreamLink.destroyRecursive();

		if (this._upstreamLinkRemoveHandler) dojo.disconnect(this._upstreamLinkRemoveHandler);
		if (this._downstreamLinkRemoveHandler) dojo.disconnect(this._downstreamLinkRemoveHandler);

		this.inherited(arguments);
	}
});

}());