/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");

dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil = {

	_baseParameters: function (actionSuffix) {
		// summary:
		//		Return a parameters object initialized with an action
		return { action: 'dm.app.analysis.impactanalysis_' + actionSuffix };
	},
	
	getWelcomeUri: function() {
		return '#' + dojo.objectToQuery(this._baseParameters('welcome'));
	},
	

	/****** BROWSE (URIS) ******/

	getAllAnalysisUri: function() {
		return '#' + dojo.objectToQuery(this._baseParameters('allImpacts'));
	},
	getAllConfigAnalysisUri: function() {
		return '#' + dojo.objectToQuery(this._baseParameters('allIAConfigs'));
	},
	getAllProfileAnalysisUri: function() {
		return '#' + dojo.objectToQuery(this._baseParameters('allIAProfiles'));
	},


	/****** VIEW SPECIFIC RESOURCE (URIS) ******/

	getAnalysisUri: function(/*String*/uri) {
		var parameters = this._baseParameters('impactAnalysis');
		if (uri) {
			parameters.resourceUri = uri;
		}
		return '#' + dojo.objectToQuery(parameters);
	},
	getAnalysisConfigUri: function(/*String*/uri) {
		var parameters = this._baseParameters('iaConfig');
		parameters.resourceUri = uri;
		return '#' + dojo.objectToQuery(parameters);
	},
	getAnalysisProfileUri: function(/*String*/uri) {
		var parameters = this._baseParameters('iaProfile');
		parameters.resourceUri = uri;
		return '#' + dojo.objectToQuery(parameters);
	},


	/****** CREATE (URIS) ******/

	getNewAnalysisUri: function (/*String(Optional)*/resourceUri, /*String(Optional)*/ contextConfigVersionUri) {
		var parameters = this._baseParameters('impactAnalysis');
		parameters.ts = new Date().getTime();
		parameters.editMode = true;

		if (resourceUri) {
			parameters.targetResource = resourceUri;
		}
		
		if (contextConfigVersionUri) {
			parameters.contextConfigVersionUri = contextConfigVersionUri;
		}

		return '#' + dojo.objectToQuery(parameters);
	},
	getNewAnalysisConfigUri: function () {
		var parameters = this._baseParameters('iaConfig');
		parameters.ts = new Date().getTime();

		return '#' + dojo.objectToQuery(parameters);
	},
	
	getNewAnalysisProfileUri: function () {
		var parameters = this._baseParameters('iaProfile');
		parameters.ts = new Date().getTime();

		return '#' + dojo.objectToQuery(parameters);
	},


	
	/** 
	 * CREATE Resource:
	 * Will actually navigate the browser to the create pages
	 */
	createAnalysis: function() {
		dojo.hash(this.getNewAnalysisUri());
	},
	createAnalysisForResource: function(/*String*/resourceId, contextConfigVersionUri, windowName) {
		if (windowName == undefined)
			dojo.hash(this.getNewAnalysisUri(resourceId, contextConfigVersionUri));
		else
		{
			var currentLocation = window.location.href;
			var hashStart = currentLocation.indexOf('#');
			if (hashStart < 0) {
				hashStart = currentLocation.length;
			}			
			url = currentLocation.substring(0, hashStart) + this.getNewAnalysisUri(resourceId, contextConfigVersionUri);
			window.open(url, windowName);
		}
	},
	createAnalysisConfig: function() {
		location.href = this.getNewAnalysisConfigUri();
	},
	createAnalysisProfile: function() {
		dojo.hash(this.getNewAnalysisProfileUri());
	},

	isConfigVersion: function(uri)
	{
		var isConfigVersion = /\/productdef\/resource\/configurationVersion+_\d+$/.exec(uri);
		return !!isConfigVersion;
	},
};
}());