/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function () {
	dojo.provide('dm.analysis.app.ui.internal.impactanalysis.util.DiagramMenuUtil');

	dojo.require("jazz.ui.ResourceLink");
	dojo.require("dm.analysis.app.ui.DiagramUtilityMethods");
	dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");

	var AnalysisUtil = dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil;
	var DiagramUtilityMethods = dm.analysis.app.ui.DiagramUtilityMethods;
	
	var EXPANSION_KEY = "dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram.expansionSequence";

	var diagram = null;
	var editor = null;

	var _resourceLinks = [];

	dojo.mixin(dm.analysis.app.ui.internal.impactanalysis.util.DiagramMenuUtil, {
		init: function(args){
			diagram = args.diagram;
			editor = args.editor;
		},

		disableCallback_preview: function(node) {
			var data = node.getData();
			var disable = !data._resourceLink;
			return disable;
		},
		enableForNodes_preview: function(nodes){
			if (nodes.length !== 1) return false; //can only operate on a single node

			this.positionPreview(nodes[0]); //required to update the node with Preview information to determine whether we can even show a Preview
			return !this.disableCallback_preview(nodes[0]);
		},

		disableCallback_hide: function(node) {
			var data = node.getData();
			var uri  = data.item.URI;
			var disable = (uri == editor.editorInput.impactAnalysis.targetResourceUri);

			if (!disable){
				disable = dojo.some(editor.editorInput.impactAnalysis.addedElements, function(addedElement){
					return uri == addedElement;
				});
			}

			return disable;
		},
		enableForNodes_hide: function(nodes){
			if (nodes.length === 0) return false; //can only operate on a single node

			if (nodes.length === 1){
				return !this.disableCallback_hide(nodes[0]);
			}

			if (nodes.length > 1){
				var enable = true;
				//Can only enable this multi-node operation if ALL nodes allow the operation
				dojo.forEach(nodes, dojo.hitch(this, function(node){
					enable = enable && !this.disableCallback_hide(node);
				}));
				return enable;
			}
		},

		disableCallback_hideType: function(node) {
			var data = node.getData();
			var uri  = data.item.URI;
			var disable = (uri == editor.editorInput.impactAnalysis.targetResourceUri);

			if (!disable){
				disable = dojo.some(editor.editorInput.impactAnalysis.addedElements, function(addedElement){
					return uri == addedElement;
				});
			}

			if (!disable){
				var hasDmType = data.item.type && !!data.item.type[0];
				var hasRdfType = data.item.rdfTypes && !!data.item.rdfTypes[0];
				disable = !hasDmType && !hasRdfType;
			}

			return disable;
		},
		enableForNodes_hideType: function(nodes){
			if (nodes.length === 0) return false; //can only operate on a single node
			else if (nodes.length === 1) return !this.disableCallback_hideType(nodes[0]);
			else if (nodes.length > 1) return true;
		},

		/* Target artifacts, added artifacts, and already-expanded artifacts should not be expandable */
		disableCallback_expand: function(node) {
			var data = node.getData();
			var uri  = data.item.URI[0];
			var disable = (uri == editor.editorInput.impactAnalysis.targetResourceUri);

			if (!disable){
				disable = AnalysisUtil.isConfigVersion(uri);
			}
			
			if (!disable){
				disable = dojo.some(editor.editorInput.impactAnalysis.addedElements, function(addedElement){
					return uri == addedElement;
				});
			}

			if (!disable){
				disable = dojo.some(diagram.expansionSequence, function(existingExpansionUri) {
					return (uri == existingExpansionUri);
				});
			}

			if (!disable && data.item.distance && data.item.distance.length > 0){
				var nodeDistance = data.item.distance[0];
				var profile = editor.getProfileInfo();
				
				var inLinks = [];
				var outLinks = [];

				dojo.forEach(diagram.allRelations.items, function(link){
					if (link.start === uri){
						outLinks.push(link);
					} else if (link.end === uri){
						inLinks.push(link);
					}
				});

				disable = inLinks.length > 0 && outLinks.length > 0;
			}

			return disable;
		},
		enableForNodes_expand: function(nodes){
			if (nodes.length !== 1) return false; //can only operate on a single node
			return !this.disableCallback_expand(nodes[0]);
		},

		/* Only nodes that were previously expanded may be collapsed */
		disableCallback_collapse: function(node) {
			var data = node.getData();
			var uri  = data.item.URI[0];
			var disable = (uri == editor.editorInput.impactAnalysis.targetResourceUri);

			//If it has expansionChildren (minimum requirement of links), then it is collapsible
			if (!disable){
				disable = (!data.item.expansionChildren_Links || data.item.expansionChildren_Links.length === 0);
			}

			return disable;
		},
		enableForNodes_collapse: function(nodes){
			if (nodes.length !== 1) return false; //can only operate on a single node
			return !this.disableCallback_collapse(nodes[0]);
		},

		//if debug=true is on, show this menu item
		disableCallback_debug: function(node) {
			return !window.djConfig.isDebug;
		},
		enableForNodes_debug: function(nodes){
			if (nodes.length !== 1) return false; //can only operate on a single node
			return !this.disableCallback_debug(nodes[0]);
		},




		determineTypeName: function(uri){
			var split;
			var typeName = messages.unknownName;
			if (uri.indexOf("/") > 0) {
				split = uri.split("/");
				typeName = split[split.length-1];
			}
			if (uri.indexOf("#") > 0) {
				split = uri.split("#");
				typeName = split[split.length-1];
			}

			return typeName;
		},

		getNodeTypes: function(node){
			var data = node.getData();

			if (data.item.URI.length != 1){
				console.warn("For quick filtering, exactly one element should be selected. " + data.item.URI.length + " selected instead."); //$NON-NLS-1$  //$NON-NLS-2$
				return;
			}

			var uri  = data.item.URI[0];
			var name = data.item.name[0];
			var type = (data.item.type && data.item.type[0]) ? data.item.type[0] : ""; //this is from the node, set by the server, but the server only passes it in some DM-specific cases
			var rdfTypes = (data.item.rdfTypes) ? data.item.rdfTypes : [type];

			return rdfTypes;
		},



		previewNode: function(node){
			this.positionPreview(node);

			var data = node.getData();
			var resourceLink = data._resourceLink;

			// Show the preview action compact rendering.
			if (resourceLink) {
				
				//override the default message when info is not available
				resourceLink.popup.disable = function() {
					if (this._isOpen) {
						this._renderError(messages.infoNotAvailable);
						this._disableOnClose = true;
						return;
					}
					if (this.onDisable)
						this.onDisable();
					this.destroy();
				};
				
				setTimeout(dojo.hitch(resourceLink, function() {
					if (this.popup && !this.popup._isOpen) {
						this.popup.open();
					}
				}), 10);
			}
		},


		positionPreview: function(node){
			var resourceLinkDiv = this.createPreviewResourceLink(node);
			
			// Determine the coordinates for the popup menu.
			// The bounds for the node are relative to its ilog diagram.
			// Adjust the coordinates for the actual screen
			var viewport = diagram.impactDiagram.getViewport();
			var bounds = node.getBounds(null);
			var nodePos = viewport.pointContentToClient(bounds);
			var diagramPos = dojo.position(diagram.impactDiagram.domNode);
			var topLeft_x = Math.round(nodePos.x + diagramPos.x);
			var topLeft_y = Math.round(nodePos.y + diagramPos.y);
			
			// Set the position of the preview 'div" node to the coordinates 
			// of the top left corner of the graph element.
			if (resourceLinkDiv) {
				dojo.style(resourceLinkDiv, { "left": topLeft_x+'px', "top": topLeft_y+'px' });
			}
		},


		/** Create the preview resource link widget. The resource
		 *  link widget will position itself near to its parent
		 *  node. To accomplish this, a special div node is 
		 *  created as a child of the graph element's raw node
		 *  but it is will be given an absolute position on the
		 *  page just before the pop-up menu is displayed.
		 *
		 * @param graphElement the analysis Node or Link.
		 * @return the 'div" node for positioning the resource link widget (or null)
		 */
		createPreviewResourceLink: function(graphElement) {
			var data = graphElement.getData();
			
			// If the resource link has not been created yet
			if (!data._resourceLink) {
				// Compute the URI for rendering the resource information
				var resourceUri = data.item.URI[0];
				var renderingUri = DiagramUtilityMethods.call_getCompactRenderingUri(resourceUri);
			
				// Compute the presentation URI to open the resource
				var presentationUri = DiagramUtilityMethods.call_getResourceEditUri(resourceUri);
				if (!presentationUri) {
					presentationUri = resourceUri;
				}
				
				// If the resource does not have a URI for compact rendering
				if (!renderingUri) {
					// Disable the preview action for this resource
					data._resourceLink = null;
					data._resourceLinkDiv = null;
				}
				
				else {		
					// Create a <div> node that will be the parent of the 
					// resource link widget. This div node and the resource
					// link are created once per graph element but the div
					// node will be positioned each time the pop-up menu
					// is displayed to absolute coordinates.
					var domNode = diagram.domNode;
					var div = document.createElement("div");
					dojo.style(div, {
						"visibility": "hidden",
						"display":    "block",
						"position":   "fixed",
						"width":      "5px",
						"height":     "5px"
					});
					div.className = "resourceLinkDiv";
					domNode.appendChild(div);
					
					// Use our own 'srcNodeRef' so that we can set a target
					// which opens the link in a new window/tab
					var link = dojo.create('a', {
						href: presentationUri,
						target: '_blank'
					});

					// Create the resource link
					var resourceLink = new jazz.ui.ResourceLink({
						uri: renderingUri,
						presentationUri: presentationUri,
						lazyFetch : true,
						label: "-",
					}, link);
					
					div.appendChild(resourceLink.domNode);
					resourceLink.startup();
			
					// Save the resource link widget
					data._resourceLink = resourceLink;
					data._resourceLinkDiv = div;

					_resourceLinks.push(resourceLink);
				}
			}

			// Return the 'div' node to be positioned
			return data._resourceLinkDiv;
		},


		hideArtifacts: function(nodes){
			if (nodes.length === 1){
				this.hideArtifact(nodes[0], false);
			} else {
				if (confirm(messages.confirmHideArtifacts)){
					var self = this;
					dojo.forEach(nodes, function(node){
						self.hideArtifact(node, true);
					});
				}
				diagram.reloadDiagramAndTree(true);
			}
		},

		//node - Node to hide
		//multi - if true, will NOT reload the diagram after hiding the artifact, it's up the caller then to reload it once it's finished
		//				   also will NOT bring up the confirm dialog, that's also up to the caller
		hideArtifact: function(node, multi){
			var data = node.getData();

			if (data.item.URI.length != 1){
				console.warn("For removal, exactly one element should be selected. " + data.item.URI.length + " selected instead.");
				return;
			}

			var uri  = data.item.URI[0];
			var name = data.item.name[0];
			var type = (data.item.type && data.item.type[0]) ? data.item.type[0] : "";
			
			// Determine the type of the resource
			type = this.determineTypeName(type);

			if (multi) {
				diagram.removeElement(uri, name, type);
			} else if (confirm(messages.confirmHideArtifact)){
				diagram.removeElementAndReload(uri, name, type);
			}
		},

		expandNode: function(node){
			var data = node.getData();
			var uris = data.item.URI;

			if (!uris || uris.length === 0){
				console.warn("No node selected..."); //$NON-NLS-1$
				return;
			}

			var deferred = diagram.expandNode(uris[0], function() {
				var expansions = diagram.expansionSequence;
				expansions.push(uris[0]);
				editor._updateDirty(editor.editorInput.impactAnalysis.expansionSequence, expansions, EXPANSION_KEY);
			});
		},

		collapseNode: function(node){
			var data = node.getData();
			var uris = data.item.URI;

			if (!uris || uris.length === 0){
				console.warn("No node selected..."); //$NON-NLS-1$
				return;
			}

			diagram.collapseNode(uris[0], function(collapsedUris) {
				var expansions = diagram.expansionSequence;
				dojo.forEach(collapsedUris, function(uri){
					expansions.splice(dojo.indexOf(expansions, uri), 1);
				});
				editor._updateDirty(editor.editorInput.impactAnalysis.expansionSequence, expansions, EXPANSION_KEY);
			});
		},

		debugNode: function(node){
			var data = node.getData();

			console.debug(data.item);
			console.debug(editor.editorInput.impactAnalysis);
		},

		destroyResourceLinks: function(){
			dojo.forEach(_resourceLinks, function(resourceLink, i) { resourceLink.destroy(); });

			// Remove all of the special 'div' resource link nodes from the diagram.
			var diagramDomNode = diagram.domNode;
			dojo.query(('.resourceLinkDiv'), diagramDomNode).forEach(function(divNode, index) {
				diagramDomNode.removeChild(divNode);
			});
		}
	});
}());