/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.common.app.ui.internal.rhlink.IAInfoClient");

dojo.require("dojox.xml.DomParser");
dojo.require("rmps.core.resource.ResourceInfo");

dojo.require("jazz.client.xhr");

dm.common.app.ui.internal.rhlink.IAInfoClient = {
	
	getInfo: function(resourceUris, properties, baseline) {
		return this._getInfos(resourceUris, properties || ['*'], baseline);
	},

	_getInfos: function(resourceUris, properties, baseline) {
		var uris = resourceUris;
		if (!dojo.isArray(uris)) {
			uris = [uris];
		}
		var queryParams = {};
		var key = uris.join(',');
		if (properties) {
			properties.sort();
			queryParams.properties = properties.join(',');
			key += queryParams.properties;
		}
		
		// Baseline
		
		if(!baseline){
			baseline = jazz.app.context.get().baseline;
		}
		if (baseline) {
			queryParams['rmps.context'] = baseline;
		}
		
		var args = {
			url: net.jazz.ajax._contextRoot + "/analysisinfo?"+  dojo.objectToQuery(queryParams),
			handleAs: "json",
			headers: {
				"Content-Type": "application/rdf+xml",
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			load: function(response, ioArgs) {
				var descriptions = response["rdf:Description"];
				var resourceInfos = [];
				
				if (descriptions) {
					for (var i = 0; i < descriptions.length; ++i) {
						var resourceInfo = new rmps.core.resource.ResourceInfo();
						resourceInfo.parseDescription(descriptions[i]);
						resourceInfos.push(resourceInfo);
					}
					
					if (resourceInfos.length > 0) {
						return resourceInfos;
					}
				} 
				
				var dError = new dojo.Deferred();
				dError.errback(rmps.core.error.createWarning("Requested resource \'" + resourceUris + "\' is not available."));
				return dError;
			},
			error: function(response, ioArgs) {
				console.error(response);
			},
			haseBody: true,
			postData: this._createInfoRequest(uris),
			cacheKey: key
		};
		return rmps.core.xhrPost(args);
	},
	
	_createInfoRequest: function(uris) {
		function encodeXML(text) {
			var div = dojo.doc.createElement("div");
			div.appendChild(dojo.doc.createTextNode(text));
			return div.innerHTML;
		}
		
		var result = '<?xml version="1.0" encoding="UTF-8"?>\n<rmpsInfo:elements xmlns:rmpsInfo="http://jazz.net/xmlns/rmps/info/1.0/">\n';
		for (var i = 0; i < uris.length; ++i) {
			result += '<rmpsInfo:serverUri>';
			result += encodeXML(uris[i]);
			result += '</rmpsInfo:serverUri>\n';
		}
		result += '</rmpsInfo:elements>\n';
		return result;
	}

};

})();