/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
/*jslint nomen: true, sloppy: true, indent: 4, maxerr: 50 */
(function () {
	dojo.provide("dm.analysis.app.ui.picker");

	console.info('dm.analysis.app.ui.picker - initializing');
	
	// the default picker function does nothing
	var defaultPickerFunction = function () {
		console.error('dm.analysis.app.ui.picker: No Resource Picker specified. Use dm.analysis.app.ui.picker.setPicker(Function)');
	},
		pickerFunction = defaultPickerFunction;

	dm.analysis.app.ui.picker = {

		setPicker: function (picker) {
			// summary:
			//		Install a function that opens a host supplied resource picker.
			//		The passed function should accept no arguments, and return a resource URI (as a String)
			//		or 'undefined' if the picker is cancelled.
			// description:
			//		To install a picker function, create a Java script module similar to the following
			//		<code>
			//		(function () {
			//			dojo.provide('my.picker.provider');
			//			dojo.require('dm.analysis.app.ui.picker');
			//
			//			var pickerFunction = function (callbackFunction) {
			//				var resourceUri; // to be set by this function
			//				// logic to open a picker, and get a resource URI.
			//				if (resourceUri) {
			//					callbackFunction(resourceUri);
			//				}
			//			}
			//
			//			dm.analysis.app.ui.picker.setPicker(pickerFunction);
			//		}());
			//		</code>
			//		The picker provider should then be registered with the host applications 'webUiInitializer' extension point. Example:
			//		<code>
			//			<extension point="com.ibm.xtools.rmps.web.app.webUiInitializers">
			//				<initializer jsModule="my.picker.provider">
			//				</initializer>
			//			</extension>
			//		</code>
			pickerFunction = picker || defaultPickerFunction;
		},

		showPicker: function (callbackFunction) {
			// summary:
			//		Open a host-supplied picker to locate a host resource, returning the URI of the selected resource
			// description:
			//		Open the picker established by a call to setPicker

			if(!dojo.isFunction(callbackFunction)) {
				console.error('dm.analysis.app.ui.picker.showPicker: first argument is not a function');
			}
			pickerFunction(callbackFunction);
		}
	};

}());